//
//  Graph.h
//  QDMGraphsDouble
//
//  Created by Dominik Ixert on 31.01.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#ifndef QDMGraphsDouble_Graph_h
#define QDMGraphsDouble_Graph_h

#include <iostream>
#include <vector>
#include <fstream>
#include <iomanip>
#include <math.h>
#include <set>
#include <bitset>
#include <sstream>
#include <utility>
#include "globalVariables.h"
#include "nr3.h"
#include "StateTemplate.h"
#include "CouplingStruct.h"
#include <gmp.h>
#include <gmpxx.h>

using namespace std;
typedef vector<StateDouble> StateVector;

class Graph {
	friend Graph mergeGraphs(Graph const &g1, Graph const &g2);
    friend ostream& operator<<(ostream &o, Graph const &g);
    friend istream& operator>>(istream &o, Graph &g);
private:
    unsigned int n;
    unsigned int order;
    unsigned int graphNumber;

	vector<Doub> t; // contains all coupling-strengths
	list<CouplingStruct> couplingsCont;

	bool withoutSqrt;
	unsigned int unitaryCUT;

	static StateDouble hubbardSinglet;
	static StateDouble hubbardTriplet0;
    StateVector basis;
	StateVector tempBasis;

	unsigned int lls;
	StateVector initialStates;

	vector<unsigned int> QPNumbers;
    
	VecDoub H;
	VecDoub Heff;
	VecDoub UCUT;
	VecDoub Sorth;
    
    void createEigenstatesFromED(unsigned int matrixSize, MatDoub &evec, StateVector &vs, StateVector &eigenstates);
    
    bool performCUTQP(Doub stepEnd, Doub stepWidth, Doub epsAbs, Doub epsRel, bool createResFile, stringstream &ssResFile, bool abortAtMin);
    bool performCUTWegner(Doub stepEnd, Doub stepWidth, Doub epsAbs, Doub epsRel, bool createResFile, stringstream &ssResFile);
public:
	Graph();
    Graph(unsigned int nVal, unsigned int orderVal, unsigned int graphNumberVal, vector<Doub> const &tVal, unsigned int unitaryCUTVal=0,bool withoutSqrtVal=false);
	Graph(unsigned int nVal, unsigned int orderVal, unsigned int graphNumberVal, vector<Doub> const &tVal, list<CouplingStruct> const &couplingsContVal, unsigned int unitaryCUTVal=0, bool withoutSqrtVal=false);

	Graph(const Graph &g);
    Graph& operator=(const Graph &g);
	~Graph();
    StateDouble hubbardHamiltonian(StateDouble const &a, Doub const &U);
	StateDouble hubbardHamiltonian(StateDouble const &a, Doub const &U, vector<Doub> const &tVal);

	void createSingletCovering(list<list<pair<unsigned int,unsigned int> > > const &dimerCont, unsigned int orthAlg=2, bool unorthInitialStates=false);
	void createDimerCovering(list<list<pair<unsigned int,unsigned int> > > const &dimerCont, unsigned int qp=1, unsigned int orthAlg=2, bool unorthInitialStates=false);
	void createUpDownCovering(mpq_class const &SzVal=0, bool const &halfFilling=true);
	void createCoveringFromBasis(StateVector const &basisVal);
    
    StateDouble getState(unsigned int i);
    StateDouble getInitialState(unsigned int i);
    unsigned int size();
    unsigned int basisSize();
    unsigned int llsSize();
    Doub getH(unsigned int i, unsigned int j) const;
    Doub getHeff(unsigned int i, unsigned int j) const;
    Doub getU(unsigned int i, unsigned int j) const;
    Doub getSorth(unsigned int i, unsigned int j) const;
    void addHeff(unsigned int i, unsigned int j, Doub HeffVal);
    
    bool performCUT(Doub stepEnd, Doub stepWidth, Doub epsAbs, Doub epsRel, bool createResFile, stringstream &ssResFile, unsigned int generator=1, bool abortAtMin=false);
    
    unsigned int createBasis(unsigned int cutoff=0,unsigned int i=0,unsigned int appliedH=0);
    unsigned int createBasisForUpDownStates(unsigned int cutoff=0,unsigned int i=0,unsigned int appliedH=0);
    
    bool calculateLanczosGS(unsigned int maxCutoff, Doub epsAbs);

    void createHMatrix(bool iterative=false, bool unitaryTransformed=false);
    void createHMatrix(vector<Doub> const &tVal,bool iterative=false, bool unitaryTransformed=false);
    void transformHMatrixWithU();

    void createEmbeddedHeff(Graph const &gSub, list<vector<unsigned int> > const &embeddings, int const &embeddingFactor=1);

    VecDoub deriveEffSpinModel();

    void orthogonalizeBasis();
    void transformHeffWithSorth();

    void applyU(unsigned int blocks=1);
    StateVector getUnitaryTransformedBasis(unsigned int blocks=1);
    
    void unorthogonalizeHeff();
    void transformBasis(StateVector const &newBasis);

    unsigned int performED(VecDoub &eval, MatDoub &evec, vector<StateDouble> &vs, bool zeroSz=true, bool halfFilling=true, bool eigenstates=true, bool StotEigenstates=true);
	unsigned int calculateHubbardEigenvalues(VecDoub &eval);
	unsigned int countHubbardSinglets();
};

ostream& operator<<(ostream &o, Graph const &g);
istream& operator>>(istream &o, Graph &g);
Graph mergeGraphs(Graph const &g1, Graph const &g2);

#endif
