//
//  GraphController.cpp
//  QDMGraphsDouble
//
//  Created by Dominik Ixert on 31.01.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#include "GraphController.h"
#include <math.h>
#include <time.h>
#include <cctype>
#include "StateTemplate.h"
#include "CouplingStruct.h"
#include "numericLimits.h"
#include "functions.h"
#include "eigen_sym.h"
#include "CGKey.h"
#include <utility>

using namespace std;

GraphController::GraphController(stringstream &ssParametersConfigFile, bool testOutput){
	// Assign default values TODO: for all parameters!
	createResFile=false;
	abortAtMin=false;
	saveInStateForm=false;

	//// READ CONFIG FILE AND ASSIGN PARAMETER VALUES ////    
	// read parameters from parameters.config
	fstream parametersConfigFile;
	parametersConfigFile.open(ssParametersConfigFile.str().c_str(),fstream::in);

	string tempString;
	char tempChar;
	while(parametersConfigFile>>tempString){
		cout << tempString << "\n";
		if(tempString=="graphname"){
			if(!(parametersConfigFile>>graphname)){
				cerr << "reading error: graphname\n";
			}
		}
		if(tempString=="siteNumber"){
			if(!(parametersConfigFile>>siteNumber)){
				cerr << "reading error: siteNumber\n";
			}
		}
		if(tempString=="order"){
			if(!(parametersConfigFile>>order)){
				cerr << "reading error: order\n";
			}
		}
		if(tempString=="graphNumber"){
			if(!(parametersConfigFile>>graphNumber)){
				cerr << "reading error: graphNumber\n";
			}
		}
		if(tempString=="loadExistingGraph"){
			if(!(parametersConfigFile>>loadExistingGraph)){
				cerr << "reading error: loadExistingGraph\n";
			}
		}
		if(tempString=="calculationScheme"){
			if(!(parametersConfigFile>>calculationScheme)){
				cerr << "reading error: calculationScheme\n";
			}
		}
		if(tempString=="basisCreation"){
			if(!(parametersConfigFile>>basisCreation)){
				cerr << "reading error: basisCreation\n";
			}
		}
		if(tempString=="orthAlg"){
			if(!(parametersConfigFile>>orthAlg)){
				cerr << "reading error: orthAlg\n";
			}
		}
		if(tempString=="spinCalculation"){
			if(!(parametersConfigFile>>spinCalculation)){
				cerr << "reading error: spinCalculation\n";
			}
		}
		if(tempString=="withoutSqrt"){
			if(!(parametersConfigFile>>withoutSqrt)){
				cerr << "reading error: withoutSqrt\n";
			}
		}
		if(tempString=="onlyAfterCalculations"){
			if(!(parametersConfigFile>>onlyAfterCalculations)){
				cerr << "reading error: onlyAfterCalculations\n";
			}
		}
		if(tempString=="superpositionToSingleBasis"){
			if(!(parametersConfigFile>>superpositionToSingleBasis)){
				cerr << "reading error: superpositionToSingleBasis\n";
			}
		}
		if(tempString=="createPureGraph"){
			if(!(parametersConfigFile>>createPureGraph)){
				cerr << "reading error: createPureGraph\n";
			}
		}
		if(tempString=="HeffEigenvalues"){
			if(!(parametersConfigFile>>HeffEigenvalues)){
				cerr << "reading error: HeffEigenvalues\n";
			}
		}
		if(tempString=="effSpinModel"){
			if(!(parametersConfigFile>>effSpinModel)){
				cerr << "reading error: effSpinModel\n";
			}
		}
		if(tempString=="saveInStateForm"){
			if(!(parametersConfigFile>>saveInStateForm)){
				cerr << "reading error: saveInStateForm\n";
			}
		}
		if(tempString=="minCutoff"){
			if(!(parametersConfigFile>>minCutoff)){
				cerr << "reading error: minCutoff\n";
			}
		}
		if(tempString=="maxCutoff"){
			if(!(parametersConfigFile>>maxCutoff)){
				cerr << "reading error: maxCutoff\n";
			}
		}
		if(tempString=="t"){
			if(!(parametersConfigFile>>tStart)){
				cerr << "reading error: tStart\n";
			}
			if(!(parametersConfigFile>>tStep)){
				cerr << "reading error: tStep\n";
			}
			if(!(parametersConfigFile>>tEnd)){
				cerr << "reading error: tEnd\n";
			}
		}
		if(tempString=="s"){
			Doub x=0;
			while(parametersConfigFile.peek()!='\n'){
				parametersConfigFile>>x;
				tsFactor.push_back(x);
			}
		}
		if(tempString=="generator"){
			if(!(parametersConfigFile>>generator)){
				cerr << "reading error: generator\n";
			}
		}
		if(tempString=="stepEnd"){
			if(!(parametersConfigFile>>stepEnd)){
				cerr << "reading error: stepEnd\n";
			}
		}
		if(tempString=="stepWidth"){
			if(!(parametersConfigFile>>stepWidth)){
				cerr << "reading error: stepWidth\n";
			}
		}
		if(tempString=="epsAbs"){
			if(!(parametersConfigFile>>epsAbs)){
				cerr << "reading error: epsAbs\n";
			}
		}
		if(tempString=="epsRel"){
			if(!(parametersConfigFile>>epsRel)){
				cerr << "reading error: epsRel\n";
			}
		}
		if(tempString=="createResFile"){
			if(!(parametersConfigFile>>createResFile)){
				cerr << "reading error: createResFile\n";
			}
		}
		if(tempString=="abortAtMin"){
			if(!(parametersConfigFile>>abortAtMin)){
				cerr << "reading error: abortAtMin\n";
			}
		}

		// zero parameters
		Doub numLimEps=numeric_limits_epsilon<Doub>();
		if(tempString=="zeroEPS"){
			if(!(parametersConfigFile>>zeroEPS)){
				cerr << "reading error: zeroEPS\n";
			}
			if(zeroEPS<0){
				zeroEPS=numLimEps;
			}
		}
		if(tempString=="zeroNormQuadEPS"){
			if(!(parametersConfigFile>>zeroNormQuadEPS)){
				cerr << "reading error: zeroNormQuadEPS\n";
			}
			if(zeroNormQuadEPS<0){
				zeroNormQuadEPS=numLimEps;
			}
		}
		if(tempString=="zeroOrthoEPS"){
			if(!(parametersConfigFile>>zeroOrthoEPS)){
				cerr << "reading error: zeroOrthoEPS\n";
			}
			if(zeroOrthoEPS<0){
				zeroOrthoEPS=numLimEps;
			}
		}
		if(tempString=="zeroResEPS"){
			if(!(parametersConfigFile>>zeroResEPS)){
				cerr << "reading error: zeroResEPS\n";
			}
			if(zeroResEPS<0){
				zeroResEPS=numLimEps;
			}
		}
		if(tempString=="zeroGlsEPS"){
			if(!(parametersConfigFile>>zeroGlsEPS)){
				cerr << "reading error: zeroGlsEPS\n";
			}
			if(zeroGlsEPS<0){
				zeroGlsEPS=numLimEps;
			}
		}
		if(tempString=="zeroCutoffEPS"){
			if(!(parametersConfigFile>>zeroCutoffEPS)){
				cerr << "reading error: zeroCutoffEPS\n";
			}
			if(zeroCutoffEPS<0){
				zeroCutoffEPS=numLimEps;
			}
		}
	}

	parametersConfigFile.close();

	// read lattice
	stringstream ssLatticeConfigFile;
	ssLatticeConfigFile << "lattice_o" << order << "g" << graphNumber << ".config";
	fstream latticeConfigFile;
	latticeConfigFile.open(ssLatticeConfigFile.str().c_str(),fstream::in);
	getline(latticeConfigFile, tempString);
	latticeConfigFile >> siteNumber;
	latticeConfigFile >> skipws >> tempChar;
	getline(latticeConfigFile, tempString);
	unsigned int x;
	while(latticeConfigFile>>x){
		CouplingStruct csTemp;
		csTemp.from=x;
		latticeConfigFile >> csTemp.to;
		latticeConfigFile >> csTemp.coupling;
		couplingsCont.push_back(csTemp);
	}
	latticeConfigFile.close();


	// read dimer coverings
	if((basisCreation==1) || (calculationScheme==3)){
		stringstream ssDimerCoveringConfigFile;
		ssDimerCoveringConfigFile << "dimerCovering_o" << order << "g" << graphNumber << ".config";
		fstream dimerCoveringConfigFile;
		dimerCoveringConfigFile.open(ssDimerCoveringConfigFile.str().c_str(),fstream::in);

		getline(dimerCoveringConfigFile, tempString);
		getline(dimerCoveringConfigFile, tempString);
		unsigned int dimerNumber=0;
		dimerCoveringConfigFile >> dimerNumber;
		unsigned int x;
		while(dimerCoveringConfigFile>>x){
			list<pair<unsigned int,unsigned int> > lpuiui;
            pair<unsigned int,unsigned int> puiui;
            puiui.first=x;
            dimerCoveringConfigFile>>tempChar;
            dimerCoveringConfigFile>>puiui.second;
            lpuiui.push_back(puiui);
			for(unsigned int i=1;i<dimerNumber;i++){
				dimerCoveringConfigFile>>puiui.first;
				dimerCoveringConfigFile>>tempChar;
				dimerCoveringConfigFile>>puiui.second;
				lpuiui.push_back(puiui);
			}
			dimerCont.push_back(lpuiui);
		}
		dimerCoveringConfigFile.close();
	}

	// read parameters from embeddings.config
	if(createPureGraph || (calculationScheme==3)){
		stringstream ssEmbeddingsFile;
		ssEmbeddingsFile << "embeddings.config";
		fstream embeddingsFile;
		embeddingsFile.open(ssEmbeddingsFile.str().c_str());
		string tempstring;
		char tempChar;
		getline(embeddingsFile,tempString);
		getline(embeddingsFile, tempString);
		while(embeddingsFile>>skipws>>tempString){
			// read filename
			string filename=tempString;
			// read number of embeddings
			getline(embeddingsFile,tempString);
			int numberOfEmbeddings=0;
			int embeddingFactor=1;
			embeddingsFile >> numberOfEmbeddings;
			if(numberOfEmbeddings<0){
				embeddingFactor=-1;
				numberOfEmbeddings=-numberOfEmbeddings;
			}
			embeddingsFile >> skipws >> tempChar;
			// read number of sites
			getline(embeddingsFile,tempString);
			unsigned int numberOfSites=0;
			embeddingsFile >> numberOfSites;
			embeddingsFile >> skipws >> tempChar;
			// read embeddings
			getline(embeddingsFile,tempString);
			list<vector<unsigned int> > lvs;
			for(unsigned int j=0;j<numberOfEmbeddings;j++){
				vector<unsigned int> vs;
				for(unsigned int i=0;i<numberOfSites;i++){
					unsigned int x=0;
					embeddingsFile >> x;
					vs.push_back(x);
				}
				lvs.push_back(vs);
			}
			embeddings.insert(pair<string,pair<int,list<vector<unsigned int> > > >(filename,pair<int,list<vector<unsigned int> > >(embeddingFactor,lvs)));
			embeddingsFile >> noskipws >> tempChar >> skipws;
		}
		embeddingsFile.close();
	}

	// test output
	if (testOutput) {
		cout << "dataType=" << dataType << "\n";
		cout << "graphname=" << graphname << "\n";
		cout << "siteNumber=" << siteNumber << "\n";
		cout << "order=" << order << "\n";
		cout << "graphNumber=" << graphNumber << "\n";
		cout << "loadExistingGraph=" << loadExistingGraph << "\n";
		cout << "calculationScheme=" << calculationScheme << "\n";
		cout << "basisCreation=" << basisCreation <<  "\n";
		cout << "orthAlg=" << orthAlg << "\n";
		cout << "spinCalculation=" << spinCalculation << "\n";
		cout << "withoutSqrt=" << withoutSqrt << "\n";
		cout << "onlyAfterCalculations=" << onlyAfterCalculations << "\n";
		cout << "superpositionToSingleBasis=" << superpositionToSingleBasis << "\n";
		cout << "createPureGraph=" << createPureGraph << "\n";
		cout << "HeffEigenvalues=" << HeffEigenvalues << "\n";
		cout << "effSpinModel=" << effSpinModel << "\n";
		cout << "saveInStateForm=" << saveInStateForm << "\n";
		cout << "minCutoff=" << minCutoff << "\n";
		cout << "maxCutoff=" << maxCutoff << "\n";
		cout << "tStart=" << tStart << "\n";
		cout << "tEnd=" << tEnd << "\n";
		cout << "tStep=" << tStep << "\n";
		cout << "tsFactorSize=" << tsFactor.size() << "\n";
		cout << "tsFactor=";
		for (unsigned int i = 0; i < tsFactor.size(); i++) {
			cout << tsFactor[i] << " ";
		}
		cout << "\n";
		cout << "generator=" << generator << "\n";
		cout << "createResFile=" << createResFile << "\n";
		cout << "stepEnd=" << stepEnd << "\n";
		cout << "stepWidth=" << stepWidth << "\n";
		cout << "epsAbs=" << epsAbs << "\n";
		cout << "epsRel=" << epsRel << "\n";
		cout << "zeroEPS=" << zeroEPS << "\n";
		cout << "zeroNormQuadEPS=" << zeroNormQuadEPS << "\n";
		cout << "zeroOrthoEPS=" << zeroOrthoEPS << "\n";
		cout << "zeroResEPS=" << zeroResEPS << "\n";
		cout << "zeroGlsEPS=" << zeroGlsEPS << "\n";
		cout << "zeroCutoffEPS=" << zeroCutoffEPS << "\n";

		cout << "couplings:\n";
		for(list<CouplingStruct>::iterator it=couplingsCont.begin(); it!=couplingsCont.end(); it++){
			cout << it->from << "-" << it->coupling << "->" << it->to << "\n";
		}

		cout << "dimers:\n";
		for(list<list<pair<unsigned int,unsigned int> > >::iterator it1=dimerCont.begin();it1!=dimerCont.end();it1++){
			for(list<pair<unsigned int,unsigned int> >::iterator it2=it1->begin();it2!=it1->end();it2++){
				cout << it2->first << ">" << it2->second << " ";
			}
			cout << "\n";
		}

		cout << "embeddings:\n";
		for(std::map<string,pair<int,list<vector<unsigned int> > > >::iterator it1=embeddings.begin();it1!=embeddings.end();it1++){
			cout << it1->second.first << " * " << it1->first << ":\n";
			for(std::list<vector<unsigned int> >::iterator it2=it1->second.second.begin();it2!=it1->second.second.end();it2++){
				for(unsigned int i=0;i<it2->size();i++){
					cout << " " << (*it2)[i];
				}
				cout << "\n";
			}
		}
	}

}
GraphController::GraphController(const GraphController &gc){
	graphname=gc.graphname;
	siteNumber=gc.siteNumber;
	order=gc.order;
	graphNumber=gc.graphNumber;
	loadExistingGraph=gc.loadExistingGraph;
	calculationScheme=gc.calculationScheme;
	basisCreation=gc.basisCreation;
    orthAlg=gc.orthAlg;
    spinCalculation=gc.spinCalculation;
    withoutSqrt=gc.withoutSqrt;
    onlyAfterCalculations=gc.onlyAfterCalculations;
    superpositionToSingleBasis=gc.superpositionToSingleBasis;
    createPureGraph=gc.createPureGraph;
	HeffEigenvalues=gc.HeffEigenvalues;
	effSpinModel=gc.effSpinModel;
	saveInStateForm=gc.saveInStateForm;
	minCutoff=gc.minCutoff;
	maxCutoff=gc.maxCutoff;
	tStart=gc.tStart;
	tEnd=gc.tEnd;
	tStep=gc.tStep;
	tsFactor=gc.tsFactor;
	generator=gc.generator;
	createResFile=gc.createResFile;
	stepEnd=gc.stepEnd;
	stepWidth=gc.stepWidth;
	epsAbs=gc.epsAbs;
	epsRel=gc.epsRel;
	couplingsCont=gc.couplingsCont;
	dimerCont=gc.dimerCont;
	embeddings=gc.embeddings;
}

GraphController& GraphController::operator=(const GraphController &gc){
    if(this!=&gc){
    	graphname=gc.graphname;
    	siteNumber=gc.siteNumber;
    	order=gc.order;
    	graphNumber=gc.graphNumber;
    	loadExistingGraph=gc.loadExistingGraph;
    	calculationScheme=gc.calculationScheme;
    	basisCreation=gc.basisCreation;
        orthAlg=gc.orthAlg;
        spinCalculation=gc.spinCalculation;
        withoutSqrt=gc.withoutSqrt;
        onlyAfterCalculations=gc.onlyAfterCalculations;
        superpositionToSingleBasis=gc.superpositionToSingleBasis;
        createPureGraph=gc.createPureGraph;
    	HeffEigenvalues=gc.HeffEigenvalues;
    	effSpinModel=gc.effSpinModel;
    	saveInStateForm=gc.saveInStateForm;
    	minCutoff=gc.minCutoff;
    	maxCutoff=gc.maxCutoff;
    	tStart=gc.tStart;
    	tEnd=gc.tEnd;
    	tStep=gc.tStep;
    	tsFactor=gc.tsFactor;
    	generator=gc.generator;
    	createResFile=gc.createResFile;
    	stepEnd=gc.stepEnd;
    	stepWidth=gc.stepWidth;
    	epsAbs=gc.epsAbs;
    	epsRel=gc.epsRel;
    	couplingsCont=gc.couplingsCont;
    	dimerCont=gc.dimerCont;
    	embeddings=gc.embeddings;
    }
    return *this;
}

GraphController::~GraphController(){
}

void GraphController::performCalculation(){
	if(!onlyAfterCalculations){
		switch(calculationScheme){
			case 1:{
				performCalculationWithCutoffFlow();
				break;
			}
			case 2:{
				performCalculationWithLanczosED();
				break;
			}
			case 3:{
				performCalculationWithUpDownEmbedding();
				break;
			}
		}
	}
	if(superpositionToSingleBasis){
		performSuperpositionBasisToSingleBasisTransformation();
	}
	if(createPureGraph){
		performPureGraphSubtraction();
	}
	if(HeffEigenvalues){
		performHeffEigenvaluesCalculation();
	}
	if(effSpinModel){
		performEffSpinModelCalculation();
	}
	if(saveInStateForm){
		performSaveInStateForm();
	}
}

void GraphController::performCalculationWithCutoffFlow(){
	// create Basis (independent of couplings) TODO: also for DC and Sz Basis
	map<pair<mpq_class,mpq_class>,StateVector> stateMap;
	if(basisCreation==3){
		createSU2Basis(stateMap);
	}
	bool killLoops = false;
	for (unsigned int m = 0; m < tsFactor.size(); m++) {

		/*
		//hardcoded test
		Doub t1=0.14;
		Doub t2=0.15;
		for(unsigned int t_i=1;t_i<=2;t_i++){
			Doub t=0;
			if(t_i==1){
				t=t1;
			}
			if(t_i==2){
				t=t2;
			}
			*/


		for (Doub t = tStart; t <= tEnd; t += tStep) {
			// create instance of gComplete (this Graph contains all S-Channels)
			Doub s = t * tsFactor[m];

			//t=0.15;
			//s=-0.45;

			vector<Doub> tVal;
			tVal.push_back(t);
			tVal.push_back(s);
			Graph gComplete(siteNumber,order,graphNumber,tVal,couplingsCont,0,withoutSqrt);

			// set Smin and Smax dependent on chosen basisCreation
			mpq_class Smin=0;
			mpq_class Smax=0;
			switch(basisCreation){
				case 2:{
					Smax=mpq_class(siteNumber,2);
					break;
				}
				case 3:{
					if(siteNumber%2!=0){
						Smin=mpq_class(1,2);
					}
					Smax=mpq_class(siteNumber,2);
					break;
				}
			}

			for(mpq_class S=Smin;S<=Smax;S+=mpq_class(1,1)){
				for(mpq_class Sz=-S;Sz<=S;Sz+=mpq_class(1,1)){
					// initialize Graph for calculation in specialized S-Sector
					Graph g(siteNumber,order,graphNumber,tVal,couplingsCont,0,withoutSqrt);
	                unsigned int iTemp = 0;
	                unsigned int lastCutoff=0;
	                unsigned int basisSize = 0;
	                unsigned int lls = 0;


	                // declare filenames
	                fstream cutoffFile;
	                stringstream ssCutoffResults;
	                ssCutoffResults << "cutoffResults_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "S" << S << "Sz" << Sz << "c" << calculationScheme << "b" << basisCreation << ".txt";

	                fstream HbeforeCUTFile;
	                stringstream ssHbeforeCUTFile;
	                ssHbeforeCUTFile << "HbeforeCUT_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "S" << S << "Sz" << Sz << "c" << calculationScheme << "b" << basisCreation << ".txt";

	                fstream HafterCUTFile;
	                stringstream ssHafterCUTFile;
	                ssHafterCUTFile << "HafterCUT_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "S" << S << "Sz" << Sz << "c" << calculationScheme << "b" << basisCreation << ".txt";

	                fstream spinValueFile;
	                stringstream ssSpinValueFilename;
	              	ssSpinValueFilename << "spinValue_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "S" << S << "Sz" << Sz << "c" << calculationScheme << "b" << basisCreation << ".txt";

	                // choose apropriate covering creation
	                switch(basisCreation){
	                	case 1:{
	                		g.createSingletCovering(dimerCont,orthAlg,false);
	                		break;
	                	}
	                	case 2:{
	                		g.createUpDownCovering(Sz,true);
	                		break;
	                	}
	                	case 3:{
	                		g.createCoveringFromBasis(stateMap[pair<mpq_class,mpq_class>(S,Sz)]);
	                		break;
	                	}
	                }
	                basisSize = g.basisSize();
	                lls = g.llsSize();

	                /*
	                cutoffFile.open(ssCutoffResults.str().c_str(),fstream::out);
	                cutoffFile.close();
	                */

	                HbeforeCUTFile.open(ssHbeforeCUTFile.str().c_str(),fstream::out);
	                HbeforeCUTFile << setprecision(16) << fixed << showpoint;
	                HbeforeCUTFile << showpos;
	                HbeforeCUTFile.close();

	                HafterCUTFile.open(ssHafterCUTFile.str().c_str(), fstream::out);
	                HafterCUTFile << setprecision(16) << fixed;
	                HafterCUTFile << showpos;
	                HafterCUTFile.close();


	                if(spinCalculation!=0){
	                	spinValueFile.open(ssSpinValueFilename.str().c_str(),fstream::out);
	                	spinValueFile.close();
	                }

	                VecDoub HeffTemp(lls*lls);

	                std::streamsize ssPrec = std::cout.precision();
	                cout << setprecision(16) << "--t=" << t << " s=" << s << " S=" << S << " Sz=" << Sz << "--\n";
	                cout << setprecision(ssPrec);
	                for (unsigned int cutoff = minCutoff; cutoff <= maxCutoff; cutoff++) {
	                	cout << " cutoff=" << cutoff << ":\n";

	                	// create basis and hamilton matrix
	                	switch(basisCreation){
	                		case 1:{
	                			iTemp=g.createBasis(cutoff,iTemp,lastCutoff);
	                			break;
	                		}
	                		case 2:{
	                			iTemp = g.createBasisForUpDownStates(cutoff,iTemp,lastCutoff);
	                			break;
	                		}
	                		case 3:{
	                			iTemp=g.createBasis(cutoff,iTemp,lastCutoff);
	                			break;
	                		}
	                	}
	                	g.createHMatrix(true,false);
	                	lastCutoff=cutoff;
	                	basisSize = g.basisSize();
	                	cout << "  llsSize=" << g.llsSize() << "\n";
	                	cout << "  basisSize=" << basisSize << "\n";


	                	// saving H before CUT
	                	HbeforeCUTFile.open(ssHbeforeCUTFile.str().c_str(),fstream::out | fstream::app);
	                	for (unsigned int i = 0; i < basisSize; i++) {
	                		for (unsigned int j = 0; j < basisSize; j++) {
	                			HbeforeCUTFile << g.getH(i, j) << " ";
	                		}
	                		HbeforeCUTFile << "\n";
	                	}
	                	HbeforeCUTFile << "\n";
	                	HbeforeCUTFile.close();



	                	// perform CUT
		                stringstream ssResFile;
		                ssResFile << "resValue_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "S" << S << "Sz" << Sz << "cutoff" << cutoff << "c" << calculationScheme << "b" << basisCreation << ".txt";

	                	cout << "  start CUT\n";
	                	time_t startingTime = time(NULL);
	                	if (g.performCUT(stepEnd, stepWidth, epsAbs, epsRel, createResFile, ssResFile, generator, abortAtMin)) {
	                		cout << "  CUT=true\n";
	                	} else {
	                		cout << "  CUT=false\n";
	                		killLoops = true;
	                		break;
	                	}
	                	cout << "  CUT Processing Time: ";
	                	printProcessingTime(startingTime);

	    				// calculate spin values
	    				if(spinCalculation==2){
	    					spinValueFile.open(ssSpinValueFilename.str().c_str(),fstream::out | fstream::app);
	    					spinValueFile << cutoff;
	    					for(unsigned int i=0;i<basisSize;i++){
	    						spinValueFile << " " << g.getState(i).Stot();
	    			        }
	    					spinValueFile << "\n";
	    					spinValueFile.close();
	    			    }


	                	// saving H after CUT
	                	HafterCUTFile.open(ssHafterCUTFile.str().c_str(),fstream::out | fstream::app);
	                	for (unsigned int i = 0; i < basisSize; i++) {
	                		for (unsigned int j = 0; j < basisSize; j++) {
	                			HafterCUTFile << g.getHeff(i, j) << " ";
	                		}
	                		HafterCUTFile << "\n";
	                	}
	                	HafterCUTFile << "\n";
	                	HafterCUTFile.close();


	                	// save convergence value
	                	Doub convergenceValue=0;
	                	if(cutoff>minCutoff){
	                		for(unsigned int i=0;i<lls;i++){
	                			for(unsigned int j=0;j<lls;j++){
	                				convergenceValue+=(g.getHeff(i,j)-HeffTemp[i*lls+j])*(g.getHeff(i,j)-HeffTemp[i*lls+j]);
	                			}
	                		}
	                		convergenceValue=sqrt(convergenceValue)/(lls*lls);
	                	}

	                	/*
	                	// save cutoffResults
	                	cutoffFile.open(ssCutoffResults.str().c_str(),fstream::out | fstream::app);
	                	cutoffFile << cutoff << " ";
	                	cutoffFile << basisSize << " ";
	                	cutoffFile << setprecision(16);
	                	for (unsigned int i = 0; i < lls; i++) {
	                		for (unsigned int j = 0; j < lls; j++) {
	                			cutoffFile << g.getHeff(i, j) << " ";
	                		}
	                	}
	                	cutoffFile << setprecision(6);
	                	cutoffFile << convergenceValue;
	                	cutoffFile << "\n";
	                	cutoffFile.close();
	                	cout << "\n";
	                	*/

	                	// proof for convergence of Heff
	                	if(cutoff>minCutoff){
	                		bool cutoffConverged=true;
	                		for(unsigned int i=0;i<lls;i++){
	                			for(unsigned int j=0;j<lls;j++){
	                				if(myAbs(HeffTemp[i*lls+j]-g.getHeff(i,j))>zeroCutoffEPS){
	                					cutoffConverged=false;
	                					break;
	                				}
	                			}
	                			if(!cutoffConverged){
	                				break;
	                			}
	                		}
	                		if(cutoffConverged){
	                			break;
	                		}
	                	}
	                	for(unsigned int i=0;i<lls;i++){
	                		for(unsigned int j=0;j<lls;j++){
	                			HeffTemp[i*lls+j]=g.getHeff(i,j);
	                		}
	                	}
	                	if (killLoops) {
	                		break;
	                	}
	                }
	                if(killLoops){
	                	break;
	                }

	    			// spin calculation after complete calculation
	    			if(spinCalculation==1){
	    				spinValueFile.open(ssSpinValueFilename.str().c_str(),fstream::out | fstream::app);
	    			    spinValueFile << "0";
	    			    for(unsigned int i=0;i<basisSize;i++){
	    			    	spinValueFile << " " << g.getState(i).Stot();
	    			    }
	    			    spinValueFile << "\n";
	    			    spinValueFile.close();
	    			}

	                // merge g with gComplete
	                gComplete=mergeGraphs(gComplete,g);
				}
	            if (killLoops) {
	            	break;
	            }
			}
            // saving of graph after complete calculation
            stringstream ssGraphSavingComplete;
            ssGraphSavingComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
            fstream graphSavingCompleteFile;
            graphSavingCompleteFile.open(ssGraphSavingComplete.str().c_str(),fstream::out);
            graphSavingCompleteFile << gComplete;
            graphSavingCompleteFile.close();
		}
	}
}

void GraphController::performCalculationWithLanczosED(){
	// create all DimerCoverings
	vector<vector<pair<unsigned int,unsigned int> > > allDC;
	if(basisCreation==4){
		createAllDimerCoverings(allDC);
	}
	for (unsigned int m = 0; m < tsFactor.size(); m++) {
		for (Doub t = tStart; t <= tEnd; t += tStep) {
			// time output for each t/U step
			time_t startingTime = time(NULL);

			Doub s = t * tsFactor[m];
			vector<Doub> tVal;
			tVal.push_back(t);
			tVal.push_back(s);

			Graph g(siteNumber,order,graphNumber,tVal,couplingsCont,0,withoutSqrt);
			unsigned int basisSize = 0;
			unsigned int lls = 0;

			// declare filenames
			fstream spinValueFile;
			stringstream ssSpinValueFilename;
			ssSpinValueFilename << "spinValue_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".txt";

            switch(basisCreation){
                case 1:{
                    g.createSingletCovering(dimerCont,orthAlg,false);
                    break;
                }
                case 4:{
                	StateVector randomStateBasis;
            		StateDouble randomState=createRandomDimerCoveringSuperposition(allDC);
            		randomStateBasis.push_back(randomState);
                    g.createCoveringFromBasis(randomStateBasis);
                    break;
                }
            }
			basisSize = g.basisSize();
			lls = g.llsSize();

			if(spinCalculation!=0){
				spinValueFile.open(ssSpinValueFilename.str().c_str(),fstream::out);
				spinValueFile.close();
			}

			cout << "--t=" << t << " s=" << s << "--\n";
			if(g.calculateLanczosGS(maxCutoff,epsAbs)){
				cout << " Lanczos converged\n";
			}
			else{
				cout << " Lanczos NOT converged\n";
			}

			// spin calculation after complete calculation
			if(spinCalculation!=0){
				spinValueFile.open(ssSpinValueFilename.str().c_str(),fstream::out | fstream::app);
				spinValueFile << "0";
				for(unsigned int i=0;i<basisSize;i++){
					spinValueFile << " " << g.getState(i).Stot();
				}
				spinValueFile << "\n";
				spinValueFile.close();
			}
			// saving of graph after complete calculation
			stringstream ssGraphSavingComplete;
			ssGraphSavingComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphSavingCompleteFile;
			graphSavingCompleteFile.open(ssGraphSavingComplete.str().c_str(),fstream::out);
			graphSavingCompleteFile << g;
			graphSavingCompleteFile.close();

			printProcessingTime(startingTime);
		}
	}
}

void GraphController::performCalculationWithUpDownEmbedding(){
	for (unsigned int m = 0; m < tsFactor.size(); m++) {
		for (Doub t = tStart; t <= tEnd; t += tStep) {
			Doub s = t * tsFactor[m];
			vector<Doub> tVal;
			tVal.push_back(t);
			tVal.push_back(s);
			cout << "--t=" << t << " s=" << s << "--\n";
			// create dimer covering
			Graph g(siteNumber,order,graphNumber,tVal,couplingsCont,0,withoutSqrt);
			g.createSingletCovering(dimerCont,orthAlg,false);

			// iterate through all embeddings
			for(std::map<string,pair<int,list<vector<unsigned int> > > >::iterator it1=embeddings.begin();it1!=embeddings.end();it1++){
				// load subgraph
				Graph gSub;
				stringstream ssgSubFile;
				ssgSubFile << it1->first << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
				//cout << "gSubFilename=" << ssgSubFile.str() << "\n";
				fstream gSubFile;
				gSubFile.open(ssgSubFile.str().c_str(),fstream::in);
				gSubFile >> gSub;
				gSubFile.close();

				list<vector<unsigned int> > tempEmbeddings;
				for(std::list<vector<unsigned int> >::iterator it2=it1->second.second.begin();it2!=it1->second.second.end();it2++){
					vector<unsigned int> tempVector;
					for(unsigned int i=0;i<it2->size();i++){
						unsigned int x=2*(*it2)[i];
						tempVector.push_back(x);
						tempVector.push_back(x+1);
					}
					tempEmbeddings.push_back(tempVector);
				}
				g.createEmbeddedHeff(gSub,tempEmbeddings,it1->second.first);
			}

			// if orthAlg=0 orthogonalize states after calculation with overlap matrix
			if(orthAlg==0){
				g.orthogonalizeBasis();
				g.transformHeffWithSorth();
			}
			//////////////////////////////////////////////////////////////////////////
			// if loadExistingGraph is true, then load graph and add the new Heff
			if(loadExistingGraph){
				Graph gLoad(siteNumber,order,graphNumber,tVal,couplingsCont,0,withoutSqrt);
				stringstream ssgFile;
				ssgFile << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
				fstream gFile;
				gFile.open(ssgFile.str().c_str());
				gFile >> gLoad;
				gFile.close();

				for(unsigned int i=0;i<g.basisSize();i++){
					for(unsigned int j=0;j<g.basisSize();j++){
						g.addHeff(i,j,gLoad.getHeff(i,j));
					}
				}
			}
			//////////////////////////////////////////////////////////////////////////
			stringstream ssEmbeddedHeffFile;
			ssEmbeddedHeffFile << "embeddedHeff_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".txt";
			fstream embeddedHeffFile;
			embeddedHeffFile.open(ssEmbeddedHeffFile.str().c_str(),fstream::out);
			for(unsigned int i=0;i<g.basisSize();i++){
				for(unsigned int j=0;j<g.basisSize();j++){
					embeddedHeffFile << g.getHeff(i,j) << " ";
				}
				embeddedHeffFile << "\n";
			}
			embeddedHeffFile.close();

			// saving of graph after complete calculation
			stringstream ssGraphSavingComplete;
			ssGraphSavingComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphSavingCompleteFile;
			graphSavingCompleteFile.open(ssGraphSavingComplete.str().c_str(),fstream::out);
			graphSavingCompleteFile << g;
			graphSavingCompleteFile.close();
		}
	}
}

void GraphController::performSuperpositionBasisToSingleBasisTransformation(){
	StateVector newBasis;
	for(mpq_class Sz=-mpq_class(siteNumber,2);Sz<=mpq_class(siteNumber,2);Sz+=mpq_class(1,1)){
		vector<Doub> tVal;
		tVal.push_back(1.0);
		tVal.push_back(1.0);
		Graph gNew(siteNumber,order,graphNumber,tVal,couplingsCont,0,withoutSqrt);
		gNew.createUpDownCovering(Sz,true);
		for(unsigned int i=0;i<gNew.llsSize();i++){
			newBasis.push_back(gNew.getState(i));
		}
	}
	for (unsigned int m = 0; m < tsFactor.size(); m++) {
		for (Doub t = tStart; t <= tEnd; t += tStep) {
			Doub s = t * tsFactor[m];
			// Load Graph
			Graph g;
			stringstream ssGraphComplete;
			ssGraphComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphCompleteFile;
			graphCompleteFile.open(ssGraphComplete.str().c_str(),fstream::in);
			graphCompleteFile >> g;
			graphCompleteFile.close();

			// transformBasis
			g.transformBasis(newBasis);

			// save Graph
			stringstream ssGraphSavingComplete;
			ssGraphSavingComplete << graphname << "_UpDownBasis" << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphSavingCompleteFile;
			graphSavingCompleteFile.open(ssGraphSavingComplete.str().c_str(),fstream::out);
			graphSavingCompleteFile << g;
			graphSavingCompleteFile.close();
		}
	}
}

void GraphController::performPureGraphSubtraction(){
	for (unsigned int m = 0; m < tsFactor.size(); m++) {
		for (Doub t = tStart; t <= tEnd; t += tStep) {
			Doub s = t * tsFactor[m];
			// Load Graph
			Graph g;
			stringstream ssGraphComplete;
			ssGraphComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphCompleteFile;
			graphCompleteFile.open(ssGraphComplete.str().c_str(),fstream::in);
			graphCompleteFile >> g;
			graphCompleteFile.close();
			// iterate through all embeddings
			for(std::map<string,pair<int,list<vector<unsigned int> > > >::iterator it1=embeddings.begin();it1!=embeddings.end();it1++){
				// load subgraph
				Graph gSub;
				stringstream ssgSubFile;
				ssgSubFile << it1->first << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
				//cout << "gSubFilename=" << ssgSubFile.str() << "\n";
				fstream gSubFile;
				gSubFile.open(ssgSubFile.str().c_str(),fstream::in);
				gSubFile >> gSub;
				gSubFile.close();

				list<vector<unsigned int> > tempEmbeddings;
				for(std::list<vector<unsigned int> >::iterator it2=it1->second.second.begin();it2!=it1->second.second.end();it2++){
					vector<unsigned int> tempVector;
					for(unsigned int i=0;i<it2->size();i++){
						unsigned int x=2*(*it2)[i];
						tempVector.push_back(x);
						tempVector.push_back(x+1);
					}
					tempEmbeddings.push_back(tempVector);
				}

				g.createEmbeddedHeff(gSub,tempEmbeddings,it1->second.first);
			}
			// saving of graph after complete calculation
			stringstream ssGraphSavingComplete;
			ssGraphSavingComplete << graphname << "_pure_t" << t << "s" << tsFactor[m] << "calculationScheme" << "b" << basisCreation << ".graph";
			fstream graphSavingCompleteFile;
			graphSavingCompleteFile.open(ssGraphSavingComplete.str().c_str(),fstream::out);
			graphSavingCompleteFile << g;
			graphSavingCompleteFile.close();
		}
	}
}

void GraphController::performEffSpinModelCalculation(){
	// load graph for all t/U and s
	for(unsigned int m=0;m<tsFactor.size();m++){
		stringstream ssEffSpinModel;
		fstream effSpinModelFile;
		ssEffSpinModel << "effSpinModel_o" << order << "g" << graphNumber << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".txt";
		effSpinModelFile.open(ssEffSpinModel.str().c_str(),fstream::out);
		if(order==2){
			effSpinModelFile << "#t e0 J01\n";
		}
		if(order==3){
			effSpinModelFile << "#t e0 J01 J02 J12\n";
		}
		if(order==4){
			effSpinModelFile << "#t e0 J01 J02 J03 J12 J13 J23 J0123 J0213 J0312\n";
		}
		if(order==6){
			effSpinModelFile << "#t e0 J01 J02 J03 J04 J05 J12 J13 J14 J15 J23 J24 J24 J34 J34 J45 J0123 J0124 J0125 J0134 J0135 J0145 J0213 J0214 J0215 J0234 J0235 J0245 J0312 J0314 J0315 J0324 J0325 J0345 J0412 J0413 J0415 J0423 J0425 J0435 J0512 J0513 J0514 J0523 J0524 J0534 J1234 J1235 J1245 J1324 J1325 J1345 J1423 J1425 J1435 J1523 J1524 J1534 J2345 J2435 J2534 J012345 J012435 J012534 J021345 J021435 J021534 J031245 J031425 J031524 J041235 J041325 J041523 J051234 J051324 J051423\n";
		}
		effSpinModelFile << setprecision(16);
		effSpinModelFile.close();

		for(Doub t=tStart;t<=tEnd;t+=tStep){
			cout << "Derive effSpinModel for s=" << tsFactor[m] << " t=" << t << "\n";

			// Load Graph
			Graph g;
			stringstream ssGraphComplete;
			ssGraphComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphCompleteFile;
			graphCompleteFile.open(ssGraphComplete.str().c_str(),fstream::in);
			graphCompleteFile >> g;
			graphCompleteFile.close();

			// derive effSpinModel for this graph (return value is VecDoub J)
			VecDoub J=g.deriveEffSpinModel();

			// save to file
			effSpinModelFile.open(ssEffSpinModel.str().c_str(),fstream::out | fstream::app);
			effSpinModelFile << t;
			for(unsigned int i=0;i<J.size();i++){
				effSpinModelFile << " " << J[i];
			}
			/*
			for(unsigned int i=0;i<J.size();i++){
				if((i==1) || (i==3) || (i==4) || (i==6)){
					Doub x=2*J[i];
					if(i==4){
						x-=0.5*(sqrt(1+64*t*t)-1);
					}
					else{
						x-=0.5*(sqrt(1+16*t*t)-1);
					}
					effSpinModelFile << " " << x;
				}
				else{
					effSpinModelFile << " " << J[i];
				}
			}
			*/
			effSpinModelFile << "\n";
			effSpinModelFile.close();

		}
	}
}

void GraphController::performHeffEigenvaluesCalculation(){
	for (unsigned int m = 0; m < tsFactor.size(); m++) {
		stringstream ssHeffEigenvalues;
		fstream HeffEigenvaluesFile;
		ssHeffEigenvalues << "HeffEigenvalues_o" << order << "g" << graphNumber << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".txt";
		HeffEigenvaluesFile.open(ssHeffEigenvalues.str().c_str(),fstream::out);
		HeffEigenvaluesFile.close();

		stringstream ssGroundStateCoefficients;
		fstream GroundStateCoefficientsFile;
		if(calculationScheme==3){
			ssGroundStateCoefficients << "groundStateCoefficients_o" << order << "g" << graphNumber << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".txt";
			GroundStateCoefficientsFile.open(ssGroundStateCoefficients.str().c_str(),fstream::out);
			GroundStateCoefficientsFile.close();
		}

		for (Doub t = tStart; t <= tEnd; t += tStep) {
			Doub s = t * tsFactor[m];
			vector<Doub> tVal;
			tVal.push_back(t);
			tVal.push_back(s);
			// load graph
			Graph gLoad;
			stringstream ssGLoad;
			ssGLoad << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream gLoadFile;
			gLoadFile.open(ssGLoad.str().c_str(),fstream::in);
			gLoadFile >> gLoad;
			gLoadFile.close();

			// calculate eigenvalues
			unsigned int HeffMatSize=gLoad.llsSize();
			MatDoub HeffMat(HeffMatSize,HeffMatSize);
			for(unsigned int i=0;i<HeffMatSize;i++){
				for(unsigned int j=0;j<HeffMatSize;j++){
					HeffMat[i][j]=gLoad.getHeff(i,j);
				}
			}

			VecDoub eval;
			MatDoub evec;
			{
				Symmeig eigSysHeff(HeffMat);
				invertSorting(eigSysHeff.d,eigSysHeff.z);
				eval=eigSysHeff.d;
				evec=eigSysHeff.z;
			}

			// create eigenstates
			vector<StateDouble> vsTemp;
			for(unsigned int i=0;i<HeffMatSize;i++){
				StateDouble sTemp=evec[0][i]*gLoad.getState(0);
				for(unsigned int j=1;j<HeffMatSize;j++){
					sTemp+=evec[j][i]*gLoad.getState(j);
				}
				vsTemp.push_back(sTemp);
			}

			//calculate eigenstates of Stot in the blocks
			bool StotEigenstates=true;
			vector<StateDouble> eigenstatesVector;
			if(StotEigenstates){
				unsigned int i=0;
				while(i<vsTemp.size()){
					//cout << "eval[" << i << "]=" << eval[i] << " ";
					// find degenerate subspace
					vector<StateDouble> blockStates;
					vector<StateDouble> blockStatesStotOp;
					Doub E=eval[i];
					bool undefinedSpin=false;
					while((myAbs(eval[i]-E)<=zeroNormQuadEPS) && i<vsTemp.size()){
						//cout << "eval[i]-E=" << myAbs(eval[i]-E) << " zeroNormQuadEPS=" << zeroNormQuadEPS << "\n";
						StateDouble sTemp=vsTemp[i].StotOp();
						Doub StotVal=vsTemp[i].Stot(sTemp);
						//cout << "StotVal=" << StotVal << " ";
						if(StotVal<-0.5){
							undefinedSpin=true;
						}
						blockStates.push_back(vsTemp[i]);
						blockStatesStotOp.push_back(sTemp);
						i++;
					}
					unsigned int blockStatesSize=blockStates.size();

					if(undefinedSpin){
						//cout << " blockStatesSize=" << blockStatesSize << "\n";
						MatDoub StotMatrix(blockStatesSize,blockStatesSize);

						// calculate StotMatrix
						for(unsigned int j=0;j<blockStatesSize;j++){
							StotMatrix[j][j]=blockStates[j]*blockStatesStotOp[j];
							for(unsigned int k=0;k<j;k++){
								StotMatrix[j][k]=blockStates[j]*blockStatesStotOp[k];
								StotMatrix[k][j]=StotMatrix[j][k];
							}
						}

						//Symmeig eigSysStot(StotMatrix);
						Jacobi jac(StotMatrix);
						invertSorting(jac.d,jac.v);
						for(unsigned int j=0;j<blockStatesSize;j++){
							//StateDouble sTemp2=eigSysStot.z[j][0]*blockStates[0];
							StateDouble sTemp2=jac.v[0][j]*blockStates[0];
							for(unsigned int k=1;k<blockStatesSize;k++){
								//sTemp2+=eigSysStot.z[j][k]*blockStates[k];
								sTemp2+=jac.v[k][j]*blockStates[k];
							}
							eigenstatesVector.push_back(sTemp2);
							//cout << " jac.d[j]=" << jac.d[j] << " Stot=" << sTemp2.Stot() << "\n";
						}
					}
					else{
						for(unsigned int j=0;j<blockStatesSize;j++){
							eigenstatesVector.push_back(blockStates[j]);
						}
					}
				}
			}

			// save eigenvalues
			HeffEigenvaluesFile.open(ssHeffEigenvalues.str().c_str(),fstream::out | fstream::app);
			HeffEigenvaluesFile << setprecision(16);
			for(unsigned int i=0;i<eval.size();i++){
				HeffEigenvaluesFile << t << " " << eval[i] << " " << eigenstatesVector[i].Stot() << "\n";
			}
			HeffEigenvaluesFile.close();

			/*
			//TODO: save coefficients of GS eigenvector directly
			if(calculationScheme==3){
				GroundStateCoefficientsFile.open(ssGroundStateCoefficients.str().c_str(),fstream::out | fstream::app);
				GroundStateCoefficientsFile << setprecision(16);
				for(unsigned int j=0;j<eigSysHeff.z.nrows();j++){
					GroundStateCoefficientsFile << t << " " << j << " " << eigSysHeff.z[j][0] << "\n";
				}
				GroundStateCoefficientsFile.close();
			}
			*/

		}
	}
}

void GraphController::performSaveInStateForm(){
	// load graph for all t/U and s
	for(unsigned int m=0;m<tsFactor.size();m++){
		for(Doub t=tStart;t<=tEnd;t+=tStep){
			stringstream ssStateForm;
			fstream fsStateForm;
			ssStateForm << "HeffStateForm_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".txt";
			fsStateForm.open(ssStateForm.str().c_str(),fstream::out);
			fsStateForm << setprecision(16);
			fsStateForm.close();

			// save states file for the SpinModelCalculator
			stringstream ssStatesFileSMC;
			fstream fsStatesFileSMC;
			ssStatesFileSMC << "states_o" << order << "g" << graphNumber << "t" << t << "s" << tsFactor[m] << ".cfg";
			fsStatesFileSMC.open(ssStatesFileSMC.str().c_str(),fstream::out);
			fsStatesFileSMC << "#filename\n";
			fsStatesFileSMC << ssStateForm.str() << "\n";
			fsStatesFileSMC.close();
			////////////////////////////////////////////////

			// Load Graph
			Graph g;
			stringstream ssGraphComplete;
			ssGraphComplete << graphname << "_t" << t << "s" << tsFactor[m] << "c" << calculationScheme << "b" << basisCreation << ".graph";
			fstream graphCompleteFile;
			graphCompleteFile.open(ssGraphComplete.str().c_str(),fstream::in);
			graphCompleteFile >> g;
			graphCompleteFile.close();

			fsStateForm.open(ssStateForm.str().c_str(),fstream::out | fstream::app);
			unsigned int llsSize=g.llsSize();
			for(unsigned int i=0;i<llsSize;i++){
				fsStateForm << g.getState(i) << "\n";
				fsStateForm << "to ";
				StateDouble outputState=g.getHeff(i,0)*g.getState(0);
				for(unsigned int j=1;j<llsSize;j++){
					outputState+=g.getHeff(i,j)*g.getState(j);
				}
				fsStateForm << outputState;
				fsStateForm << "\n";
			}
			fsStateForm.close();
		}
	}
}

void GraphController::createSU2Basis(map<pair<mpq_class,mpq_class>,StateVector> &stateMap){
	// create all possible combinations for S
	StateDouble singleSpinUp(2,"10");
	StateDouble singleSpinDown(2,"01");
	mpq_class up(1,2);
	mpq_class down(-1,2);
	CGKey singleSpinKey(up,up);

	set<CGKey> spinTreeSet;
	spinTreeSet.insert(singleSpinKey);
	mpq_class maxJ=up;
	mpq_class minJ=up;
	for(unsigned int i=2;i<=siteNumber;i++){
		set<CGKey> tempSet;
		for(set<CGKey>::iterator it1=spinTreeSet.begin();it1!=spinTreeSet.end();it1++){
			mpq_class JJlast=it1->getJJ().back();
			CGKey tempKey=*it1;
			if(JJlast>=up){
				tempKey.addJ(up);
				tempKey.addJJ(JJlast-up);
				tempSet.insert(tempKey);
				tempKey=*it1;
			}
			tempKey.addJ(up);
			tempKey.addJJ(JJlast+up);
			tempSet.insert(tempKey);
		}
		spinTreeSet.swap(tempSet);
	}

	// create all possible m-value combinations (m1,m2,m12,m3,m123,m4,m1234,...,mn)
	set<vector<mpq_class> > mSet;
	vector<mpq_class> vm;
	vm.push_back(up);
	mSet.insert(vm);
	vm.clear();
	vm.push_back(down);
	mSet.insert(vm);
	for(unsigned int i=1;i<siteNumber-1;i++){
		set<vector<mpq_class> > tempMSet;
		for(set<vector<mpq_class> >::iterator it1=mSet.begin();it1!=mSet.end();it1++){
			vector<mpq_class> tempVector=*it1;
			tempVector.push_back(up);
			tempVector.push_back(up);
			tempMSet.insert(tempVector);
			tempVector=*it1;
			tempVector.push_back(down);
			tempVector.push_back(down);
			tempMSet.insert(tempVector);
		}
		mSet.swap(tempMSet);
	}
	set<vector<mpq_class> > tempMSet;
	for(set<vector<mpq_class> >::iterator it1=mSet.begin();it1!=mSet.end();it1++){
		vector<mpq_class> tempVector=*it1;
		tempVector.push_back(up);
		tempMSet.insert(tempVector);
		tempVector=*it1;
		tempVector.push_back(down);
		tempMSet.insert(tempVector);
	}
	mSet.swap(tempMSet);
	tempMSet.clear();


	// now take the set and calculate the states
	//map<pair<mpq_class,mpq_class>,StateVector> stateMap; //stores all states with (S,Sz)
	unsigned int spinTreeSetSize=spinTreeSet.size();
	unsigned int spinTreeSetIndex=1;
	for(set<CGKey>::iterator it1=spinTreeSet.begin();it1!=spinTreeSet.end();it1++){
		cout << spinTreeSetIndex << "/" << spinTreeSetSize << "\n";
		spinTreeSetIndex++;
		mpq_class J=it1->getJJ().back();
		for(mpq_class M=-J;M<=J;M+=mpq_class(1,1)){
			StateDouble tempState(2*siteNumber);
			cout << " J=" << J << " M=" << M << "\n";
			for(set<vector<mpq_class> >::iterator it2=mSet.begin();it2!=mSet.end();it2++){
				vector<mpq_class> vj;
				vj.push_back(it1->getJ(0));
				for(unsigned int i=1;i<it1->getJSize();i++){
					vj.push_back(it1->getJ(i));
					vj.push_back(it1->getJJ(i));
				}
				vector<mpq_class> vm=*it2;
				vm.push_back(M);

				/*
				cout << "vj=";
				for(unsigned int i=0;i<vj.size();i++){
					cout << vj[i] << " ";
				}
				cout << "\n";
				cout << "vm=";
				for(unsigned int i=0;i<vm.size();i++){
					cout << vm[i] << " ";
				}
				cout << "\n";
				*/

				Doub CG=calculateGeneralClebschGordanCoefficient(vj,vm);
				//cout << "CG=" << CG << "\n";
				// create product state
				StateDouble prodState;
				if(vm[0]==up){
					prodState=StateDouble(2,"10");
				}
				else{
					if(vm[0]==down){
						prodState=StateDouble(2,"01");
					}
				}
				for(unsigned int i=0;i<siteNumber-1;i++){
					if(vm[2*i+1]==up){
						prodState=productState(prodState,StateDouble(2,"10"));
					}
					else{
						if(vm[2*i+1]==down){
							prodState=productState(prodState,StateDouble(2,"01"));
						}
					}
				}
				tempState+=CG*prodState;
			}
			if(!tempState.isEmpty()){
				StateVector tempStateVector=stateMap[pair<mpq_class,mpq_class>(J,M)];
				tempStateVector.push_back(tempState);
				stateMap[pair<mpq_class,mpq_class>(J,M)]=tempStateVector;
			}
		}
	}
}

void GraphController::createAllDimerCoverings(vector<vector<pair<unsigned int,unsigned int> > > &allDC){
	vector<vector<bool> > isFixed;
	unsigned int fixedNumber=0;
	vector<bool> notFixed(siteNumber,false);
	pair<unsigned int,unsigned int> puiui;
	puiui.first=0;
	for(unsigned int i=1;i<siteNumber;i++){
		vector<pair<unsigned int,unsigned int> > DC;
		puiui.second=i;
		DC.push_back(puiui);
		allDC.push_back(DC);
		isFixed.push_back(notFixed);
		isFixed.back()[0]=true;
		isFixed.back()[i]=true;
	}
	fixedNumber+=2;
	while(fixedNumber<siteNumber){
		vector<vector<pair<unsigned int,unsigned int> > > allDC2;
		vector<vector<bool> > isFixed2;
		for(unsigned int k=0;k<allDC.size();k++){
			for(unsigned int i=1;i<siteNumber-1;i++){
				if(!isFixed[k][i]){
					for(unsigned int j=i+1;j<siteNumber;j++){
						if(!isFixed[k][j]){
							allDC2.push_back(allDC[k]);
							pair<unsigned int,unsigned int> puiui(i,j);
							allDC2.back().push_back(puiui);
							isFixed2.push_back(isFixed[k]);
							isFixed2.back()[i]=true;
							isFixed2.back()[j]=true;
						}
					}
				}
			}
		}
		allDC.swap(allDC2);
		isFixed.swap(isFixed2);
		fixedNumber+=2;
	}
}
StateDouble GraphController::createRandomDimerCoveringSuperposition(const vector<vector<pair<unsigned int,unsigned int> > > &allDC){
    StateDouble hubbardSinglet=(StateDouble(4,"1001")-StateDouble(4,"0110"))/Doub(sqrt(2));
    StateDouble randomState(2*siteNumber);

    //StateVector allDCStates;
    vector<bool> vb(2*siteNumber,false);
    for(unsigned int i=0;i<allDC.size();i++){
        StateDouble s1(vb);
        for(unsigned int j=0;j<allDC[i].size();j++){
            vector<unsigned int> pos;
            pos.push_back(2*(allDC[i][j].first));
            pos.push_back(2*(allDC[i][j].first)+1);
            pos.push_back(2*(allDC[i][j].second));
            pos.push_back(2*(allDC[i][j].second)+1);
            s1=insertState(s1,hubbardSinglet,pos);
            //cout << "s1=" << s1 << "\n";
        }
        randomState+=Doub(rand())*s1;
    }
    
    randomState/=sqrt(randomState.normQuad());
    
    return randomState;
}
