//
//  GraphController.h
//  QDMGraphsDouble
//
//  Created by Dominik Ixert on 31.01.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#ifndef QDMGraphsDouble_GraphController_h
#define QDMGraphsDouble_GraphController_h

#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include <utility>
#include <gmp.h>
#include <gmpxx.h>
#include "Graph.h"
#include "globalVariables.h"
#include "nr3.h"

using namespace std;

class GraphController {
private:
	string graphname;
	unsigned int siteNumber;
	unsigned int order;
	unsigned int graphNumber;
	bool loadExistingGraph;

	unsigned int calculationScheme;
	unsigned int basisCreation;
	unsigned int orthAlg;
	unsigned int spinCalculation;
	bool withoutSqrt;

	bool onlyAfterCalculations;

	bool superpositionToSingleBasis;
	bool createPureGraph;
	bool HeffEigenvalues;
	bool effSpinModel;
	bool saveInStateForm;

	unsigned int minCutoff;
	unsigned int maxCutoff;

	Doub tStart;
	Doub tStep;
	Doub tEnd;
	vector<Doub> tsFactor;

	unsigned int generator;
	Doub stepEnd;
	Doub stepWidth;
	Doub epsAbs;
	Doub epsRel;
	bool createResFile;
	bool abortAtMin;
    
	list<CouplingStruct> couplingsCont;
	list<list<pair<unsigned int,unsigned int> > > dimerCont;
	map<string,pair<int,list<vector<unsigned int> > > > embeddings;

	void performCalculationWithCutoffFlow();
	void performCalculationWithLanczosED();
	void performCalculationWithUpDownEmbedding();

	void performSuperpositionBasisToSingleBasisTransformation();
	void performHeffEigenvaluesCalculation();
	void performPureGraphSubtraction();

	void performEffSpinModelCalculation();

	void performSaveInStateForm();

	void createSU2Basis(map<pair<mpq_class,mpq_class>,StateVector> &stateMap);
	void createAllDimerCoverings(vector<vector<pair<unsigned int,unsigned int> > > &allDC);
    StateDouble createRandomDimerCoveringSuperposition(const vector<vector<pair<unsigned int,unsigned int> > > &allDC);

public:
	GraphController(stringstream &ssParametersConfigFile, bool testOutput);
	GraphController(const GraphController &gc);
	GraphController& operator=(const GraphController &gc);
	~GraphController();
    
    void performCalculation();
};

#endif
