/*
 * SpinStateTemplate.h
 *
 *  Created on: Feb 4, 2013
 *      Author: ixert
 */

#ifndef SPINSTATETEMPLATE_H_
#define SPINSTATETEMPLATE_H_

#include "StateTemplate.h"

template<class T>
class SpinStateTemplate : public StateTemplate<T> {
public:
	SpinStateTemplate<T> SUp(unsigned int k) const;
	SpinStateTemplate<T> SDown(unsigned int k) const;
	SpinStateTemplate<T> SUpDown(unsigned int k, unsigned int j) const;
	SpinStateTemplate<T> Sz(unsigned int k) const;

	SpinStateTemplate<T> StotOp();
};

//// implementation ////
//// class functions ////

template<class T>
SpinStateTemplate<T> SpinStateTemplate<T>::SUp(unsigned int k) const{
	// applies the operator S^+_k to the state
	if(k<n){
		typename std::map<vector<bool>,T>::const_iterator it;
		SpinStateTemplate<T> s(n);
		vector<bool> v;
		it=stateCont.begin();
		while(it!=stateCont.end()){
			v=it->first;
			if(!(it->first[k])){
				v=it->first;
				v[k]=true;
				s.stateCont.insert(pair<vector<bool>,T>(v,it->second));
			}
		}
		return s;
	}
	else{
		cerr << "SpinState SpinState::SUp(unsigned int k): index out of range\n";
		return *this;
	}
}
template<class T>
SpinStateTemplate<T> SpinStateTemplate<T>::SDown(unsigned int k) const{
	// applies the operator S^-_k to the state
	if(k<n){
		typename std::map<vector<bool>,T>::const_iterator it;
		SpinStateTemplate<T> s(n);
		vector<bool> v;
		it=stateCont.begin();
		while(it!=stateCont.end()){
			v=it->first;
			if((it->first[k])){
				v=it->first;
				v[k]=false;
				s.stateCont.insert(pair<vector<bool>,T>(v,it->second));
			}
		}
		return s;
	}
	else{
		cerr << "SpinState SpinState::SDown(unsigned int k): index out of range\n";
		return *this;
	}
}
template<class T>
SpinStateTemplate<T> SpinStateTemplate<T>::SUpDown(unsigned int k, unsigned int j) const{
	// applies the operator S^+_k S^-_j to the state
	// this method is faster than the stand-alone routines SUp(k) and SDown(j)
	if((k<n) && (j<n)){
		typename std::map<vector<bool>,T>::const_iterator it;
		SpinStateTemplate<T> s(n);
		vector<bool> v;
		it=stateCont.begin();
		while(it!=stateCont.end()){
			if(!(it->first[k]) && (it->first[j])){
				v=it->first;
				v[k]=true;
				v[j]=false;
				s.stateCont.insert(pair<vector<bool>,T>(v,it->second));
			}
		}
		return s;
	}
	else{
		cerr << "SpinState SpinState::SUpDown(unsigned int k, unsigned int j): index out of range\n";
		return *this;
	}
}

template<class T>
SpinStateTemplate<T> SpinStateTemplate<T>::StotOp(){
	// applies the total spin operator S^2 to the state
	SpinStateTemplate<T> c(n);
	// run over all sites
	for (unsigned int i = 0; i < n; i++) {
		for(unsigned int j=0;j<n;j++) {
			// S_i^+ S_j^-
			c+=(this->SUpDown(i,j))/T(2);
			// S_i^- S_j^+
			c+=(this->SUpDown(j,i))/T(2);
			// S_i^z S_j^z
			c+=(this->Sz(i)).Sz(j);
		}
	}
	return c;
}

typedef SpinStateTemplate<Doub> SpinStateDouble;

#endif /* SPINSTATETEMPLATE_H_ */
