/*
 * StateTemplate.h
 *
 *  Created on: 19.12.2011
 *      Author: ixert
 */

#ifndef STATETEMPLATE_H_
#define STATETEMPLATE_H_

#include <iostream>
#include <list>
#include <map>
#include <vector>
#include <math.h>
#include <sstream>
#include <set>
#include "globalVariables.h"
#include "mathFunctions.h"
using namespace std;

template<class T>
class StateTemplate {
template<class S>
friend ostream& operator<<(ostream &o, StateTemplate<S> const &s);
template<class S>
friend istream& operator>>(istream &o, StateTemplate<S> &s);
template<class S>
friend S operator*(StateTemplate<S> const &s1, StateTemplate<S> const &s2);
template<class S>
friend StateTemplate<S> productState(StateTemplate<S> const &s1, StateTemplate<S> const &s2);
template<class S>
friend StateTemplate<S> insertState(StateTemplate<S> const &s1, StateTemplate<S> const &s2, unsigned int *pos);
template<class S>
friend StateTemplate<S> insertState(StateTemplate<S> const &s1, StateTemplate<S> const &s2, vector<unsigned int> const &pos);
template<class S>
friend S partialScalarProduct(StateTemplate<S> const &s1, StateTemplate<S> const &s2, vector<unsigned int> const &pos);
template<class S>
friend S partialEntangledScalarProduct(StateTemplate<S> const &s1, StateTemplate<S> const &s2, StateTemplate<S> const &sSub1, StateTemplate<S> const &sSub2, vector<unsigned int> const &pos, vector<unsigned int> const &posNot);
template<class S>
friend S partialEntangledScalarProduct(const typename std::vector<StateTemplate<S> >::const_iterator s1, const typename std::vector<StateTemplate<S> >::const_iterator s2, const typename std::vector<StateTemplate<S> >::const_iterator sSub1, const typename std::vector<StateTemplate<S> >::const_iterator sSub2, vector<unsigned int> const &pos, vector<unsigned int> const &posNot);
public: //TODO: wieder private machen, dafuer muss createEmbeddedHeff in Graph.cpp angepasst werden
	unsigned int n;
	map<vector<bool>,T> stateCont;

	unsigned int QP(vector<bool> v);
public:
	StateTemplate();
	StateTemplate(vector<bool> &v);
	StateTemplate(vector<bool> &v, T const &x);
	StateTemplate(vector<bool> const &v, vector<unsigned int> const &pos);
	StateTemplate(unsigned int nVal);
	StateTemplate(unsigned int nVal,string c);
	StateTemplate<T> c(unsigned int k) const;
	StateTemplate<T> cDagger(unsigned int k) const;
	StateTemplate<T> ccDagger(unsigned int k, unsigned int j) const;
	StateTemplate<T> nState(unsigned int k) const;
	StateTemplate<T> nnState(unsigned int k) const;
	T normQuad() const;
	T Stot();
    T Stot(StateTemplate<T> const &s);
	T StotMean();
    StateTemplate<T> StotOp();
	int QP();
	StateTemplate<T> getQP(unsigned int k);
	unsigned int size();
	vector<vector<bool> > getSingleStates() const;
	bool isEmpty() const;

	StateTemplate<T> permutateState(vector<unsigned int> const &pos) const;
	bool checkPermutationSymmetry(vector<unsigned int> const &pos) const;

    StateTemplate<T>& operator+=(StateTemplate<T> const &s);
    StateTemplate<T>& operator-=(StateTemplate<T> const &s);
    StateTemplate<T>& operator*=(T const &x);
    StateTemplate<T>& operator/=(T const &x);
};

//// friend functions declaration ////
template<class T>
ostream& operator<<(ostream &o, StateTemplate<T> const &s);
template<class T>
istream& operator>>(istream &o, StateTemplate<T> &s);
template<class T>
T operator*(StateTemplate<T> const &s1, StateTemplate<T> const &s2);
template<class T>
StateTemplate<T> productState(StateTemplate<T> const &s1, StateTemplate<T> const &s2);
template<class T>
StateTemplate<T> insertState(StateTemplate<T> const &s1, StateTemplate<T> const &s2, unsigned int *pos);
template<class T>
T partialScalarProduct(StateTemplate<T> const &s1, StateTemplate<T> const &s2, vector<unsigned int> const &pos);
template<class T>
T partialEntangledScalarProduct(StateTemplate<T> const &s1, StateTemplate<T> const &s2, StateTemplate<T> const &sSub1, StateTemplate<T> const &sSub2, vector<unsigned int> const &pos, vector<unsigned int> const &posNot);
template<class T>
T partialEntangledScalarProduct(const typename std::vector<StateTemplate<T> >::const_iterator s1, const typename std::vector<StateTemplate<T> >::const_iterator s2, const typename std::vector<StateTemplate<T> >::const_iterator sSub1, const typename std::vector<StateTemplate<T> >::const_iterator sSub2, vector<unsigned int> const &pos, vector<unsigned int> const &posNot);

//// non class/friend functions declaration ////
template<class T>
const StateTemplate<T> operator+(StateTemplate<T> const &s1, StateTemplate<T> const &s2);
template<class T>
const StateTemplate<T> operator-(StateTemplate<T> const &s1, StateTemplate<T> const &s2);
template<class T>
const StateTemplate<T> operator*(StateTemplate<T> const &s, T const &x);
template<class T>
const StateTemplate<T> operator*(T const &x, StateTemplate<T> const &s);
template<class T>
const StateTemplate<T> operator/(StateTemplate<T> const &s, T const &x);

//// implementation ////
//// class functions ////
template<class T>
unsigned int StateTemplate<T>::QP(vector<bool> v){
	// calculates the number of double occupancies for the hubbard state v (v is NOT a superposition of states)
	unsigned int q=0;
	for(unsigned int i=0;i<n;i+=2){
		if(v[i] && v[i+1]){
			q++;
		}
	}
	return q;
}

template<class T>
StateTemplate<T>::StateTemplate():n(2){
}
template<class T>
StateTemplate<T>::StateTemplate(vector<bool> &v){
	n=v.size();
	if(n>0){
		stateCont.insert(pair<vector<bool>,T>(v,1));
	}
}
template<class T>
StateTemplate<T>::StateTemplate(vector<bool> &v, T const &x){
	n=v.size();
	if(n>0){
		stateCont.insert(pair<vector<bool>,T>(v,x));
	}
}
template<class T>
StateTemplate<T>::StateTemplate(vector<bool> const &v, vector<unsigned int> const &pos){
	n=pos.size();
	if(n>0){
		vector<bool> vv(n);
		for(unsigned int i=0;i<n;i++){
			vv[i]=*(v.begin()+pos[i]);
		}
		stateCont.insert(pair<vector<bool>,T>(vv,T(1)));
	}
}
template<class T>
StateTemplate<T>::StateTemplate(unsigned int nVal):n(nVal){
}
template<class T>
StateTemplate<T>::StateTemplate(unsigned int nVal,string c):n(nVal){
	vector<bool> bs;
	for(unsigned int i=0;i<n;i++){
		bs.push_back(c[i]-'0');
	}
	if(n>0){
		stateCont.insert(pair<vector<bool>,T>(bs,1));
	}
}

template<class T>
StateTemplate<T> StateTemplate<T>::c(unsigned int k) const{
	// applies the annihilation operator c_k to the state
	if (k < n) {
		typename std::map<vector<bool>,T>::const_iterator it;
		StateTemplate<T> s(n);
		vector<bool> v;
		T x = 0;

		it = stateCont.begin();
		while (it != stateCont.end()) {
			int vz = 0;
			if ((it->first[k])) {
				for (unsigned int i = 0; i < k; i++) {
					if (it->first[i]) {
						vz++;
					}
				}
				x = (it->second) * (1 - 2 * (vz % 2));
				v = it->first;
				v[k] = !v[k];
				s.stateCont.insert(pair<vector<bool>, T>(v, x));
			}
			it++;
		}
		return s;
	}
	else {
		cerr << "State State::c(unsigned int k): index out of range.\n";
		return *this;
	}
}
template<class T>
StateTemplate<T> StateTemplate<T>::cDagger(unsigned int k) const{
	// applies the annihilation operator cDagger_k to the state
	if (k < n) {
		typename std::map<vector<bool>,T>::const_iterator it;
		StateTemplate<T> s(n);
		vector<bool> v;
		T x = 0;

		it = stateCont.begin();
		while (it != stateCont.end()) {
			int vz = 0;
			if (!(it->first[k])) {
				for (unsigned int i = 0; i < k; i++) {
					if (it->first[i]) {
						vz++;
					}
				}
				x = (it->second) * (1 - 2 * (vz % 2));
				v = it->first;
				v[k] = !v[k];
				s.stateCont.insert(pair<vector<bool>, T>(v, x));
			}
			it++;
		}
		return s;
	}
	else {
		cerr << "State State::cDagger(unsigned int k): index out of range.\n";
		return *this;
	}
}
template<class T>
StateTemplate<T> StateTemplate<T>::ccDagger(unsigned int k, unsigned int j) const{
	// applies first the annihilation operator c_k and then the creation operator c_j
	// this method is faster than the stand-alone routines c and cDagger
	if ((k < n) && (j < n)) {
		if (k == j) {
			return (this->nState(k));
		}
		else {
			typename std::map<vector<bool>,T>::const_iterator it;
			StateTemplate<T> s(n);
			vector<bool> v;
			T x = 0;

			it = stateCont.begin();
			while (it != stateCont.end()) {
				int vz = 0;
				if ((it->first[k]) && !(it->first[j])) {
					v = it->first;
					for (unsigned int i = 0; i < k; i++) {
						if (it->first[i]) {
							vz++;
						}
					}
					v[k] = !v[k];
					for (unsigned int i = 0; i < j; i++) {
						if (v[i]) {
							vz++;
						}
					}
					v[j] = !v[j];
					x = (it->second) * (1 - 2 * (vz % 2));
					s.stateCont.insert(pair<vector<bool>, T>(v, x));
				}
				it++;
			}
			return s;
		}
	} else {
		cerr << "State State::ccDagger(unsigned int k, unsigned int j): index out of range.\n";
		return *this;
	}
}
template<class T>
StateTemplate<T> StateTemplate<T>::nState(unsigned int k) const{
	// the number operator n=c_k*cDagger_k
	// this method is faster than the stand-alone routines c and cDagger; it is also faster than ccDagger(k,k)
	if (k < n) {
		typename std::map<vector<bool>,T>::const_iterator it;
		StateTemplate<T> s(n);

		it = stateCont.begin();
		while (it != stateCont.end()) {
			if ((it->first[k])) {
				s.stateCont.insert(*it);
			}
			it++;
		}
		return s;
	} else {
		cerr << "void State::nState(unsigned int k): index out of range.\n";
		return *this;
	}
}
template<class T>
StateTemplate<T> StateTemplate<T>::nnState(unsigned int k) const{
	// calculates n(k,up)*n(k,down)=n(k)*n(k+1)
	// this method is faster than any combination of stand-alone routines
	if (k < n-1) {
		typename std::map<vector<bool>,T>::const_iterator it;
		StateTemplate<T> s(n);

		it = stateCont.begin();
		while (it != stateCont.end()) {
			if ((it->first[k]) && (it->first[k+1])) {
				s.stateCont.insert(*it);
			}
			it++;
		}
		return s;
	} else {
		cerr << "void State::nnState(unsigned int k): index out of range.\n";
		return *this;
	}
}
template<class T>
T StateTemplate<T>::normQuad() const{
	// returns the square of the norm
	T x=0;
	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=stateCont.begin();it!=stateCont.end();it++){
		T y=it->second;
		x+=y*y;
	}
	return x;
}

template<class T>
T StateTemplate<T>::Stot(){
	// returns the total spin S (S^2|state>=S*(S+1)|state>) if it is defined
	// if it is not defined -1 is returned
	StateTemplate<T> s1=this->StotOp();
	return Stot(s1);
}
template<class T>
T StateTemplate<T>::Stot(StateTemplate<T> const &s){
    // returns the total spin S (S^2|state>=S*(S+1)|state>) if it is defined
	// if it is not defined -1 is returned
    // s is a state on which StotOp was already applied
	/*
    for(T S=0;S<=T(n)/T(2);S+=T(1)/T(2)){
		StateTemplate<T> s2=s-S*(S+1)*(*this);
		if(s2.normQuad()<=zeroNormQuadEPS){
			return S;
		}
	}
	*/
	T lambda=((*this)*s)/this->normQuad();
	StateTemplate<T> s2=s-lambda*(*this);
	if(s2.normQuad()<=zeroNormQuadEPS){
		return lambda;
	}
	return T(-1);
}
template<class T>
T StateTemplate<T>::StotMean(){
	// returns the mean value of the total spin <S^2>
	return (*this*this->StotOp());
}
template<class T>
StateTemplate<T> StateTemplate<T>::StotOp(){
	// applies the total spin operator S^2 to the state
    StateTemplate<T> c(n);
	// run over all sites
	for (unsigned int i = 0; i < n/2; i++) {
		for(unsigned int j=0;j<n/2;j++) {
			// S_i^+ S_j^-
			c+=((this->ccDagger(2*j,2*j+1)).ccDagger(2*i+1,2*i))/T(2);
			// S_i^- S_j^+
			c+=((this->ccDagger(2*j+1,2*j)).ccDagger(2*i,2*i+1))/T(2);
			// S_i^z S_j^z
			for (int s1 = 0; s1 <= 1; s1++) {
				for (int s2 = 0; s2 <= 1; s2++) {
					c+=T(1-2*((s1+s2)%2))*(this->nState(2*j+s1)).nState(2*i+s2)/T(4);
				}
			}
		}
	}
	return c;
}

template<class T>
int StateTemplate<T>::QP(){
    // returns Number of QP if it is defined
    // returns -1 if Number is not defined and -2 if state is empty
    if(stateCont.size()!=0){
    	typename std::map<vector<bool>,T>::iterator it;
        it=stateCont.begin();
        unsigned int k=QP(it->first);
        it++;
        while(it!=stateCont.end()){
            if(QP(it->first)!=k){
                return -1;
            }
            it++;
        }
        return int(k);
    }
    else{
        return -2;
    }
}

template<class T>
StateTemplate<T> StateTemplate<T>::getQP(unsigned int k){
	// returns the (NOT NORMALIZED) parts of the state with QP-Number k
	StateTemplate<T> s1(n);
	typename std::map<vector<bool>,T>::iterator it;
	for(it=stateCont.begin();it!=stateCont.end();++it){
		if((QP(it->first))==k){
			s1.stateCont.insert(*it);
		}
	}
	return s1;
}

template<class T>
unsigned int StateTemplate<T>::size(){
	return n;
}

template<class T>
vector<vector<bool> > StateTemplate<T>::getSingleStates() const{
	vector<vector<bool> > singleStates;
	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=stateCont.begin(); it!=stateCont.end();it++){
		singleStates.push_back(it->first);
	}
	return singleStates;
}

template<class T>
bool StateTemplate<T>::isEmpty() const{
	if(stateCont.size()==0){
		return true;
	}
	else{
		return false;
	}
}

template<class T>
StateTemplate<T> StateTemplate<T>::permutateState(vector<unsigned int> const &pos) const{
	StateTemplate<T> s(n);
	for(typename std::map<vector<bool>,T>::const_iterator it1=stateCont.begin();it1!=stateCont.end();it1++){
		vector<bool> vb(n);
		for(unsigned int i=0;i<n/2;i++){
			vb[2*pos[i]]=it1->first[2*i];
			vb[2*pos[i]+1]=it1->first[2*i+1];
		}
		StateTemplate<T> sTemp(vb,it1->second);
		s+=sTemp;
	}

	return s;
}
template<class T>
bool StateTemplate<T>::checkPermutationSymmetry(vector<unsigned int> const &pos) const{
	StateTemplate<T> s=this->permutateState(pos);
	StateTemplate<T> sTemp=s-(*this);
	//cout << sTemp.normQuad() << "\n";
	return (sTemp.normQuad()<=zeroNormQuadEPS);
}

template<class T>
StateTemplate<T>& StateTemplate<T>::operator+=(StateTemplate<T> const &s){
	if (n == s.n) {
		typename std::map<vector<bool>,T>::const_iterator it2;
		for (it2 = s.stateCont.begin(); it2 != s.stateCont.end(); ++it2) {
			pair<typename std::map<vector<bool>,T>::iterator,bool> ret=stateCont.insert(*it2);
			if(!(ret.second)){
				ret.first->second+=it2->second;
				if(myAbs(ret.first->second)<=zeroEPS){
					stateCont.erase(ret.first);
				}
			}
		}
	}
	else {
		cerr << "State& State::operator+=(State const &s): States not same length\n";
	}
    return *this;
}
template<class T>
StateTemplate<T>& StateTemplate<T>::operator-=(StateTemplate<T> const &s){
	if (n == s.n) {
		typename std::map<vector<bool>,T>::const_iterator it2;
		for (it2 = s.stateCont.begin(); it2 != s.stateCont.end(); ++it2) {
			pair<typename std::map<vector<bool>,T>::iterator,bool> ret=stateCont.insert(pair<vector<bool>,T>(it2->first,-it2->second));
			if(!(ret.second)){
				ret.first->second-=it2->second;
				if(myAbs(ret.first->second)<=zeroEPS){
					stateCont.erase(ret.first);
				}
			}
		}
	}
	else {
		cerr << "State& State::operator-=(State const &s): States not same length\n";
	}
    return *this;
}
template<class T>
StateTemplate<T>& StateTemplate<T>::operator*=(T const &x){
	typename std::map<vector<bool>,T>::iterator it=stateCont.begin();
	typename std::map<vector<bool>,T>::iterator tempIt;
    while(it!=stateCont.end()){
    	it->second*=x;
    	if(myAbs(it->second)<=zeroEPS){
    		tempIt=it;
    		it++;
    		stateCont.erase(tempIt);
    	}
    	else{
    		it++;
    	}
    }
    return *this;
}
template<class T>
StateTemplate<T>& StateTemplate<T>::operator/=(T const &x){
	typename std::map<vector<bool>,T>::iterator it=stateCont.begin();
	typename std::map<vector<bool>,T>::iterator tempIt;
    while(it!=stateCont.end()){
    	it->second/=x;
    	if(myAbs(it->second)<=zeroEPS){
    		tempIt=it;
    		it++;
    		stateCont.erase(tempIt);
    	}
    	else{
    		it++;
    	}
    }
    return *this;
}

//// friend functions implementations ////
template<class T>
ostream& operator<<(ostream &o, StateTemplate<T> const &s){
	bool showposset=false;
	ios::fmtflags f;
	f = cout.flags();
	if(f & ios::showpos){
		showposset=true;
	}

	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=s.stateCont.begin(); it!=s.stateCont.end();++it){
		o << showpos;
		o << (it->second) << "|" << noshowpos;
		for(unsigned int i=0;i<s.n;i++){
			o << it->first[i];
		}
		o << ">";
	}

	if(showposset){
		o << showpos;
	}

	return o;
}
template<class T>
istream& operator>>(istream &o, StateTemplate<T> &s){
	char tempChar;
	tempChar=o.peek();
	if(tempChar=='\n'){
		o>>noskipws>>tempChar>>skipws;
	}
	string tempstring;
	getline(o,tempstring);
	istringstream iss(tempstring);

	s.stateCont.clear();

	T x=0;
	char temp;
	bool btemp;
	//iss>>temp;
	while(iss>>x){
		iss>>temp;
		vector<bool> vb;
		iss>>temp;
		while(temp!='>'){
			if(temp=='0'){
				btemp=false;
			}
			else{
				btemp=true;
			}
			vb.push_back(btemp);
			iss>>temp;
		}
		s.n=vb.size();
		s.stateCont.insert(pair<vector<bool>,T>(vb,x));
	}

	return o;
}

template<class T>
T operator*(StateTemplate<T> const &s1, StateTemplate<T> const &s2){
	if(s1.n>0){
		T x=0;
		typename std::map<vector<bool>,T>::const_iterator it1;
		typename std::map<vector<bool>,T>::const_iterator it2;
		if (s1.stateCont.size() < s2.stateCont.size()) {
			for (it1 = s1.stateCont.begin(); it1 != s1.stateCont.end(); ++it1) {
				it2 = s2.stateCont.find(it1->first);
				if (it2 != s2.stateCont.end()) {
					x += (it1->second) * (it2->second);
				}
			}
		}
		else {
			for (it1 = s2.stateCont.begin(); it1 != s2.stateCont.end(); ++it1) {
				it2 = s1.stateCont.find(it1->first);
				if (it2 != s1.stateCont.end()) {
					x += (it1->second) * (it2->second);
				}
			}
		}
		return x;
	}
	else{
		return T(1);
	}
}

template<class T>
StateTemplate<T> productState(StateTemplate<T> const &s1, StateTemplate<T> const &s2){
    StateTemplate<T> s(s1.n+s2.n);
    typename std::map<vector<bool>,T>::const_iterator it1;
	for(it1=s1.stateCont.begin(); it1!=s1.stateCont.end();++it1){
		typename std::map<vector<bool>,T>::const_iterator it2;
		for(it2=s2.stateCont.begin(); it2!=s2.stateCont.end();++it2){
			vector<bool> v;
			for(unsigned int i=0;i<s1.n;i++){
				v.push_back(it1->first[i]);
			}

			for(unsigned int i=0;i<s2.n;i++){
				v.push_back(it2->first[i]);
			}
			s.stateCont.insert(pair<vector<bool>,T>(v,(it1->second)*(it2->second)));
		}
	}
    return s;
}
template<class T>
StateTemplate<T> insertState(StateTemplate<T> const &s1, StateTemplate<T> const &s2, unsigned int *pos){
	// inserts s2 in s1 at the positions specified in pos
	// therefore it is required that length(s2)<=length(s1)!
    StateTemplate<T> s(s1.n);
    typename std::map<vector<bool>,T>::const_iterator it1;
	for(it1=s1.stateCont.begin(); it1!=s1.stateCont.end();++it1){
		typename std::map<vector<bool>,T>::const_iterator it2;
		for(it2=s2.stateCont.begin(); it2!=s2.stateCont.end();++it2){
			vector<bool> v=it1->first;
			for(unsigned int j=0;j<s2.n;j++){
				v[pos[j]]=it2->first[j];
			}
			s.stateCont.insert(pair<vector<bool>,T>(v,(it1->second)*(it2->second)));
		}
	}
    return s;
}
template<class T>
StateTemplate<T> insertState(StateTemplate<T> const &s1, StateTemplate<T> const &s2, vector<unsigned int> const &pos){
	// inserts s2 in s1 at the positions specified in pos
	// therefore it is required that length(s2)<=length(s1)!
    StateTemplate<T> s(s1.n);
    typename std::map<vector<bool>,T>::const_iterator it1;
	for(it1=s1.stateCont.begin(); it1!=s1.stateCont.end();++it1){
		typename std::map<vector<bool>,T>::const_iterator it2;
		for(it2=s2.stateCont.begin(); it2!=s2.stateCont.end();++it2){
			vector<bool> v=it1->first;
			for(unsigned int j=0;j<s2.n;j++){
				v[pos[j]]=it2->first[j];
			}
			s.stateCont.insert(pair<vector<bool>,T>(v,(it1->second)*(it2->second)));
		}
	}
    return s;
}

template<class T>
T partialScalarProduct(StateTemplate<T> const &s1, StateTemplate<T> const &s2, vector<unsigned int> const &pos){
	// calculates the scalar product between s1 and s2 by only considering the positions at s1 which are specified in pos
	// therefore it is required that length(s2)<length(s1) and length(pos)=length(s2)
	typename std::map<vector<bool>,T>::const_iterator it1;
	StateTemplate<T> s(s2.n);
	for(it1=s1.stateCont.begin();it1!=s1.stateCont.end();++it1){
		vector<bool> v;
		for(unsigned int i=0;i<s2.n;i++){
			v.push_back((it1->first)[pos[i]]);
		}
		StateTemplate<T> sTemp(v,it1->second);
		s+=sTemp;
	}

	//cout << "s=" << s << "\n";
	//cout << "s2=" << s2 << "\n";

	return s*s2;
}
template<class T>
T partialEntangledScalarProduct(StateTemplate<T> const &s1, StateTemplate<T> const &s2, StateTemplate<T> const &sSub1, StateTemplate<T> const &sSub2, vector<unsigned int> const &pos, vector<unsigned int> const &posNot){
	// calculates the partial scalar product between s1 and sSub1 as well as s2 and sSub2 by only considering the positions at s1/s2 which are specified in pos
	// in addition the partial scalar product between s1 and s2 is calculated by only considering the positions which are NOT specified in pos
	// this is performed for each element of the map individually!
	// therefore it is required that length(sSub)<length(s) and length(pos)=length(sSub)

	T x=0;

	typename std::map<vector<bool>,T>::const_iterator it1;
	typename std::map<vector<bool>,T>::const_iterator it2;
	for(it1=s1.stateCont.begin();it1!=s1.stateCont.end();it1++){
		/*
		vector<bool> v11(posNot.size());
		for(unsigned int i=0;i<posNot.size();i++){
			v11.push_back();
		}
		*/
		/*
		vector<bool> v12(pos.size());
		for(unsigned int i=0;i<sSub1.n;i++){
			v12.push_back(it1->first[pos[i]]);
		}
		*/
		StateTemplate<T> s1p(it1->first,pos);
		//StateTemplate<T> s1pNot(v11,T(1));
		for(it2=s2.stateCont.begin();it2!=s2.stateCont.end();it2++){
			/*
			vector<bool> v21;
			for(unsigned int i=0;i<posNot.size();i++){
				v21.push_back(it2->first[posNot[i]]);
			}
			*/
			bool v11equals21=true;
			for(unsigned int i=0;i<posNot.size();i++){
				if((it2->first[posNot[i]])!=(it1->first[posNot[i]])){
					v11equals21=false;
					break;
				}
			}
			if(v11equals21){
				/*
				vector<bool> v22(pos.size());
				for(unsigned int i=0;i<sSub2.n;i++){
					v22.push_back(it2->first[pos[i]]);
				}
				StateTemplate<T> s2p(v22,T(1));
				*/
				StateTemplate<T> s2p(it2->first,pos);
				//StateTemplate<T> s2pNot(v21,T(1));
				x+=(it1->second*it2->second)*(s1p*sSub1)*(s2p*sSub2);
				//x+=(it1->second*it2->second)*(s1p*sSub1)*(s2p*sSub2)*(s1pNot*s2pNot);
			}
		}
	}

	return x;
}

template<class T>
T partialEntangledScalarProduct(const typename std::vector<StateTemplate<T> >::const_iterator s1, const typename std::vector<StateTemplate<T> >::const_iterator s2, const typename std::vector<StateTemplate<T> >::const_iterator sSub1, const typename std::vector<StateTemplate<T> >::const_iterator sSub2, vector<unsigned int> const &pos, vector<unsigned int> const &posNot){
	// calculates the partial scalar product between s1 and sSub1 as well as s2 and sSub2 by only considering the positions at s1/s2 which are specified in pos
	// in addition the partial scalar product between s1 and s2 is calculated by only considering the positions which are NOT specified in pos
	// this is performed for each element of the map individually!
	// therefore it is required that length(sSub)<length(s) and length(pos)=length(sSub)

	T x=0;

	typename std::map<vector<bool>,T>::const_iterator it1;
	typename std::map<vector<bool>,T>::const_iterator it2;
	for(it1=s1->stateCont.begin();it1!=s1->stateCont.end();it1++){
		StateTemplate<T> s1p(it1->first,pos);
		for(it2=s2->stateCont.begin();it2!=s2->stateCont.end();it2++){
			bool v11equals21=true;
			for(unsigned int i=0;i<posNot.size();i++){
				if((it2->first[posNot[i]])!=(it1->first[posNot[i]])){
					v11equals21=false;
					break;
				}
			}
			if(v11equals21){
				StateTemplate<T> s2p(it2->first,pos);
				x+=(it1->second*it2->second)*(s1p*(*sSub1))*(s2p*(*sSub2));
			}
		}
	}

	return x;
}

//// non class/friend functions implementations ////
template<class T>
const StateTemplate<T> operator+(StateTemplate<T> const &s1, StateTemplate<T> const &s2){
    StateTemplate<T> s=s1;
    s+=s2;
    return s;
}
template<class T>
const StateTemplate<T> operator-(StateTemplate<T> const &s1, StateTemplate<T> const &s2){
    StateTemplate<T> s=s1;
    s-=s2;
    return s;
}
template<class T>
const StateTemplate<T> operator*(StateTemplate<T> const &s, T const &x){
    StateTemplate<T> s2=s;
    s2*=x;
    return s2;
}
template<class T>
const StateTemplate<T> operator*(T const &x, StateTemplate<T> const &s){
    StateTemplate<T> s2=s;
    s2*=x;
    return s2;
}
template<class T>
const StateTemplate<T> operator/(StateTemplate<T> const &s, T const &x){
    StateTemplate<T> s2=s;
    s2/=x;
    return s2;
}

typedef StateTemplate<Doub> StateDouble;

#endif /* STATETEMPLATE_H_ */
