/*
 * functors.cpp
 *
 *  Created on: Jul 12, 2012
 *      Author: ixert
 */

#include "functors.h"
#include "mathFunctions.h"
#include "nr3.h"
#include "eigen_sym.h"
using namespace std;

//// implementation of QP-Generator ////
funcQP::funcQP(unsigned int basisSizeI, vector<unsigned int> qI, unsigned int unitaryI) : basisSize(basisSizeI),q(qI),unitary(unitaryI){
}
void funcQP::operator()(const Doub x, VecDoub_I &y, VecDoub_O &dydx){
    // the QP-Generator (stores full matrix, but uses symmetry for calculation)
    unsigned int nn=basisSize*basisSize;
    
	Doub temp1 = 0;
	Doub temp2 = 0;
    Doub temp4 = 0;
	for(unsigned int i=0;i<basisSize;i++){
		for(unsigned int j=i;j<basisSize;j++){
			temp1=0;
			temp2=0;
			for(unsigned int k=0;k<basisSize;k++){
				temp1+=Doub(sgn<int>(int(q[i])-int(q[k]))-sgn<int>(int(q[k])-int(q[j])))*y[basisSize*i+k]*y[basisSize*k+j];
			}
			dydx[basisSize*i+j]=temp1;
            dydx[basisSize*j+i]=temp1;
		}
	}
    
    
    // calculate unitary transformation matrix
    if(unitary==1){
        for(unsigned int i=0;i<basisSize;i++){
            for(unsigned int j=0;j<basisSize;j++){
                temp4=0;
                for(unsigned int k=0;k<basisSize;k++){
                    temp4+=Doub(sgn<int>(int(q[i])-int(q[k])))*y[basisSize*i+k]*y[nn+basisSize*k+j];
                }
                dydx[nn+basisSize*i+j]=temp4;
            }
        }
    }

}

Doub funcQP::offBlockDiagonalElements(VecDoub_I &y){
	// calculates the sum over squares of the off block diagonal Elements of H and returns the square root
	Doub res=0;
	for(unsigned int i=0;i<basisSize;i++){
		for(unsigned int j=0;j<basisSize;j++){
			if(q[i]!=q[j]){
				res+=y[i*basisSize+j]*y[i*basisSize+j];
			}
		}
	}
	return sqrt(res);
}

//// implementation of Wegner-Generator ////
funcWegner::funcWegner(unsigned int basisSizeI, unsigned int llsSizeI, unsigned int unitaryI) : basisSize(basisSizeI),llsSize(llsSizeI),unitary(unitaryI){
}
void funcWegner::operator()(const Doub x, VecDoub_I &y, VecDoub_O &dydx){
	// the Wegner-Generator (stores full matrix, but uses symmetry for calculation)
	unsigned int nn=basisSize*basisSize;

	Doub temp1=0;
	Doub temp2=0;
	Doub temp3=0;
	Doub temp4=0;
	Doub temp5=0;
	for(unsigned int i=0;i<basisSize;i++){
		for(unsigned int j=i;j<basisSize;j++){
			temp1=0;
			temp4=0;
			for(unsigned int k=0;k<basisSize;k++){
				temp2=0;
				temp3=0;
				for(unsigned int l=0;l<basisSize;l++){
					if(((i<llsSize) && (l<llsSize)) || ((i>=llsSize) && (l>=llsSize))){
						temp2+=y[i*basisSize+l]*y[l*basisSize+k];
					}
					if(((l<llsSize) && (k<llsSize)) || ((l>=llsSize) && (k>=llsSize))){
						temp2-=y[i*basisSize+l]*y[l*basisSize+k];
					}
					if(((k<llsSize) && (l<llsSize)) || ((k>=llsSize) && (l>=llsSize))){
						temp3+=y[k*basisSize+l]*y[l*basisSize+j];
					}
					if(((l<llsSize) && (j<llsSize)) || ((l>=llsSize) && (j>=llsSize))){
						temp3-=y[k*basisSize+l]*y[l*basisSize+j];
					}
				}
				temp1+=temp2*y[k*basisSize+j]-temp3*y[i*basisSize+k];
			}
			dydx[basisSize*i+j]=temp1;
			dydx[basisSize*j+i]=temp1;
		}

	}
    // calculate unitary transformation matrix
    if(unitary==1){
        for(unsigned int i=0;i<basisSize;i++){
            for(unsigned int j=0;j<basisSize;j++){
                temp4=0;
                for(unsigned int k=0;k<basisSize;k++){
                    for(unsigned int l=0;l<basisSize;l++){
                        temp5=0;
                        if(((i<llsSize) && (k<llsSize)) || ((i>=llsSize) && (k>=llsSize))){
                            temp5+=y[i*basisSize+k]*y[k*basisSize+l];
                        }
                        if(((k<llsSize) && (l<llsSize)) || ((k>=llsSize) && (l>=llsSize))){
                            temp5-=y[i*basisSize+k]*y[k*basisSize+l];
                        }
                        temp4+=temp5*y[nn+l*basisSize+j];
                    }
                }
                dydx[nn+basisSize*i+j]=temp4;
            }
        }
    }
}

Doub funcWegner::offBlockDiagonalElements(VecDoub_I &y){
	// calculates the sum over squares of the off block diagonal Elements of H and returns the square root
	Doub res=0;
	for(unsigned int i=0;i<llsSize;i++){
		for(unsigned int j=llsSize;j<basisSize;j++){
            res+=y[i*basisSize+j]*y[i*basisSize+j];
            res+=y[j*basisSize+i]*y[j*basisSize+i];
		}
	}

	return sqrt(res);
}
