/*
 * orthogonalization.cpp
 *
 *  Created on: Jul 24, 2012
 *      Author: ixert
 */

#include "orthogonalization.h"
#include <iostream>
#include "eigen_sym.h"
#include "functions.h"

using namespace std;

void MGS(StateDouble &s, vector<StateDouble> &basis){
	// applies the modified Gram-Schmidt algorithm to state s with respect to basis
	// This function checks if the square of the norm is greater than zeroNormQuadEPS AND if the new state is still orthogonal to the basis
	// if one of the above condiditions is not fullfilled the state is not added to the basis
	unsigned int size=basis.size();
	Doub norm=0;
	for (unsigned int j = 0; j < size; j++) {
		s -=(s * basis[j]) * basis[j];
	}
	norm = s.normQuad();
    cout << "normQuad=" << norm << "\n";
	if (norm > zeroNormQuadEPS) {
		s/=sqrt(norm);
        bool isOrtho=true;
        for(unsigned int j=0;j<size;j++){
        	Doub orthValue=s*basis[j];
        	cout << "<s|" << j << ">=" << orthValue << "\n";
        	if(orthValue>zeroOrthoEPS){
        		isOrtho=false;
        		cout << "State not orthogonal and thus not added to basis!\n";
            }
        }
        if(isOrtho){
        	basis.push_back(s);
        }
	}
}

void MGSWithoutSqrt(StateDouble &s, vector<StateDouble> &basis){
	// applies the modified Gram-Schmidt algorithm without taking the sqrt to state s with respect to basis
	// This function checks if the square of the norm is greater than zeroNormQuadEPS AND if the new state is still orthogonal to the basis
	// if one of the above condiditions is not fullfilled the state is not added to the basis
	unsigned int size=basis.size();
	Doub norm=0;
	for (unsigned int j = 0; j < size; j++) {
		s -=(s * basis[j]) * basis[j]/(basis[j].normQuad());
	}
	norm = s.normQuad();
    cout << "normQuad=" << norm << "\n";
	if (norm > zeroNormQuadEPS) {
        bool isOrtho=true;
        /*
        for(unsigned int j=0;j<size;j++){
        	Doub orthValue=s*basis[j];
        	cout << "<s|" << j << ">=" << orthValue << "\n";
        	if(orthValue>zeroOrthoEPS){
        		isOrtho=false;
        		cout << "State not orthogonal and thus not added to basis!\n";
            }
        }
        */
        if(isOrtho){
        	basis.push_back(s);
        }
	}
}


void MGSQP(StateDouble &s, vector<StateDouble> &basis, vector<unsigned int> &q){
	// applies the modified Gram-Schmidt algorithm to state s with respect to basis
	// furthermore the states are sorted by QP
	// This function checks if the square of the norm is greater than zeroNormQuadEPS AND if the new state is still orthogonal to the basis
	// if one of the above condiditions is not fullfilled the state is not added to the basis

	Doub norm=0;
	unsigned int size=basis.size();
	// split s up into QP-parts
	for(unsigned int i=0;i<=s.size();i++){
		StateDouble sQP=s.getQP(i);
		for(unsigned int k=0;k<size;k++){
			if(q[k]==i){
				sQP-=(sQP*basis[k])*basis[k];
			}
		}
		norm=sQP.normQuad();
		cout << "normQuad=" << norm << "\n";
		if(norm>zeroNormQuadEPS){
			sQP/=sqrt(norm);
			bool isOrtho=true;
			for(unsigned int j=0;j<size;j++){
				Doub orthValue=sQP*basis[j];
				if(orthValue>zeroOrthoEPS){
					isOrtho=false;
					cout << "State not orthogonal and thus not added to basis!\n";
				}
			}
			if(isOrtho){
				basis.push_back(sQP);
                q.push_back(i);
			}
		}
	}
}

void MGSQPWithoutSqrt(StateDouble &s, vector<StateDouble> &basis, vector<unsigned int> &q){
    // applies the modified Gram-Schmidt algorithm without sqrt to state s with respect to basis
	// furthermore the states are sorted by QP
	// This function checks if the square of the norm is greater than zeroNormQuadEPS AND if the new state is still orthogonal to the basis
	// if one of the above condiditions is not fullfilled the state is not added to the basis
    
	Doub norm=0;
	unsigned int size=basis.size();
	// split s up into QP-parts
	for(unsigned int i=0;i<=s.size();i++){
		StateDouble sQP=s.getQP(i);
		for(unsigned int k=0;k<size;k++){
			if(q[k]==i){
				sQP-=(sQP*basis[k])*basis[k]/(basis[k].normQuad());
			}
		}
		norm=sQP.normQuad();
		cout << "normQuad=" << norm << "\n";
		if(norm>zeroNormQuadEPS){
			bool isOrtho=true;
			for(unsigned int j=0;j<size;j++){
				Doub orthValue=sQP*basis[j];
				if(orthValue>zeroOrthoEPS){
					isOrtho=false;
					cout << "State not orthogonal and thus not added to basis!\n";
				}
			}
			if(isOrtho){
				basis.push_back(sQP);
                q.push_back(i);
			}
		}
	}
}
