/*
 * ExponentsTemplate.h
 *
 *  Created on: Jun 11, 2013
 *      Author: ixert
 */

#ifndef EXPONENTSTEMPLATE_H_
#define EXPONENTSTEMPLATE_H_

#include <iostream>
#include <map>

using namespace std;

template<class T>
class ExponentsTemplate {
template<class S>
friend bool operator==(ExponentsTemplate<S> const&lhs, ExponentsTemplate<S> const& rhs);
template<class S>
friend ostream& operator<<(ostream &o, ExponentsTemplate<S> const &s);
template<class S>
friend istream& operator>>(istream &o, ExponentsTemplate<S> &s);
private:
	map<unsigned int,T> variables;
public:
	ExponentsTemplate();
	ExponentsTemplate(T const &x);
	void setExponent(unsigned int const &i, T const &x);
	T getExponent(unsigned int const &i) const;
	void increaseExponent(unsigned int const &i, T const &x);
	void decreaseExponent(unsigned int const &i, T const &x);

	template<class S>
	S replaceCouplingByOtherCoupling(unsigned int const &c1, unsigned int const &c2, S const &factor=S(1));

	template<class S>
	S evaluate(vector<S> const &h) const;

	ExponentsTemplate<T>& operator*=(ExponentsTemplate<T> const& rhs);
};

template<class T>
bool operator != (ExponentsTemplate<T> const& lhs, ExponentsTemplate<T> const& rhs);

template<class T>
const ExponentsTemplate<T> operator*(ExponentsTemplate<T> const& lhs, ExponentsTemplate<T> const& rhs);

template<class T>
ExponentsTemplate<T>::ExponentsTemplate() {
}

template<class T>
ExponentsTemplate<T>::ExponentsTemplate(T const &x){
}

template<class T>
void ExponentsTemplate<T>::setExponent(unsigned int const &i, T const &x){
	if(x!=0){
		typename map<unsigned int,T>::iterator it;
		it=variables.find(i);
		if(it!=variables.end()){
			it->second=x;
		}
		else{
			variables.insert(pair<unsigned int,T>(i,x));
		}
	}
	else{
		variables.erase(i);
	}
}

template<class T>
T ExponentsTemplate<T>::getExponent(unsigned int const &i) const{
	typename map<unsigned int,T>::const_iterator it;
	it=variables.find(i);
	if(it!=variables.end()){
		return it->second;
	}
	else{
		return T(0);
	}
}

template<class T>
void ExponentsTemplate<T>::increaseExponent(unsigned int const &i, T const &x){
	if(x!=0){
		typename map<unsigned int,T>::iterator it;
		it=variables.find(i);
		if(it!=variables.end()){
			it->second+=x;
			if(it->second==0){
				variables.erase(it);
			}
		}
		else{
			variables.insert(pair<unsigned int,T>(i,x));
		}
	}
}

template<class T>
void ExponentsTemplate<T>::decreaseExponent(unsigned int const &i, T const &x){
	increaseExponent(i,T(-1)*x);
}


template<class T>
template<class S>
S ExponentsTemplate<T>::replaceCouplingByOtherCoupling(unsigned int const &c1, unsigned int const &c2, S const &factor){
	// replace the coupling c1, by c2
	// the factor S is returned to the power of the exponent of c1
	T exp1=this->getExponent(c1);
	this->setExponent(c1,0);
	this->increaseExponent(c2,exp1);
	return pow(factor,exp1);
}

template<class T>
template<class S>
S ExponentsTemplate<T>::evaluate(vector<S> const &h) const{
	S result=S(1);
	typename map<unsigned int,T>::const_iterator it;
	for(it=variables.begin();it!=variables.end();it++){
		//cout << h[it->first] << "^" << it->second << "=" << pow(h[it->first],it->second) << "\n";
		result*=pow(h[it->first],it->second);
	}
	return result;
}

template<class T>
ExponentsTemplate<T>& ExponentsTemplate<T>::operator*=(ExponentsTemplate<T> const& rhs){
	typename map<unsigned int,T>::const_iterator it1;
	for(it1=rhs.variables.begin();it1!=rhs.variables.end();it1++){
		typename map<unsigned int,T>::iterator it2=variables.find(it1->first);
		if(it2!=variables.end()){
			it2->second+=it1->second;
			if(it2->second==0){
				variables.erase(it2);
			}
		}
		else{
			variables.insert(pair<unsigned int,T>(it1->first,it1->second));
		}
	}

	return *this;
}

template<class T>
bool operator==(ExponentsTemplate<T> const& lhs, ExponentsTemplate<T> const& rhs){
	return (lhs.variables==rhs.variables);
}

template<class T>
bool operator != (ExponentsTemplate<T> const& lhs, ExponentsTemplate<T> const& rhs)
{
  return ! (lhs==rhs);
}

template<class T>
ostream& operator<<(ostream &o, ExponentsTemplate<T> const &s){
	typename map<unsigned int,T>::const_iterator it;
	for(it=s.variables.begin();it!=s.variables.end();it++){
		if(it!=s.variables.begin()){
			o << "*";
		}
		o << "h" << it->first;
		if(it->second!=T(1)){
			o << "^" << it->second;
		}
	}

	return o;
}

template<class T>
istream& operator>>(istream &o, ExponentsTemplate<T> &s){
	char tempChar='*';
	while(tempChar=='*'){
		tempChar=o.peek();
		if(tempChar=='h'){
			o>>tempChar;
			unsigned int i;
			o>>i;
			T x;
			if(o.peek()=='^'){
				o>>tempChar;
				o>>x;
			}
			else{
				x=T(1);
			}
			s.variables.insert(pair<unsigned int,T>(i,x));

			tempChar=o.peek();
			if(tempChar=='*'){
				o>>tempChar;
			}
		}
		else{
			s.variables.insert(pair<unsigned int,T>(0,0));
		}
	}

	return o;
}

template<class T>
const ExponentsTemplate<T> operator*(ExponentsTemplate<T> const& lhs, ExponentsTemplate<T> const& rhs){
	ExponentsTemplate<T> ret=lhs;
	ret*=rhs;
	return ret;
}


#endif /* EXPONENTSTEMPLATE_H_ */
