/*
 * Graph.cpp
 *
 *  Created on: May 2, 2014
 *      Author: ixert
 */

#include "Graph.h"
#include "functions.h"
#include "nr3.h"
#include <list>
#include "globalVariables.h"

Graph::Graph() : graphNumber(0), canonicalGraphNumber(0), symmetryNumber(-1) {
	// TODO Auto-generated constructor stub

}

void Graph::insert(unsigned short const &site,set<pair<unsigned short,short> > const &coupledSites){
	adj.insert(pair<unsigned short,set<pair<unsigned short,short> > >(site,coupledSites));
}

void Graph::addLink(unsigned short const &site1, unsigned short const &site2, short const &coupling){
	// this function adds a link site1->site2, BUT NOT THE INVERSE!!!
	map<unsigned short,set<pair<unsigned short,short> > >::iterator it1;
	it1=adj.find(site1);
	if(it1!=adj.end()){
		it1->second.insert(pair<unsigned short,short>(site2,coupling));
	}
	else{
		set<pair<unsigned short,short> > tempSet;
		tempSet.insert(pair<unsigned short,short>(site2,coupling));
		adj.insert(pair<unsigned short,set<pair<unsigned short,short> > >(site1,tempSet));
	}
}

void Graph::createNewGraphs(unsigned int const &gCUTOrder, map<char,vector<unsigned int> > const &maxCoordinationNumberByCoupling, list<Graph>& newGraphs){
	// run over all sites and couplings and if the coordinationNumber of this site and coupling is less than the maxCoordinationNumberByCoupling, create a new Graph with a new link
	unsigned int numberOfCouplings=couplingIndex.size();
	unsigned int numberOfSites=adj.size();
	map<unsigned short,set<pair<unsigned short,short> > >::iterator itAdj;
	for(itAdj=adj.begin();itAdj!=adj.end();++itAdj){
		// find coordination numbers by couplings and not coupled sites
		vector<unsigned int> coordinationNumberByCoupling(numberOfCouplings,0);
		vector<bool> coupledSites(numberOfSites,false);
		coupledSites[itAdj->first]=true;
		set<pair<unsigned short,short> >::iterator itSet;
		for(itSet=itAdj->second.begin();itSet!=itAdj->second.end();++itSet){
			coordinationNumberByCoupling[couplingIndex[itSet->second]]++;
			coupledSites[itSet->first]=true;
		}
		/*
		cout << "coordinationNumberByCoupling=";
		for(unsigned int i=0;i<numberOfCouplings;i++){
			cout << coordinationNumberByCoupling[i] << " ";
		}
		cout << "\n";
		*/
		// find legal couplings
		vector<bool> legalCouplings(numberOfCouplings,false);
		map<char,vector<unsigned int> >::const_iterator itMaxCoord;
		for(itMaxCoord=maxCoordinationNumberByCoupling.begin();itMaxCoord!=maxCoordinationNumberByCoupling.end();++itMaxCoord){
			vector<bool> tempLegal(numberOfCouplings,true);
			bool isLegal=true;
			for(unsigned int i=0;i<numberOfCouplings;i++){
				if(coordinationNumberByCoupling[i]>itMaxCoord->second[i]){
					isLegal=false;
					break;
				}
				else{
					if(coordinationNumberByCoupling[i]==itMaxCoord->second[i]){
						tempLegal[i]=false;
					}
				}
			}
			if(isLegal){
				for(unsigned int i=0;i<numberOfCouplings;i++){
					legalCouplings[i]=legalCouplings[i] || tempLegal[i];
				}
			}
		}

		/*
		cout << "legalCouplings=";
		for(unsigned int i=0;i<numberOfCouplings;i++){
			cout << legalCouplings[i] << " ";
		}
		cout << "\n";
		*/

		// now iterate over all (legal) couplings
		map<short,unsigned short>::const_iterator itCoupl;
		for(itCoupl=couplingIndex.begin();itCoupl!=couplingIndex.end();++itCoupl){
			//cout << "itCoupl->second=" << itCoupl->second << "\n";
			if(legalCouplings[itCoupl->second]){
				// if the coordination number is smaller iterate over all sites
				for(unsigned int s=0;s<numberOfSites;s++){
					if(!coupledSites[s]){
						// if the site is not already coupled add the link between these two sites
						Graph g;
						g.adj=adj;
						g.addLink(itAdj->first,s,itCoupl->first);
						g.addLink(s,itAdj->first,inverseCouplings[itCoupl->first]);
						newGraphs.push_back(g);
					}
				}
				// if the number of sites on the graphs is smaller than the gCUT Order add a new site
				if(numberOfSites<gCUTOrder){
					Graph g;
					g.adj=adj;
					g.addLink(itAdj->first,numberOfSites,itCoupl->first);
					g.addLink(numberOfSites,itAdj->first,inverseCouplings[itCoupl->first]);
					newGraphs.push_back(g);
				}
			}
		}
	}
}

void Graph::calculateGraphNumber(){
	//cout << "calculateGraphNumber\n";

	mpz_class numberOfCouplings=couplingIndex.size();

	graphNumber=0;
	unsigned int siteNumber=adj.size();
	vector<unsigned short> siteMap(siteNumber);
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	unsigned int i=0;
	for(it1=adj.begin();it1!=adj.end();it1++){
		siteMap[i]=it1->first;
	}

	unsigned int powerFactor=(siteNumber*(siteNumber-1))/2-1; //this is the maximal exponent
	//cout << "powerFactor=" << powerFactor << "\n";
	mpz_class powerBase=numberOfCouplings+1;
	mpz_class powerNumber=pow(powerBase,powerFactor);
	//cout << "powerNumber=" << powerNumber << "\n";

	unsigned int j=0;
	for(it1=adj.begin();it1!=adj.end();it1++){
		set<pair<unsigned short,short> >::const_iterator it2;
		for(unsigned int k=j+1;k<siteNumber;k++){
			for(it2=it1->second.begin();it2!=it1->second.end();it2++){
				if(it2->first==siteMap[k]){
					graphNumber+=powerNumber*(couplingIndex[it2->second]+1);
					break;
				}
			}
			powerNumber/=powerBase;
		}
		j++;
	}
}
void Graph::calculateCanonicalGraphNumber(){
	// alle permutationen von 2 sitekopplungen durchgehen
	// anzahl permutationen ist p=siteNumber*(siteNumber-1)/2

	// TODO: kann effizienter implementiert werden indem siteMapInv[it1->first] nciht jedes mal ausgefuehrt wird, sondern vorher in einer Variable gespeichert wird

	//cout << "calculate canonicalGraphNumber\n";

	mpz_class numberOfCouplings=couplingIndex.size();

	unsigned int siteNumber=adj.size();

	vector<unsigned short> siteMap(siteNumber);
	map<unsigned short,unsigned short> siteMapInv;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	unsigned int i=0;
	for(it1=adj.begin();it1!=adj.end();it1++){
		siteMap[i]=it1->first;
		siteMapInv.insert(pair<unsigned short,unsigned short>(it1->first,i));
		i++;
	}



	unsigned int p=siteNumber;
	/////////////////////////////////////////////////
	// create real graph adjacency matrix
	MatSignedChar fullAdjMat(siteNumber,siteNumber);
	for(unsigned int i=0;i<siteNumber;i++){
		for(unsigned int j=0;j<siteNumber;j++){
			fullAdjMat[i][j]=0;
		}
	}
	// run over all (interbetween) sites and create adjacency matrix
	for(it1=adj.begin();it1!=adj.end();it1++){
		set<pair<unsigned short,short> >::const_iterator it2;
		for(it2=it1->second.begin();it2!=it1->second.end();it2++){
			fullAdjMat[siteMapInv[it1->first]][siteMapInv[it2->first]]=couplingIndex[it2->second]+1;
		}
	}
	/////////////////////////////////////////////////
	// calculate canonical graphNumber
	canonicalGraphNumber=0;


	/////////////// more efficient way //////////
	vector<MatSignedChar> allAdjMatPerms;
	unsigned int fixedNumbers=0;
	unsigned int actualRow=0;

	{
	vector<vector<unsigned int> > allEntries; //allEntries[site][couplingNumber]
	unsigned int maxEntries=0;

	vector<unsigned int> maxCouplingNumbers(numberOfCouplings.get_ui(),0);

	// find rows with largest numbers of entries
	for(it1=adj.begin();it1!=adj.end();it1++){
		//cout << "it1->first=" << it1->first << "\n";


		vector<unsigned int> maxTemp(numberOfCouplings.get_ui(),0);
		set<pair<unsigned short,short> >::const_iterator it2;
		for(it2=it1->second.begin();it2!=it1->second.end();it2++){
			maxTemp[couplingIndex[it2->second]]++;
		}
		allEntries.push_back(maxTemp);

		// if the number of entries is the same take the ones with the largest number of 2s, if this is the same, the largest numbers of 1s, and so on...
		if(it1->second.size()>maxEntries){
			maxEntries=it1->second.size();
			maxCouplingNumbers=maxTemp;
		}
		else{
			if(it1->second.size()==maxEntries){
				for(int i=maxTemp.size()-1;i>=0;i--){
					if(maxTemp[i]>maxCouplingNumbers[i]){
						maxCouplingNumbers=maxTemp;
						break;
					}
					else{
						if(maxTemp[i]<maxCouplingNumbers[i]){
							break;
						}
					}
				}
			}
		}
	}

	//cout << "maxCouplingNumbers=(" << maxCouplingNumbers[0] << "," << maxCouplingNumbers[1] << "," << maxCouplingNumbers[2] << ")\n";

	for(it1=adj.begin();it1!=adj.end();it1++){
		if(allEntries[siteMapInv[it1->first]]==maxCouplingNumbers){
			if(siteMapInv[it1->first]!=actualRow){
				MatSignedChar newMat=fullAdjMat;
				// switch rows
				for(unsigned int k1=0;k1<p;k1++){
					short temp=newMat[siteMapInv[it1->first]][k1];
					newMat[siteMapInv[it1->first]][k1]=newMat[actualRow][k1];
					newMat[actualRow][k1]=temp;
				}
				// switch columns
				for(unsigned int k1=0;k1<p;k1++){
					short temp=newMat[k1][siteMapInv[it1->first]];
					newMat[k1][siteMapInv[it1->first]]=newMat[k1][actualRow];
					newMat[k1][actualRow]=temp;
				}
				allAdjMatPerms.push_back(newMat);
			}
			else{
				allAdjMatPerms.push_back(fullAdjMat);
			}
		}
	}
	fixedNumbers++;
	}

	while((fixedNumbers<p) && (actualRow<p)){
		vector<MatSignedChar> allAdjMatPerms2;
		for(unsigned int i=0;i<allAdjMatPerms.size();i++){
			//find biggest number in this row
			short maxNumberInRow=0;
			for(unsigned int l=fixedNumbers;l<p;l++){
				if(allAdjMatPerms[i][actualRow][l]>maxNumberInRow){
					maxNumberInRow=allAdjMatPerms[i][actualRow][l];
				}
			}
			for(unsigned int j=fixedNumbers;j<p;j++){
				if(allAdjMatPerms[i][actualRow][j]==maxNumberInRow){ // hier kann man bereits die zeile mit der groessten zahl nehmen.
					if(j!=fixedNumbers){
						MatSignedChar newMat=allAdjMatPerms[i];
						// switch rows
						for(unsigned int k1=0;k1<p;k1++){
							short temp=newMat[fixedNumbers][k1];
							newMat[fixedNumbers][k1]=newMat[j][k1];
							newMat[j][k1]=temp;
						}
						// switch columns
						for(unsigned int k1=0;k1<p;k1++){
							short temp=newMat[k1][fixedNumbers];
							newMat[k1][fixedNumbers]=newMat[k1][j];
							newMat[k1][j]=temp;
						}
						allAdjMatPerms2.push_back(newMat);
					}
					else{
						allAdjMatPerms2.push_back(allAdjMatPerms[i]);
					}
				}
			}
		}
		if(allAdjMatPerms2.size()==0){
			actualRow++;
		}
		else{
			fixedNumbers++;
			allAdjMatPerms.swap(allAdjMatPerms2);
		}
	}

	unsigned int powerFactor=(siteNumber*(siteNumber-1))/2-1;
	//cout << "powerFactor=" << powerFactor << "\n";
	mpz_class powerBase=numberOfCouplings+1;
	mpz_class powerNumber=pow(powerBase,powerFactor);
	//cout << "powerNumber=" << powerNumber << "\n";
	for(unsigned int ps=0;ps<allAdjMatPerms.size();ps++){
		mpz_class cgnTemp=0;
		mpz_class powerNumberTemp=powerNumber;
		for(unsigned int i=0;i<p-1;i++){
			for(unsigned int j=i+1;j<p;j++){
				cgnTemp+=allAdjMatPerms[ps][i][j]*powerNumberTemp;
				//powerFactor--;
				powerNumberTemp/=powerBase;
			}
		}
		if(cgnTemp>canonicalGraphNumber){
			canonicalGraphNumber=cgnTemp;
		}
	}
}

void Graph::calculateSymmetryNumber(){
	// symmetryNumber can be calculated using all permutations of the sites
	// TODO: maybe is is useful for the whole program to have one routine "calculateAllPermutations?"

	symmetryNumber=0;

	// 1) create a vector which contains all sites of the graph
	vector<unsigned short> allSites;
	map<unsigned short,unsigned short> invSiteMap;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	unsigned short index=0;
	for(it1=adj.begin();it1!=adj.end();it1++){
		allSites.push_back(it1->first);
		invSiteMap.insert(pair<unsigned short,unsigned short>(it1->first,index));
		index++;
	}

	// 2) calculate all permutations of the sites
	vector<vector<unsigned short> > allPermutations1;
	calculateAllPermutations(allSites,allPermutations1);

	// 3) apply the permutation to the sites and check if the adj matrix is the same
	for(unsigned int i=0;i<allPermutations1.size();i++){
		// apply permutation to graph
		map<unsigned short,set<pair<unsigned short,short> > > permutatedAdj;
		for(it1=adj.begin();it1!=adj.end();it1++){
			set<pair<unsigned short,short> > tempSet;
			set<pair<unsigned short,short> >::const_iterator itSet1;
			for(itSet1=it1->second.begin();itSet1!=it1->second.end();itSet1++){
				tempSet.insert(pair<unsigned short,short>(allPermutations1[i][invSiteMap[itSet1->first]],itSet1->second));
			}
			permutatedAdj.insert(pair<unsigned short,set<pair<unsigned short,short> > >(allPermutations1[i][invSiteMap[it1->first]],tempSet));
		}

		// check if permutatedAdj equals adj
		if(permutatedAdj==adj){
			// if they are equal increase the symmetryNumber
			symmetryNumber++;
		}
	}
}

map<unsigned short,unsigned short> Graph::calculateDistanceToAllNodes(unsigned short const &startNode){
	// initialize priority queue
	map<unsigned short,unsigned short> allDistances;
	map<unsigned short,unsigned short> distance;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator itAdj;
	for(itAdj=adj.begin();itAdj!=adj.end();itAdj++){
		if(itAdj->first==startNode){
			distance.insert(pair<unsigned short,unsigned short>(itAdj->first,0));
		}
		else{
			distance.insert(pair<unsigned short,unsigned short>(itAdj->first,adj.size())); //maximal distance is number of sites
		}
	}


	// perform dijkstra algoritm
	while(distance.size()>0){
		// find site with smallest actual distance
		map<unsigned short,unsigned short>::iterator minDistance=distance.begin();
		map<unsigned short,unsigned short>::iterator itDistance=distance.begin();
		itDistance++;
		while(itDistance!=distance.end()){
			if(itDistance->second<minDistance->second){
				minDistance=itDistance;
			}
			itDistance++;
		}
		// find actual site in adj
		itAdj=adj.find(minDistance->first);
		// run over all neighbors from this site
		set<pair<unsigned short,short> >::const_iterator itSet;
		for(itSet=itAdj->second.begin();itSet!=itAdj->second.end();itSet++){
			// find neighbor in distance
			map<unsigned short,unsigned short>::iterator itDistance2=distance.find(itSet->first);
			// update distance
			if(itDistance2!=distance.end()){
				if((minDistance->second+1)<itDistance2->second){
					itDistance2->second=minDistance->second+1;
				}
			}
		}
		// remove actual site from distance
		allDistances.insert(*minDistance);
		distance.erase(minDistance);
	}

	return allDistances;
}

mpz_class Graph::getGraphNumber(){
	if((graphNumber==0) && (adj.size()>1)){
		this->calculateGraphNumber();
	}
	return graphNumber;
}
mpz_class Graph::getCanonicalGraphNumber(){
	if((canonicalGraphNumber==0) && (adj.size()>1)){
		this->calculateCanonicalGraphNumber();
	}
	return canonicalGraphNumber;
}

int Graph::getSymmetryNumber(){
	if((symmetryNumber==-1) && (adj.size()>1)){
		this->calculateSymmetryNumber();
	}
	return symmetryNumber;
}

vector<Graph> Graph::removeLinks(){
	// this method removes one link at a time and so creates a subgraph
	// If isolated sites are found they are removed.

	vector<Graph> subgraphs;
	if(adj.size()<=2){
		// if only two sites are present, there is only one link!
		return subgraphs;
	}

	map<unsigned short,set<pair<unsigned short,short> > >::iterator it1;
	for(it1=adj.begin();it1!=adj.end();it1++){
		set<pair<unsigned short,short> >::iterator it2;
		for(it2=it1->second.begin();it2!=it1->second.end();it2++){
			if(it1->first < it2->first){
				// if the first site has a smaller number than the second remove the link
				bool firstSiteRemoved=false;
				bool secondSiteRemoved=false;
				map<unsigned short,set<pair<unsigned short,short> > > tempAdj=adj;
				map<unsigned short,set<pair<unsigned short,short> > >::iterator it3=tempAdj.find(it1->first);
				it3->second.erase(*it2);
				if(it3->second.size()==0){
					// if site is isolated remove this site from the map
					tempAdj.erase(it3);
					firstSiteRemoved=true;
				}

				// now remove the corresponding link into the other direction
				it3=tempAdj.find(it2->first);
				map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(it2->second);
				it3->second.erase(pair<unsigned short,short>(it1->first,itInvCoupl->second));

				if(it3->second.size()==0){
					tempAdj.erase(it3);
					secondSiteRemoved=true;
				}

				subgraphs.push_back(Graph());
				subgraphs.back().adj=tempAdj;
			}
		}
	}

	return subgraphs;
}

vector<Graph> Graph::removeLinks(vector<unsigned short> const &refSites){
	// this method removes one link at a time and so creates a subgraph
	// If isolated sites are found they are removed.
	// in contrast to the "normal" removeLinks() method this method only keeps the subgraphs in which the sites specified in refSites are contained.

	vector<Graph> subgraphs;
	if(adj.size()<=2){
		// if only two sites are present, there is only one link!
		return subgraphs;
	}

	map<unsigned short,set<pair<unsigned short,short> > >::iterator it1;
	for(it1=adj.begin();it1!=adj.end();it1++){
		set<pair<unsigned short,short> >::iterator it2;
		for(it2=it1->second.begin();it2!=it1->second.end();it2++){
			if(it1->first < it2->first){
				// if the first site has a smaller number than the second remove the link
				map<unsigned short,set<pair<unsigned short,short> > > tempAdj=adj;
				map<unsigned short,set<pair<unsigned short,short> > >::iterator it3=tempAdj.find(it1->first);
				it3->second.erase(*it2);

				bool removedRefSite=false;
				if(it3->second.size()==0){
					// if site is isolated remove this site from the map
					for(unsigned int r=0;r<refSites.size();r++){
						if(it3->first==refSites[r]){
							removedRefSite=true;
							break;
						}
					}
					tempAdj.erase(it3);
				}
				if(!removedRefSite){
					// now remove the corresponding link into the other direction
					it3=tempAdj.find(it2->first);
					map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(it2->second);
					it3->second.erase(pair<unsigned short,short>(it1->first,itInvCoupl->second));

					if(it3->second.size()==0){
						for(unsigned int r=0;r<refSites.size();r++){
							if(it3->first==refSites[r]){
								removedRefSite=true;
								break;
							}
						}
						tempAdj.erase(it3);
					}

					if(!removedRefSite){
						subgraphs.push_back(Graph());
						subgraphs.back().adj=tempAdj;
					}
				}
			}
		}
	}

	return subgraphs;
}

vector<unsigned short> Graph::identifySites(Graph const &refGraph, vector<unsigned short> const &refSites){
	// refSites are sites on refGraph
	// this method finds the sites on *this which correspond to the refSites on refGraph

	vector<unsigned short> identifiedSites;

	cout << "G1:\n";
	cout << *this << "\n";
	cout << "G2:\n";
	cout << refGraph << "\n";

	vector<unsigned short> allSitesG1;
	map<unsigned short,unsigned short> invSiteMapG1;
	vector<unsigned short> allSitesG2;
	map<unsigned short,unsigned short> invSiteMapG2;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	unsigned short index=0;
	for(it1=adj.begin();it1!=adj.end();it1++){
		allSitesG1.push_back(it1->first);
		invSiteMapG1.insert(pair<unsigned short,unsigned short>(it1->first,index));
		index++;
	}
	index=0;
	for(it1=refGraph.adj.begin();it1!=refGraph.adj.end();it1++){
		allSitesG2.push_back(it1->first);
		invSiteMapG2.insert(pair<unsigned short,unsigned short>(it1->first,index));
		index++;
	}

	cout << "allSitesG1:\n";
	for(unsigned int i=0;i<allSitesG1.size();i++){
		cout << allSitesG1[i] << " ";
	}
	cout << "\n";
	cout << "invSiteMapG1:\n";
	map<unsigned short,unsigned short>::const_iterator it;
	for(it=invSiteMapG1.begin();it!=invSiteMapG1.end();it++){
		cout << "(" << it->first << " " << it->second << ") ";
	}
	cout << "\n";

	cout << "allSitesG2:\n";
	for(unsigned int i=0;i<allSitesG2.size();i++){
		cout << allSitesG2[i] << " ";
	}
	cout << "\n";
	cout << "invSiteMapG2:\n";
	for(it=invSiteMapG2.begin();it!=invSiteMapG2.end();it++){
		cout << "(" << it->first << " " << it->second << ") ";
	}
	cout << "\n";

	cout << "refSites=";
	for(unsigned int i=0;i<refSites.size();i++){
		cout << refSites[i] << " ";
	}
	cout << "\n";


	// calculate all site permutations of g2
	vector<vector<unsigned short> > allPermutations1;
	calculateAllPermutations(allSitesG2,allPermutations1);
	/*
	// this routine is now outsourced to the function "calculateAllPermutations"
	vector<unsigned short> vTemp;
	allPermutations1.push_back(vTemp);
	for(unsigned i=0;i<allSitesG2.size();i++){
		vector<vector<unsigned short> > allPermutations2;
		for(unsigned int j=0;j<allSitesG2.size();j++){
			for(unsigned int k=0;k<allPermutations1.size();k++){
				// check if allSitesG2[j] exists already in allPermutations1[k]
				bool existsAlready=false;
				for(unsigned int m=0;m<allPermutations1[k].size();m++){
					if(allPermutations1[k][m]==allSitesG2[j]){
						existsAlready=true;
						break;
					}
				}
				if(!existsAlready){
					vTemp=allPermutations1[k];
					vTemp.push_back(allSitesG2[j]);
					allPermutations2.push_back(vTemp);
				}
			}
		}
		allPermutations1.swap(allPermutations2);
	}
	*/

	// test output all permutations
	cout << "permutations:\n";
	for(unsigned int i=0;i<allPermutations1.size();i++){
		for(unsigned int j=0;j<allPermutations1[i].size();j++){
			cout << allPermutations1[i][j] << " ";
		}
		cout << "\n";
	}

	// now apply every permutation to g1 and check if the new permutated graph is equal to refGraph=g2
	for(unsigned int i=0;i<allPermutations1.size();i++){
		// apply permutation to graph
		Graph permutatedGraph;
		for(it1=adj.begin();it1!=adj.end();it1++){
			set<pair<unsigned short,short> > tempSet;
			set<pair<unsigned short,short> >::const_iterator itSet1;
			for(itSet1=it1->second.begin();itSet1!=it1->second.end();itSet1++){
				tempSet.insert(pair<unsigned short,short>(allPermutations1[i][invSiteMapG1[itSet1->first]],itSet1->second));
			}
			permutatedGraph.adj.insert(pair<unsigned short,set<pair<unsigned short,short> > >(allPermutations1[i][invSiteMapG1[it1->first]],tempSet));
		}

		cout << "permutatedGraph:\n";
		cout << permutatedGraph << "\n";

		// check if permutatedGraph equals refGraph
		if(permutatedGraph==refGraph){
			cout << "foundPermutatedGraph!\n";
			// if they are equal find out the indexes in the permutation which correspond to the refSites (= the sites you want to identify)
			vector<unsigned short> refIndexes;
			for(unsigned int j=0;j<refSites.size();j++){
				for(unsigned int k=0;k<allPermutations1[i].size();k++){
					if(allPermutations1[i][k]==refSites[j]){
						refIndexes.push_back(k);
						break;
					}
				}
			}
			for(unsigned int j=0;j<refIndexes.size();j++){
				identifiedSites.push_back(allSitesG1[refIndexes[j]]);
			}
			return identifiedSites;
		}
	}

	return identifiedSites;

	// COMMENT: this scheme allows to obtain the complete mapping directly!
	// so if one wants to calculate this for every link on one specific graph, this calculation has only to be performed once!
	// TODO: implement this idea!
}

vector<vector<unsigned short> > Graph::getAllNSiteCombinations(unsigned short const &N){
	vector<vector<unsigned short> > allCombinations;
	vector<unsigned short> allSitesG1;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	for(it1=adj.begin();it1!=adj.end();it1++){
		allSitesG1.push_back(it1->first);
	}


	vector<unsigned short> vus;
	for(unsigned int i=0;i<allSitesG1.size();i++){
		vector<unsigned short> vus;
		vus.push_back(allSitesG1[i]);
		allCombinations.push_back(vus);
	}
	for(unsigned int i=1;i<N;i++){
		vector<vector<unsigned short> > allCombinations2;
		for(unsigned int j=0;j<allCombinations.size();j++){
			for(unsigned int k=0;k<allSitesG1.size();k++){
				vector<unsigned short> vus=allCombinations[j];
				if(vus.back()<allSitesG1[k]){
					vus.push_back(allSitesG1[k]);
					allCombinations2.push_back(vus);
				}
			}
		}
		allCombinations.swap(allCombinations2);
	}

	cout << "test output:\n";
	for(unsigned int i=0;i<allCombinations.size();i++){
		for(unsigned int j=0;j<allCombinations[i].size();j++){
			cout << allCombinations[i][j] << " ";
		}
		cout << "\n";
	}

	return allCombinations;
}

vector<pair<unsigned int,vector<unsigned short> > > Graph::calculateEmbeddingsOnThisCluster(vector<Graph> &allGraphs, unsigned short const &initialSite){
	// calculates the embeddings from allGraphs onto this graph starting at the initialSite
	// the return value is a vector of the embeddings: the unsigned int is the position of the graph in allGraphs and vector<unsigned short> are the identified sites

	typedef map<unsigned short,set<pair<unsigned short,short> > > adjType;
	vector<pair<unsigned int,vector<unsigned short> > > allEmbeddings;

	// find maximum numberOfLinks
	unsigned int maxNumberOfLinks=0;
	unsigned int maxNumberOfSites=0;
	for(unsigned int i=0;i<allGraphs.size();i++){
		unsigned int sumOfLinks=allGraphs[i].sumOfLinks();
		unsigned int numberOfSites=allGraphs[i].gCUTOrder();
		if(sumOfLinks>maxNumberOfLinks){
			maxNumberOfLinks=sumOfLinks;
		}
		if(numberOfSites>maxNumberOfSites){
			maxNumberOfSites=numberOfSites;
		}
	}
	maxNumberOfLinks/=2;

	// create all usedBonds
	list<set<pair<unsigned short,unsigned short> > > usedBondsList; // usedBonds on the masterCluster; the pair (in the set) must be sorted, i.e. pair.first < pair.second

	// start with inserting all couplings from the initialSite into usedBonds
	adjType::const_iterator itInitialSite=adj.find(initialSite);
	set<pair<unsigned short,short>  >::const_iterator itInitialSiteSet;
	for(itInitialSiteSet=itInitialSite->second.begin();itInitialSiteSet!=itInitialSite->second.end();itInitialSiteSet++){
		pair<unsigned short,unsigned short> pusus;
		if(initialSite<itInitialSiteSet->first){
			pusus.first=initialSite;
			pusus.second=itInitialSiteSet->first;
		}
		else{
			pusus.first=itInitialSiteSet->first;
			pusus.second=initialSite;
		}
		set<pair<unsigned short,unsigned short> > tempSet;
		tempSet.insert(pusus);
		usedBondsList.push_back(tempSet);
	}

	cout << "create usedBonds:\n";
	cout << "maxNumberOfLinks=" << maxNumberOfLinks << "\n";
	cout << "maxNumberOfSites=" << maxNumberOfSites << "\n";

	unsigned int index=0;

	list<set<pair<unsigned short,unsigned short> > >::iterator it_usedBondsList=usedBondsList.begin();
	while(it_usedBondsList!=usedBondsList.end()){
		cout << index << "/" << usedBondsList.size() << "\n";
		cout << "links in actual usedBonds=" << it_usedBondsList->size() << "\n";
		if(it_usedBondsList->size()<maxNumberOfLinks){
			set<unsigned short> usedSites;
			set<pair<unsigned short,unsigned short> >::iterator it_usedBondsList_set;
			for(it_usedBondsList_set=it_usedBondsList->begin();it_usedBondsList_set!=it_usedBondsList->end();it_usedBondsList_set++){
				usedSites.insert(it_usedBondsList_set->first);
				usedSites.insert(it_usedBondsList_set->second);
			}

			if(usedSites.size()<=maxNumberOfSites){

				set<unsigned short>::iterator it_usedSites;

				cout << "usedSites=";
				for(it_usedSites=usedSites.begin();it_usedSites!=usedSites.end();it_usedSites++){
					cout << *it_usedSites << " ";
				}
				cout << "\n";

				for(it_usedSites=usedSites.begin();it_usedSites!=usedSites.end();it_usedSites++){
					adjType::const_iterator itMasterClusterAdj=adj.find(*it_usedSites); // find starting site
					set<pair<unsigned short,short> >::const_iterator itMasterClusterSet;
					for(itMasterClusterSet=itMasterClusterAdj->second.begin();itMasterClusterSet!=itMasterClusterAdj->second.end();itMasterClusterSet++){ // run over all couplings that go out from the starting site
						// if the maximum Number of Sites is already reached do not add new sites!
						// check if the "new site" (itMasterClusterSet->first) is already within the set
						if((usedSites.size()<maxNumberOfSites) || (usedSites.find(itMasterClusterSet->first)!=usedSites.end())){
							pair<unsigned short,unsigned short> pusus;
							if(*it_usedSites<itMasterClusterSet->first){
								pusus.first=*it_usedSites;
								pusus.second=itMasterClusterSet->first;
							}
							else{
								pusus.first=itMasterClusterSet->first;
								pusus.second=*it_usedSites;
							}
							set<pair<unsigned short,unsigned short> > tempSet=*it_usedBondsList;
							pair<set<pair<unsigned short,unsigned short> >::iterator,bool> ret=tempSet.insert(pusus);
							if(ret.second){
								// now check if this set already exists in usedBonds
								bool existsAlready=false;
								list<set<pair<unsigned short,unsigned short> > >::iterator it_usedBondsList2;
								for(it_usedBondsList2=usedBondsList.begin();it_usedBondsList2!=usedBondsList.end();it_usedBondsList2++){
									if(*it_usedBondsList2==tempSet){
										existsAlready=true;
										break;
									}
								}
								if(!existsAlready){
									cout << "push tempSet into usedBondsList!\n";
									usedBondsList.push_back(tempSet);
								}
								else{
									cout << "tempSet exists already in usedBondsList!\n";
								}
							}
						}
					}
				}
			}
		}
		it_usedBondsList++;
		index++;

		//char t;
		//cin >> t;
	}

	// test-output of usedBonds
	cout << "test output of usedBonds\n";
	for(it_usedBondsList=usedBondsList.begin();it_usedBondsList!=usedBondsList.end();it_usedBondsList++){
		set<pair<unsigned short,unsigned short> >::iterator itSet;
		for(itSet=it_usedBondsList->begin();itSet!=it_usedBondsList->end();itSet++){
			cout << itSet->first << " " << itSet->second << "\n";
		}
		cout << "\n";
	}


	// now, after all usedBonds are created:
	// 	a) create a Graph from the usedBonds
	//	b) find the corresponding graph in allGraphs
	//	c) identify the corresponding sites
	for(it_usedBondsList=usedBondsList.begin();it_usedBondsList!=usedBondsList.end();it_usedBondsList++){
		// create Graph
		Graph tempGraph;
		set<pair<unsigned short,unsigned short> >::iterator itSet;
		for(itSet=it_usedBondsList->begin();itSet!=it_usedBondsList->end();itSet++){
			// find linkType (this can be performed in the step where all bonds are created for more performance
			adjType::const_iterator itMasterClusterAdj=adj.find(itSet->first);
			if(itMasterClusterAdj==adj.end()){
				cout << "ERROR: Site not Found in masterCluster.adj!\n";
			}
			set<pair<unsigned short,short> >::const_iterator itMasterClusterSet;
			for(itMasterClusterSet=itMasterClusterAdj->second.begin();itMasterClusterSet!=itMasterClusterAdj->second.end();itMasterClusterSet++){
				if(itMasterClusterSet->first==itSet->second){
					tempGraph.addLink(itSet->first,itSet->second,itMasterClusterSet->second);
					break;
				}
			}
			// find the link in opposite direction
			itMasterClusterAdj=adj.find(itSet->second);
			for(itMasterClusterSet=itMasterClusterAdj->second.begin();itMasterClusterSet!=itMasterClusterAdj->second.end();itMasterClusterSet++){
				if(itMasterClusterSet->first==itSet->first){
					tempGraph.addLink(itSet->second,itSet->first,itMasterClusterSet->second);
					break;
				}
			}
		}

		for(unsigned int i=0;i<allGraphs.size();i++){
			// check if graph is the same (topologically)
			if(tempGraph.getCanonicalGraphNumber()==allGraphs[i].getCanonicalGraphNumber()){
				// identify sites
				vector<unsigned short> refSites=allGraphs[i].getAllSites();
				vector<unsigned short> identifiedSites=tempGraph.identifySites(allGraphs[i],refSites);
				allEmbeddings.push_back(pair<unsigned int,vector<unsigned short> >(i,identifiedSites));
				break;
			}
		}
	}

	return allEmbeddings;
}

void Graph::embedGraphOnThisCluster(Graph &refGraph, unsigned short const &initialSite, list<vector<unsigned short> > &graphEmbeddings){
	// 0) first of all, clear the list
	graphEmbeddings.clear();

	unsigned short numberOfSites=refGraph.gCUTOrder();
	vector<unsigned short> permutation(numberOfSites);
	vector<unsigned short> vus(numberOfSites);

	for(unsigned short firstSite=0;firstSite<numberOfSites;firstSite++){
		list<vector<int> > identifiedSites;
		vector<int> initialVector(numberOfSites,-1);
		initialVector[firstSite]=initialSite;
		identifiedSites.push_back(initialVector);
		// iterate over all identifiedSites
		list<vector<int> >::iterator itIdSites1=identifiedSites.begin();
		while(itIdSites1!=identifiedSites.end()){
			// iterate over all identified sites
			for(unsigned short i=0;i<numberOfSites;i++){
				int NNSite=(*itIdSites1)[i];
				if(NNSite!=-1){
					// iterate over all neighbors of i
					set<pair<unsigned short,short> >::const_iterator itNNg;
					for(itNNg=refGraph.adj[i].begin();itNNg!=refGraph.adj[i].end();++itNNg){
						// if this neighbor of refGraph is "free" iterate over all free neighbors of G
						if((*itIdSites1)[itNNg->first]==-1){
							set<pair<unsigned short,short> >::const_iterator itNNbig;
							for(itNNbig=adj[NNSite].begin();itNNbig!=adj[NNSite].end();++itNNbig){
								// check if this site has the same coupling type
								if(itNNbig->second==itNNg->second){
									// check if this site is free
									bool free=true;
									unsigned short NNSiteBig=itNNbig->first;
									for(unsigned short j=0;j<numberOfSites;j++){
										if((*itIdSites1)[j]==NNSiteBig){
											free=false;
											break;
										}
									}
									if(free){
										// identify both sites with each other and push this back to identifiedSites
										vector<int> idVec=(*itIdSites1);
										idVec[itNNg->first]=NNSiteBig;
										identifiedSites.push_back(idVec);
									}
								}
							}
						}
					}
				}
			}
			++itIdSites1;
		}

		// check which of the obtained identifications are valid
		// => assign mapped sites (on G) to the adjacency matrix of g
		// => then check if this adj-matrix is a sub-adj-matrix of G (sub-adj matrix is the part of the full adj-matrix, when only the sites which are connected to each other are taken into account)
		itIdSites1=identifiedSites.begin();
		while(itIdSites1!=identifiedSites.end()){
			// check wether this combination is complete (i.e. contains no -1 elements)
			bool isComplete=true;
			for(unsigned int i=0;i<numberOfSites;i++){
				if((*itIdSites1)[i]==-1){
					isComplete=false;
					break;
				}
			}
			if(isComplete){
				bool deleteComb=false;
				map<unsigned short,set<pair<unsigned short,short> > >::const_iterator itAdj;
				for(itAdj=refGraph.adj.begin();itAdj!=refGraph.adj.end();++itAdj){
					unsigned short site1OnG=(*itIdSites1)[itAdj->first];
					set<pair<unsigned short,short> >::const_iterator itSet;
					for(itSet=itAdj->second.begin();itSet!=itAdj->second.end();++itSet){
						unsigned short site2OnG=(*itIdSites1)[itSet->first];
						// check if this link exists in G
						set<pair<unsigned short,short> >::iterator itFind=adj[site1OnG].find(pair<unsigned short,short>(site2OnG,itSet->second));
						if(itFind==adj[site1OnG].end()){
							deleteComb=true;
							break;
						}
					}
					if(deleteComb){
						break;
					}
				}

				if(!deleteComb){
					// if the adj matrices are the same, check if this siteCombination is exactly the same as a previous one
					// => assign the sites to the adj-matrix of g and check if they are the same under permutations.

					list<vector<unsigned short> >::iterator itGraphEmb;
					for(itGraphEmb=graphEmbeddings.begin();itGraphEmb!=graphEmbeddings.end();++itGraphEmb){
						// 1) first of all the sites must be identical (in the same step the permutation can be found)
						bool sameSites=true;
						for(unsigned int i=0;i<numberOfSites;i++){
							bool foundSite=false;
							for(unsigned int j=0;j<numberOfSites;j++){
								if((*itGraphEmb)[j]==(*itIdSites1)[i]){
									foundSite=true;
									permutation[i]=j;
									break;
								}
							}
							if(!foundSite){
								sameSites=false;
								break;
							}
						}
						//cout << "sameSites=" << sameSites << "\n";
						if(sameSites){
							// 2) check if the adj matrix is invariant under this permutation
							bool isIdentical=true;
							map<unsigned short,set<pair<unsigned short,short> > >::const_iterator itAdj2;
							for(itAdj2=refGraph.adj.begin();itAdj2!=refGraph.adj.end();++itAdj2){
								unsigned short permutatedSite=permutation[itAdj2->first];
								set<pair<unsigned short,short> >::const_iterator itSet2;
								for(itSet2=itAdj2->second.begin();itSet2!=itAdj2->second.end();++itSet2){
									unsigned short permutatedSite2=permutation[itSet2->first];
									// iterate through the whole adj matrix and check if this matrix element is the same in the permutated one
									set<pair<unsigned short,short> >::const_iterator itSetFind=refGraph.adj[permutatedSite].find(pair<unsigned short,short>(permutatedSite2,itSet2->second));
									if(itSetFind==refGraph.adj[permutatedSite].end()){
										isIdentical=false;
										break;
									}
								}
								if(!isIdentical){
									break;
								}
							}
							if(isIdentical){
								deleteComb=true;
							}
						}

					}
				}
				//cout << "deleteComb=" << deleteComb << "\n";
				//cout << "\n";
				if(!deleteComb){
					//itIdSites1=identifiedSites.erase(itIdSites1); deleting is not necessary! Push back to graphEmbeddings!
					for(unsigned int i=0;i<numberOfSites;i++){
						vus[i]=(*itIdSites1)[i];
					}
					graphEmbeddings.push_back(vus);
				}
			}
			++itIdSites1;
		}
	}
}

unsigned int Graph::gCUTOrder(){
	return adj.size();
}

unsigned int Graph::sumOfLinks() const{
	unsigned int sum=0;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	for(it1=adj.begin();it1!=adj.end();it1++){
		sum+=it1->second.size();
	}
	return sum;
}

vector<unsigned int> Graph::sumOfBondTypes(){
	vector<unsigned int> sum(couplingIndex.size(),0);
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	for(it1=adj.begin();it1!=adj.end();it1++){
		set<pair<unsigned short,short> >::const_iterator it2;
		for(it2=it1->second.begin();it2!=it1->second.end();it2++){
			sum[couplingIndex[it2->second]]++;
		}
	}
	return sum;
}

vector<unsigned short> Graph::getAllSites(){
	vector<unsigned short> allSites(adj.size());
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it1;
	unsigned int index=0;
	for(it1=adj.begin();it1!=adj.end();it1++){
		allSites[index]=it1->first;
		index++;
	}
	return allSites;
}

unsigned short Graph::getFirstSite(){
	if(adj.size()!=0){
		return adj.begin()->first;
	}
	else{
		return 0;
	}
}

unsigned short Graph::getMaxDistance(){
	unsigned short maxDistance=0;
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator itAdj;
	for(itAdj=adj.begin();itAdj!=adj.end();itAdj++){
		map<unsigned short,unsigned short> allDistances=calculateDistanceToAllNodes(itAdj->first);
		map<unsigned short,unsigned short>::const_iterator it;
		for(it=allDistances.begin();it!=allDistances.end();it++){
			if(it->second>maxDistance){
				maxDistance=it->second;
			}
		}
	}
	return maxDistance;
}

bool Graph::isPCUTGraph(unsigned short const &pCUTOrder){
	// check if the graph can occur in the given pCUT order
	// 1) check if the sum over all links is larger than the pCUTOrder
	// 2) run over all links and remove this particular link (use method removeLinks())
	// check if the resulting graph a) has the same number of sites and then b) is connected
	// if it has not the same number of sites, or is not connected, then increase the pCUTCounter by 2, else increase by 1 (i.e. the link was part of a loop)
	if(this->sumOfLinks()/2>pCUTOrder){
		return false;
	}
	unsigned short pCUTCounter=0;
	vector<Graph> subgraphs=this->removeLinks();
	for(unsigned int i=0;i<subgraphs.size();i++){
		if(subgraphs[i].adj.size()<adj.size()){
			pCUTCounter+=2;
		}
		else{
			if(subgraphs[i].isConnected()){
				pCUTCounter+=1;
			}
			else{
				pCUTCounter+=2;
			}
		}
		if(pCUTCounter>pCUTOrder){
			return false;
		}
	}

	return true;
}

bool Graph::isConnected(){
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator itAdj;
	set<unsigned short> allSites;
	set<unsigned short> newSites;
	allSites.insert(adj.begin()->first);
	newSites.insert(adj.begin()->first);
	while(newSites.size()>0){
		set<unsigned short> newSites2;
		set<unsigned short>::const_iterator it1;
		for(it1=newSites.begin();it1!=newSites.end();it1++){
			itAdj=adj.find(*it1);
			set<pair<unsigned short,short> >::const_iterator itSet;
			for(itSet=itAdj->second.begin();itSet!=itAdj->second.end();itSet++){
				pair<set<unsigned short>::iterator,bool> ret=allSites.insert(itSet->first);
				if(ret.second){
					newSites2.insert(itSet->first);
				}
			}
		}
		newSites.swap(newSites2);
	}

	return (allSites.size()==adj.size());
}

void Graph::numberOfEdgeSpins(vector<unsigned short> const &sites, vector<vector<unsigned short> > &edgeSpins){
	// this function returns the numberOfEdge spins for the in sites defined sites

	// run over all sites
	for(unsigned int i=0;i<sites.size();i++){
		unsigned short actualSite=sites[i];
		// run over all connected sites
		set<pair<unsigned short,short> >::const_iterator itSet;
		for(itSet=adj[actualSite].begin();itSet!=adj[actualSite].end();++itSet){
			unsigned short connectedSite=itSet->first;
			unsigned short couplingType=couplingIndex[itSet->second];
			// check if the connectedSite is contained within sites
			bool contained=false;
			for(unsigned int j=0;j<sites.size();j++){
				if(connectedSite==sites[j]){
					contained=true;
					break;
				}
			}
			if(!contained){
				edgeSpins[i][couplingType]++;
			}
		}
	}
}

bool Graph::isStrongSubgraph(Graph const &refGraph){
	// this routine checks if the Graph is a strong subgraph of refGraph
	// this function assumes:
	// 1) that the sites of (*this) have the same numbers as the corresponding sites on refGraph
	// 2) that (*this) is a valid subgraph of refGraph
	// if one of this conditions isn't met, than the behaviour is undefined

	// run over all sites of (*this)
	map<unsigned short,set<pair<unsigned short,short> > >::iterator itAdj1;
	for(itAdj1=this->adj.begin();itAdj1!=this->adj.end();++itAdj1){
		// run over all links from the site on refGraph
		map<unsigned short,set<pair<unsigned short,short> > >::const_iterator itAdjRef=refGraph.adj.find(itAdj1->first);
		if(itAdjRef!=refGraph.adj.end()){
			set<pair<unsigned short,short> >::const_iterator itSetRef;
			for(itSetRef=itAdjRef->second.begin();itSetRef!=itAdjRef->second.end();++itSetRef){
				// check if the linked site exists on (*this)
				map<unsigned short,set<pair<unsigned short,short> > >::iterator itAdj2=this->adj.find(itSetRef->first);
				if(itAdj2!=this->adj.end()){
					// if the linked site exists, than check if the link is contained within (*this)
					set<pair<unsigned short,short> >::iterator itSet=itAdj1->second.find(*itSetRef);
					if(itSet==itAdj1->second.end()){
						// link does not exist
						return false;
					}
				}
			}
		}
		else{
			cout << "ERROR: site not found on refGraph!\n";
		}
	}

	return true;
}

void Graph::assignSites(Graph &newGraph, vector<unsigned short> const &newSites){
	// this method generates a newGraph where the sites are assigned to the sites from newSites
	map<unsigned short,set<pair<unsigned short,short> > >::iterator itAdj;
	for(itAdj=this->adj.begin();itAdj!=this->adj.end();++itAdj){
		set<pair<unsigned short,short> >::iterator itSet;
		for(itSet=itAdj->second.begin();itSet!=itAdj->second.end();++itSet){
			newGraph.addLink(newSites[itAdj->first],newSites[itSet->first],itSet->second);
		}
	}
}

void Graph::writeSiteCombinations(ostream &o){
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it;

	for(it=adj.begin();it!=adj.end();it++){
		set<pair<unsigned short,short> >::const_iterator it2;
		for(it2=it->second.begin();it2!=it->second.end();it2++){
			map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(it2->second);
			if(itInvCoupl->second==it2->second){
				// if the coupling has no inverse coupling (which means the inverse is the coupling itself only add the bond if the first site is smaller than the second site
				if(it->first<it2->first){
					o << it2->second << " " << it->first << " " << it2->first << "\n";
				}
			}
			else{
				// if the coupling has an inverse coupling only add it if the coupling has the smaller coupling index than its inverse coupling
				if(couplingIndex[it2->second]<couplingIndex[itInvCoupl->second]){
					o << it2->second << " " << it->first << " " << it2->first << "\n";
				}
			}
		}
	}
}

ostream& operator<<(ostream &o, Graph const &g){
	map<unsigned short,set<pair<unsigned short,short> > >::const_iterator it;


	for(it=g.adj.begin();it!=g.adj.end();it++){
		set<pair<unsigned short,short> >::const_iterator it2;
		for(it2=it->second.begin();it2!=it->second.end();it2++){
			map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(it2->second);
			if(itInvCoupl->second==it2->second){
				// if the coupling has no inverse coupling (which means the inverse is the coupling itself only add the bond if the first site is smaller than the second site
				if(it->first<it2->first){
					o << it2->second << " " << it->first << " " << it2->first << ";" << "\n";
				}
			}
			else{
				// if the coupling has an inverse coupling only add it if the coupling has the smaller coupling index than its inverse coupling
				if(couplingIndex[it2->second]<couplingIndex[itInvCoupl->second]){
					o << it2->second << " " << it->first << " " << it2->first << ";" << "\n";
				}
			}
		}
	}


	/*
	// test full output
	for(it=g.adj.begin();it!=g.adj.end();it++){
		set<pair<unsigned short,short> >::const_iterator it2;
		for(it2=it->second.begin();it2!=it->second.end();it2++){
			o << it2->second << " " << it->first << " " << it2->first << ";" << "\n";
		}
	}
	*/

	return o;
}

istream& operator>>(istream &o, Graph &g){
	string tempstring;
	unsigned short x;
	unsigned short y;
	unsigned short c;
	char tempChar;
	while(o>>c){
		o>>x;
		o>>y;

		g.addLink(x,y,c);
		map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(c);
		g.addLink(y,x,itInvCoupl->second);
		o>>tempChar;
	}

	return o;
}

bool operator==(Graph const& lhs, Graph const& rhs){
	return (lhs.adj==rhs.adj);
}

bool operator!=(Graph const& lhs, Graph const& rhs){
	return !(lhs==rhs);
}

