/*
 * GraphController.cpp
 *
 *  Created on: Jul 1, 2014
 *      Author: ixert
 */

#include "GraphController.h"
#include "Graph.h"
#include "LatticeGraph.h"
#include<sstream>
#include<fstream>
#include<iostream>
#include "SpinModelClass.h"
#include "SpinModelTDLClass.h"
#include "functions.h"
#include<tuple>

using namespace std;

GraphController::GraphController(string const &sInverseCouplingsFile, string const &sLatticeFile){
	string tempString;
	char tempChar;
	int x,y;
	unsigned short index=0;
	// read inverseCouplingsFile
	if(sInverseCouplingsFile!=""){
		fstream fsInverseCouplingsFile;
		fsInverseCouplingsFile.open(sInverseCouplingsFile.c_str(),fstream::in);
		getline(fsInverseCouplingsFile,tempString);
		while(fsInverseCouplingsFile>>x){
			fsInverseCouplingsFile >> y;
			inverseCouplings.insert(pair<short,short>(x,y));
			couplingIndex.insert(pair<short,unsigned short>(x,index++));
		}
		fsInverseCouplingsFile.close();
	}

	// read latticeFile
	if(sLatticeFile!=""){
		fstream fsLatticeFile;
		fsLatticeFile.open(sLatticeFile.c_str(),fstream::in);
		getline(fsLatticeFile,tempString);
		while(fsLatticeFile>>tempChar){
			if(tempChar>='A' && tempChar<='Z'){
				vector<int> tempVec(5);
				tempVec[0]=tempChar-'A';
				char siteTypeFrom=tempChar;
				fsLatticeFile >> x;
				tempVec[1]=x;
				fsLatticeFile >> y;
				tempVec[2]=y;
				fsLatticeFile >> tempChar;
				tempVec[3]=tempChar-'A';
				fsLatticeFile >> x;
				tempVec[4]=x;
				latticeVectors.push_back(tempVec);
				map<char,vector<unsigned int> >::iterator itFind=coordinationNumberByCouplingForSiteTypes.find(siteTypeFrom);
				if(itFind==coordinationNumberByCouplingForSiteTypes.end()){
					vector<unsigned int> tempCoordVec(couplingIndex.size(),0);
					(tempCoordVec[couplingIndex[x]])++;
					coordinationNumberByCouplingForSiteTypes.insert(pair<char,vector<unsigned int> >(siteTypeFrom,tempCoordVec));
				}
				else{
					(itFind->second[couplingIndex[x]])++;
				}
			}
		}
		fsLatticeFile.close();
	}

	//test output of latticeVectors
	cout << "latticeVectors:\n";
	for(unsigned int i=0;i<latticeVectors.size();i++){
		for(unsigned int j=0;j<latticeVectors[i].size();j++){
			cout << latticeVectors[i][j] << " ";
		}
		cout << "\n";
	}

	//test output of inverseCouplings
	cout << "inverseCouplings:\n";
	map<short,short>::const_iterator it;
	for(it=inverseCouplings.begin();it!=inverseCouplings.end();it++){
		cout << it->first << " " << it->second << "\n";
	}

	//test output of couplingIndex
	cout << "couplingIndex:\n";
	map<short,unsigned short>::const_iterator it2;
	for(it2=couplingIndex.begin();it2!=couplingIndex.end();it2++){
		cout << it2->first << " " << it2->second << "\n";
	}

	cout << "coordinationNumberByCoupling:\n";
	map<char,vector<unsigned int> >::iterator itCoordNumb;
	//cout << "coordinationNumberByCouplingForSiteTypes.size()=" << coordinationNumberByCouplingForSiteTypes.size() << "\n";
	for(itCoordNumb=coordinationNumberByCouplingForSiteTypes.begin();itCoordNumb!=coordinationNumberByCouplingForSiteTypes.end();++itCoordNumb){
		cout << char(itCoordNumb->first) << "\n";
		for(unsigned int i=0;i<(itCoordNumb->second.size());i++){
			cout  << " " << i << " " << (itCoordNumb->second[i]) << "\n";
		}
	}
}

void GraphController::createGraphs(unsigned int const &gCUTOrder, bool const &pCUTGraphs, string const &sMasterCluster, string const &sInitialSiteList){
	unsigned int x,y;
	string tempString;

	vector<pair<unsigned short,string> > initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.push_back(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();

	// load masterCluster
	Graph gMasterCluster;
	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::in);
	fsMasterCluster >> gMasterCluster;
	fsMasterCluster.close();


	// 1) start with a single site and create all Graphs by adding all possible couplings to this site
	// (the information how much couplings are allowed and which are the corresponding coord. numbers are saved in coordinatinNumberByCoupling)
	list<pair<unsigned int,Graph> > allGraphs; // the unsigned int is a unique number which identifies each graph
	vector<list<pair<unsigned int,list<vector<unsigned short> > > > > allEmbeddings(initialSites.size()); // unsigned short is the graphIndex within allGraphs
	unsigned int graphIndex=0;

	set<mpz_class> existingDiscardedGraphs; // all graphs, which are saved and all graphs which are discarded, because they are not embeddable, are saved here (to be precise only their graphNumber is saved).

	map<short,unsigned short>::const_iterator itCouplingIndex;
	for(itCouplingIndex=couplingIndex.begin();itCouplingIndex!=couplingIndex.end();++itCouplingIndex){
		if((itCouplingIndex->second)<=couplingIndex[inverseCouplings[itCouplingIndex->first]]){
			Graph g;
			g.addLink(0,1,itCouplingIndex->first);
			g.addLink(1,0,inverseCouplings[itCouplingIndex->first]);
			allGraphs.push_back(pair<unsigned int,Graph>(graphIndex,g));
			existingDiscardedGraphs.insert(g.getCanonicalGraphNumber());
			// calculate embedding onto masterCluster
			map<unsigned short,string>::iterator itInitialSite;
			for(unsigned int i=0;i<initialSites.size();i++){
				list<vector<unsigned short> > graphEmbeddings;
				gMasterCluster.embedGraphOnThisCluster(g, initialSites[i].first, graphEmbeddings);
				allEmbeddings[i].push_back(pair<unsigned short,list<vector<unsigned short> > >(graphIndex,graphEmbeddings));
			}
			graphIndex++;
		}
	}

	cout << "create Graphs\n";
	// 2) now iterate as long as necessary over all graphs and add links
	list<pair<unsigned int,Graph> >::iterator itAllGraphs=allGraphs.begin();
	unsigned int controlIndex=0;
	while(itAllGraphs!=allGraphs.end()){
		cout << controlIndex++ << "/" << allGraphs.size() << "\n";
		//cout << "Graph " << itAllGraphs->first << ":\n";
		//cout << itAllGraphs->second << "\n";
		// add all possible links to this graph
		list<Graph> newGraphs;
		cout << " createNewGraphs\n";
		itAllGraphs->second.createNewGraphs(gCUTOrder,coordinationNumberByCouplingForSiteTypes,newGraphs);
		// now iterate over all previous graphs and check:
		// a) if this new graphs are top. equivalent
		list<Graph>::iterator itNewGraphs;
		/*
		cout << "newGraphs:\n";
		for(itNewGraphs=newGraphs.begin();itNewGraphs!=newGraphs.end();++itNewGraphs){
			cout << (*itNewGraphs) << "\n";
		}
		*/
		//cout << " newGraphs.size()=" << newGraphs.size() << "\n";
		unsigned int newGraphIndex=0;
		for(itNewGraphs=newGraphs.begin();itNewGraphs!=newGraphs.end();++itNewGraphs){
			cout << "  newGraph " << newGraphIndex++ << " (" << itNewGraphs->gCUTOrder() << ")\n";
			//cout << (*itNewGraphs) << "\n";
			cout << "   check existence\n";
			/*
			bool graphExists=false;
			list<pair<unsigned int,Graph> >::iterator itAllGraphs2;

			mpz_class cgnNewGraph=itNewGraphs->getCanonicalGraphNumber();
			for(itAllGraphs2=allGraphs.begin();itAllGraphs2!=allGraphs.end();++itAllGraphs2){
				// check if the numberOfSites is the same
				//cout << "   Graph " << testOutputIndex++ << "\n";
					// check if the graphs have the same canonical graph number
				if(itAllGraphs2->second.getCanonicalGraphNumber()==cgnNewGraph){
					//cout << "    " << itAllGraphs2->second.getCanonicalGraphNumber() << "==" << itNewGraphs->getCanonicalGraphNumber() << "\n";
					graphExists=true;
					break;
				}
			}
			*/
			mpz_class cgnNewGraph=itNewGraphs->getCanonicalGraphNumber();
			pair<set<mpz_class>::iterator,bool> itSetInsert=existingDiscardedGraphs.insert(cgnNewGraph);
			//cout << "   graphExists=" << graphExists << "\n";
			if(itSetInsert.second){
				cout << "    check embedding\n";
				// if the Graph does not exist already check
				// b) if this Graph can be embedded onto the masterCluster
				map<unsigned short,string>::iterator itInitialSite;
				bool pushedBack=false;
				for(unsigned int i=0;i<initialSites.size();i++){
					list<vector<unsigned short> > graphEmbeddings;
					gMasterCluster.embedGraphOnThisCluster(*itNewGraphs, initialSites[i].first, graphEmbeddings);
					if(graphEmbeddings.size()>0){
						if(!pushedBack){
							allGraphs.push_back(pair<unsigned int,Graph>(graphIndex,*itNewGraphs));
							pushedBack=true;
						}
						allEmbeddings[i].push_back(pair<unsigned short,list<vector<unsigned short> > >(graphIndex,graphEmbeddings));
					}
				}
				if(pushedBack){
					//cout << "   pushedBack\n";
					graphIndex++;
				}
			}
		}
		++itAllGraphs;
	}

	// 3) after all graphs are created check if the graph is allowed in the maximum pCUT order
	if(pCUTGraphs){
		cout << "check for pCUT\n";
		itAllGraphs=allGraphs.begin();
		while(itAllGraphs!=allGraphs.end()){
			if(itAllGraphs->second.isPCUTGraph(gCUTOrder)){
				++itAllGraphs;
			}
			else{
				// if the graph is not allowed in the corresponding pCUT order run over allEmbeddings and delete all embeddings which reference this graph
				for(unsigned int i=0;i<initialSites.size();i++){
					list<pair<unsigned int,list<vector<unsigned short> > > >::iterator itAllEmb=allEmbeddings[i].begin();
					while(itAllEmb!=allEmbeddings[i].end()){
						if((itAllEmb->first)==(itAllGraphs->first)){
							itAllEmb=allEmbeddings[i].erase(itAllEmb);
						}
						else{
							++itAllEmb;
						}
					}
				}
				itAllGraphs=allGraphs.erase(itAllGraphs);
			}
		}
	}

	cout << "sort graphs\n";
	// 4) sort graphs by number of Sites and links (TODO: this is not necessary anymore in the subtraction step, but leave it in there for consistency; actually this should not take too much time, so who cares...)
	itAllGraphs=allGraphs.begin();
	while(itAllGraphs!=allGraphs.end()){
		unsigned int minOrder=itAllGraphs->second.gCUTOrder();
		unsigned int minSumOfLinks=itAllGraphs->second.sumOfLinks();
		list<pair<unsigned int,Graph> >::iterator itMin=itAllGraphs;

		// search for "smaller" graphs
		list<pair<unsigned int,Graph> >::iterator itListSearch=itAllGraphs;
		itListSearch++;
		while(itListSearch!=allGraphs.end()){
			unsigned int tempOrder=itListSearch->second.gCUTOrder();
			if(tempOrder<minOrder){
				minSumOfLinks=itListSearch->second.sumOfLinks();
				minOrder=tempOrder;
				itMin=itListSearch;
			}
			else{
				if(tempOrder==minOrder){
					unsigned int tempSumOfLinks=itListSearch->second.sumOfLinks();
					if(tempSumOfLinks<minSumOfLinks){
						minOrder=tempOrder;
						minSumOfLinks=tempSumOfLinks;
						itMin=itListSearch;
					}
				}
			}
			itListSearch++;
		}

		// now swap both graphs within allGraphs
		if(itAllGraphs!=itMin){
			pair<unsigned int,Graph> tempGraph=(*itAllGraphs);
			(*itAllGraphs)=(*itMin);
			(*itMin)=tempGraph;
		}
		++itAllGraphs;
	}

	cout << "save graphs\n";
	// 5) save all Graphs as BondFiles and embeddings
	stringstream ssGraphList;
	ssGraphList << "graphList_o" << gCUTOrder << ".cfg";
	fstream fsGraphList;
	fsGraphList.open(ssGraphList.str().c_str(), fstream::out);
	fsGraphList.close();

	for(unsigned int i=0;i<initialSites.size();i++){
		fstream fsEmbeddings;
		fsEmbeddings.open(initialSites[i].second.c_str(),fstream::out);
		fsEmbeddings << "#order graphNumber sitesOnMasterCluster\n";
		fsEmbeddings.close();
	}

	vector<unsigned int> graphNumbersByOrder(gCUTOrder+1,0);
	list<tuple<unsigned int,unsigned int,unsigned int> > graphNumbersByIndex; //graphIndex, order, graphNumber

	for(itAllGraphs=allGraphs.begin();itAllGraphs!=allGraphs.end();++itAllGraphs){
		unsigned int actualOrder=itAllGraphs->second.gCUTOrder();
		graphNumbersByOrder[actualOrder]++;

		graphNumbersByIndex.push_back(tuple<unsigned int,unsigned int,unsigned int>(itAllGraphs->first,actualOrder,graphNumbersByOrder[actualOrder]));
		stringstream ssBondFile;
		ssBondFile << "BondFile_o" << actualOrder << "g" << graphNumbersByOrder[actualOrder] << ".cfg";
		fstream fsBondFile;
		fsBondFile.open(ssBondFile.str().c_str(),fstream::out);
		fsBondFile << "# This is a bond file.\n";
		fsBondFile << "# The number of lattice sites.\n";
		fsBondFile << actualOrder << "\n\n";
		fsBondFile << "# The number of bits that should be reserved within a\n";
		fsBondFile << "# BaseState object for storing extra information.\n";
		fsBondFile << "0\n\n";
		fsBondFile << "# Now the list of bonds.\n";
		fsBondFile << "# Format:\n";
		fsBondFile << "# PerturbationOperatorNumber LaticeSiteNumbers;\n";
		fsBondFile << (itAllGraphs->second);
		fsBondFile.close();

		stringstream ssGraphFile;
		ssGraphFile << "GraphFile_o" << actualOrder << "g" << graphNumbersByOrder[actualOrder] << ".cfg";
		fstream fsGraphFile;
		fsGraphFile.open(ssGraphFile.str().c_str(),fstream::out);
		fsGraphFile << (itAllGraphs->second);
		fsGraphFile.close();

		fsGraphList.open(ssGraphList.str().c_str(), fstream::out | fstream::app);
		fsGraphList << actualOrder  << " " << graphNumbersByOrder[actualOrder] << " " << ssGraphFile.str() << "\n";
		fsGraphList.close();

	//	cout << "itAllGraphs->first=" << itAllGraphs->first << "\n";

		for(unsigned int i=0;i<initialSites.size();i++){
			//TODO: this could be optimized by deleting the saved contribution(?)
			// find index of this graph and save embeddings
			list<pair<unsigned int,list<vector<unsigned short> > > >::iterator itAllEmb;
			for(itAllEmb=allEmbeddings[i].begin();itAllEmb!=allEmbeddings[i].end();++itAllEmb){
				if(itAllEmb->first==itAllGraphs->first){
					fstream fsEmbeddings;
					fsEmbeddings.open(initialSites[i].second.c_str(),fstream::out | fstream::app);
					list<vector<unsigned short> >::iterator itListEmb;
					for(itListEmb=itAllEmb->second.begin();itListEmb!=itAllEmb->second.end();++itListEmb){
						fsEmbeddings << actualOrder << " " << graphNumbersByOrder[actualOrder];
						for(unsigned int k=0;k<itListEmb->size();k++){
							fsEmbeddings << " " << (*itListEmb)[k];
						}
						fsEmbeddings << "\n";
					}
					fsEmbeddings.close();
				}
			}
		}
	}
}

void GraphController::calculateGraphSubtractions(unsigned int const &gCUTOrder, string const &sGraphList){
	vector<Graph> allGraphs;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	// load graphs
	vector<string> sGraphFiles;
	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(!fsGraphList.eof()){
		unsigned int order=0;
		unsigned int graphNumber=0;
		string tempstring;
		fsGraphList>>order;
		fsGraphList>>graphNumber;
		fsGraphList>>tempstring;
		if(tempstring!=""){
			sGraphFiles.push_back(tempstring);
			graphNumbers.push_back(pair<unsigned int,unsigned int>(order,graphNumber));
		}
	}
	fsGraphList.close();

	/*
	cout << "graphNumbers test:\n";
	cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
	for(unsigned int i=0;i<graphNumbers.size();i++){
		cout << graphNumbers[i].first << " " << graphNumbers[i].second << "\n";
	}
	*/


	// load graphList and push graphs from graphList to allGraphs
	for(unsigned int i=0;i<sGraphFiles.size();i++){
		//cout << sGraphFiles[i] << "\n";
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphFiles[i].c_str(),fstream::in);
		Graph g;
		//cout << "going to load graph...\n";
		fsGraphFiles >> g;
		//cout << "loaded graph...\n";
		//cout << "test output:\n";
		//cout << g << "\n";
		allGraphs.push_back(g);
		fsGraphFiles.close();
	}
	// sort them by order and sum of links
	// 25.07.2014: graphNumbers must also switches in the same way! Now it is added
	bool switchedsomething=false;
	for(unsigned int j=0;j<allGraphs.size();j++){
		unsigned int minimalOrder=allGraphs[j].gCUTOrder();
		unsigned int minimalSum=allGraphs[j].sumOfLinks();
		unsigned int minIndex=j;
		for(unsigned int i=j+1;i<allGraphs.size();i++){
			unsigned int order=allGraphs[i].gCUTOrder();
			if(order<minimalOrder){
				minIndex=i;
				minimalOrder=order;
				minimalSum=allGraphs[i].sumOfLinks();
			}
			else{
				if(minimalOrder==order){
					unsigned int sum=allGraphs[i].sumOfLinks();
					if(sum<minimalSum){
						minIndex=i;
						minimalSum=sum;
					}
				}
			}
		}
		if(minIndex!=j){
			Graph tempGraph=allGraphs[j];
			allGraphs[j]=allGraphs[minIndex];
			allGraphs[minIndex]=tempGraph;

			pair<unsigned int,unsigned int> tempGraphNumbers=graphNumbers[j];
			graphNumbers[j]=graphNumbers[minIndex];
			graphNumbers[minIndex]=tempGraphNumbers;

			switchedsomething=true;
		}
	}

	/*
	cout << "graphNumbers test:\n";
	cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
	for(unsigned int i=0;i<graphNumbers.size();i++){
		cout << graphNumbers[i].first << " " << graphNumbers[i].second << "\n";
	}
	*/

	vector<vector<int> > embeddingFactors(allGraphs.size(),vector<int>(allGraphs.size(),0));
	// Step 1: remove links to create subgraphs
	// Step 2: check if created graphs are defined in the graphList
	// Step 2a: if it is defined: add this as a subgraph
	// Step 2b: if it is not defined: remove further links until 2a (or graph is empty, this is just an artificial exit condition)
	// Step 3: proceed with next graph
	for(unsigned int i=0;i<allGraphs.size();i++){
		cout << "allGraphs[" << i << "]:\n";
		cout << allGraphs[i] << "\n";
		vector<Graph> subgraphs=allGraphs[i].removeLinks();
		cout << "subgraphs.size()=" << subgraphs.size() << "\n";
		for(unsigned int j=0;j<subgraphs.size();j++){
			cout << "subgraphs[" << j << "]:\n";
			cout << "gCUTOrder="<< subgraphs[j].gCUTOrder() << "\n";
			cout << subgraphs[j] << "\n";

			for(unsigned int k=0;k<allGraphs.size();k++){
				cout << "k=" << k << "\n";
				// erst ueberpruefen ob Anzahl Sites und Anzahl bondTypes gleich ist, sonst koennen die Graphen auch nicht gleich sein
				if(subgraphs[j].gCUTOrder()==allGraphs[k].gCUTOrder()){
					if(subgraphs[j].sumOfBondTypes()==allGraphs[k].sumOfBondTypes()){
						if(subgraphs[j].getCanonicalGraphNumber()==allGraphs[k].getCanonicalGraphNumber()){
							embeddingFactors[i][k]--;

					/*
							for(unsigned int kk=0;kk<embeddingFactors[k].size();kk++){
								embeddingFactors[i][kk]-=embeddingFactors[k][kk];
							}
					 */
							break;
						}
					}
				}
			}


			// remove additional links
			cout << "remove additional links\n";
			vector<Graph> subgraphs2=subgraphs[j].removeLinks();
			cout << "subgraphs2.size()=" << subgraphs2.size() << "\n";
			for(unsigned int k=0;k<subgraphs2.size();k++){
				cout << "subgraphs2[" << k << "]:\n";
				cout << subgraphs2[k] << "\n";

				// push them back to subgraphs if this graph does not exist already in subgraphs!
				bool foundGraph=false;
				for(unsigned int kk=0;kk<subgraphs.size();kk++){
					if(subgraphs[kk]==subgraphs2[k]){
						foundGraph=true;
						break;
					}
				}

				if(!foundGraph){
					subgraphs.push_back(subgraphs2[k]);
				}
			}
		}
	}

	cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
	cout << "allGraphs.size()=" << allGraphs.size() << "\n";
	cout << "embeddingFactors.size()=" << embeddingFactors.size() << "\n";

	stringstream ssCouplingEmbeddingFactors;
	ssCouplingEmbeddingFactors << "subgraphEmbeddings_KaneMele_o" << gCUTOrder << ".cfg";
	fstream fsCouplingEmbeddingFactors;
	fsCouplingEmbeddingFactors.open(ssCouplingEmbeddingFactors.str().c_str(),fstream::out);
	fsCouplingEmbeddingFactors << "#Order(g1) GraphNumber(g1) Sites(g1) Order(g2) GraphNumber(g2) Sites(g2) EmbeddingFactor\n";
	for(unsigned int i=0;i<embeddingFactors.size();i++){
		for(unsigned int j=0;j<embeddingFactors[i].size();j++){
			if(embeddingFactors[i][j]!=0){
				// this is the groundState Subtraction!
				// the corresponding site is just a single site! this is the firstSite of the graph (should always be zero)
				cout << "i=" << i << "j=" << j << "\n";
				cout << graphNumbers[i].first << " " << graphNumbers[i].second << " " << allGraphs[i].getFirstSite() << " " << graphNumbers[j].first << " " << graphNumbers[j].second << " " << allGraphs[j].getFirstSite() << " " << embeddingFactors[i][j] << "\n";
				fsCouplingEmbeddingFactors << graphNumbers[i].first << " " << graphNumbers[i].second << " " << allGraphs[i].getFirstSite() << " " << graphNumbers[j].first << " " << graphNumbers[j].second << " " << allGraphs[j].getFirstSite() << " " << embeddingFactors[i][j] << "\n";
			}
		}
	}
	fsCouplingEmbeddingFactors.close();

	// now calculate all combinations of 2*n sites for every graph
	for(unsigned int j=0;j<allGraphs.size();j++){
		cout << "j=" << j << "\n";
		for(unsigned int s=2;s<=allGraphs[j].gCUTOrder();s+=2){
			cout << " s=" << s << "\n";
			//vector<int> embeddingFactorsCouplings(allGraphs.size(),0);
			//vector<vector<unsigned short> > correspondingSites(allGraphs.size());

			vector<vector<unsigned short> > allCombinations=allGraphs[j].getAllNSiteCombinations(s);
			cout << "combinationSize=" << allCombinations.size() << "\n";
			for(unsigned int c=0;c<allCombinations.size();c++){
				map<vector<unsigned short>,int> couplingEmbeddings;  //IMPORTANT: The First Element in the vector is the graphNumber!

				cout << "  c=" << c << "\n";
				//vector<Graph> subgraphs=allGraphs[j].removeLinks();
				vector<Graph> subgraphs=allGraphs[j].removeLinks(allCombinations[c]);
				cout << "  subgraphs.size()=" << subgraphs.size() << "\n";
				for(unsigned int jSub=0;jSub<subgraphs.size();jSub++){
					cout << "   jSub=" << jSub << "\n";
					cout << "subgraph:\n";
					cout << subgraphs[jSub] << "\n";
					for(unsigned int k=0;k<allGraphs.size();k++){
						cout << "    k=" << k << "\n";
						// check if graph could be embedded with respect to the specified sites!
						if(subgraphs[jSub].getCanonicalGraphNumber()==allGraphs[k].getCanonicalGraphNumber()){
							// if both graphs are topologically equivalent: identify the sites!
							cout << "    found Graph!\n";

							vector<unsigned short> identifiedSites=allGraphs[k].identifySites(subgraphs[jSub],allCombinations[c]);
							vector<unsigned short> tempVec;
							tempVec.push_back(k);
							cout << "identifiedSites=";
							for(unsigned int jc=0;jc<identifiedSites.size();jc++){
								tempVec.push_back(identifiedSites[jc]);
								cout << identifiedSites[jc] << " ";
							}
							cout << "\n";
							cout << "test1\n";
							pair<map<vector<unsigned short>,int>::iterator,bool> ret;
							ret=couplingEmbeddings.insert(pair<vector<unsigned short>,int>(tempVec,-1));
							if(!ret.second){
								ret.first->second--;
							}
							cout << "test2\n";

							break;
						}
					}

					// remove additional links
					vector<Graph> subgraphs2=subgraphs[jSub].removeLinks(allCombinations[c]);
					for(unsigned int k=0;k<subgraphs2.size();k++){
						// push them back to subgraphs if this graph does not exist already in subgraphs!
						bool foundGraph=false;
						for(unsigned int kk=0;kk<subgraphs.size();kk++){
							if(subgraphs[kk]==subgraphs2[k]){
								foundGraph=true;
								break;
							}
						}
						if(!foundGraph){
							subgraphs.push_back(subgraphs2[k]);
						}
					}
					cout << "test3\n";
				}
				fsCouplingEmbeddingFactors.open(ssCouplingEmbeddingFactors.str().c_str(),fstream::out | fstream::app);
				map<vector<unsigned short>,int>::iterator it;
				for(it=couplingEmbeddings.begin();it!=couplingEmbeddings.end();it++){
					fsCouplingEmbeddingFactors << graphNumbers[j].first << " " << graphNumbers[j].second; // order and number of the graph
					for(unsigned int k=0;k<allCombinations[c].size();k++){
						fsCouplingEmbeddingFactors << " " << allCombinations[c][k]; // lattice sites of the coupling on the graph
					}
					fsCouplingEmbeddingFactors  << " " << graphNumbers[it->first[0]].first << " " << graphNumbers[it->first[0]].second; // order and number of the subgraph
					for(unsigned int k=1;k<it->first.size();k++){
						fsCouplingEmbeddingFactors << " " << it->first[k]; // lattice sites of the coupling on the subgraph
					}
					fsCouplingEmbeddingFactors << " " << it->second << "\n"; // embedding factor
				}
				fsCouplingEmbeddingFactors.close();
			}
		}
	}
}

void GraphController::calculateGraphEmbeddings(unsigned int const &gCUTOrder, string const &sGraphList, string const &sMasterCluster, string const &sInitialSiteList){
	// read list-files
	vector<string> sGraphs;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	unsigned int x,y;
	string tempString;

	map<unsigned short,string> initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.insert(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();

	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(fsGraphList>>x){
		fsGraphList>>y;
		fsGraphList>>tempString;
		sGraphs.push_back(tempString);
		graphNumbers.push_back(pair<unsigned int,unsigned int>(x,y));
	}
	fsGraphList.close();

	// load Graphs
	vector<Graph> allGraphs;
	for(unsigned int i=0;i<sGraphs.size();i++){
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphs[i].c_str(),fstream::in);
		Graph g;
		fsGraphFiles >> g;
		allGraphs.push_back(g);
		fsGraphFiles.close();
	}

	// load masterCluster
	Graph gMasterCluster;
	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::in);
	fsMasterCluster >> gMasterCluster;
	fsMasterCluster.close();


	// calculate embeddings on masterCluster for all specified initialSites
	map<unsigned short,string>::const_iterator itInitialSites;
	for(itInitialSites=initialSites.begin();itInitialSites!=initialSites.end();itInitialSites++){
		vector<pair<unsigned int,vector<unsigned short> > > allEmbeddings=gMasterCluster.calculateEmbeddingsOnThisCluster(allGraphs,itInitialSites->first);
		// sort the embeddings
		for(unsigned int i=0;i<allEmbeddings.size();i++){
			unsigned int minGraphIndex=allEmbeddings[i].first;
			unsigned int minIndex=i;
			for(unsigned int j=i+1;j<allEmbeddings.size();j++){
				if(allEmbeddings[j].first<minGraphIndex){
					minGraphIndex=allEmbeddings[j].first;
					minIndex=j;
				}
			}
			if(minIndex!=i){
				pair<unsigned int,vector<unsigned short> > tempPair=allEmbeddings[i];
				allEmbeddings[i]=allEmbeddings[minIndex];
				allEmbeddings[minIndex]=tempPair;
			}
		}

		// save the embeddings
		fstream fsEmbeddings;
		fsEmbeddings.open(itInitialSites->second.c_str(),fstream::out);
		fsEmbeddings << "#order graphNumber sitesOnMasterCluster\n";
		for(unsigned int i=0;i<allEmbeddings.size();i++){
			unsigned int graphIndex=allEmbeddings[i].first;
			fsEmbeddings << graphNumbers[graphIndex].first << " " << graphNumbers[graphIndex].second;
			for(unsigned int k=0;k<allEmbeddings[i].second.size();k++){
				fsEmbeddings << " " << allEmbeddings[i].second[k];
			}
			fsEmbeddings << "\n";
		}
		fsEmbeddings.close();
	}
}

void GraphController::createMasterCluster(unsigned int const &xMax, unsigned int const &yMax, unsigned int const &n, string const &sMasterCluster, string const &sGnuplot, string const &sPlotPoints){
	LatticeGraph lg;
	lg.createRectangularLatticeCluster(latticeVectors,xMax,yMax,n);
	Graph g=lg.createGraph();

	stringstream ssMasterClusterLg;
	ssMasterClusterLg << sMasterCluster << "lg";
	fstream fsMasterClusterLg;
	fsMasterClusterLg.open(ssMasterClusterLg.str().c_str(),fstream::out);
	fsMasterClusterLg << lg;
	fsMasterClusterLg.close();

	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::out);
	fsMasterCluster << g;
	fsMasterCluster.close();

	lg.createGraphPlot(sGnuplot,sPlotPoints);
}

void GraphController::createEDCluster(unsigned int const &gCUTOrder, string const &sInterUnitCellVectors, string const &sIntraUnitCellVectors, unsigned int const &v1Max, unsigned int const &v2Max, string const &sSpinModelTDLFull){
	// load latticeVectors
	vector<vector<int> > interUnitCellVectors;
	map<int,vector<int> > intraUnitCellVectors;

	string tempstring;
	int x;
	char tempChar;
	vector<int> tempVec;

	fstream fsInterUnitCellVectors;
	fsInterUnitCellVectors.open(sInterUnitCellVectors.c_str(),fstream::in);
	getline(fsInterUnitCellVectors,tempstring);
	while(fsInterUnitCellVectors>>x){
		tempVec.push_back(x);
		if(fsInterUnitCellVectors.peek()=='\n'){
			interUnitCellVectors.push_back(tempVec);
			tempVec.clear();
		}
	}
	fsInterUnitCellVectors.close();
	if(interUnitCellVectors.size()!=2){
		cout << "number of interUnitCellVectors must be 2!\n";
		return;
	}

	cout << "test output of interUnitCellVectors:\n";
	for(unsigned int i=0;i<interUnitCellVectors.size();i++){
		for(unsigned int j=0;j<interUnitCellVectors[i].size();j++){
			cout << interUnitCellVectors[i][j] << " ";
		}
		cout << "\n";
	}

	fstream fsIntraUnitCellVectors;
	fsIntraUnitCellVectors.open(sIntraUnitCellVectors.c_str(),fstream::in);
	getline(fsIntraUnitCellVectors,tempstring);
	while(fsIntraUnitCellVectors>>tempChar){
		if(tempChar>='A' && tempChar<='Z'){
			vector<int> tempVec(2);
			if(tempChar!='A'){
				cout << "first site in intraUnitCellVectors must be A!\n";
				return;
			}
			fsIntraUnitCellVectors >> x;
			tempVec[0]=x;
			fsIntraUnitCellVectors >> x;
			tempVec[1]=x;
			fsIntraUnitCellVectors >> tempChar;
			intraUnitCellVectors.insert(pair<int,vector<int> >(tempChar-'A',tempVec));
		}
	}
	fsIntraUnitCellVectors.close();

	cout << "test output of intraUnitCellVectors:\n";
	for(map<int,vector<int> >::iterator it1=intraUnitCellVectors.begin();it1!=intraUnitCellVectors.end();it1++){
		for(unsigned int i=0;i<it1->second.size();i++){
			cout << it1->second[i] << " ";
		}
		cout << it1->first << "\n";
	}


	// load spinModelsTDL
	// spinModelList should be shorted against the different siteTypes
	vector<list<SpinModelTDLClass> > spinModelList(intraUnitCellVectors.size());
	fstream fsSpinModelTDLFull;
	fsSpinModelTDLFull.open(sSpinModelTDLFull.c_str(),fstream::in);
	getline(fsSpinModelTDLFull,tempstring);
	SpinModelTDLClass tempSpinModelTDL;
	while(fsSpinModelTDLFull>>tempSpinModelTDL){
		cout << "test output of tempSpinModelTDL :\n";
		cout << tempSpinModelTDL;
		spinModelList[tempSpinModelTDL.getOriginType()].push_back(tempSpinModelTDL);
	}
	fsSpinModelTDLFull.close();

	// create BondFile by translating the latticeVectors
	vector<int> actualSite(2,0);
	unsigned short actualSiteIndex=0;
	map<vector<int>, unsigned short> siteMap;
	// initialize siteMap
	map<int,vector<int> >::iterator itIntra;
	for(itIntra=intraUnitCellVectors.begin();itIntra!=intraUnitCellVectors.end();itIntra++){
		for(unsigned int v1=0;v1<=v1Max;v1++){
			for(unsigned int v2=0;v2<=v2Max;v2++){
				actualSite[0]=v1*interUnitCellVectors[0][0]+v2*interUnitCellVectors[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectors[0][1]+v2*interUnitCellVectors[1][1]+itIntra->second[1];
				siteMap.insert(pair<vector<int>,unsigned short>(actualSite,actualSiteIndex++));
			}
		}
	}

	//SpinModelClass spinModelED;
	vector<SpinModelClass> referenceSpinModels; // the reference spinModel is the spinModel without assigning newSites, so that the sites are 0,1,2,3...
	map<vector<unsigned short>,unsigned int> siteCombinationsToReferenceSpinModel;
	list<pair<vector<unsigned short>,unsigned int> > siteCombinationsToReferenceSpinModel_list;

	//map<int,vector<int> >::iterator itIntra;
	for(itIntra=intraUnitCellVectors.begin();itIntra!=intraUnitCellVectors.end();itIntra++){
		for(unsigned int v1=0;v1<=v1Max;v1++){
			for(unsigned int v2=0;v2<=v2Max;v2++){
				// run over interUnitCells
				cout << "v1=" << v1 << " v2=" << v2 << "\n";
				cout << " itIntra=" << itIntra->second[0] << " " << itIntra->second[1] << " " << char(itIntra->first+'A') << "\n";
				list<pair<vector<unsigned short>,unsigned int> > siteCombinationsToReferenceSpinModel_temp;

				actualSite[0]=v1*interUnitCellVectors[0][0]+v2*interUnitCellVectors[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectors[0][1]+v2*interUnitCellVectors[1][1]+itIntra->second[1];

				// run over all operators of the spinModels which have the origin itIntra->first.second
				list<SpinModelTDLClass>::iterator itList;
				for(itList=spinModelList[itIntra->first].begin();itList!=spinModelList[itIntra->first].end();itList++){
					SpinModelClass tempSpinModel=*itList;
					vector<vector<int> > siteVectors=itList->getSiteVectors();
					vector<unsigned short> newSites(siteVectors.size());
					for(unsigned int i=0;i<siteVectors.size();i++){
						cout << "  i=" << i << "\n";
						vector<int> tempVec(2,0);
						tempVec[0]=actualSite[0]+siteVectors[i][0];
						tempVec[1]=actualSite[1]+siteVectors[i][1];
						if((siteVectors[i][0]!=0) || (siteVectors[i][1]!=0)){
							map<int,vector<int> >::iterator itIntraMapFind;
							if(siteVectors[i][2]!=0){
								// if the siteType of the newVector is not the siteType A, then the composition into the interUnitCellVectors does not work.
								// therefore subtract the corresponding intraUnitCellVector
								itIntraMapFind=intraUnitCellVectors.find(siteVectors[i][2]);
								cout << "   itIntraMap=" << itIntraMapFind->second[0] << " " << itIntraMapFind->second[1] << "\n";
								tempVec[0]-=itIntraMapFind->second[0];
								tempVec[1]-=itIntraMapFind->second[1];
							}
							cout << "   tempVec(different siteTyp)=" << tempVec[0] << " " << tempVec[1] << "\n";
							// calculate vector-representation in interUnitCellBasis to find the corresponding unitCell
							int a1=interUnitCellVectors[0][0];
							int a2=interUnitCellVectors[0][1];
							int b1=interUnitCellVectors[1][0];
							int b2=interUnitCellVectors[1][1];
							int det=a1*b2-b1*a2;

							int xa=(b2*tempVec[0]-b1*tempVec[1])/det;
							int xb=(a1*tempVec[1]-a2*tempVec[0])/det;

							// check if xa or xb are out of bounds and shift them due to periodic boundary conditions
							while(xa<0){
								xa+=v1Max+1;
							}
							while(xa>v1Max){
								xa-=v1Max+1;
							}
							while(xb<0){
								xb+=v2Max+1;
							}
							while(xb>v2Max){
								xb-=v2Max+1;
							}

							// write the vector in the normal euclidean basis
							tempVec[0]=xa*interUnitCellVectors[0][0]+xb*interUnitCellVectors[1][0];
							tempVec[1]=xa*interUnitCellVectors[0][1]+xb*interUnitCellVectors[1][1];
							// add the intraUnitCellVector if it was previously subtracted
							if(siteVectors[i][2]!=0){
								tempVec[0]+=itIntraMapFind->second[0];
								tempVec[1]+=itIntraMapFind->second[1];
							}
						}
						cout << "  actualSite=" << actualSite[0] << " " << actualSite[1] << "\n";
						cout << "  siteVector=" << siteVectors[i][0] << " " << siteVectors[i][1] << " " << char(siteVectors[i][2]+'A') << "\n";
						cout << "  tempVec=" << tempVec[0] << " " << tempVec[1] << "\n";


						// now find the corresponding siteNumber to this vector from the siteMap
						map<vector<int>, unsigned short>::iterator itMapFind=siteMap.find(tempVec);
						if(itMapFind==siteMap.end()){
							cout << "error: couldn't find site in siteMap!\n";
							return;
						}
						cout << "  actualSiteIndex=" << itMapFind->second << "\n";
						newSites[i]=itMapFind->second;
					}

					// assign the new sites to the spinModel
					tempSpinModel.assignNewSites(newSites);

					// order against sites
					tempSpinModel.orderAgainstSites();

					// push sites to siteCombinations if this tempSpinModel not already exists
					// 29.05.2015: Here comes the subtle, tricky part: Due to periodic boundary conditions, the same sites could occur multiple times, on different "ways" on the lattice
					// All of this combinations must be added, but only once for each way. Since the ways are unique for each initialSite/siteType just check if the spin model is already contained within the siteCombinationsToReferenceSpinModel_list
					// if not, add it to siteCombinationsToReferenceSpinModel_temp
					// if yes, then do not add it.
					bool existsAlready=false;
					list<pair<vector<unsigned short>,unsigned int> >::iterator itSiteComb;
					for(itSiteComb=siteCombinationsToReferenceSpinModel_list.begin();itSiteComb!=siteCombinationsToReferenceSpinModel_list.end();itSiteComb++){
						SpinModelClass tempSpinModel2=referenceSpinModels[itSiteComb->second];
						tempSpinModel2.assignNewSites(itSiteComb->first);
						tempSpinModel2.orderAgainstSites();
						if(tempSpinModel==tempSpinModel2){
							existsAlready=true;
							break;
						}
					}
					if(!existsAlready){
						SpinModelClass tempSpinModel3=*itList;
						unsigned int referenceIndex=referenceSpinModels.size();
						// if it not already exists, find out the referenceSpinModel
						for(unsigned int iRef=0;iRef<referenceSpinModels.size();iRef++){
							if(tempSpinModel3==referenceSpinModels[iRef]){
								referenceIndex=iRef;
								break;
							}
						}
						if(referenceIndex==referenceSpinModels.size()){
							referenceSpinModels.push_back(tempSpinModel3);
						}
						siteCombinationsToReferenceSpinModel_temp.push_back(pair<vector<unsigned short>,unsigned int>(newSites,referenceIndex));
					}
					//spinModelED+=tempSpinModel/tempSpinModel.numberOfOperators();
				}
				// now add the temp siteCombinations to the list
				list<pair<vector<unsigned short>,unsigned int> >::iterator itListAdd;
				for(itListAdd=siteCombinationsToReferenceSpinModel_temp.begin();itListAdd!=siteCombinationsToReferenceSpinModel_temp.end();itListAdd++){
					siteCombinationsToReferenceSpinModel_list.push_back(*itListAdd);
				}
			}
		}
	}


	// save siteMap
	stringstream ssSiteMap;
	ssSiteMap << "siteMapEDCluster_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSiteMap;
	fsSiteMap.open(ssSiteMap.str().c_str(),fstream::out);
	for(map<vector<int>,unsigned short>::iterator it1=siteMap.begin();it1!=siteMap.end();it1++){
		for(unsigned int i=0;i<it1->first.size();i++){
			fsSiteMap << it1->first[i] << " ";
		}
		fsSiteMap << "-> " << it1->second << "\n";
	}
	fsSiteMap.close();

	// save siteCombinations and reference SpinModels
	stringstream ssReferenceSpinModel;
	ssReferenceSpinModel << "effSpinModelReferenceTDL_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsReferenceSpinModel;
	fsReferenceSpinModel.open(ssReferenceSpinModel.str().c_str(),fstream::out);
	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		fsReferenceSpinModel << i << "\n";
		fsReferenceSpinModel << referenceSpinModels[i] << "\n";
	}
	fsReferenceSpinModel.close();

	stringstream ssSiteCombinations;
	ssSiteCombinations << "siteCombinationsForReferenceSpinModel_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSiteCombinations;
	fsSiteCombinations.open(ssSiteCombinations.str().c_str(),fstream::out);
	fsSiteCombinations << "#referenceSpinModel site1 site2 ...\n";
	for(unsigned int j=0;j<referenceSpinModels.size();j++){
		list<pair<vector<unsigned short>,unsigned int> >::iterator itSiteComb;
		for(itSiteComb=siteCombinationsToReferenceSpinModel_list.begin();itSiteComb!=siteCombinationsToReferenceSpinModel_list.end();itSiteComb++){
			if(itSiteComb->second==j){
				fsSiteCombinations << itSiteComb->second;
				for(unsigned int i=0;i<itSiteComb->first.size();i++){
					fsSiteCombinations << " " << itSiteComb->first[i];
				}
				fsSiteCombinations << "\n";
			}
		}
	}
	fsSiteCombinations.close();

	/*
	// save spinModel
	stringstream ssSpinModelED;
	ssSpinModelED << "effSpinModelED_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSpinModelED;
	fsSpinModelED.open(ssSpinModelED.str().c_str(),fstream::out);
	fsSpinModelED << spinModelED << "\n";
	fsSpinModelED.close();
	*/
}

void GraphController::createEDClusterInDifferentUCBasis(unsigned int const &gCUTOrder, string const &sInterUnitCellVectorsOld, string const &sIntraUnitCellVectorsOld, string const &sInterUnitCellVectors, string const &sIntraUnitCellVectors, unsigned int const &v1Max, unsigned int const &v2Max, string const &sSpinModelTDLFull, string const &sReplaceCouplings){
	// load latticeVectors
	vector<vector<int> > interUnitCellVectorsNew;
	map<int,vector<int> > intraUnitCellVectorsNew;
	vector<vector<int> > interUnitCellVectorsOld;
	map<int,vector<int> > intraUnitCellVectorsOld;

	string tempstring;
	int x;
	Doub tempDoub;
	char tempChar;
	vector<int> tempVec;

	// load new latticeVectors
	fstream fsInterUnitCellVectorsNew;
	fsInterUnitCellVectorsNew.open(sInterUnitCellVectors.c_str(),fstream::in);
	getline(fsInterUnitCellVectorsNew,tempstring);
	while(fsInterUnitCellVectorsNew>>x){
		tempVec.push_back(x);
		if(fsInterUnitCellVectorsNew.peek()=='\n'){
			interUnitCellVectorsNew.push_back(tempVec);
			tempVec.clear();
		}
	}
	fsInterUnitCellVectorsNew.close();
	if(interUnitCellVectorsNew.size()!=2){
		cout << "number of interUnitCellVectors must be 2!\n";
		return;
	}

	cout << "test output of interUnitCellVectorsNew:\n";
	for(unsigned int i=0;i<interUnitCellVectorsNew.size();i++){
		for(unsigned int j=0;j<interUnitCellVectorsNew[i].size();j++){
			cout << interUnitCellVectorsNew[i][j] << " ";
		}
		cout << "\n";
	}

	fstream fsIntraUnitCellVectorsNew;
	fsIntraUnitCellVectorsNew.open(sIntraUnitCellVectors.c_str(),fstream::in);
	getline(fsIntraUnitCellVectorsNew,tempstring);
	while(fsIntraUnitCellVectorsNew>>tempChar){
		if(tempChar>='A' && tempChar<='Z'){
			vector<int> tempVec(2);
			if(tempChar!='A'){
				cout << "first site in intraUnitCellVectors must be A!\n";
				return;
			}
			fsIntraUnitCellVectorsNew >> x;
			tempVec[0]=x;
			fsIntraUnitCellVectorsNew >> x;
			tempVec[1]=x;
			fsIntraUnitCellVectorsNew >> tempChar;
			intraUnitCellVectorsNew.insert(pair<int,vector<int> >(tempChar-'A',tempVec));
		}
	}
	fsIntraUnitCellVectorsNew.close();

	cout << "test output of intraUnitCellVectorsNew:\n";
	for(map<int,vector<int> >::iterator it1=intraUnitCellVectorsNew.begin();it1!=intraUnitCellVectorsNew.end();it1++){
		for(unsigned int i=0;i<it1->second.size();i++){
			cout << it1->second[i] << " ";
		}
		cout << it1->first << "\n";
	}

	// load old latticeVectors
	fstream fsInterUnitCellVectorsOld;
	fsInterUnitCellVectorsOld.open(sInterUnitCellVectorsOld.c_str(),fstream::in);
	getline(fsInterUnitCellVectorsOld,tempstring);
	while(fsInterUnitCellVectorsOld>>x){
		tempVec.push_back(x);
		if(fsInterUnitCellVectorsOld.peek()=='\n'){
			interUnitCellVectorsOld.push_back(tempVec);
			tempVec.clear();
		}
	}
	fsInterUnitCellVectorsOld.close();
	if(interUnitCellVectorsOld.size()!=2){
		cout << "number of interUnitCellVectors must be 2!\n";
		return;
	}

	cout << "test output of interUnitCellVectorsOld:\n";
	for(unsigned int i=0;i<interUnitCellVectorsOld.size();i++){
		for(unsigned int j=0;j<interUnitCellVectorsOld[i].size();j++){
			cout << interUnitCellVectorsOld[i][j] << " ";
		}
		cout << "\n";
	}

	fstream fsIntraUnitCellVectorsOld;
	fsIntraUnitCellVectorsOld.open(sIntraUnitCellVectorsOld.c_str(),fstream::in);
	getline(fsIntraUnitCellVectorsOld,tempstring);
	while(fsIntraUnitCellVectorsOld>>tempChar){
		if(tempChar>='A' && tempChar<='Z'){
			vector<int> tempVec(2);
			if(tempChar!='A'){
				cout << "first site in intraUnitCellVectors must be A!\n";
				return;
			}
			fsIntraUnitCellVectorsOld >> x;
			tempVec[0]=x;
			fsIntraUnitCellVectorsOld >> x;
			tempVec[1]=x;
			fsIntraUnitCellVectorsOld >> tempChar;
			intraUnitCellVectorsOld.insert(pair<int,vector<int> >(tempChar-'A',tempVec));
		}
	}
	fsIntraUnitCellVectorsOld.close();

	cout << "test output of intraUnitCellVectorsOld:\n";
	for(map<int,vector<int> >::iterator it1=intraUnitCellVectorsOld.begin();it1!=intraUnitCellVectorsOld.end();it1++){
		for(unsigned int i=0;i<it1->second.size();i++){
			cout << it1->second[i] << " ";
		}
		cout << it1->first << "\n";
	}


	// load spinModelsTDL
	// spinModelList should be shorted against the different siteTypes
	vector<list<SpinModelTDLClass> > spinModelList(intraUnitCellVectorsOld.size());
	fstream fsSpinModelTDLFull;
	fsSpinModelTDLFull.open(sSpinModelTDLFull.c_str(),fstream::in);
	getline(fsSpinModelTDLFull,tempstring);
	SpinModelTDLClass tempSpinModelTDL;
	while(fsSpinModelTDLFull>>tempSpinModelTDL){
		cout << "test output of tempSpinModelTDL :\n";
		cout << tempSpinModelTDL;
		spinModelList[tempSpinModelTDL.getOriginType()].push_back(tempSpinModelTDL);
	}
	fsSpinModelTDLFull.close();


	// load replaceCouplingsFile
	vector<tuple<unsigned int,unsigned int,Doub> > replaceCouplings;
	fstream fsReplaceCouplings;
	fsReplaceCouplings.open(sReplaceCouplings.c_str(),fstream::in);
	getline(fsReplaceCouplings,tempstring);
	while(fsReplaceCouplings>>x){
		tuple<unsigned int,unsigned int,Doub> tempTuple;
		get<0>(tempTuple)=x;
		fsReplaceCouplings>>x;
		get<1>(tempTuple)=x;
		fsReplaceCouplings>>tempDoub;
		get<2>(tempTuple)=tempDoub;
		replaceCouplings.push_back(tempTuple);
	}
	fsReplaceCouplings.close();

	// test output of replace couplings
	cout << "test output of replaceCouplings:\n";
	for(unsigned int j=0;j<replaceCouplings.size();j++){
		cout << get<0>(replaceCouplings[j]) << " " << get<1>(replaceCouplings[j]) << " " << get<2>(replaceCouplings[j]) << "\n";
	}

	//21.09.15: voher stand hier replace couplings => wird nun nachher gemacht

	// create BondFile by translating the new latticeVectors
	/*
	vector<int> actualSite(2,0);
	unsigned short actualSiteIndex=0;
	map<vector<int>, unsigned short> siteMap;
	// initialize siteMap
	map<int,vector<int> >::iterator itIntra;
	for(itIntra=intraUnitCellVectorsNew.begin();itIntra!=intraUnitCellVectorsNew.end();itIntra++){
		for(unsigned int v1=0;v1<=v1Max;v1++){
			for(unsigned int v2=0;v2<=v2Max;v2++){
				actualSite[0]=v1*interUnitCellVectorsNew[0][0]+v2*interUnitCellVectorsNew[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectorsNew[0][1]+v2*interUnitCellVectorsNew[1][1]+itIntra->second[1];
				siteMap.insert(pair<vector<int>,unsigned short>(actualSite,actualSiteIndex++));
			}
		}
	}
	*/

	// create BondFile by translating the old latticeVectors
	vector<int> actualSite(2,0);
	unsigned short actualSiteIndex=0;
	map<vector<int>, unsigned short> siteMap;
	// initialize siteMap
	map<int,vector<int> >::iterator itIntra;
	for(itIntra=intraUnitCellVectorsOld.begin();itIntra!=intraUnitCellVectorsOld.end();itIntra++){
		for(unsigned int v1=0;v1<=v1Max/2;v1++){ // the 1/2 is because v1Max is in units of the old interUnitCellVectors
			for(unsigned int v2=0;v2<=v2Max;v2++){
				actualSite[0]=v1*interUnitCellVectorsOld[0][0]+v2*interUnitCellVectorsOld[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectorsOld[0][1]+v2*interUnitCellVectorsOld[1][1]+itIntra->second[1];
				siteMap.insert(pair<vector<int>,unsigned short>(actualSite,actualSiteIndex++));
			}
		}
	}

	//SpinModelClass spinModelED;
	vector<SpinModelClass> referenceSpinModels; // the reference spinModel is the spinModel without assigning newSites, so that the sites are 0,1,2,3...
	list<pair<vector<unsigned short>,unsigned int> > siteCombinationsToReferenceSpinModel_list;

	// run over all sites and create coordinates in eukl. basis
	for(itIntra=intraUnitCellVectorsNew.begin();itIntra!=intraUnitCellVectorsNew.end();itIntra++){
		for(unsigned int v1=0;v1<=v1Max;v1++){
			for(unsigned int v2=0;v2<=v2Max;v2++){
				// run over interUnitCells
				cout << "v1=" << v1 << " v2=" << v2 << "\n";

				cout << " itIntra=" << itIntra->second[0] << " " << itIntra->second[1] << " " << char(itIntra->first+'A') << "\n";

				list<pair<vector<unsigned short>,unsigned int> > siteCombinationsToReferenceSpinModel_temp;

				actualSite[0]=v1*interUnitCellVectorsNew[0][0]+v2*interUnitCellVectorsNew[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectorsNew[0][1]+v2*interUnitCellVectorsNew[1][1]+itIntra->second[1];

				// find siteType for actualSite in the OLD basis
				int siteTypeOld=findSiteTypeInBasis(actualSite,interUnitCellVectorsOld,intraUnitCellVectorsOld);

				// run over all operators of the spinModels which have the origin at siteTypeOld
				list<SpinModelTDLClass>::iterator itList;
				for(itList=spinModelList[siteTypeOld].begin();itList!=spinModelList[siteTypeOld].end();itList++){
					SpinModelClass tempSpinModel=*itList;
					vector<vector<int> > siteVectors=itList->getSiteVectors();
					vector<unsigned short> newSites(siteVectors.size());
					// run over all siteVectors
					for(unsigned int i=0;i<siteVectors.size();i++){
						cout << "  i=" << i << "\n";
						vector<int> tempVec(2,0);
						tempVec[0]=actualSite[0]+siteVectors[i][0];
						tempVec[1]=actualSite[1]+siteVectors[i][1];
						if((siteVectors[i][0]!=0) || (siteVectors[i][1]!=0)){ // not the origin
							// find siteType in newBasis
							int siteTypeNew=findSiteTypeInBasis(tempVec,interUnitCellVectorsNew,intraUnitCellVectorsNew);
							map<int,vector<int> >::iterator itIntraMapFind;
							if(siteTypeNew!=0){
								// if the siteType of the newVector is not the siteType A, then the composition into the interUnitCellVectors does not work.
								// therefore subtract the corresponding intraUnitCellVector
								itIntraMapFind=intraUnitCellVectorsNew.find(siteTypeNew);
								cout << "itIntraMap=" << itIntraMapFind->second[0] << " " << itIntraMapFind->second[1] << "\n";
								tempVec[0]-=itIntraMapFind->second[0];
								tempVec[1]-=itIntraMapFind->second[1];
							}
							cout << "tempVec(different siteTyp)=" << tempVec[0] << " " << tempVec[1] << "\n";
							// calculate vector-representation in NEW interUnitCellBasis to find the corresponding unitCell
							int a1=interUnitCellVectorsNew[0][0];
							int a2=interUnitCellVectorsNew[0][1];
							int b1=interUnitCellVectorsNew[1][0];
							int b2=interUnitCellVectorsNew[1][1];
							int det=a1*b2-b1*a2;

							int xa=(b2*tempVec[0]-b1*tempVec[1])/det;
							int xb=(a1*tempVec[1]-a2*tempVec[0])/det;

							// check if xa or xb are out of bounds and shift them due to periodic boundary conditions
							// 16.04.2015: by using a differenc UC Basis xa,xb could be more than on v1Max out of bonds => use while
							while(xa<0){
								xa+=v1Max+1;
							}
							while(xa>v1Max){
								xa-=v1Max+1;
							}
							while(xb<0){
								xb+=v2Max+1;
							}
							while(xb>v2Max){
								xb-=v2Max+1;
							}

							// write the vector in the normal euclidean basis
							tempVec[0]=xa*interUnitCellVectorsNew[0][0]+xb*interUnitCellVectorsNew[1][0];
							tempVec[1]=xa*interUnitCellVectorsNew[0][1]+xb*interUnitCellVectorsNew[1][1];
							// add the intraUnitCellVector if it was previously subtracted
							if(siteTypeNew!=0){
								tempVec[0]+=itIntraMapFind->second[0];
								tempVec[1]+=itIntraMapFind->second[1];
							}
						}
						cout << "  actualSite=" << actualSite[0] << " " << actualSite[1] << "\n";
						cout << "  siteVector=" << siteVectors[i][0] << " " << siteVectors[i][1] << " " << char(siteVectors[i][2]+'A') << "\n";
						cout << "  tempVec=" << tempVec[0] << " " << tempVec[1] << "\n";


						// now find the corresponding siteNumber to this vector from the siteMap
						map<vector<int>, unsigned short>::iterator itMapFind=siteMap.find(tempVec);
						if(itMapFind==siteMap.end()){
							cout << "error: couldn't find site in siteMap!\n";
							return;
						}
						cout << "  actualSiteIndex=" << itMapFind->second << "\n";
						newSites[i]=itMapFind->second;
					}

					// assign the new sites to the spinModel
					tempSpinModel.assignNewSites(newSites);

					// order against sites
					tempSpinModel.orderAgainstSites();

					// push sites to siteCombinations if this tempSpinModel not already exists
					// 29.05.2015: Here comes the subtle, tricky part: Due to periodic boundary conditions, the same sites could occur multiple times, on different "ways" on the lattice
					// All of this combinations must be added, but only once for each way. Since the ways are unique for each initialSite/siteType just check if the spin model is already contained within the siteCombinationsToReferenceSpinModel_list
					// if not, add it to siteCombinationsToReferenceSpinModel_temp
					// if yes, then do not add it.
					bool existsAlready=false;
					list<pair<vector<unsigned short>,unsigned int> >::iterator itSiteComb;
					for(itSiteComb=siteCombinationsToReferenceSpinModel_list.begin();itSiteComb!=siteCombinationsToReferenceSpinModel_list.end();itSiteComb++){
						SpinModelClass tempSpinModel2=referenceSpinModels[itSiteComb->second];
						tempSpinModel2.assignNewSites(itSiteComb->first);
						tempSpinModel2.orderAgainstSites();
						if(tempSpinModel==tempSpinModel2){
							existsAlready=true;
							break;
						}
					}
					if(!existsAlready){
						SpinModelClass tempSpinModel3=*itList;
						unsigned int referenceIndex=referenceSpinModels.size();
						// if it not already exists, find out the referenceSpinModel
						for(unsigned int iRef=0;iRef<referenceSpinModels.size();iRef++){
							if(tempSpinModel3==referenceSpinModels[iRef]){
								referenceIndex=iRef;
								break;
							}
						}
						if(referenceIndex==referenceSpinModels.size()){
							referenceSpinModels.push_back(tempSpinModel3);
						}
						siteCombinationsToReferenceSpinModel_temp.push_back(pair<vector<unsigned short>,unsigned int>(newSites,referenceIndex));
					}
					//spinModelED+=tempSpinModel/tempSpinModel.numberOfOperators();
				}
				// now add the temp siteCombinations to the list
				list<pair<vector<unsigned short>,unsigned int> >::iterator itListAdd;
				for(itListAdd=siteCombinationsToReferenceSpinModel_temp.begin();itListAdd!=siteCombinationsToReferenceSpinModel_temp.end();itListAdd++){
					siteCombinationsToReferenceSpinModel_list.push_back(*itListAdd);
				}
			}
		}
	}

	// now replace all couplings from all spin models accordingly
	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		for(unsigned int j=0;j<replaceCouplings.size();j++){
			referenceSpinModels[i].replaceCouplingByOtherCoupling(get<0>(replaceCouplings[j]),get<1>(replaceCouplings[j]),get<2>(replaceCouplings[j]));
		}
	}


	// save siteMap
	stringstream ssSiteMap;
	ssSiteMap << "siteMapEDCluster_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSiteMap;
	fsSiteMap.open(ssSiteMap.str().c_str(),fstream::out);
	for(map<vector<int>,unsigned short>::iterator it1=siteMap.begin();it1!=siteMap.end();it1++){
		for(unsigned int i=0;i<it1->first.size();i++){
			fsSiteMap << it1->first[i] << " ";
		}
		fsSiteMap << "-> " << it1->second << "\n";
	}
	fsSiteMap.close();

	// save siteCombinations and reference SpinModels
	stringstream ssReferenceSpinModel;
	ssReferenceSpinModel << "effSpinModelReferenceTDL_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsReferenceSpinModel;
	fsReferenceSpinModel.open(ssReferenceSpinModel.str().c_str(),fstream::out);
	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		fsReferenceSpinModel << i << "\n";
		fsReferenceSpinModel << referenceSpinModels[i] << "\n";
	}
	fsReferenceSpinModel.close();

	stringstream ssSiteCombinations;
	ssSiteCombinations << "siteCombinationsForReferenceSpinModel_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSiteCombinations;
	fsSiteCombinations.open(ssSiteCombinations.str().c_str(),fstream::out);
	fsSiteCombinations << "#referenceSpinModel site1 site2 ...\n";
	for(unsigned int j=0;j<referenceSpinModels.size();j++){
		list<pair<vector<unsigned short>,unsigned int> >::iterator itSiteComb;
		for(itSiteComb=siteCombinationsToReferenceSpinModel_list.begin();itSiteComb!=siteCombinationsToReferenceSpinModel_list.end();itSiteComb++){
			if(itSiteComb->second==j){
				fsSiteCombinations << itSiteComb->second;
				for(unsigned int i=0;i<itSiteComb->first.size();i++){
					fsSiteCombinations << " " << itSiteComb->first[i];
				}
				fsSiteCombinations << "\n";
			}
		}
	}
	fsSiteCombinations.close();

	/*
	// save spinModel
	stringstream ssSpinModelED;
	ssSpinModelED << "effSpinModelED_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSpinModelED;
	fsSpinModelED.open(ssSpinModelED.str().c_str(),fstream::out);
	fsSpinModelED << spinModelED << "\n";
	fsSpinModelED.close();
	*/
}

void GraphController::createEDClusterForGeneralClusterForm(unsigned int const &gCUTOrder, string const &sInterUnitCellVectorsOld, string const &sIntraUnitCellVectorsOld, string const &sSiteBoundariesFile, string const &sPeriodicBoundariesFile, string const &sSpinModelTDLFull, string const &sReplaceCouplings, string const &sClusterName){
	// load latticeVectors
	vector<vector<int> > interUnitCellVectorsOld;
	map<int,vector<int> > intraUnitCellVectorsOld;

	string tempstring;
	int x;
	Doub tempDoub;
	char tempChar;
	vector<int> tempVec;

	// load old latticeVectors
	fstream fsInterUnitCellVectorsOld;
	fsInterUnitCellVectorsOld.open(sInterUnitCellVectorsOld.c_str(),fstream::in);
	getline(fsInterUnitCellVectorsOld,tempstring);
	while(fsInterUnitCellVectorsOld>>x){
		tempVec.push_back(x);
		if(fsInterUnitCellVectorsOld.peek()=='\n'){
			interUnitCellVectorsOld.push_back(tempVec);
			tempVec.clear();
		}
	}
	fsInterUnitCellVectorsOld.close();
	if(interUnitCellVectorsOld.size()!=2){
		cout << "number of interUnitCellVectors must be 2!\n";
		return;
	}

	cout << "test output of interUnitCellVectorsOld:\n";
	for(unsigned int i=0;i<interUnitCellVectorsOld.size();i++){
		for(unsigned int j=0;j<interUnitCellVectorsOld[i].size();j++){
			cout << interUnitCellVectorsOld[i][j] << " ";
		}
		cout << "\n";
	}

	fstream fsIntraUnitCellVectorsOld;
	fsIntraUnitCellVectorsOld.open(sIntraUnitCellVectorsOld.c_str(),fstream::in);
	getline(fsIntraUnitCellVectorsOld,tempstring);
	while(fsIntraUnitCellVectorsOld>>tempChar){
		if(tempChar>='A' && tempChar<='Z'){
			vector<int> tempVec(2);
			if(tempChar!='A'){
				cout << "first site in intraUnitCellVectors must be A!\n";
				return;
			}
			fsIntraUnitCellVectorsOld >> x;
			tempVec[0]=x;
			fsIntraUnitCellVectorsOld >> x;
			tempVec[1]=x;
			fsIntraUnitCellVectorsOld >> tempChar;
			intraUnitCellVectorsOld.insert(pair<int,vector<int> >(tempChar-'A',tempVec));
		}
	}
	fsIntraUnitCellVectorsOld.close();

	cout << "test output of intraUnitCellVectorsOld:\n";
	for(map<int,vector<int> >::iterator it1=intraUnitCellVectorsOld.begin();it1!=intraUnitCellVectorsOld.end();it1++){
		for(unsigned int i=0;i<it1->second.size();i++){
			cout << it1->second[i] << " ";
		}
		cout << it1->first << "\n";
	}

	// load spinModelsTDL
	// spinModelList should be shorted against the different siteTypes
	vector<list<SpinModelTDLClass> > spinModelList(intraUnitCellVectorsOld.size());
	fstream fsSpinModelTDLFull;
	fsSpinModelTDLFull.open(sSpinModelTDLFull.c_str(),fstream::in);
	getline(fsSpinModelTDLFull,tempstring);
	SpinModelTDLClass tempSpinModelTDL;
	while(fsSpinModelTDLFull>>tempSpinModelTDL){
		cout << "test output of tempSpinModelTDL :\n";
		cout << tempSpinModelTDL;
		spinModelList[tempSpinModelTDL.getOriginType()].push_back(tempSpinModelTDL);
	}
	fsSpinModelTDLFull.close();


	// load replaceCouplingsFile
	vector<tuple<unsigned int,unsigned int,Doub> > replaceCouplings;
	fstream fsReplaceCouplings;
	fsReplaceCouplings.open(sReplaceCouplings.c_str(),fstream::in);
	getline(fsReplaceCouplings,tempstring);
	while(fsReplaceCouplings>>x){
		tuple<unsigned int,unsigned int,Doub> tempTuple;
		get<0>(tempTuple)=x;
		fsReplaceCouplings>>x;
		get<1>(tempTuple)=x;
		fsReplaceCouplings>>tempDoub;
		get<2>(tempTuple)=tempDoub;
		replaceCouplings.push_back(tempTuple);
	}
	fsReplaceCouplings.close();

	// test output of replace couplings
	cout << "test output of replaceCouplings:\n";
	for(unsigned int j=0;j<replaceCouplings.size();j++){
		cout << get<0>(replaceCouplings[j]) << " " << get<1>(replaceCouplings[j]) << " " << get<2>(replaceCouplings[j]) << "\n";
	}

	// now replace all couplings from all spin models accordingly
	for(unsigned int i=0;i<spinModelList.size();i++){
		list<SpinModelTDLClass>::iterator itList;
		for(itList=spinModelList[i].begin();itList!=spinModelList[i].end();itList++){
			for(unsigned int j=0;j<replaceCouplings.size();j++){
				itList->replaceCouplingByOtherCoupling(get<0>(replaceCouplings[j]),get<1>(replaceCouplings[j]),get<2>(replaceCouplings[j]));
			}
		}
	}

	// load siteBoundaries and createSiteMap from siteBoundaries
	map<vector<int>, unsigned short> siteMap;
	vector<int> actualSite(2,0);
	{
	vector<vector<int> > siteBoundaries;
	fstream fsSiteBoundaries;
	fsSiteBoundaries.open(sSiteBoundariesFile.c_str(),fstream::in);
	getline(fsSiteBoundaries,tempstring);
	while(fsSiteBoundaries>>x){
		vector<int> tempVec(4);
		tempVec[0]=x;
		fsSiteBoundaries>>x;
		tempVec[1]=x;
		fsSiteBoundaries>>x;
		tempVec[2]=x;
		fsSiteBoundaries>>x;
		tempVec[3]=x;
		siteBoundaries.push_back(tempVec);
	}
	fsSiteBoundaries.close();

	// create siteMap from SiteBoundaries
	unsigned short actualSiteIndex=0;
	for(unsigned int i=0;i<siteBoundaries.size();i++){
		for(int y=siteBoundaries[i][1];y<=siteBoundaries[i][2];y+=siteBoundaries[i][3]){
			actualSite[0]=siteBoundaries[i][0];
			actualSite[1]=y;
			siteMap.insert(pair<vector<int>,unsigned short>(actualSite,actualSiteIndex++));
		}
	}
	}
	// load periodicBoundaries
	vector<pair<int,int> > periodicBoundaries;
	fstream fsPeriodicBoundaries;
	fsPeriodicBoundaries.open(sPeriodicBoundariesFile.c_str(),fstream::in);
	while(fsPeriodicBoundaries>>x){
		pair<int,int> pii;
		pii.first=x;
		fsPeriodicBoundaries>>x;
		pii.second=x;
		periodicBoundaries.push_back(pii);
	}
	fsPeriodicBoundaries.close();

 	//SpinModelClass spinModelED;
	vector<SpinModelClass> referenceSpinModels; // the reference spinModel is the spinModel without assigning newSites, so that the sites are 0,1,2,3...
	map<vector<unsigned short>,unsigned int> siteCombinationsToReferenceSpinModel;

	// run over all sites
	map<vector<int>, unsigned short>::iterator itSiteMap;
	for(itSiteMap=siteMap.begin();itSiteMap!=siteMap.end();itSiteMap++){
		// find siteType for actualSite in the OLD basis
		actualSite=itSiteMap->first;
		int siteTypeOld=findSiteTypeInBasis(actualSite,interUnitCellVectorsOld,intraUnitCellVectorsOld);

		// run over all operators of the spinModels which have the origin at siteTypeOld
		list<SpinModelTDLClass>::iterator itList;
		for(itList=spinModelList[siteTypeOld].begin();itList!=spinModelList[siteTypeOld].end();itList++){
			SpinModelClass tempSpinModel=*itList;
			vector<vector<int> > siteVectors=itList->getSiteVectors();
			vector<unsigned short> newSites(siteVectors.size());
			// run over all siteVectors
			for(unsigned int i=0;i<siteVectors.size();i++){
				cout << "  i=" << i << "\n";
				vector<int> tempVec(2,0);
				tempVec[0]=actualSite[0]+siteVectors[i][0];
				tempVec[1]=actualSite[1]+siteVectors[i][1];

				// find site in siteMap
				map<vector<int>, unsigned short>::iterator itSiteMapFind=siteMap.find(tempVec);
				// if the site could not be found it is not contained within the cluster boundaries => periodic boundary conditions!
				if(itSiteMapFind==siteMap.end()){
					for(unsigned int j=0;j<periodicBoundaries.size();j++){
						// add periodic boundary vector and check if the new site is contained within siteMap
						vector<int> tempVec2=tempVec;
						tempVec2[0]+=periodicBoundaries[j].first;
						tempVec2[1]+=periodicBoundaries[j].second;
						itSiteMapFind=siteMap.find(tempVec2);
						if(itSiteMapFind!=siteMap.end()){
							break;
						}
					}
				}
				// itSiteMap is now a pointer to the correct site within siteMap
				if(itSiteMapFind==siteMap.end()){
					cout << "error: couldn't find site in siteMap!\n";
					return;
				}
				newSites[i]=itSiteMapFind->second;
			}

			// assign the new sites to the spinModel
			tempSpinModel.assignNewSites(newSites);

			// order against sites
			tempSpinModel.orderAgainstSites();

			// push sites to siteCombinations if this tempSpinModel not already exists
			bool existsAlready=false;
			map<vector<unsigned short>,unsigned int>::iterator itSiteComb;
			for(itSiteComb=siteCombinationsToReferenceSpinModel.begin();itSiteComb!=siteCombinationsToReferenceSpinModel.end();itSiteComb++){
				SpinModelClass tempSpinModel2=referenceSpinModels[itSiteComb->second];
				tempSpinModel2.assignNewSites(itSiteComb->first);
				tempSpinModel2.orderAgainstSites();
				if(tempSpinModel==tempSpinModel2){
					existsAlready=true;
					break;
				}
			}
			if(!existsAlready){
				SpinModelClass tempSpinModel3=*itList;
				unsigned int referenceIndex=referenceSpinModels.size();
				// if it not already exists, find out the referenceSpinModel
				for(unsigned int iRef=0;iRef<referenceSpinModels.size();iRef++){
					if(tempSpinModel3==referenceSpinModels[iRef]){
						referenceIndex=iRef;
						break;
					}
				}
				if(referenceIndex==referenceSpinModels.size()){
					referenceSpinModels.push_back(tempSpinModel3);
				}
				siteCombinationsToReferenceSpinModel.insert(pair<vector<unsigned short>,unsigned int>(newSites,referenceIndex));
			}

			//spinModelED+=tempSpinModel/tempSpinModel.numberOfOperators();
		}
	}


	// save siteMap
	stringstream ssSiteMap;
	ssSiteMap << "siteMapEDCluster_" << sClusterName << ".txt";
	fstream fsSiteMap;
	fsSiteMap.open(ssSiteMap.str().c_str(),fstream::out);
	for(map<vector<int>,unsigned short>::iterator it1=siteMap.begin();it1!=siteMap.end();it1++){
		for(unsigned int i=0;i<it1->first.size();i++){
			fsSiteMap << it1->first[i] << " ";
		}
		fsSiteMap << "-> " << it1->second << "\n";
	}
	fsSiteMap.close();

	// save siteCombinations and reference SpinModels
	stringstream ssReferenceSpinModel;
	ssReferenceSpinModel << "effSpinModelReferenceTDL_o" << gCUTOrder << "_" << sClusterName << ".txt";
	fstream fsReferenceSpinModel;
	fsReferenceSpinModel.open(ssReferenceSpinModel.str().c_str(),fstream::out);
	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		fsReferenceSpinModel << i << "\n";
		fsReferenceSpinModel << referenceSpinModels[i] << "\n";
	}
	fsReferenceSpinModel.close();

	stringstream ssSiteCombinations;
	ssSiteCombinations << "siteCombinationsForReferenceSpinModel_o" << gCUTOrder << "_" << sClusterName << ".txt";
	fstream fsSiteCombinations;
	fsSiteCombinations.open(ssSiteCombinations.str().c_str(),fstream::out);
	fsSiteCombinations << "#referenceSpinModel site1 site2 ...\n";
	for(unsigned int j=0;j<referenceSpinModels.size();j++){
		map<vector<unsigned short>,unsigned int>::iterator itSiteComb;
		for(itSiteComb=siteCombinationsToReferenceSpinModel.begin();itSiteComb!=siteCombinationsToReferenceSpinModel.end();itSiteComb++){
			if(itSiteComb->second==j){
				fsSiteCombinations << itSiteComb->second;
				for(unsigned int i=0;i<itSiteComb->first.size();i++){
					fsSiteCombinations << " " << itSiteComb->first[i];
				}
				fsSiteCombinations << "\n";
			}
		}
	}
	fsSiteCombinations.close();
}

void GraphController::createHubbardEDCluster(string const &sInterUnitCellVectors, string const &sIntraUnitCellVectors, unsigned int const &v1Max, unsigned int const &v2Max){
	// load latticeVectors
	vector<vector<int> > interUnitCellVectors;
	map<int,vector<int> > intraUnitCellVectors;

	string tempstring;
	int x;
	char tempChar;
	vector<int> tempVec;

	fstream fsInterUnitCellVectors;
	fsInterUnitCellVectors.open(sInterUnitCellVectors.c_str(),fstream::in);
	getline(fsInterUnitCellVectors,tempstring);
	while(fsInterUnitCellVectors>>x){
		tempVec.push_back(x);
		if(fsInterUnitCellVectors.peek()=='\n'){
			interUnitCellVectors.push_back(tempVec);
			tempVec.clear();
		}
	}
	fsInterUnitCellVectors.close();
	if(interUnitCellVectors.size()!=2){
		cout << "number of interUnitCellVectors must be 2!\n";
		return;
	}

	cout << "test output of interUnitCellVectors:\n";
	for(unsigned int i=0;i<interUnitCellVectors.size();i++){
		for(unsigned int j=0;j<interUnitCellVectors[i].size();j++){
			cout << interUnitCellVectors[i][j] << " ";
		}
		cout << "\n";
	}

	fstream fsIntraUnitCellVectors;
	fsIntraUnitCellVectors.open(sIntraUnitCellVectors.c_str(),fstream::in);
	getline(fsIntraUnitCellVectors,tempstring);
	while(fsIntraUnitCellVectors>>tempChar){
		if(tempChar>='A' && tempChar<='Z'){
			vector<int> tempVec(2);
			if(tempChar!='A'){
				cout << "first site in intraUnitCellVectors must be A!\n";
				return;
			}
			fsIntraUnitCellVectors >> x;
			tempVec[0]=x;
			fsIntraUnitCellVectors >> x;
			tempVec[1]=x;
			fsIntraUnitCellVectors >> tempChar;
			intraUnitCellVectors.insert(pair<int,vector<int> >(tempChar-'A',tempVec));
		}
	}
	fsIntraUnitCellVectors.close();

	cout << "test output of intraUnitCellVectors:\n";
	for(map<int,vector<int> >::iterator it1=intraUnitCellVectors.begin();it1!=intraUnitCellVectors.end();it1++){
		for(unsigned int i=0;i<it1->second.size();i++){
			cout << it1->second[i] << " ";
		}
		cout << it1->first << "\n";
	}

	// inter unitCellVectors
	int a1=interUnitCellVectors[0][0];
	int a2=interUnitCellVectors[0][1];
	int b1=interUnitCellVectors[1][0];
	int b2=interUnitCellVectors[1][1];
	int det=a1*b2-b1*a2;

	// create siteMap by translating LatticeVectors
	vector<int> actualSite(2,0);
	unsigned short actualSiteIndex=0;
	map<vector<int>, unsigned short> siteMap;
	// initialize siteMap
	for(unsigned int v1=0;v1<=v1Max;v1++){
		for(unsigned int v2=0;v2<=v2Max;v2++){
			map<int,vector<int> >::iterator itIntra;
			for(itIntra=intraUnitCellVectors.begin();itIntra!=intraUnitCellVectors.end();itIntra++){
				actualSite[0]=v1*interUnitCellVectors[0][0]+v2*interUnitCellVectors[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectors[0][1]+v2*interUnitCellVectors[1][1]+itIntra->second[1];
				siteMap.insert(pair<vector<int>,unsigned short>(actualSite,actualSiteIndex++));
			}
		}
	}

	Graph edCluster;

	// run over all sites
	map<vector<int>, unsigned short>::iterator itMap1;
	for(itMap1=siteMap.begin();itMap1!=siteMap.end();itMap1++){
		// find siteType of this site
		int siteType1=findSiteTypeInBasis(itMap1->first,interUnitCellVectors,intraUnitCellVectors);
		// add all vectors from the lattice definition
		for(unsigned int i=0;i<latticeVectors.size();i++){
			if(latticeVectors[i][0]==siteType1){
				// add lattice vector
				vector<int> nextSite=itMap1->first;
				nextSite[0]+=latticeVectors[i][1];
				nextSite[1]+=latticeVectors[i][2];
				// find site within siteMap
				map<vector<int>, unsigned short>::iterator itMap2=siteMap.find(nextSite);
				if(itMap2==siteMap.end()){
					// if site is not found, periodic boundary conditions apply
					// 1) subtract intraUnitCellVector
					vector<int> tempVec=nextSite;
					tempVec[0]-=intraUnitCellVectors[latticeVectors[i][3]][0];
					tempVec[1]-=intraUnitCellVectors[latticeVectors[i][3]][1];
					// 2) calculate vector-representation in interUnitCellBasis to find the corresponding unitCell
					int xa=(b2*tempVec[0]-b1*tempVec[1])/det;
					int xb=(a1*tempVec[1]-a2*tempVec[0])/det;
					// 3) check if xa or xb are out of bounds and shift them due to periodic boundary conditions
					if(xa<0){
						xa+=v1Max+1;
					}
					else{
						if(xa>v1Max){
							xa-=v1Max+1;
						}
					}
					if(xb<0){
						xb+=v2Max+1;
					}
					else{
						if(xb>v2Max){
							xb-=v2Max+1;
						}
					}
					// 4) write vector in normal euclidean basis
					tempVec[0]=xa*interUnitCellVectors[0][0]+xb*interUnitCellVectors[1][0];
					tempVec[1]=xa*interUnitCellVectors[0][1]+xb*interUnitCellVectors[1][1];
					// 5) add the intraUnitCellVector
					tempVec[0]+=intraUnitCellVectors[latticeVectors[i][3]][0];
					tempVec[1]+=intraUnitCellVectors[latticeVectors[i][3]][1];
					// 6) find site within siteMap
					itMap2=siteMap.find(tempVec);
				}
				edCluster.addLink(itMap1->second,itMap2->second,latticeVectors[i][4]);
			}
		}
	}

	// save siteMap
	stringstream ssSiteMap;
	ssSiteMap << "siteMapHubbardEDCluster_" <<  v1Max << "x" << v2Max << ".txt";
	fstream fsSiteMap;
	fsSiteMap.open(ssSiteMap.str().c_str(),fstream::out);
	for(map<vector<int>,unsigned short>::iterator it1=siteMap.begin();it1!=siteMap.end();it1++){
		for(unsigned int i=0;i<it1->first.size();i++){
			fsSiteMap << it1->first[i] << " ";
		}
		fsSiteMap << "-> " << it1->second << "\n";
	}
	fsSiteMap.close();

	// save as bondFile
	stringstream ssHubbardEDCluster;
	ssHubbardEDCluster << "HubbardEDCluster_" << v1Max << "x" << v2Max << ".cfg";
	fstream fsHubbardEDCluster;
	fsHubbardEDCluster.open(ssHubbardEDCluster.str().c_str(),fstream::out);
	fsHubbardEDCluster << edCluster;
	// on-site hubbard interaction (must be saved manually, as the graph does not contain information about on-site interactions
	// what is the maximum coupling number
	int maxCouplingNumber=0;
	map<short,short>::iterator itInvCoupl;
	for(itInvCoupl=inverseCouplings.begin();itInvCoupl!=inverseCouplings.end();itInvCoupl++){
		if(couplingIndex[itInvCoupl->first]<=couplingIndex[itInvCoupl->second]){ // this ensures, that only the correct couplings, ie the first of the inverse coupling pair is counted
			if(itInvCoupl->first>maxCouplingNumber){
				maxCouplingNumber=itInvCoupl->first;
			}
		}
	}
	// the hubbard interaction is the maximumCouplingNumber+1
	for(unsigned int i=0;i<edCluster.gCUTOrder();i++){
		fsHubbardEDCluster << maxCouplingNumber+1 << " " << i << ";\n";
	}
	fsHubbardEDCluster.close();
}

void GraphController::performGraphSubtractionsAndEmbeddings(unsigned int const &gCUTOrder, string const &sGraphList, string const &sInitialSiteList, string const &sSubtractionList, string const &sSpinModelFormList, string const &sEffSpinModelList, string const &sMasterClusterLg){
	// 1) read effSpinModels
	// a) read list-files
	vector<string> sGraphs;
	vector<string> sVecSpinModelForm;
	vector<string> sVecEffSpinModel;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	unsigned int x,y;
	string tempString;

	map<unsigned short,string> initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.insert(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();


	char tempChar;
	unsigned int index=0;


	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(fsGraphList>>x){
		fsGraphList>>y;
		fsGraphList>>tempString;
		sGraphs.push_back(tempString);
		graphNumbers.push_back(pair<unsigned int,unsigned int>(x,y));
	}
	fsGraphList.close();

	fstream fsSpinModelFormList;
	fsSpinModelFormList.open(sSpinModelFormList.c_str(),fstream::in);
	while(fsSpinModelFormList>>x){
		fsSpinModelFormList>>y;
		if(pair<unsigned int,unsigned int>(x,y)!=graphNumbers[index]){
			cout << "error! spinModelFormList is not ordered in the same manner as spinModelFormList!\n";
		}
		fsSpinModelFormList>>tempString;
		sVecSpinModelForm.push_back(tempString);
		index++;
	}
	fsSpinModelFormList.close();

	index=0;
	fstream fsEffSpinModelList;
	fsEffSpinModelList.open(sEffSpinModelList.c_str(),fstream::in);
	while(fsEffSpinModelList>>x){
		fsEffSpinModelList>>y;
		if(pair<unsigned int,unsigned int>(x,y)!=graphNumbers[index]){
			cout << "error! effSpinModelList is not ordered in the same manner as spinModelFormList!\n";
		}
		fsEffSpinModelList>>tempString;
		sVecEffSpinModel.push_back(tempString);
		index++;
	}
	fsEffSpinModelList.close();

	vector<SpinModelClass> vecSpinModels;
	for(unsigned int i=0;i<sVecSpinModelForm.size();i++){
		vector<SpinModelClass> vecSpinModels_singleOperators;
		// b) effSpinModelForm gives the general form of the operators
		// read this and create effSpinModels
		SpinModelClass tempSpinModel;
		fstream fsSpinModelForm;
		fsSpinModelForm.open(sVecSpinModelForm[i].c_str(),fstream::in);
		//first of all skip over the comment lines, and the number of couplings integer
		while(fsSpinModelForm.peek()=='#'){
			getline(fsSpinModelForm,tempString);
		}
		fsSpinModelForm >> x;
		fsSpinModelForm >> noskipws >> tempChar >> skipws;
		while(fsSpinModelForm.peek()=='#'){
			getline(fsSpinModelForm,tempString);
		}
		/////////////////////////////////////////

		while(fsSpinModelForm>>x){
			fsSpinModelForm>>y;
			fsSpinModelForm>>tempSpinModel;
			vecSpinModels_singleOperators.push_back(tempSpinModel);
		}
		fsSpinModelForm.close();

		// c) effSpinModel gives the corresponding sites to the operator
		SpinModelClass fullSpinModel;
		fstream fsEffSpinModel;
		fsEffSpinModel.open(sVecEffSpinModel[i].c_str(),fstream::in);
		while(fsEffSpinModel>>tempChar){
			fsEffSpinModel>>x;

			vector<unsigned short> vecSites;
			unsigned short z;
			for(unsigned int j=0;j<vecSpinModels_singleOperators[x].numberOfOperators();j++){
				fsEffSpinModel >> z;
				vecSites.push_back(z);
			}
			fsEffSpinModel >> noskipws >> tempChar >> skipws;
			pair<SeriesDouble,SeriesDouble> prefactor;
			fsEffSpinModel>>prefactor;
			//fsEffSpinModel.clear();
			SpinModelClass newSpinModel=vecSpinModels_singleOperators[x];
			newSpinModel.assignNewSites(vecSites);
			newSpinModel.orderAgainstSites();
			fullSpinModel+=prefactor*newSpinModel;
		}
		fsEffSpinModel.close();
		vecSpinModels.push_back(fullSpinModel);
	}

	// TEST OUTPUT
	cout << "test output of effSpinModel:\n";
	for(unsigned int i=0;i<vecSpinModels.size();i++){
		cout << graphNumbers[i].first << " " << graphNumbers[i].second << "\n";
		cout << " " << vecSpinModels[i] << "\n\n";
	}


	// load Graphs
	vector<Graph> allGraphs;
	for(unsigned int i=0;i<sGraphs.size();i++){
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphs[i].c_str(),fstream::in);
		Graph g;
		fsGraphFiles >> g;
		allGraphs.push_back(g);
		fsGraphFiles.close();
	}

	// 2) read subtractionList
	vector<vector<int> > vecSubtractions;
	fstream fsSubtractionList;
	fsSubtractionList.open(sSubtractionList.c_str(),fstream::in);
	int c,d,e;
	vector<int> tempVecInt;
	getline(fsSubtractionList,tempString);
	while(fsSubtractionList>>c){
		tempVecInt.push_back(c);
		if(fsSubtractionList.peek()=='\n'){
			vecSubtractions.push_back(tempVecInt);
			tempVecInt.clear();
		}
	}
	fsSubtractionList.close();

	cout << "vecSubtractions.size()=" << vecSubtractions.size() << "\n";

	// 3) calculate subtracted graphs
	//TODO: this should not be done! they should be embedded directly onto the lattice without purifying them!
	cout << "calulate subtracted graphs:\n";
	for(unsigned int i=0;i<vecSubtractions.size();i++){
		// push sites of g1 and g2 into single vectors
		unsigned int numberOfSites=(vecSubtractions[i].size()-5)/2;
		unsigned int order1=vecSubtractions[i][0];
		unsigned int graphNumber1=vecSubtractions[i][1];
		vector<unsigned short> sites1;
		vector<unsigned short> sites2;
		for(unsigned int j=2;j<=numberOfSites+1;j++){
			sites1.push_back(vecSubtractions[i][j]);
			sites2.push_back(vecSubtractions[i][j+numberOfSites+2]);
		}
		unsigned int order2=vecSubtractions[i][2+numberOfSites];
		unsigned int graphNumber2=vecSubtractions[i][3+numberOfSites];

		// create a sorted version of sites2
		// also create a permutation vector
		vector<unsigned short> sortedSites2=sites2;
		vector<unsigned short> permutationVector(sites2.size());
		for(unsigned int j=0;j<permutationVector.size();j++){
			permutationVector[j]=j;
		}
		for(unsigned int j=0;j<sortedSites2.size();j++){
			unsigned int minIndex=j;
			for(unsigned int k=j+1;k<sortedSites2.size();k++){
				if(sortedSites2[k]<sortedSites2[minIndex]){
					minIndex=k;
				}
			}
			if(minIndex!=j){
				unsigned short temp=sortedSites2[j];
				sortedSites2[j]=sortedSites2[minIndex];
				sortedSites2[minIndex]=temp;

				temp=permutationVector[j];
				permutationVector[j]=permutationVector[minIndex];
				permutationVector[minIndex]=temp;
			}
		}

		cout << "sites1=";
		for(unsigned int j=0;j<sites1.size();j++){
			cout << sites1[j] << " ";
		}
		cout << "\n";
		cout << "sites2=";
		for(unsigned int j=0;j<sites2.size();j++){
			cout << sites2[j] << " ";
		}
		cout << "\n";

		cout << "sortedSites2=";
		for(unsigned int j=0;j<sortedSites2.size();j++){
			cout << sortedSites2[j] << " ";
		}
		cout << "\n";
		cout << "permutationVector=";
		for(unsigned int j=0;j<permutationVector.size();j++){
			cout << permutationVector[j] << " ";
		}
		cout << "\n";


		// find index of the spinModels g1 and g2
		unsigned int index1=0;
		unsigned int index2=0;
		for(unsigned int j=0;j<graphNumbers.size();j++){
			if((graphNumbers[j].first==order1) && (graphNumbers[j].second==graphNumber1)){
				index1=j;
			}
			if((graphNumbers[j].first==order2) && (graphNumbers[j].second==graphNumber2)){
				index2=j;
			}
		}

		// get the part of the spinModel of g2 with the sortedSites2
		SpinModelClass sortedSites2SpinModel=vecSpinModels[index2].getOperatorsOnSpecifiedSites(sortedSites2);
		cout << "output of the sortedsites spinModel:\n";
		cout << sortedSites2SpinModel << "\n";

		// use assignNewSites() to apply the corresponding sites of g1 to g2!
		// Important: consider the ordering
		// so create the newSitesVector with considering the ordering:
		vector<unsigned short> newSites(sortedSites2.size());
		for(unsigned int j=0;j<permutationVector.size();j++){
			newSites[j]=sites1[permutationVector[j]];
		}
		sortedSites2SpinModel.assignNewSites(newSites);
		// order the operators against the sites
		sortedSites2SpinModel.orderAgainstSites();
		// use the embeddingFactor to add this spinModel to the spinModel of g1
		cout << "index1=" << index1 << "\n";
		cout << "order1=" << order1 << "\n";
		cout << "graphNumber1=" << graphNumber1 << "\n";
		cout << "index2=" << index2 << "\n";
		cout << "order2=" << order2 << "\n";
		cout << "graphNumber1=" << graphNumber1 << "\n";
		cout << "embeddingFactor=" << vecSubtractions[i][vecSubtractions[i].size()-1] << "\n";
		vecSpinModels[index1]+=vecSubtractions[i][vecSubtractions[i].size()-1]*sortedSites2SpinModel;
	}

	// 4) save subtracted effSpinModels
	for(unsigned int i=0;i<vecSpinModels.size();i++){
		stringstream ssEffSpinModelPure;
		ssEffSpinModelPure << "effSpinModelPure_o" << graphNumbers[i].first << "g" << graphNumbers[i].second << ".txt";
		fstream fsEffSpinModelPure;
		fsEffSpinModelPure.open(ssEffSpinModelPure.str().c_str(), fstream::out);
		fsEffSpinModelPure << vecSpinModels[i];
		fsEffSpinModelPure.close();
	}


	// now this is important for the ground-state energy:
	// up to now the ground state energy was usually NOT per site!
	// so, now divide the groundState energy through the numberOfSites
	// later one must set the site of the groundState Energy to the initialSite
	for(unsigned int i=0;i<vecSpinModels.size();i++){
		//cout << "o" << graphNumbers[i].first << " g" << graphNumbers[i].second << "\n";
		vecSpinModels[i].divideGroundStateEnergyBy(graphNumbers[i].first);
	}

	// 5) embed on masterCluster
	// load embeddings
	//TODO: this must be done for all initialSites separately

	vector<SpinModelClass> thermodynamicSpinModels(initialSites.size());
	unsigned int tdlIndex=0;
	map<unsigned short,string>::const_iterator itInitialSites;
	for(itInitialSites=initialSites.begin();itInitialSites!=initialSites.end();itInitialSites++){
		cout << "initialSites=" << itInitialSites->first << " " << itInitialSites->second << "\n";
		vector<pair<unsigned int,vector<unsigned short> > > allEmbeddings;
		fstream fsEmbeddingList;
		fsEmbeddingList.open(itInitialSites->second.c_str(),fstream::in);
		getline(fsEmbeddingList,tempString);
		while(fsEmbeddingList>>c){
			fsEmbeddingList>>d;
			unsigned int graphIndex=0;
			// find the index corresponding to order=c, graphNumber=d
			for(unsigned int i=0;i<graphNumbers.size();i++){
				if((graphNumbers[i].first==c) && (graphNumbers[i].second==d)){
					graphIndex=i;
					break;
				}
			}
			getline(fsEmbeddingList,tempString);
			stringstream ssTemp(tempString);
			vector<unsigned short> tempVecShort;
			while(ssTemp>>e){
				tempVecShort.push_back(e);
			}
			allEmbeddings.push_back(pair<unsigned int,vector<unsigned short> >(graphIndex,tempVecShort));
		}
		fsEmbeddingList.close();

		cout << "embed on masterCluster\n";
		cout << "vecSpinModels.size()=" << vecSpinModels.size() << "\n";
		cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
		cout << "allGraphs.size()=" << allGraphs.size() << "\n";

		// Now calcualte the thermodynamic spinModel
		for(unsigned int i=0;i<allEmbeddings.size();i++){ // run over all graphs
			unsigned int graphIndex=allEmbeddings[i].first;
			cout << "graphIndex=" << graphIndex << "\n";
			cout << "order=" << graphNumbers[graphIndex].first << " graphNumber=" << graphNumbers[graphIndex].second << "\n";

			// assign sites and add the spinModel
			vector<unsigned short> oldSites=allGraphs[graphIndex].getAllSites();
			SpinModelClass tempSpinModel=vecSpinModels[graphIndex];
			cout << "tempSpinModel=" << tempSpinModel << "\n";
			cout << "oldSites=";
			for(unsigned int j=0;j<oldSites.size();j++){
				cout << oldSites[j] << " ";
			}
			cout << "\n";
			cout << "newSites=";
			for(unsigned int j=0;j<allEmbeddings[i].second.size();j++){
				cout << allEmbeddings[i].second[j] << " ";
			}
			cout << "\n";
			tempSpinModel.assignNewSites(oldSites,allEmbeddings[i].second);
			cout << "tempSpinModel=" << tempSpinModel << "\n";
			tempSpinModel.orderAgainstSites();
			cout << "tempSpinModel=" << tempSpinModel << "\n";
			// now set the site of the groundState energy (the unity) to the initialSite
			tempSpinModel.setGroundStateSiteTo(itInitialSites->first);
			// now kick out all operator-terms which dont include the initialSite!
			tempSpinModel.deleteOperatorsWithoutSite(itInitialSites->first);

			thermodynamicSpinModels[tdlIndex]+=tempSpinModel;
		}

		// 6) save thermodynamic spinModel
		stringstream ssSpinModelTDL;
		ssSpinModelTDL << "effSpinModelTDL_o" << gCUTOrder << "_initialSite" << itInitialSites->first << ".txt";
		fstream fsSpinModelTDL;
		fsSpinModelTDL.open(ssSpinModelTDL.str().c_str(),fstream::out);
		fsSpinModelTDL << thermodynamicSpinModels[tdlIndex] << "\n";
		fsSpinModelTDL.close();
		tdlIndex++;
		cout << "tdlIndex=" << tdlIndex << "\n";
	}


	// 7) load masterCluster and save the spinModel with ascending sites, where only the sites that occur in the spinModel are numbered. So at N sites the occuring sites are numbered from 0 to N-1
	// initialSite is set to 0
	// additionally save the vectors which connect the different sites
	// must be performed for all thermodynamicSpinModels (i.e. for all initialSites)

	LatticeGraph masterClusterLg;

	fstream fsMasterClusterLg;
	fsMasterClusterLg.open(sMasterClusterLg.c_str(),fstream::in);
	fsMasterClusterLg >> masterClusterLg;
	fsMasterClusterLg.close();

	stringstream ssSpinModelTDLFull;
	ssSpinModelTDLFull << "effSpinModelTDLFull_o" << gCUTOrder << ".txt";
	fstream fsSpinModelTDLFull;
	fsSpinModelTDLFull.open(ssSpinModelTDLFull.str().c_str(),fstream::out);
	fsSpinModelTDLFull << "#siteType1 connectionVector1 siteType2 connectionVector2 ...\n";
	fsSpinModelTDLFull.close();

	vector<vector<int> > inverseSiteMap=masterClusterLg.getInverseSiteMap();
	tdlIndex=0;
	for(itInitialSites=initialSites.begin();itInitialSites!=initialSites.end();itInitialSites++){
		vector<int> initialSiteVector=inverseSiteMap[itInitialSites->first];

		cout << "allSiteCombinations\n";
		list<vector<unsigned short> > siteCombinations=thermodynamicSpinModels[tdlIndex].getAllSiteCombinations();

		list<vector<unsigned short> >::iterator itSiteComb;
		cout << "test output of siteCombinations:\n";
		for(itSiteComb=siteCombinations.begin();itSiteComb!=siteCombinations.end();itSiteComb++){
			for(unsigned int i=0;i<itSiteComb->size();i++){
				cout << (*itSiteComb)[i] << " ";
			}
			cout << "\n";
		}

		for(itSiteComb=siteCombinations.begin();itSiteComb!=siteCombinations.end();itSiteComb++){
			SpinModelClass siteCombSpinModel=thermodynamicSpinModels[tdlIndex].getOperatorsOnSpecifiedSites(*itSiteComb);
			siteCombSpinModel.assignNewSites();
			fsSpinModelTDLFull.open(ssSpinModelTDLFull.str().c_str(),fstream::out | fstream::app);
			for(unsigned int i=0;i<itSiteComb->size();i++){
				if(i!=0){
					fsSpinModelTDLFull << " ";
				}
				fsSpinModelTDLFull << char(inverseSiteMap[(*itSiteComb)[i]][2]+'A');
				fsSpinModelTDLFull << " " << inverseSiteMap[(*itSiteComb)[i]][0]-initialSiteVector[0];
				fsSpinModelTDLFull << " " << inverseSiteMap[(*itSiteComb)[i]][1]-initialSiteVector[1];
			}
			fsSpinModelTDLFull << "\n";
			fsSpinModelTDLFull << siteCombSpinModel << "\n";
			fsSpinModelTDLFull.close();
		}
		tdlIndex++;
	}
}

void GraphController::createSiteCombinationsWithEdgeSpins(unsigned int const &gCUTOrder, string const &sGraphList, string const &sMasterCluster, string const &sInitialSiteList){
	// this function creates all possible graphs with edge spins (from the already calculated graphs, embeddings and the masterCluster)
	// in order to have an unique edge spin configuration for each graph only WEAK embeddings are considered (i.e. that to the embedding of a specific graph only graphs are considered where all possible links between sites are drawn)

	// => siteCombination and siteMap files
	// => embedding files
	// => edgeSpinSubtractions file

	unsigned int x,y;
	int c,d,e;
	string tempString;

	// load masterCluster
	Graph gMasterCluster;
	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::in);
	fsMasterCluster >> gMasterCluster;
	fsMasterCluster.close();

	// find maximum number of couplings
	// now, for each coupling there is a possible edge spin coupling (since the edge spins are stationary, one needs to have an edge spin even for the inverse couplings
	unsigned short numberOfCouplings=couplingIndex.size();
	unsigned short minEdgeSpinCoupling=0;
	map<short,short>::const_iterator itInvCoupl;
	for(itInvCoupl=inverseCouplings.begin();itInvCoupl!=inverseCouplings.end();++itInvCoupl){
		if(itInvCoupl->first<=itInvCoupl->second){
			minEdgeSpinCoupling++;
		}
	}


	// load all graphs (neccessary for creation of siteCombinations and siteMaps)
	vector<string> sGraphs;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(fsGraphList>>x){
		fsGraphList>>y;
		fsGraphList>>tempString;
		sGraphs.push_back(tempString);
		graphNumbers.push_back(pair<unsigned int,unsigned int>(x,y));
	}
	fsGraphList.close();

	vector<Graph> allGraphs;
	for(unsigned int i=0;i<sGraphs.size();i++){
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphs[i].c_str(),fstream::in);
		Graph g;
		fsGraphFiles >> g;
		allGraphs.push_back(g);
		fsGraphFiles.close();
	}

	// edgeSpinGraphs
	// for each graph save the one edge spin configuration
	vector<vector<vector<unsigned short> > > edgeSpinConf(allGraphs.size()); // vector(Graphs)<vector(all sites of the graph)<vector(number of edge spins for each edgeSpin coupling of this site)> >

	// load initialSiteList
	vector<pair<unsigned short,string> > initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.push_back(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();

	// run over all initial sites
	for(unsigned int initialSiteIndex=0;initialSiteIndex<initialSites.size();initialSiteIndex++){
		// load embeddings for this initialSite
		list<pair<unsigned int,vector<unsigned short> > > allEmbeddings;
		fstream fsEmbeddingList;
		fsEmbeddingList.open(initialSites[initialSiteIndex].second.c_str(),fstream::in);
		getline(fsEmbeddingList,tempString);
		while(fsEmbeddingList>>c){
			fsEmbeddingList>>d;

			unsigned int graphIndex=0;
			// find the index corresponding to order=c, graphNumber=d
			for(unsigned int i=0;i<graphNumbers.size();i++){
				if((graphNumbers[i].first==c) && (graphNumbers[i].second==d)){
					graphIndex=i;
					break;
				}
			}

			getline(fsEmbeddingList,tempString);
			stringstream ssTemp(tempString);
			vector<unsigned short> tempVecShort;
			while(ssTemp>>e){
				tempVecShort.push_back(e);
			}
			allEmbeddings.push_back(pair<unsigned int,vector<unsigned short> >(graphIndex,tempVecShort));
		}
		fsEmbeddingList.close();

		// run over all embeddings
		list<pair<unsigned int,vector<unsigned short> > >::iterator itAllEmb=allEmbeddings.begin();
		while(itAllEmb!=allEmbeddings.end()){
			unsigned int actualGraph=itAllEmb->first;
			// check if this embedding is a strong or weak embedding
			Graph assignedGraph;
			allGraphs[actualGraph].assignSites(assignedGraph,itAllEmb->second);
			if(assignedGraph.isStrongSubgraph(gMasterCluster)){
				// if it is a strong embedding, check if the edge spin configuration for this graph was already obtained
				if(edgeSpinConf[actualGraph].size()==0){
					// call method: numberOfEdgeSpins on masterCluster
					edgeSpinConf[actualGraph].resize(itAllEmb->second.size(),vector<unsigned short>(numberOfCouplings,0)); //initialize vector in which the number of edge spins is saved
					gMasterCluster.numberOfEdgeSpins(itAllEmb->second,edgeSpinConf[actualGraph]);
				}
				++itAllEmb;
			}
			else{
				// if it is not a strong embedding, delete this embedding from the list
				itAllEmb=allEmbeddings.erase(itAllEmb);
			}
		}

		// save embeddings with edgeSpins
		stringstream ssEdgeSpinEmbeddings;
		ssEdgeSpinEmbeddings << "edgeSpinEmbeddings_o" << gCUTOrder << "_initialSite" << initialSites[initialSiteIndex].first << ".cfg";
		fstream fsEdgeSpinEmbeddings;
		fsEdgeSpinEmbeddings.open(ssEdgeSpinEmbeddings.str().c_str(),fstream::out);
		fsEdgeSpinEmbeddings << "#order graphNumber sitesOnMasterCluster\n";
		for(itAllEmb=allEmbeddings.begin();itAllEmb!=allEmbeddings.end();++itAllEmb){
			fsEdgeSpinEmbeddings << graphNumbers[itAllEmb->first].first << " " << graphNumbers[itAllEmb->first].second;
			for(unsigned int j=0;j<itAllEmb->second.size();j++){
				fsEdgeSpinEmbeddings << " " << itAllEmb->second[j];
			}
			fsEdgeSpinEmbeddings << "\n";
		}
		fsEdgeSpinEmbeddings.close();
	}

	// calculate the edgeSubtractions (subtraction of subgraphs with edge Spins)
	// for each subgraph, check the edgeSpinConfiguration
	vector<vector<int> > subtractionFactors(allGraphs.size(),vector<int>(allGraphs.size(),0));

	// run over all graphs (which are strong embeddings)
	for(unsigned int i=0;i<allGraphs.size();i++){
		// check if this graph is a strong embedding (i.e. it has a defined edge spin configuration)
		if(edgeSpinConf[i].size()>0){
			// calculate ALL subgraphs, but increase the subtraction factor only if the subgraph is a strong embedding onto the original graph (BUT weak graphs must also be considered on the way, because by removing links these weak graphs can become strong)
			// Step 1: remove links to create subgraphs
			// Step 2: check if created graphs are defined in the graphList
			// Step 2a: if it is defined: add this as a subgraph
			// Step 2b: if it is not defined: remove further links until 2a (or graph is empty, this is just an artificial exit condition)
			// Step 3: proceed with next graph
			vector<Graph> subgraphs=allGraphs[i].removeLinks();
			cout << "allGraphs[" << i << "]:\n";
			cout << allGraphs[i] << "\n";
			cout << "subgraphs.size()=" << subgraphs.size() << "\n";
			for(unsigned int j=0;j<subgraphs.size();j++){
				// check if this is a strong graph
				if(subgraphs[j].isStrongSubgraph(allGraphs[i])){
					// if this is a strong graph, then find the index of this graph
					cout << "subgraphs[" << j << "]:\n";
					cout << subgraphs[j] << "\n";
					// get subGraphIndex
					for(unsigned int subGraphIndex=0;subGraphIndex<allGraphs.size();subGraphIndex++){
						if(subgraphs[j].getCanonicalGraphNumber()==allGraphs[subGraphIndex].getCanonicalGraphNumber()){
							// subgraph is found, now increase subtractionFactor
							subtractionFactors[i][subGraphIndex]--;
							break;
						}
					}
				}

				// remove additional links
				vector<Graph> subgraphs2=subgraphs[j].removeLinks();
				for(unsigned int k=0;k<subgraphs2.size();k++){
					// push them back to subgraphs if this graph does not exist already in subgraphs!
					bool foundGraph=false;
					for(unsigned int kk=0;kk<subgraphs.size();kk++){
						if(subgraphs[kk]==subgraphs2[k]){
							foundGraph=true;
							break;
						}
					}
					if(!foundGraph){
						subgraphs.push_back(subgraphs2[k]);
					}
				}
			}
		}
	}

	// subtraction of edge spins and siteCombinations for each graph with edge spins
	// important: the method writeSiteCombinations assumes that each coupling belongs to one instance of an referenceSpinModel

	stringstream ssEdgeSpinSubtractions;
	ssEdgeSpinSubtractions << "edgeSpinSubtractions_o" << gCUTOrder << ".cfg";
	fstream fsEdgeSpinSubtractions;
	fsEdgeSpinSubtractions.open(ssEdgeSpinSubtractions.str().c_str(),fstream::out);
	fsEdgeSpinSubtractions << "#order graphNumber edgeSpinCoupling subtractionFactor\n";
	fsEdgeSpinSubtractions.close();

	for(unsigned int i=0;i<edgeSpinConf.size();i++){
		// now just use the summed number of edgeSpins (not for each site individually) for the edgeSpinSubtractions
		// save also the stuff within siteCombinations

		if(edgeSpinConf[i].size()>0){ //only save strong graphs
			stringstream ssSiteCombinations;
			ssSiteCombinations << "siteCombinations_o" << graphNumbers[i].first << "g" << graphNumbers[i].second << ".cfg";
			fstream fsSiteCombinations;
			fsSiteCombinations.open(ssSiteCombinations.str().c_str(),fstream::out);
			fsSiteCombinations << "#referenceSpinModel site1 site2 ...\n";
			allGraphs[i].writeSiteCombinations(fsSiteCombinations);

			vector<int> summedEdgeSpins(numberOfCouplings,0);
			for(unsigned int j=0;j<edgeSpinConf[i].size();j++){
				for(unsigned int k=0;k<numberOfCouplings;k++){
					summedEdgeSpins[k]-=edgeSpinConf[i][j][k];
					for(unsigned int nn=0;nn<edgeSpinConf[i][j][k];nn++){
						fsSiteCombinations << minEdgeSpinCoupling+k << " " << j << "\n";
					}
				}
			}
			fsSiteCombinations.close();

			fsEdgeSpinSubtractions.open(ssEdgeSpinSubtractions.str().c_str(),fstream::out | fstream::app);
			for(unsigned int k=0;k<numberOfCouplings;k++){
				fsEdgeSpinSubtractions << graphNumbers[i].first << " " << graphNumbers[i].second << " " << minEdgeSpinCoupling+k << " " << summedEdgeSpins[k] << "\n";
			}
			fsEdgeSpinSubtractions.close();
		}
	}

	// save subtraction factors
	stringstream ssEdgeSpinSubgraphEmbeddings;
	ssEdgeSpinSubgraphEmbeddings << "edgeSpinSubgraphEmbeddings_o" << gCUTOrder << ".cfg";
	fstream fsEdgeSpinSubgraphEmbeddings;
	fsEdgeSpinSubgraphEmbeddings.open(ssEdgeSpinSubgraphEmbeddings.str().c_str(),fstream::out);
	fsEdgeSpinSubgraphEmbeddings << "#Order(g1) GraphNumber(g1) Order(g2) GraphNumber(g2) EmbeddingFactor\n";
	for(unsigned int i=0;i<subtractionFactors.size();i++){
		pair<unsigned int,unsigned int> graphNumbers1=graphNumbers[i];
		for(unsigned int j=0;j<subtractionFactors.size();j++){
			if(subtractionFactors[i][j]!=0){
				pair<unsigned int,unsigned int> graphNumbers2=graphNumbers[j];
				fsEdgeSpinSubgraphEmbeddings << graphNumbers1.first << " " << graphNumbers1.second << " " << graphNumbers2.first << " " << graphNumbers2.second << " " << subtractionFactors[i][j] << "\n";
			}
		}
	}
	fsEdgeSpinSubgraphEmbeddings.close();

	// now save the remaining stuff
	// edgeSpinCouplings (which real coupling corresponds to the edgeSpinCoupling)
	stringstream ssEdgeSpinCouplings;
	ssEdgeSpinCouplings << "edgeSpinCouplings.cfg";
	fstream fsEdgeSpinCouplings;
	fsEdgeSpinCouplings.open(ssEdgeSpinCouplings.str().c_str(),fstream::out);
	fsEdgeSpinCouplings << "#edgeSpinCoupling latticeCouplings\n";
	map<short,unsigned short>::iterator itCouplInd;
	for(itCouplInd=couplingIndex.begin();itCouplInd!=couplingIndex.end();++itCouplInd){
		fsEdgeSpinCouplings << minEdgeSpinCoupling+itCouplInd->second << " " << itCouplInd->first << "\n";
	}
	fsEdgeSpinCouplings.close();
}
