/*
 * LatticeGraph.cpp
 *
 *  Created on: May 5, 2014
 *      Author: ixert
 */

#include "LatticeGraph.h"
#include<fstream>
#include "globalVariables.h"

LatticeGraph::LatticeGraph(){
}

void LatticeGraph::createRectangularLatticeCluster(vector<vector<int> > const &latticeVectors, unsigned int const &xMax, unsigned int const &yMax, unsigned int const &n){
	//TODO: how long should this program add links to the sites?
	// Of course links with (x,y)>(xMax,yMax) should not be added, but how do I know that i've found all sites?
	// maybe just saying do it n times is sufficient? => for the next nearest neighbors this does not work
	// do it n times and check if x,y is within xMAx,yMax
	// d=latticeVectors[i] is specified in the following way:
	// d[0]=siteTypeFrom
	// d[1]=x
	// d[2]=y
	// d[3]=siteTypeTo
	// d[4]=couplingType

	// add links to the origin
	vector<int> originA(3,0);
	for(unsigned int i=0;i<latticeVectors.size();i++){
		if(latticeVectors[i][0]==0){ //check if the siteTypeFrom corresponds to the first siteType=A
			if((latticeVectors[i][1]<=xMax) && (latticeVectors[i][2]<=yMax)){
				set<pair<vector<int>,short> > tempSet;
				// create latticeVector
				vector<int> d(3);
				d[0]=latticeVectors[i][1];
				d[1]=latticeVectors[i][2];
				d[2]=latticeVectors[i][3];
				tempSet.insert(pair<vector<int>,short>(d,latticeVectors[i][4]));
				adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(originA,tempSet));
				tempSet.clear();
				map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(latticeVectors[i][4]);
				if(itInvCoupl==inverseCouplings.end()){
					cout << "ERROR: inverse coupling not found!\n";
					return;
				}
				tempSet.insert(pair<vector<int>,short>(originA,itInvCoupl->second));
				adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(d,tempSet));
			}
		}
	}

	// run over all sites
	for(unsigned int i=0;i<n;i++){
		map<vector<int>,set<pair<vector<int>,short> > >::iterator it1;
		for(it1=adj.begin();it1!=adj.end();it1++){
			for(unsigned int i=0;i<latticeVectors.size();i++){
				if(latticeVectors[i][0]==it1->first[2]){ // actual site is of the same type as the lattice vector
					vector<int> newVec=it1->first;
					// create latticeVector
					newVec[0]+=latticeVectors[i][1];
					newVec[1]+=latticeVectors[i][2];
					newVec[2]=latticeVectors[i][3];

					// add new link if the new lattice site lies within (x,y)<=(xMax,yMax)
					if((newVec[0]<=xMax) && (newVec[1]<=yMax)){
						set<pair<vector<int>,short> > tempSet=it1->second;
						pair<set<pair<vector<int>,short> >::iterator,bool> retSet=tempSet.insert(pair<vector<int>,short>(newVec,latticeVectors[i][4]));
						if(retSet.second){
							map<vector<int>,set<pair<vector<int>,short> > >::iterator it2=adj.find(it1->first);
							it2->second=tempSet;
							it2=adj.find(newVec);
							map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(latticeVectors[i][4]);
							if(itInvCoupl==inverseCouplings.end()){
								cout << "ERROR: inverse coupling not found!\n";
								return;
							}
							if(it2!=adj.end()){
								it2->second.insert(pair<vector<int>,short>(it1->first,itInvCoupl->second));
							}
							else{
								set<pair<vector<int>,short> > tempSet2;
								tempSet2.insert(pair<vector<int>,short>(it1->first,itInvCoupl->second));
								adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(newVec,tempSet2));
							}
						}
					}
				}
			}
		}
	}
}

vector<LatticeGraph> LatticeGraph::createNewLatticeGraphs(vector<vector<int> > const &latticeVectors){
	// d=latticeVectors[i] is specified in the following way:
	// d[0]=siteTypeFrom
	// d[1]=x
	// d[2]=y
	// d[3]=siteTypeTo
	// d[4]=couplingType

	vector<LatticeGraph> vlg;
	if(adj.size()==0){
		// if this graph is empty, then create all Graphs (from sitetype A=0)
		// this graphs are already moved that the left-bottom site (left before bottom!) is the origin
		vector<int> originA(3,0);
		for(unsigned int i=0;i<latticeVectors.size();i++){
			if(latticeVectors[i][0]==0){ //check if the siteTypeFrom corresponds to the first siteType=A
				LatticeGraph lg;
				set<pair<vector<int>,short> > tempSet;
				// create latticeVector
				vector<int> d(3);
				d[0]=latticeVectors[i][1];
				d[1]=latticeVectors[i][2];
				d[2]=latticeVectors[i][3];
				tempSet.insert(pair<vector<int>,short>(d,latticeVectors[i][4]));
				lg.adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(originA,tempSet));
				tempSet.clear();
				map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(latticeVectors[i][4]);
				if(itInvCoupl==inverseCouplings.end()){
					cout << "ERROR: inverse coupling not found!\n";
					return vector<LatticeGraph>();
				}
				tempSet.insert(pair<vector<int>,short>(originA,itInvCoupl->second));
				lg.adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(d,tempSet));
				vlg.push_back(lg);
			}
		}
	}
	else{
		LatticeGraph lg;

		// run over all sites
		map<vector<int>,set<pair<vector<int>,short> > >::iterator it1;
		for(it1=adj.begin();it1!=adj.end();it1++){
			// run over all latticeVectors
			for(unsigned int i=0;i<latticeVectors.size();i++){
				if(latticeVectors[i][0]==it1->first[2]){ // check if siteType corresponds to the siteType of the actual site
					vector<int> newVec=it1->first;

					// create latticeVector
					newVec[0]+=latticeVectors[i][1];
					newVec[1]+=latticeVectors[i][2];
					newVec[2]=latticeVectors[i][3];
					set<pair<vector<int>,short> > tempSet=it1->second;

					pair<set<pair<vector<int>,short> >::iterator,bool> retSet;
					retSet=tempSet.insert(pair<vector<int>,short>(newVec,latticeVectors[i][4]));
					if(retSet.second){
						LatticeGraph lg;
						lg.adj=adj;
						map<vector<int>,set<pair<vector<int>,short> > >::iterator it2=lg.adj.find(it1->first);
						it2->second=tempSet;

						it2=lg.adj.find(newVec);
						map<short,short>::const_iterator itInvCoupl=inverseCouplings.find(latticeVectors[i][4]);
						if(it2!=lg.adj.end()){
							it2->second.insert(pair<vector<int>,short>(it1->first,itInvCoupl->second));
						}
						else{
							set<pair<vector<int>,short> > tempSet2;
							tempSet2.insert(pair<vector<int>,short>(it1->first,itInvCoupl->second));
							lg.adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(newVec,tempSet2));
						}

						// move graph to origin
						lg.moveGraphToOrigin();

						// add only if graph does not exist
						bool foundGraph=false;
						for(unsigned int j=0;j<vlg.size();j++){
							if(vlg[j]==lg){
								foundGraph=true;
								break;
							}
						}
						if(!foundGraph){
							vlg.push_back(lg);
						}
					}
				}
			}
		}
	}
	return vlg;
}

void LatticeGraph::moveGraphToOrigin(){
	// this method moves all GraphSites so that the left-bottom site (left before bottom!) is the origin

	// find left bottom site
	map<vector<int>,set<pair<vector<int>,short> > >::iterator it;
	int minx=0;
	int miny=0;
	for(it=adj.begin();it!=adj.end();it++){
		if(it->first[0]<minx){
			minx=it->first[0];
			miny=it->first[1];
		}
		else{
			if(it->first[0]==minx){
				if(it->first[1]<miny){
					minx=it->first[0];
					miny=it->first[1];
				}
			}
		}
	}

	// now move all sites about -(minx,miny)
	map<vector<int>,set<pair<vector<int>,short> > > tempAdj;
	for(it=adj.begin();it!=adj.end();it++){
		vector<int> tempVector=it->first;
		tempVector[0]-=minx;
		tempVector[1]-=miny;

		set<pair<vector<int>,short> > tempSet;
		set<pair<vector<int>,short> >::iterator itSet;
		for(itSet=it->second.begin();itSet!=it->second.end();itSet++){
			vector<int> tempVector2=itSet->first;
			tempVector2[0]-=minx;
			tempVector2[1]-=miny;
			tempSet.insert(pair<vector<int>,short>(tempVector2,itSet->second));
		}

		tempAdj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(tempVector,tempSet));
	}
	adj.swap(tempAdj);
}

unsigned int LatticeGraph::gCUTOrder(){
	return adj.size();
}

unsigned int LatticeGraph::sumOfLinks(){
	unsigned int sum=0;
	map<vector<int>,set<pair<vector<int>,short> > >::const_iterator it1;
	for(it1=adj.begin();it1!=adj.end();it1++){
		sum+=it1->second.size();
	}
	return sum;
}

void LatticeGraph::buildSiteMap(){
	// build siteMap
	unsigned short siteIndex=0;
	map<vector<int>,set<pair<vector<int>,short> > >::iterator it;
	for(it=adj.begin();it!=adj.end();it++){
		siteMap.insert(pair<vector<int>,unsigned short>(it->first,siteIndex));
		siteIndex++;
	}
}

vector<vector<int> > LatticeGraph::getInverseSiteMap(){
	// if siteMap is not build, build it!
	if(siteMap.size()==0){
		buildSiteMap();
	}

	vector<vector<int> > inverseSiteMap(siteMap.size());
	map<vector<int>, unsigned short>::const_iterator it;
	for(it=siteMap.begin();it!=siteMap.end();it++){
		inverseSiteMap[it->second]=it->first;
	}

	return inverseSiteMap;
}

Graph LatticeGraph::createGraph(){
	// if siteMap is not build, build it!
	if(siteMap.size()==0){
		buildSiteMap();
	}

	/*
	map<vector<int>, unsigned short>::iterator it2;
	for(it2=siteMap.begin();it2!=siteMap.end();it2++){
		for(unsigned int i=0;i<it2->first.size();i++){
			cout << it2->first[i] << " ";
		}
		cout << "-> " << it2->second << "\n";
	}
	*/


	Graph g;
	// add sites to g
	map<vector<int>,set<pair<vector<int>,short> > >::iterator it;
	for(it=adj.begin();it!=adj.end();it++){
		map<vector<int>, unsigned short>::iterator itFind=siteMap.find(it->first);

		set<pair<unsigned short,short> > tempSet;
		set<pair<vector<int>,short> >::iterator itSet;
		for(itSet=it->second.begin();itSet!=it->second.end();itSet++){
			map<vector<int>, unsigned short>::iterator itSetFind=siteMap.find(itSet->first);
			tempSet.insert(pair<unsigned short,short>(itSetFind->second,itSet->second));
		}

		g.insert(itFind->second,tempSet);
	}

	return g;
}

void LatticeGraph::createGraphPlot(string const &sGnuplot, string const &sPlotPoints){
	// 1) create Plotpoints
	fstream fsPointFile;
	fsPointFile.open(sPlotPoints.c_str(),fstream::out);
	map<vector<int>,set<pair<vector<int>,short> > >::const_iterator it;
	for(it=adj.begin();it!=adj.end();it++){
		fsPointFile << it->first[0] << " " << it->first[1] << "\n";
	}
	fsPointFile.close();

	// 2) create gnuplot file
	fstream fsGnuplotFile;
	fsGnuplotFile.open(sGnuplot.c_str(),fstream::out);
	fsGnuplotFile << "set samples 10000;\n";
	fsGnuplotFile << "set terminal epslatex  size 15cm,10cm color standalone header \"\\\\usepackage{amsmath}\"\n";
	string sTex=sGnuplot;
	sTex.replace(sGnuplot.find("."),8,".tex");
	fsGnuplotFile << "set output \"" << sTex << "\"\n";
	fsGnuplotFile << "set nokey;\n";
	fsGnuplotFile << "set xlabel '$x$';\n";
	fsGnuplotFile << "set ylabel '$y$';\n";
	fsGnuplotFile << "unset border;\n";
	fsGnuplotFile << "unset tics;\n";

	// plot couplings
	unsigned int arrowIndex=1;
	for(it=adj.begin();it!=adj.end();it++){
		set<pair<vector<int>,short> >::const_iterator it2;
		for(it2=it->second.begin();it2!=it->second.end();it2++){
			if(it2->second==0){
				fsGnuplotFile << "set arrow " << arrowIndex << " from " << it->first[0] << "," << it->first[1] << " to " << it2->first[0] << "," << it2->first[1] << " nohead front;\n";
				arrowIndex++;
			}
			if(it2->second==1){
				fsGnuplotFile << "set arrow " << arrowIndex << " from " << it->first[0] << "," << it->first[1] << " to " << it2->first[0] << "," << it2->first[1] << " lt 5 nohead front;\n";
				arrowIndex++;
			}
			//TODO: plotting with more than two couplings!
		}
	}

	// plot sites from siteMap
	map<vector<int>, unsigned short>::const_iterator itSiteMap;
	for(itSiteMap=siteMap.begin();itSiteMap!=siteMap.end();itSiteMap++){
		fsGnuplotFile << "set label '" << itSiteMap->second << "' at " << itSiteMap->first[0] << "," << itSiteMap->first[1] << ";\n";
	}

	fsGnuplotFile << "plot '" << sPlotPoints << "' using 1:2 with points lc rgb 'black' pt 7 ps 1.4 title '';\n";
	fsGnuplotFile << "set output;\n";

	fsGnuplotFile.close();
}

ostream& operator<<(ostream &o, LatticeGraph const &lg){
	map<vector<int>,set<pair<vector<int>,short> > >::const_iterator it;

	for(it=lg.adj.begin();it!=lg.adj.end();it++){
		set<pair<vector<int>,short> >::const_iterator it2;
		for(it2=it->second.begin();it2!=it->second.end();it2++){
			o << it->first[0] << " " << it->first[1] << " " << char(it->first[2]+'A') << " " << it2->first[0] << " " << it2->first[1] << " " << char(it2->first[2]+'A') << " " << it2->second << "\n";
		}
	}

	return o;
}

istream& operator>>(istream &o, LatticeGraph &lg){
	string tempstring;
	int x1,x2;
	int y1,y2;
	int c;
	char siteType1;
	char siteType2;
	while(o>>x1){
		o>>y1;
		o>>skipws>>siteType1;
		o>>x2;
		o>>y2;
		o>>skipws>>siteType2;
		o>>c;

		vector<int> tempVec1(3);
		tempVec1[0]=x1;
		tempVec1[1]=y1;
		tempVec1[2]=siteType1-'A';

		vector<int >tempVec2(3);
		tempVec2[0]=x2;
		tempVec2[1]=y2;
		tempVec2[2]=siteType2-'A';


		map<vector<int>,set<pair<vector<int>,short> > >::iterator itAdj=lg.adj.find(tempVec1);
		if(itAdj!=lg.adj.end()){
			itAdj->second.insert(pair<vector<int>,short>(tempVec2,c));
		}
		else{
			set<pair<vector<int>,short> > tempSet;
			tempSet.insert(pair<vector<int>,short>(tempVec2,c));
			lg.adj.insert(pair<vector<int>,set<pair<vector<int>,short> > >(tempVec1,tempSet));
		}
	}

	return o;
}

bool operator==(LatticeGraph const& lhs, LatticeGraph const& rhs){
	return (lhs.adj==rhs.adj);
}

bool operator!=(LatticeGraph const& lhs, LatticeGraph const& rhs){
	return !(lhs==rhs);
}
