/*
 * LatticeGraph.h
 *
 *  Created on: May 5, 2014
 *      Author: ixert
 */

#ifndef LATTICEGRAPH_H_
#define LATTICEGRAPH_H_

#include<map>
#include<set>
#include<vector>
#include<utility>
#include<sstream>
#include "Graph.h"

using namespace std;

class LatticeGraph {
	friend ostream& operator<<(ostream &o, LatticeGraph const &lg);
	friend istream& operator>>(istream &o, LatticeGraph &lg);
	friend bool operator==(LatticeGraph const& lhs, LatticeGraph const& rhs);
	// LatticeGraphs "live" on a particular lattice
	// in contrast to Graphs their sites have a position (i.e. Coordinates)
	// additionally the sites have a type (A=0, B=1, ...)
private:
	// vector<int> is (x,y,siteType)
	// short defines coupling; for example on the KMH-Model:
	// 0 is normal coupling (nearest neighbors)
	// 1 is directed coupling from site to coupled site (next nearest neighbors)
	// 2 is directed coupling from coupled site to site (next nearest neighbors)
	map<vector<int>,set<pair<vector<int>,short> > > adj;

	map<vector<int>, unsigned short> siteMap; //for mapping to graph
public:
	LatticeGraph();

	void createRectangularLatticeCluster(vector<vector<int> > const &latticeVectors, unsigned int const &xMax, unsigned int const &yMax, unsigned int const &n);

	vector<LatticeGraph> createNewLatticeGraphs(vector<vector<int> > const &latticeVectors);

	void moveGraphToOrigin();

	unsigned int gCUTOrder();
	unsigned int sumOfLinks();

	void buildSiteMap();
	vector<vector<int> > getInverseSiteMap();
	Graph createGraph();

	void createGraphPlot(string const &sGnuplot, string const &sPlotPoints);
};

bool operator!=(LatticeGraph const& lhs, LatticeGraph const& rhs);

#endif /* LATTICEGRAPH_H_ */
