/*
 * SpinModelClass.h
 *
 *  Created on: Jul 17, 2014
 *      Author: ixert
 */

#ifndef SPINMODELCLASS_H_
#define SPINMODELCLASS_H_

#include "nr3.h"
#include<list>
#include<utility>
#include<set>
#include "SeriesTemplate.h"
#include "SpinOperatorsOnSites.h"
#include "pairFunctions.h"

using namespace std;

class SpinModelClass {
friend istream& operator>>(istream &o, SpinModelClass &s);
friend ostream& operator<<(ostream &o, SpinModelClass &s);
friend bool operator==(SpinModelClass const &lhs, SpinModelClass const &rhs);
protected:
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > > spinModel;

public:
	SpinModelClass();

	int numberOfOperators();

	void assignNewSites();
	void assignNewSites(vector<unsigned short> const &newSites);
	void assignNewSites(vector<unsigned short> const &oldSites, vector<unsigned short> const &newSites);
	void orderAgainstSites();

	void divideGroundStateEnergyBy(Doub const &n);
	void setGroundStateSiteTo(unsigned short const &initialSite);

	list<vector<unsigned short> > getAllSiteCombinations();
	SpinModelClass getOperatorsOnSpecifiedSites(vector<unsigned short> const &newSites);
	SpinModelClass getOperatorsWhichContainSite(unsigned short const &site);
	void deleteOperatorsWithoutSite(unsigned short const &site);

	void replaceCouplingByOtherCoupling(unsigned int const &c1, unsigned int const &c2, Doub const &factor);

	SpinModelClass& operator+=(SpinModelClass const& rhs);
	SpinModelClass& operator+=(SpinOperatorsOnSites<Complex> const& rhs);
	SpinModelClass& operator-=(SpinModelClass const& rhs);
	SpinModelClass& operator*=(Doub const& rhs);
	SpinModelClass& operator*=(pair<SeriesDouble,SeriesDouble> const& rhs);
	SpinModelClass& operator/=(Doub const& rhs);
};

// free functions declaration //
const SpinModelClass operator+(SpinModelClass const& lhs, SpinModelClass const& rhs);
const SpinModelClass operator+(SpinModelClass const& lhs, SpinOperatorsOnSites<Complex> const& rhs);
const SpinModelClass operator+(SpinOperatorsOnSites<Complex> const& lhs, SpinModelClass const& rhs);
const SpinModelClass operator-(SpinModelClass const& lhs, SpinModelClass const& rhs);
const SpinModelClass operator*(SpinModelClass const& lhs, Doub const& rhs);
const SpinModelClass operator*(Doub const& lhs, SpinModelClass const& rhs);
const SpinModelClass operator*(SpinModelClass const& lhs, pair<SeriesDouble,SeriesDouble> const& rhs);
const SpinModelClass operator*(pair<SeriesDouble,SeriesDouble> const& lhs, SpinModelClass const& rhs);
const SpinModelClass operator/(SpinModelClass const& lhs, Doub const& rhs);

bool operator!=(SpinModelClass const &lhs, SpinModelClass const &rhs);

#endif /* SPINMODELCLASS_H_ */
