/*
 * SpinModelTDLClass.cpp
 *
 *  Created on: 28.08.2014
 *      Author: dominik-ixert
 */

#include "SpinModelTDLClass.h"
#include "pairFunctions.h"

SpinModelTDLClass::SpinModelTDLClass():originType(-1){
}

int SpinModelTDLClass::getOriginType(){
	if(originType<0){
		for(unsigned int i=0;i<siteVectors.size();i++){
			if((siteVectors[i][0]==0) && (siteVectors[i][1]==0)){
				originType=siteVectors[i][2];
				break;
			}
		}
	}

	return originType;
}

vector<vector<int> > SpinModelTDLClass::getSiteVectors(){
	return siteVectors;
}

istream& operator>>(istream &o, SpinModelTDLClass &s){
	// clear all
	s.siteVectors.clear();
	s.spinModel.clear();

	// first read the sites and their vectors
	// get complete line of stream
	char tempChar;
	tempChar=o.peek();
	if(tempChar=='\n'){
		o>>noskipws>>tempChar>>skipws;
	}
	string tempstring;
	getline(o,tempstring);
	istringstream iss(tempstring);

	int x;

	vector<int> tempVec(3);
	while(iss>>tempChar){
		tempVec[2]=tempChar-'A';
		iss>>x;
		tempVec[0]=x;
		iss>>x;
		tempVec[1]=x;
		if((tempVec[0]==0) && (tempVec[1]==0)){
			s.originType=tempVec[2];
		}
		s.siteVectors.push_back(tempVec);
	}

	if(s.siteVectors.size()==0){
		o.setstate(ios_base::failbit);
		return o;
	}

	//if there is a + or - in front of the prefactor kick this out
	Doub VZ=1;
	if(o.peek()=='+'){
		o>>tempChar;
	}
	else{
		if(o.peek()=='-'){
			o>>tempChar;
			VZ=-1;
		}
	}

	pair<SeriesDouble,SeriesDouble> prefactor;
	while(o.peek()=='('){
		o>>prefactor;
		if(VZ!=1){
			prefactor*=VZ;
			VZ=1;
		}

		string tempstring;
		getline(o,tempstring);
		istringstream iss(tempstring);

		if(iss.peek()=='*'){
			iss>>tempChar;
		}

		SpinOperatorsOnSites<Complex> tempSpinOperator;
		while(iss>>tempSpinOperator){
			s.spinModel.push_back(pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> >(prefactor,tempSpinOperator));
		}
		if(o.peek()=='+'){
			o>>tempChar;
		}
		else{
			if(o.peek()=='-'){
				o>>tempChar;
				VZ=-1;
			}
		}
	}

	if(s.spinModel.size()==0){
		o.setstate(ios_base::failbit);
	}
	else{
		o.clear();
	}

	return o;
}

ostream& operator<<(ostream &o, SpinModelTDLClass &s){
	for(unsigned int i=0;i<s.siteVectors.size();i++){
		if(i!=0){
			o << " ";
		}
		o << char(s.siteVectors[i][2]+'A');
		o << " " << s.siteVectors[i][0];
		o << " " << s.siteVectors[i][1];
	}
	o << "\n";
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it1;
	for(it1=s.spinModel.begin();it1!=s.spinModel.end();it1++){
		o << "+" << it1->first << "*" << it1->second << "\n";
	}
	return o;
}
