/*
 * main.cpp
 *
 *  Created on: May 2, 2014
 *      Author: ixert
 */

#include<iostream>
#include<sstream>
#include<fstream>
#include "LatticeGraph.h"
#include "GraphController.h"
#include "functions.h"

using namespace std;

int main(int argc, char *argv[]){
	// argument-list:
	// 6 arguments (argc=7):
	// calculationType=1, gCUTOrder, inverseCouplingsFile, latticeFile, masterCluster, initialSiteList
	// => graphCreation of graphs on the lattice specified in latticeFile
	// => leads to graphList
	// => also calculates embeddings on masterCluster
	// => leads to embeddingList

	// 6 arguments (argc=7):
	// calculationType=2, pCUTOrder, inverseCouplingsFile, latticeFile, masterCluster, initialSiteList
	// => graphCreation of graphs on the lattice specified in latticeFile
	// => leads to graphList, but kicks out all non pCUT Graphs
	// => also calculates embeddings on masterCluster
	// => leads to embeddingList

	// 4 arguments (argc=5):
	// calculationType=3, gCUTOrder, inverseCouplingsFile, graphList
	// => graphSubtraction
	// => leads to subtractionList

	// 6 arguments (argc=7):
	// calculationType=4, gCUTOrder, inverseCouplingsFile, graphList, masterCluster, initialSiteList
	// => graphEmbedding onto the masterCluster (always including the initialSite)
	// => leads to embeddingList

	// 9 arguments (argc=10):
	// calculationType=5, gCUTOrder, inverseCouplingsFile, graphList, initialSiteList, subtractionList, spinModelFormList, effSpinModelList, masterClusterLg
	// => actual subtraction/embedding for the effective Spin-Models specified in effSpinModelList
	// => leads to thermodynamic spin model

	// 9 arguments (argc=10):
	// calculationType=6, inverseCouplingsFile, latticeFile, xMax, yMax, n, masterClusterFilename, gnuplotFilename, plotPointsFilename
	// => createMasterCluster
	// => leads to MasterCluster

	// 7 arguments (argc=8)
	// calculationType=7, gCUTOrder, interUnitCellVectors, intraUnitCellVectors, xMax, yMax, spinModelTDL
	// => createEDCluster
	// => creates an EDCluster with the SpinHamiltonian from spinModelTDL

	// 10 arguments (argc=11)
	// calculationType=8, gCUTOrder, interUnitCellVectorsOld, intraUnitCellVectorsOld, interUnitCellVectors, intraUnitCellVectors, xMax, yMax, spinModelTDL, replaceCouplingsFile
	// => createEDClusterInDifferentUnitCellBasis
	// => creates an EDCluster with the SpinHamiltonian from spinModelTDL in the through inter/intraUnitCellVectors specified basis

	// 9 arguments (argc=10)
	// calculationType=9, gCUTOrder, interUnitCellVectorsOld, intraUnitCellVectorsOld, siteBoundariesFile, periodicBoundariesFile, spinModelTDLFUll, replaceCouplingsFile, clusterName
	// => siteBoundaries definies ymin(x) and ymax(x) and the x values; periodicBoundaries are Vectors which connect the same site, due to periodic boundary conditions
	// => creates an EDCluster with the SpinHamiltonian from spinModelTDL on the lattice specified through siteBoundaries und periodicBoundaries

	// 7 arguments (argc=8)
	// calculationType=10, inverseCouplingsFile, latticeFile, interUnitCellVectors, intraUnitCellVectors, xMax, yMax
	// => createHubbardEDCluster
	// => creates an EDCluster for the (Kane-Mele-)Hubbard-model

	// 7 arguments (argc=8)
	// calculationType=11, gCUTOrder, inverseCouplingsFile, latticeFile, graphList, masterCluster, initialSiteList
	// => creates all different edge spin configurations on graphs
	// => leads to siteCombinations and new embedding files


	time_t startingTime = time(NULL);
	srand(time(NULL));


	// read parameters
	if(argc<2){
		cout << "no arguments!\n";
		return 1;
	}


	unsigned int calculationType=0;
	unsigned int gCUTOrder=2;
	string sInverseCouplingsFile="";
	string sLatticeFile="";
	string sGraphList="";
	string sMasterCluster="";
	string sMasterClusterLg="";
	string sInitialSiteList="";
	string sCouplingList="";
	string sSubtractionList="";
	string sSpinModelFormList="";
	string sEffSpinModelList="";
	string sGnuplot="";
	string sPlotPoints="";
	string sEffSpinModelTDL="";
	string sInterUnitCellVectors="";
	string sIntraUnitCellVectors="";
	string sInterUnitCellVectorsOld="";
	string sIntraUnitCellVectorsOld="";
	string sReplaceCouplings="";
	string sSiteBoundaries="";
	string sPeriodicBoundaries="";
	string sClusterName="";

	unsigned int yMax=0;
	unsigned int xMax=0;
	unsigned int n=100; // this is the samples parameter for the creation of the master cluster. larger value means longer calculation time for the master cluster. Should be increased if the masterCluster gets bigger!

	stringstream ssCalculationType(argv[1]);
	ssCalculationType >> calculationType;

	switch(calculationType){
	case 1:{
		if(argc!=7){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[4]);
		ssLatticeFile >> sLatticeFile;
		stringstream ssMasterCluster(argv[5]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssInitialSite(argv[6]);
		ssInitialSite >> sInitialSiteList;
		break;
	}
	case 2:{
		if(argc!=7){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[4]);
		ssLatticeFile >> sLatticeFile;
		stringstream ssMasterCluster(argv[5]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssInitialSite(argv[6]);
		ssInitialSite >> sInitialSiteList;
		break;
	}
	case 3:{
		if(argc!=5){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssGraphList(argv[4]);
		ssGraphList >> sGraphList;
		break;
	}
	case 4:{
		if(argc!=7){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssGraphList(argv[4]);
		ssGraphList >> sGraphList;
		stringstream ssMasterCluster(argv[5]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssInitialSite(argv[6]);
		ssInitialSite >> sInitialSiteList;
		break;
	}
	case 5:{
		if(argc!=10){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssGraphList(argv[4]);
		ssGraphList >> sGraphList;
		stringstream ssInitialSite(argv[5]);
		ssInitialSite >> sInitialSiteList;
		stringstream ssSubtractionList(argv[6]);
		ssSubtractionList >> sSubtractionList;
		stringstream ssSpinModelFormList(argv[7]);
		ssSpinModelFormList >> sSpinModelFormList;
		stringstream ssEffSpinModelList(argv[8]);
		ssEffSpinModelList >> sEffSpinModelList;
		stringstream ssMasterClusterLg(argv[9]);
		ssMasterClusterLg >> sMasterClusterLg;
		break;
	}
	case 6:{
		if(argc!=10){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssInverseCouplingsFile(argv[2]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[3]);
		ssLatticeFile >> sLatticeFile;
		stringstream ssXMax(argv[4]);
		ssXMax >> xMax;
		stringstream ssYMax(argv[5]);
		ssYMax >> yMax;
		stringstream ssN(argv[6]);
		ssN >> n;
		stringstream ssMasterCluster(argv[7]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssGnuplot(argv[8]);
		ssGnuplot >> sGnuplot;
		stringstream ssPlotPoints(argv[9]);
		ssPlotPoints >> sPlotPoints;
		break;
	}
	case 7:{
		if(argc!=8){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInterUnitCellVectors(argv[3]);
		ssInterUnitCellVectors >> sInterUnitCellVectors;
		stringstream ssIntraUnitCellVectors(argv[4]);
		ssIntraUnitCellVectors >> sIntraUnitCellVectors;
		stringstream ssXMax(argv[5]);
		ssXMax >> xMax;
		stringstream ssYMax(argv[6]);
		ssYMax >> yMax;
		stringstream ssSpinModelTDL(argv[7]);
		ssSpinModelTDL >> sEffSpinModelTDL;
		break;
	}
	case 8:{
		if(argc!=11){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder>>gCUTOrder;
		stringstream ssInterUnitCellVectorsOld(argv[3]);
		ssInterUnitCellVectorsOld>>sInterUnitCellVectorsOld;
		stringstream ssIntraUnitCellVectorsOld(argv[4]);
		ssIntraUnitCellVectorsOld>>sIntraUnitCellVectorsOld;
		stringstream ssInterUnitCellVectors(argv[5]);
		ssInterUnitCellVectors>>sInterUnitCellVectors;
		stringstream ssIntraUnitCellVectors(argv[6]);
		ssIntraUnitCellVectors>>sIntraUnitCellVectors;
		stringstream ssXMax(argv[7]);
		ssXMax >> xMax;
		stringstream ssYMax(argv[8]);
		ssYMax >> yMax;
		stringstream ssSpinModelTDL(argv[9]);
		ssSpinModelTDL >> sEffSpinModelTDL;
		stringstream ssReplaceCouplings(argv[10]);
		ssReplaceCouplings >> sReplaceCouplings;
		break;
	}
	case 9:{
		if(argc!=10){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder>>gCUTOrder;
		stringstream ssInterUnitCellVectorsOld(argv[3]);
		ssInterUnitCellVectorsOld>>sInterUnitCellVectorsOld;
		stringstream ssIntraUnitCellVectorsOld(argv[4]);
		ssIntraUnitCellVectorsOld>>sIntraUnitCellVectorsOld;
		stringstream ssSiteBoundaries(argv[5]);
		ssSiteBoundaries>>sSiteBoundaries;
		stringstream ssPeriodicBoundaries(argv[6]);
		ssPeriodicBoundaries>>sPeriodicBoundaries;
		stringstream ssSpinModelTDL(argv[7]);
		ssSpinModelTDL >> sEffSpinModelTDL;
		stringstream ssReplaceCouplings(argv[8]);
		ssReplaceCouplings >> sReplaceCouplings;
		stringstream ssClusterName(argv[9]);
		ssClusterName>>sClusterName;
		break;
	}
	case 10:{
		if(argc!=8){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssInverseCouplingsFile(argv[2]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[3]);
		ssLatticeFile >> sLatticeFile;
		stringstream ssInterUnitCellVectors(argv[4]);
		ssInterUnitCellVectors>>sInterUnitCellVectors;
		stringstream ssIntraUnitCellVectors(argv[5]);
		ssIntraUnitCellVectors>>sIntraUnitCellVectors;
		stringstream ssXMax(argv[6]);
		ssXMax >> xMax;
		stringstream ssYMax(argv[7]);
		ssYMax >> yMax;
		break;
	}
	case 11:{
		if(argc!=8){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[4]);
		ssLatticeFile >> sLatticeFile;
		stringstream ssGraphList(argv[5]);
		ssGraphList >> sGraphList;
		stringstream ssMasterCluster(argv[6]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssInitialSite(argv[7]);
		ssInitialSite >> sInitialSiteList;
		break;
	}
	}

	if(gCUTOrder<2){
		cout << "gCUTOrder must be at least 2!\n";
		return 1;
	}

	cout << "calculationType=" << calculationType << "\n";
	cout << "gCUTOrder=" << gCUTOrder << "\n";
	cout << "sInverseCouplingsFile=" << sInverseCouplingsFile << "\n";
	cout << "sLatticeFile=" << sLatticeFile << "\n";
	cout << "sGraphList=" << sGraphList << "\n";
	cout << "sMasterCluster=" << sMasterCluster << "\n";
	cout << "sMasterClusterLg=" << sMasterClusterLg << "\n";
	cout << "sInitialSiteList=" << sInitialSiteList << "\n";
	cout << "sCouplingList=" << sCouplingList << "\n";
	cout << "sSubtractionList=" << sSubtractionList << "\n";
	cout << "sSpinModelFormList=" << sSpinModelFormList << "\n";
	cout << "sEffSpinModelList=" << sEffSpinModelList << "\n";
	cout << "sGnuplot=" << sGnuplot << "\n";
	cout << "sPlotPoints=" << sPlotPoints << "\n";
	cout << "yMax=" << yMax << "\n";
	cout << "xMax=" << xMax << "\n";
	cout << "n=" << n << "\n";
	cout << "sEffSpinModelTDL=" << sEffSpinModelTDL << "\n";
	cout << "sInterUnitCellVectors=" << sInterUnitCellVectors << "\n";
	cout << "sIntraUnitCellVectors=" << sIntraUnitCellVectors << "\n";
	cout << "sInterUnitCellVectorsOld=" << sInterUnitCellVectorsOld << "\n";
	cout << "sIntraUnitCellVectorsOld=" << sIntraUnitCellVectorsOld << "\n";
	cout << "sReplaceCouplings=" << sReplaceCouplings << "\n";
	cout << "sSiteBoundaries=" << sSiteBoundaries << "\n";
	cout << "sPeriodicBoundaries=" << sPeriodicBoundaries << "\n";
	cout << "sClusterName=" << sClusterName << "\n";;

	// set inverseCouplings (specifically for the KMH-Model on the honeycomb lattice)

	// calculation
	GraphController gc(sInverseCouplingsFile,sLatticeFile);

	/*
	// quick and dirty creation of 12er masterCluster
	LatticeGraph LG12er;
	stringstream ssLG12er;
	ssLG12er<< "masterCluster_9er.cfglg";
	fstream fsLG12er;
	fsLG12er.open(ssLG12er.str().c_str(),fstream::in);
	fsLG12er>>LG12er;
	fsLG12er.close();

	cout << LG12er << "\n";

	Graph G12er=LG12er.createGraph();
	stringstream ssG12er;
	ssG12er << "masterCluster_9er.cfg";
	fstream fsG12er;
	fsG12er.open(ssG12er.str().c_str(),fstream::out);
	fsG12er<<G12er;
	fsG12er.close();

	return 0;
	*/


	switch(calculationType){
		case 1:{
			gc.createGraphs(gCUTOrder,false,sMasterCluster,sInitialSiteList);
			break;
		}
		case 2:{
			gc.createGraphs(gCUTOrder,true,sMasterCluster,sInitialSiteList);
			break;
		}
		case 3:{
			gc.calculateGraphSubtractions(gCUTOrder,sGraphList);
			break;
		}
		case 4:{
			gc.calculateGraphEmbeddings(gCUTOrder,sGraphList,sMasterCluster,sInitialSiteList);
			break;
		}
		case 5:{
			gc.performGraphSubtractionsAndEmbeddings(gCUTOrder,sGraphList,sInitialSiteList,sSubtractionList,sSpinModelFormList,sEffSpinModelList,sMasterClusterLg);
			break;
		}
		case 6:{
			gc.createMasterCluster(xMax,yMax,n,sMasterCluster,sGnuplot,sPlotPoints);
			break;
		}
		case 7:{
			gc.createEDCluster(gCUTOrder,sInterUnitCellVectors,sIntraUnitCellVectors,xMax,yMax,sEffSpinModelTDL);
			break;
		}
		case 8:{
			gc.createEDClusterInDifferentUCBasis(gCUTOrder,sInterUnitCellVectorsOld,sIntraUnitCellVectorsOld,sInterUnitCellVectors,sIntraUnitCellVectors,xMax,yMax,sEffSpinModelTDL,sReplaceCouplings);
			break;
		}
		case 9:{
			gc.createEDClusterForGeneralClusterForm(gCUTOrder,sInterUnitCellVectorsOld,sIntraUnitCellVectorsOld,sSiteBoundaries,sPeriodicBoundaries,sEffSpinModelTDL,sReplaceCouplings,sClusterName);
			break;
		}
		case 10:{
			gc.createHubbardEDCluster(sInterUnitCellVectors,sIntraUnitCellVectors,xMax,yMax);
			break;
		}
		case 11:{
			gc.createSiteCombinationsWithEdgeSpins(gCUTOrder,sGraphList,sMasterCluster,sInitialSiteList);
			break;
		}
	}

	// processing time //
    cout << "Processing Time: ";
	printProcessingTime(startingTime);


	return 0;
}
