/*
 * Graph.h
 *
 *  Created on: May 2, 2014
 *      Author: ixert
 */

#ifndef GRAPH_H_
#define GRAPH_H_

#include<map>
#include<set>
#include<list>
#include<vector>
#include<utility>
#include<gmpxx.h>
#include<gmp.h>

using namespace std;

class Graph {
	friend ostream& operator<<(ostream &o, Graph const &g);
	friend istream& operator>>(istream &o, Graph &g);

	//friend void outputWithLinkCouplings(ostream &o, Graph const &g);

	friend void dotGraphOutput(ostream &o, Graph const &g, unsigned int const &order, unsigned int const &number);

	friend bool operator==(Graph const& lhs, Graph const& rhs);
	// saves the relation between different sites (links) => Bondfile
public:
	// site -> {(coupled site 1, from), (coupled site 2, from), ...}
	// short (as an integer type!) defines coupling
	// 0 is normal coupling (nearest neighbors)
	// 1 is directed coupling from site to coupled site (next nearest neighbors)
	// 2 is directed coupling from coupled site to site (next nearest neighbors)
	map<unsigned short,set<pair<unsigned short,short> > > adj;

	mpz_class graphNumber;
	mpz_class canonicalGraphNumber;

	mpz_class linkGraphNumber;

	map<pair<unsigned short,unsigned short>,set<pair<unsigned short,unsigned short> > > linkAdj;
	// save which links (in the format (site1,site2), with site1<site2) are coupled to other links
	// works only for 1 couplingType at the moment and only for undirected couplings.

	list<pair<unsigned short,unsigned short> > originalKagomeLinks;

	int symmetryNumber;

	void calculateGraphNumber();
	void calculateCanonicalGraphNumber();

	void calculateSymmetryNumber();

	map<unsigned short,unsigned short> calculateDistanceToAllNodes(unsigned short const &startNode);
public:
	Graph();

	void insert(unsigned short const &site,set<pair<unsigned short,short> > const &coupledSites);

	void addLinkCoupling(pair<unsigned short,unsigned short> const &link,pair<unsigned short,unsigned short> const &coupledLink);

	void addOriginalKagomeLink(pair<unsigned short,unsigned short> const &link);

	void addLink(unsigned short const &site1, unsigned short const &site2, short const &coupling);

	mpz_class getGraphNumber();
	mpz_class getCanonicalGraphNumber();
	mpz_class getLinkGraphNumber();

	int getSymmetryNumber();

	vector<Graph> removeLinks();
	vector<Graph> removeLinks(vector<unsigned short> const &refSites);

	vector<unsigned short> identifySites(Graph const &refGraph, vector<unsigned short> const &refSites);

	vector<vector<unsigned short> > getAllNSiteCombinations(unsigned short const &N);

	vector<pair<unsigned int,vector<unsigned short> > > calculateEmbeddingsOnThisCluster(vector<Graph> &allGraphs, unsigned short const &initialSite);

	void embedGraphOnThisCluster(Graph &refGraph, unsigned short const &initialSite, list<vector<unsigned short> > &graphEmbeddings);

	unsigned int gCUTOrder();

	unsigned int sumOfLinks() const;
	vector<unsigned int> sumOfBondTypes();

	vector<unsigned short> getAllSites();
	unsigned short getFirstSite();

	unsigned short getMaxDistance();

	bool isPCUTGraph(unsigned short const &pCUTOrder);

	bool isConnected();

	void numberOfEdgeSpins(vector<unsigned short> const &sites, vector<vector<unsigned short> > &edgeSpins);

	bool isStrongSubgraph(Graph const &refGraph);

	void assignSites(Graph &newGraph, vector<unsigned short> const &newSites);

	void writeSiteCombinations(ostream &o);

	Graph convertToKagomeGraph() const;
	Graph convertToTriangularGraph() const;
};

bool operator!=(Graph const& lhs, Graph const& rhs);

#endif /* GRAPH_H_ */
