/*
 * GraphController.cpp
 *
 *  Created on: Jul 1, 2014
 *      Author: ixert
 */

#include "GraphController.h"
#include "Graph.h"
#include "LatticeGraph.h"
#include<sstream>
#include<fstream>
#include<iostream>
#include "SpinModelClass.h"
#include "SpinModelTDLClass.h"
#include "pairFunctions.h"

using namespace std;

GraphController::GraphController(string const &sInverseCouplingsFile, string const &sLatticeFile){
	string tempString;
	char tempChar;
	int x,y;
	unsigned short index=0;
	// read inverseCouplingsFile
	if(sInverseCouplingsFile!=""){
		fstream fsInverseCouplingsFile;
		fsInverseCouplingsFile.open(sInverseCouplingsFile.c_str(),fstream::in);
		getline(fsInverseCouplingsFile,tempString);
		while(fsInverseCouplingsFile>>x){
			fsInverseCouplingsFile >> y;
			inverseCouplings.insert(pair<short,short>(x,y));
			couplingIndex.insert(pair<short,unsigned short>(x,index++));
		}
		fsInverseCouplingsFile.close();
	}

	// read latticeFile
	if(sLatticeFile!=""){
		fstream fsLatticeFile;
		fsLatticeFile.open(sLatticeFile.c_str(),fstream::in);
		getline(fsLatticeFile,tempString);
		while(fsLatticeFile>>tempChar){
			if(tempChar>='A' && tempChar<='Z'){
				vector<int> tempVec(5);
				tempVec[0]=tempChar-'A';
				fsLatticeFile >> x;
				tempVec[1]=x;
				fsLatticeFile >> y;
				tempVec[2]=y;
				fsLatticeFile >> tempChar;
				tempVec[3]=tempChar-'A';
				fsLatticeFile >> x;
				tempVec[4]=x;
				latticeVectors.push_back(tempVec);
			}
		}
		fsLatticeFile.close();
	}

	//test output of latticeVectors
	cout << "latticeVectors:\n";
	for(unsigned int i=0;i<latticeVectors.size();i++){
		for(unsigned int j=0;j<latticeVectors[i].size();j++){
			cout << latticeVectors[i][j] << " ";
		}
		cout << "\n";
	}

	//test output of inverseCouplings
	cout << "inverseCouplings:\n";
	map<short,short>::const_iterator it;
	for(it=inverseCouplings.begin();it!=inverseCouplings.end();it++){
		cout << it->first << " " << it->second << "\n";
	}

	//test output of couplingIndex
	cout << "couplingIndex:\n";
	map<short,unsigned short>::const_iterator it2;
	for(it2=couplingIndex.begin();it2!=couplingIndex.end();it2++){
		cout << it2->first << " " << it2->second << "\n";
	}
}

void GraphController::createGraphs(unsigned int const &gCUTOrder, bool const &pCUTGraphs){
	LatticeGraph lg;
	vector<LatticeGraph> vlg=lg.createNewLatticeGraphs(latticeVectors);

	unsigned int creationIndex=0;
	while(creationIndex<vlg.size()){
		cout << creationIndex << "/" << vlg.size() << "\n";
		if(!pCUTGraphs || ((vlg[creationIndex].sumOfLinks()/2)<gCUTOrder)){
			vector<LatticeGraph> tempVec=vlg[creationIndex].createNewLatticeGraphs(latticeVectors);
			for(unsigned int k1=0;k1<tempVec.size();k1++){
				//cout << "k1=" << k1 << "\n";
				//cout << tempVec[k1] << "\n";;
				if(tempVec[k1].gCUTOrder()<=gCUTOrder){
					bool foundGraph=false;
					for(unsigned int k2=0;k2<vlg.size();k2++){
						if(tempVec[k1]==vlg[k2]){
							foundGraph=true;
							break;
						}
					}
					if(!foundGraph){
						vlg.push_back(tempVec[k1]);
					}
				}
			}
		}
		creationIndex++;
	}

	// test output
	/*
	cout << "output of all graphs:\n";
	for(unsigned int i=0;i<vlg.size();i++){
		cout << "Order " << vlg[i].gCUTOrder() << "\n";
		cout << vlg[i] << "\n";
	}
	*/

	//// now delete topological equivalent graphs
	// step 1: create Graph from LatticeGraph
	vector<vector<Graph> > vvg(gCUTOrder-1);
	//cout << "vvg.size()=" << vvg.size() << "\n";
	for(unsigned int i=0;i<vlg.size();i++){
		cout << "Graph " << i << "\n";
		vvg[vlg[i].gCUTOrder()-2].push_back(vlg[i].createGraph());
		cout << vvg[vlg[i].gCUTOrder()-2].back() << "\n";
		//cout << "vvg[i].size()=" << vvg[i].size() << "\n";
	}

	// step 2: compare canonicalGraphNumbers and only use one of them
	// this step also checks if pCUTGraphs are required if the graph is necessary for pCUT
	for(unsigned int i=0;i<vvg.size();i++){
		cout << "i=" << i << "\n";
		vector<Graph> vg;
		for(unsigned int j=0;j<vvg[i].size();j++){
			// check if this graph is pCUTGraph
			bool isInvalidGraph=false;
			if(pCUTGraphs){
				cout << "o" << i+2 << "g" << j << "\n";
				isInvalidGraph=!(vvg[i][j].isPCUTGraph(gCUTOrder));
				cout << "isInvalidGraph=" << isInvalidGraph << "\n";
			}
			if(!isInvalidGraph){
				cout << " j=" << j << "\n";
				cout << " graphNumber=" << vvg[i][j].getCanonicalGraphNumber() <<  "\n";
				cout << "Graph:\n";
				cout << vvg[i][j] << "\n";
				for(unsigned int k=0;k<vg.size();k++){
					cout << "  k=" << k << "\n";
					cout << "  graphNumber=" << vg[k].getCanonicalGraphNumber() << "\n";
					cout << "  Graph:\n";
					cout << vg[k] << "\n";
					if(vvg[i][j].getCanonicalGraphNumber()==vg[k].getCanonicalGraphNumber()){
						if(vvg[i][j].getLinkGraphNumber()==vg[k].getLinkGraphNumber()){
							isInvalidGraph=true;
							break;
						}
					}
				}
				if(!isInvalidGraph){
					vg.push_back(vvg[i][j]);
					cout << "pushed back!\n";
				}
			}
		}
		cout << "vg.size()=" << vg.size() << "\n";
		vvg[i]=vg;
		cout << "vvg[i.size()=" << vvg[i].size() << "\n";
	}


	// test output

	cout << "output of topological inequivalent graphs:\n";
	for(unsigned int i=0;i<vvg.size();i++){
		cout << "Order " << 2+i << "\n";
		for(unsigned int j=0;j<vvg[i].size();j++){
			cout << "Graph " << j+1 << "\n";
			//cout << vvg[i][j] << "\n";
			//cout << "maxDistance=" << vvg[i][j].getMaxDistance() << "\n";
			cout << "canonicalGraphNumber=" << vvg[i][j].getCanonicalGraphNumber() << "\n";
			cout << "linkGraphNumber=" << vvg[i][j].getLinkGraphNumber() << "\n";
			cout << "calculateCanonicalGraphNumnber:\n";
			vvg[i][j].calculateCanonicalGraphNumber();
		}
	}

	//TODO: sort graphs by their sum of links
	// if the sorting is done in this step, then it is not necessary to sort the graphs in the subtraction step

	// save all graphs as BondFiles
	stringstream ssGraphList;
	ssGraphList << "graphList_Kagome_o" << gCUTOrder << ".cfg";
	fstream fsGraphList;
	fsGraphList.open(ssGraphList.str().c_str(), fstream::out);
	fsGraphList.close();

	for(unsigned int i=0;i<vvg.size();i++){
		for(unsigned int j=0;j<vvg[i].size();j++){
			/*
			stringstream ssBondFile;
			ssBondFile << "BondFile_KaneMele_o" << i+2 << "g" << j+1 << ".cfg";
			fstream fsBondFile;
			fsBondFile.open(ssBondFile.str().c_str(),fstream::out);
			fsBondFile << "# This is a bond file.\n";
			fsBondFile << "# The number of lattice sites.\n";
			fsBondFile << vvg[i][j].gCUTOrder() << "\n\n";
			fsBondFile << "# The number of bits that should be reserved within a\n";
			fsBondFile << "# BaseState object for storing extra information.\n";
			fsBondFile << "0\n\n";
			fsBondFile << "# Now the list of bonds.\n";
			fsBondFile << "# Format:\n";
			fsBondFile << "# PerturbationOperatorNumber LaticeSiteNumbers;\n";
			fsBondFile << vvg[i][j];
			fsBondFile.close();
			*/

			Graph kagomeGraph=vvg[i][j].convertToKagomeGraph();

			stringstream ssGraphFile;
			ssGraphFile << "GraphFile_Kagome_o" << i+2 << "g" << j+1 << ".cfg";
			fstream fsGraphFile;
			fsGraphFile.open(ssGraphFile.str().c_str(),fstream::out);
			fsGraphFile << kagomeGraph;
			fsGraphFile.close();

			fsGraphList.open(ssGraphList.str().c_str(), fstream::out | fstream::app);
			fsGraphList << i+2  << " " << j+1 << " " << ssGraphFile.str() << "\n";
			fsGraphList.close();

			stringstream ssDotFile;
			ssDotFile << "DotFile_Kagome_o" << i+2 << "g" << j+1 << ".gv";
			fstream fsDotFile;
			fsDotFile.open(ssDotFile.str().c_str(),fstream::out);
			dotGraphOutput(fsDotFile,kagomeGraph,i+2,j+1);
			fsDotFile.close();

			/*
			stringstream ssGraphCouplingFile;
			ssGraphCouplingFile << "GraphCouplingFile_Kagome_o" << i+2 << "g" << j+1 << ".cfg";
			fstream fsGraphCouplingFile;
			fsGraphCouplingFile.open(ssGraphCouplingFile.str().c_str(),fstream::out);
			fsGraphCouplingFile<<vvg[i][j].convertToKagomeGraph();
			fsGraphCouplingFile.close();
			*/

		}
	}
}

void GraphController::calculateGraphSubtractions(unsigned int const &gCUTOrder, string const &sGraphList){
	vector<Graph> allGraphs;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	// load graphs
	vector<string> sGraphFiles;
	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(!fsGraphList.eof()){
		unsigned int order=0;
		unsigned int graphNumber=0;
		string tempstring;
		fsGraphList>>order;
		fsGraphList>>graphNumber;
		fsGraphList>>tempstring;
		if(tempstring!=""){
			sGraphFiles.push_back(tempstring);
			graphNumbers.push_back(pair<unsigned int,unsigned int>(order,graphNumber));
		}
	}
	fsGraphList.close();

	/*
	cout << "graphNumbers test:\n";
	cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
	for(unsigned int i=0;i<graphNumbers.size();i++){
		cout << graphNumbers[i].first << " " << graphNumbers[i].second << "\n";
	}
	*/

	// load graphList and push graphs from graphList to allGraphs
	for(unsigned int i=0;i<sGraphFiles.size();i++){
		//cout << sGraphFiles[i] << "\n";
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphFiles[i].c_str(),fstream::in);
		Graph g;
		//cout << "going to load graph...\n";
		fsGraphFiles >> g;
		//cout << "test output:\n";
		//cout << g << "\n";
		cout << "o" << graphNumbers[i].first << "g" << graphNumbers[i].second << "\n";
		allGraphs.push_back(g.convertToTriangularGraph());
		fsGraphFiles.close();
	}

	// sort them by order and sum of links
	// 25.07.2014: graphNumbers must also switches in the same way! Now it is added
	bool switchedsomething=false;
	for(unsigned int j=0;j<allGraphs.size();j++){
		unsigned int minimalOrder=allGraphs[j].gCUTOrder();
		unsigned int minimalSum=allGraphs[j].sumOfLinks();
		unsigned int minIndex=j;
		for(unsigned int i=j+1;i<allGraphs.size();i++){
			unsigned int order=allGraphs[i].gCUTOrder();
			if(order<minimalOrder){
				minIndex=i;
				minimalOrder=order;
				minimalSum=allGraphs[i].sumOfLinks();
			}
			else{
				if(minimalOrder==order){
					unsigned int sum=allGraphs[i].sumOfLinks();
					if(sum<minimalSum){
						minIndex=i;
						minimalSum=sum;
					}
				}
			}
		}
		if(minIndex!=j){
			Graph tempGraph=allGraphs[j];
			allGraphs[j]=allGraphs[minIndex];
			allGraphs[minIndex]=tempGraph;

			pair<unsigned int,unsigned int> tempGraphNumbers=graphNumbers[j];
			graphNumbers[j]=graphNumbers[minIndex];
			graphNumbers[minIndex]=tempGraphNumbers;

			switchedsomething=true;
		}
	}

	/*
	cout << "graphNumbers test:\n";
	cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
	for(unsigned int i=0;i<graphNumbers.size();i++){
		cout << graphNumbers[i].first << " " << graphNumbers[i].second << "\n";
	}
	*/

	vector<vector<int> > embeddingFactors(allGraphs.size(),vector<int>(allGraphs.size(),0));
	// Step 1: remove links to create subgraphs
	// Step 2: check if created graphs are defined in the graphList
	// Step 2a: if it is defined: add this as a subgraph
	// Step 2b: if it is not defined: remove further links until 2a (or graph is empty, this is just an artificial exit condition)
	// Step 3: proceed with next graph
	for(unsigned int i=0;i<allGraphs.size();i++){
		cout << "allGraphs[" << i << "]:\n";
		cout << allGraphs[i] << "\n";
		vector<Graph> subgraphs=allGraphs[i].removeLinks();
		cout << "subgraphs.size()=" << subgraphs.size() << "\n";
		for(unsigned int j=0;j<subgraphs.size();j++){
			cout << "subgraphs[" << j << "]:\n";
			cout << "gCUTOrder="<< subgraphs[j].gCUTOrder() << "\n";
			cout << subgraphs[j] << "\n";

			for(unsigned int k=0;k<allGraphs.size();k++){
				cout << "k=" << k << "\n";
				// erst ueberpruefen ob Anzahl Sites und Anzahl bondTypes gleich ist, sonst koennen die Graphen auch nicht gleich sein
				if(subgraphs[j].gCUTOrder()==allGraphs[k].gCUTOrder()){
					if(subgraphs[j].sumOfBondTypes()==allGraphs[k].sumOfBondTypes()){
						if(subgraphs[j].getCanonicalGraphNumber()==allGraphs[k].getCanonicalGraphNumber()){
							if(subgraphs[j].getLinkGraphNumber()==allGraphs[k].getLinkGraphNumber()){
								cout << "foundSubgraph!\n";
								embeddingFactors[i][k]--;
								break;
							}
						}
					}
				}
			}


			// remove additional links
			cout << "remove additional links\n";
			vector<Graph> subgraphs2=subgraphs[j].removeLinks();
			cout << "subgraphs2.size()=" << subgraphs2.size() << "\n";
			for(unsigned int k=0;k<subgraphs2.size();k++){
				cout << "subgraphs2[" << k << "]:\n";
				cout << subgraphs2[k] << "\n";

				// push them back to subgraphs if this graph does not exist already in subgraphs!
				bool foundGraph=false;
				for(unsigned int kk=0;kk<subgraphs.size();kk++){
					if(subgraphs[kk]==subgraphs2[k]){
						foundGraph=true;
						break;
					}
				}

				if(!foundGraph){
					subgraphs.push_back(subgraphs2[k]);
				}
			}
		}
	}

	cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
	cout << "allGraphs.size()=" << allGraphs.size() << "\n";
	cout << "embeddingFactors.size()=" << embeddingFactors.size() << "\n";

	stringstream ssCouplingEmbeddingFactors;
	ssCouplingEmbeddingFactors << "subgraphEmbeddings_KaneMele_o" << gCUTOrder << ".cfg";
	fstream fsCouplingEmbeddingFactors;
	fsCouplingEmbeddingFactors.open(ssCouplingEmbeddingFactors.str().c_str(),fstream::out);
	fsCouplingEmbeddingFactors << "#Order(g1) GraphNumber(g1) Sites(g1) Order(g2) GraphNumber(g2) Sites(g2) EmbeddingFactor\n";
	for(unsigned int i=0;i<embeddingFactors.size();i++){
		for(unsigned int j=0;j<embeddingFactors[i].size();j++){
			if(embeddingFactors[i][j]!=0){
				// this is the groundState Subtraction!
				// the corresponding site is just a single site! this is the firstSite of the graph (should always be zero)
				cout << "i=" << i << "j=" << j << "\n";
				cout << graphNumbers[i].first << " " << graphNumbers[i].second << " " << allGraphs[i].getFirstSite() << " " << graphNumbers[j].first << " " << graphNumbers[j].second << " " << allGraphs[j].getFirstSite() << " " << embeddingFactors[i][j] << "\n";
				fsCouplingEmbeddingFactors << graphNumbers[i].first << " " << graphNumbers[i].second << " " << allGraphs[i].getFirstSite() << " " << graphNumbers[j].first << " " << graphNumbers[j].second << " " << allGraphs[j].getFirstSite() << " " << embeddingFactors[i][j] << "\n";
			}
		}
	}
	fsCouplingEmbeddingFactors.close();

	/*
	 not necessary for gsKagome
	// now calculate all combinations of 2*n sites for every graph
	for(unsigned int j=0;j<allGraphs.size();j++){
		cout << "j=" << j << "\n";
		for(unsigned int s=2;s<=allGraphs[j].gCUTOrder();s+=2){
			cout << " s=" << s << "\n";
			//vector<int> embeddingFactorsCouplings(allGraphs.size(),0);
			//vector<vector<unsigned short> > correspondingSites(allGraphs.size());

			vector<vector<unsigned short> > allCombinations=allGraphs[j].getAllNSiteCombinations(s);
			cout << "combinationSize=" << allCombinations.size() << "\n";
			for(unsigned int c=0;c<allCombinations.size();c++){
				map<vector<unsigned short>,int> couplingEmbeddings;  //IMPORTANT: The First Element in the vector is the graphNumber!

				cout << "  c=" << c << "\n";
				//vector<Graph> subgraphs=allGraphs[j].removeLinks();
				vector<Graph> subgraphs=allGraphs[j].removeLinks(allCombinations[c]);
				cout << "  subgraphs.size()=" << subgraphs.size() << "\n";
				for(unsigned int jSub=0;jSub<subgraphs.size();jSub++){
					cout << "   jSub=" << jSub << "\n";
					cout << "subgraph:\n";
					cout << subgraphs[jSub] << "\n";
					for(unsigned int k=0;k<allGraphs.size();k++){
						cout << "    k=" << k << "\n";
						// check if graph could be embedded with respect to the specified sites!
						if(subgraphs[jSub].getCanonicalGraphNumber()==allGraphs[k].getCanonicalGraphNumber()){
							// if both graphs are topologically equivalent: identify the sites!
							cout << "    found Graph!\n";

							vector<unsigned short> identifiedSites=allGraphs[k].identifySites(subgraphs[jSub],allCombinations[c]);
							vector<unsigned short> tempVec;
							tempVec.push_back(k);
							cout << "identifiedSites=";
							for(unsigned int jc=0;jc<identifiedSites.size();jc++){
								tempVec.push_back(identifiedSites[jc]);
								cout << identifiedSites[jc] << " ";
							}
							cout << "\n";
							cout << "test1\n";
							pair<map<vector<unsigned short>,int>::iterator,bool> ret;
							ret=couplingEmbeddings.insert(pair<vector<unsigned short>,int>(tempVec,-1));
							if(!ret.second){
								ret.first->second--;
							}
							cout << "test2\n";

							break;
						}
					}

					// remove additional links
					vector<Graph> subgraphs2=subgraphs[jSub].removeLinks(allCombinations[c]);
					for(unsigned int k=0;k<subgraphs2.size();k++){
						// push them back to subgraphs if this graph does not exist already in subgraphs!
						bool foundGraph=false;
						for(unsigned int kk=0;kk<subgraphs.size();kk++){
							if(subgraphs[kk]==subgraphs2[k]){
								foundGraph=true;
								break;
							}
						}
						if(!foundGraph){
							subgraphs.push_back(subgraphs2[k]);
						}
					}
					cout << "test3\n";
				}
				fsCouplingEmbeddingFactors.open(ssCouplingEmbeddingFactors.str().c_str(),fstream::out | fstream::app);
				map<vector<unsigned short>,int>::iterator it;
				for(it=couplingEmbeddings.begin();it!=couplingEmbeddings.end();it++){
					fsCouplingEmbeddingFactors << graphNumbers[j].first << " " << graphNumbers[j].second; // order and number of the graph
					for(unsigned int k=0;k<allCombinations[c].size();k++){
						fsCouplingEmbeddingFactors << " " << allCombinations[c][k]; // lattice sites of the coupling on the graph
					}
					fsCouplingEmbeddingFactors  << " " << graphNumbers[it->first[0]].first << " " << graphNumbers[it->first[0]].second; // order and number of the subgraph
					for(unsigned int k=1;k<it->first.size();k++){
						fsCouplingEmbeddingFactors << " " << it->first[k]; // lattice sites of the coupling on the subgraph
					}
					fsCouplingEmbeddingFactors << " " << it->second << "\n"; // embedding factor
				}
				fsCouplingEmbeddingFactors.close();
			}
		}
	}
	*/
}

void GraphController::calculateGraphEmbeddings(unsigned int const &gCUTOrder, string const &sGraphList, string const &sMasterCluster, string const &sInitialSiteList){
	// read list-files
	vector<string> sGraphs;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	unsigned int x,y;
	string tempString;

	map<unsigned short,string> initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.insert(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();

	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(fsGraphList>>x){
		fsGraphList>>y;
		fsGraphList>>tempString;
		sGraphs.push_back(tempString);
		graphNumbers.push_back(pair<unsigned int,unsigned int>(x,y));
	}
	fsGraphList.close();

	// load Graphs
	vector<Graph> allGraphs;
	for(unsigned int i=0;i<sGraphs.size();i++){
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphs[i].c_str(),fstream::in);
		Graph g;
		fsGraphFiles >> g;
		allGraphs.push_back(g.convertToTriangularGraph());
		fsGraphFiles.close();
	}

	// load masterCluster
	Graph gMasterCluster;
	{
	Graph gTemp;
	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::in);
	fsMasterCluster >> gTemp;
	fsMasterCluster.close();
	gMasterCluster=gTemp.convertToTriangularGraph();
	}


	// calculate embeddings on masterCluster for all specified initialSites
	map<unsigned short,string>::const_iterator itInitialSites;
	for(itInitialSites=initialSites.begin();itInitialSites!=initialSites.end();itInitialSites++){
		vector<pair<unsigned int,vector<unsigned short> > > allEmbeddings=gMasterCluster.calculateEmbeddingsOnThisCluster(allGraphs,itInitialSites->first);
		// sort the embeddings
		for(unsigned int i=0;i<allEmbeddings.size();i++){
			unsigned int minGraphIndex=allEmbeddings[i].first;
			unsigned int minIndex=i;
			for(unsigned int j=i+1;j<allEmbeddings.size();j++){
				if(allEmbeddings[j].first<minGraphIndex){
					minGraphIndex=allEmbeddings[j].first;
					minIndex=j;
				}
			}
			if(minIndex!=i){
				pair<unsigned int,vector<unsigned short> > tempPair=allEmbeddings[i];
				allEmbeddings[i]=allEmbeddings[minIndex];
				allEmbeddings[minIndex]=tempPair;
			}
		}

		// save the embeddings
		fstream fsEmbeddings;
		fsEmbeddings.open(itInitialSites->second.c_str(),fstream::out);
		fsEmbeddings << "#order graphNumber sitesOnMasterCluster\n";
		for(unsigned int i=0;i<allEmbeddings.size();i++){
			unsigned int graphIndex=allEmbeddings[i].first;
			fsEmbeddings << graphNumbers[graphIndex].first << " " << graphNumbers[graphIndex].second;
			for(unsigned int k=0;k<allEmbeddings[i].second.size();k++){
				fsEmbeddings << " " << allEmbeddings[i].second[k];
			}
			fsEmbeddings << "\n";
		}
		fsEmbeddings.close();
	}
}

void GraphController::createMasterCluster(unsigned int const &xMax, unsigned int const &yMax, unsigned int const &n, string const &sMasterCluster, string const &sGnuplot, string const &sPlotPoints){
	LatticeGraph lg;
	lg.createRectangularLatticeCluster(latticeVectors,xMax,yMax,n);
	Graph g=lg.createGraph();

	stringstream ssMasterClusterLg;
	ssMasterClusterLg << sMasterCluster << "lg";
	fstream fsMasterClusterLg;
	fsMasterClusterLg.open(ssMasterClusterLg.str().c_str(),fstream::out);
	fsMasterClusterLg << lg;
	fsMasterClusterLg.close();

	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::out);
	fsMasterCluster << g.convertToKagomeGraph();
	fsMasterCluster.close();

	lg.createGraphPlot(sGnuplot,sPlotPoints);
}

void GraphController::createEDCluster(unsigned int const &gCUTOrder, string const &sInterUnitCellVectors, string const &sIntraUnitCellVectors, unsigned int const &v1Max, unsigned int const &v2Max, string const &sSpinModelTDLFull){
	// load latticeVectors
	vector<vector<int> > interUnitCellVectors;
	map<int,vector<int> > intraUnitCellVectors;

	string tempstring;
	int x;
	char tempChar;
	vector<int> tempVec;

	fstream fsInterUnitCellVectors;
	fsInterUnitCellVectors.open(sInterUnitCellVectors.c_str(),fstream::in);
	getline(fsInterUnitCellVectors,tempstring);
	while(fsInterUnitCellVectors>>x){
		tempVec.push_back(x);
		if(fsInterUnitCellVectors.peek()=='\n'){
			interUnitCellVectors.push_back(tempVec);
			tempVec.clear();
		}
	}
	fsInterUnitCellVectors.close();
	if(interUnitCellVectors.size()!=2){
		cout << "number of interUnitCellVectors must be 2!\n";
		return;
	}

	cout << "test output of interUnitCellVectors:\n";
	for(unsigned int i=0;i<interUnitCellVectors.size();i++){
		for(unsigned int j=0;j<interUnitCellVectors[i].size();j++){
			cout << interUnitCellVectors[i][j] << " ";
		}
		cout << "\n";
	}

	fstream fsIntraUnitCellVectors;
	fsIntraUnitCellVectors.open(sIntraUnitCellVectors.c_str(),fstream::in);
	getline(fsIntraUnitCellVectors,tempstring);
	while(fsIntraUnitCellVectors>>tempChar){
		if(tempChar>='A' && tempChar<='Z'){
			vector<int> tempVec(2);
			if(tempChar!='A'){
				cout << "first site in intraUnitCellVectors must be A!\n";
				return;
			}
			fsIntraUnitCellVectors >> x;
			tempVec[0]=x;
			fsIntraUnitCellVectors >> x;
			tempVec[1]=x;
			fsIntraUnitCellVectors >> tempChar;
			intraUnitCellVectors.insert(pair<int,vector<int> >(tempChar-'A',tempVec));
		}
	}
	fsIntraUnitCellVectors.close();

	cout << "test output of intraUnitCellVectors:\n";
	for(map<int,vector<int> >::iterator it1=intraUnitCellVectors.begin();it1!=intraUnitCellVectors.end();it1++){
		for(unsigned int i=0;i<it1->second.size();i++){
			cout << it1->second[i] << " ";
		}
		cout << it1->first << "\n";
	}


	// load spinModelsTDL
	// spinModelList should be shorted against the different siteTypes
	vector<list<SpinModelTDLClass> > spinModelList(intraUnitCellVectors.size());
	fstream fsSpinModelTDLFull;
	fsSpinModelTDLFull.open(sSpinModelTDLFull.c_str(),fstream::in);
	getline(fsSpinModelTDLFull,tempstring);
	SpinModelTDLClass tempSpinModelTDL;
	while(fsSpinModelTDLFull>>tempSpinModelTDL){
		cout << "test output of tempSpinModelTDL :\n";
		cout << tempSpinModelTDL;
		spinModelList[tempSpinModelTDL.getOriginType()].push_back(tempSpinModelTDL);
	}
	fsSpinModelTDLFull.close();

	// create BondFile by translating the latticeVectors
	vector<int> actualSite(2,0);
	unsigned short actualSiteIndex=0;
	map<vector<int>, unsigned short> siteMap;
	// initialize siteMap
	for(unsigned int v1=0;v1<=v1Max;v1++){
		for(unsigned int v2=0;v2<=v2Max;v2++){
			map<int,vector<int> >::iterator itIntra;
			for(itIntra=intraUnitCellVectors.begin();itIntra!=intraUnitCellVectors.end();itIntra++){
				actualSite[0]=v1*interUnitCellVectors[0][0]+v2*interUnitCellVectors[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectors[0][1]+v2*interUnitCellVectors[1][1]+itIntra->second[1];
				siteMap.insert(pair<vector<int>,unsigned short>(actualSite,actualSiteIndex++));
			}
		}
	}

	//SpinModelClass spinModelED;
	vector<SpinModelClass> referenceSpinModels; // the reference spinModel is the spinModel without assigning newSites, so that the sites are 0,1,2,3...
	map<vector<unsigned short>,unsigned int> siteCombinationsToReferenceSpinModel;

	for(unsigned int v1=0;v1<=v1Max;v1++){
		for(unsigned int v2=0;v2<=v2Max;v2++){
			// run over interUnitCells
			cout << "v1=" << v1 << " v2=" << v2 << "\n";
			map<int,vector<int> >::iterator itIntra;
			for(itIntra=intraUnitCellVectors.begin();itIntra!=intraUnitCellVectors.end();itIntra++){
				cout << " itIntra=" << itIntra->second[0] << " " << itIntra->second[1] << " " << char(itIntra->first+'A') << "\n";
				actualSite[0]=v1*interUnitCellVectors[0][0]+v2*interUnitCellVectors[1][0]+itIntra->second[0];
				actualSite[1]=v1*interUnitCellVectors[0][1]+v2*interUnitCellVectors[1][1]+itIntra->second[1];

				// run over all operators of the spinModels which have the origin itIntra->first.second
				list<SpinModelTDLClass>::iterator itList;
				for(itList=spinModelList[itIntra->first].begin();itList!=spinModelList[itIntra->first].end();itList++){
					SpinModelClass tempSpinModel=*itList;
					vector<vector<int> > siteVectors=itList->getSiteVectors();
					vector<unsigned short> newSites(siteVectors.size());
					for(unsigned int i=0;i<siteVectors.size();i++){
						cout << "  i=" << i << "\n";
						vector<int> tempVec(2,0);
						tempVec[0]=actualSite[0]+siteVectors[i][0];
						tempVec[1]=actualSite[1]+siteVectors[i][1];
						if((siteVectors[i][0]!=0) || (siteVectors[i][1]!=0)){
							map<int,vector<int> >::iterator itIntraMapFind;
							if(siteVectors[i][2]!=0){
								// if the siteType of the newVector is not the siteType A, then the composition into the interUnitCellVectors does not work.
								// therefore subtract the corresponding intraUnitCellVector
								itIntraMapFind=intraUnitCellVectors.find(siteVectors[i][2]);
								cout << "itIntraMap=" << itIntraMapFind->second[0] << " " << itIntraMapFind->second[1] << "\n";
								tempVec[0]-=itIntraMapFind->second[0];
								tempVec[1]-=itIntraMapFind->second[1];
							}
							cout << "tempVec(different siteTyp)=" << tempVec[0] << " " << tempVec[1] << "\n";
							// calculate vector-representation in interUnitCellBasis to find the corresponding unitCell
							int a1=interUnitCellVectors[0][0];
							int a2=interUnitCellVectors[0][1];
							int b1=interUnitCellVectors[1][0];
							int b2=interUnitCellVectors[1][1];
							int det=a1*b2-b1*a2;

							int xa=(b2*tempVec[0]-b1*tempVec[1])/det;
							int xb=(a1*tempVec[1]-a2*tempVec[0])/det;

							// check if xa or xb are out of bounds and shift them due to periodic boundary conditions
							if(xa<0){
								xa+=v1Max+1;
							}
							else{
								if(xa>v1Max){
									xa-=v1Max+1;
								}
							}
							if(xb<0){
								xb+=v2Max+1;
							}
							else{
								if(xb>v2Max){
									xb-=v2Max+1;
								}
							}

							// write the vector in the normal euclidean basis
							tempVec[0]=xa*interUnitCellVectors[0][0]+xb*interUnitCellVectors[1][0];
							tempVec[1]=xa*interUnitCellVectors[0][1]+xb*interUnitCellVectors[1][1];
							// add the intraUnitCellVector if it was previously subtracted
							if(siteVectors[i][2]!=0){
								tempVec[0]+=itIntraMapFind->second[0];
								tempVec[1]+=itIntraMapFind->second[1];
							}
						}
						cout << "  actualSite=" << actualSite[0] << " " << actualSite[1] << "\n";
						cout << "  siteVector=" << siteVectors[i][0] << " " << siteVectors[i][1] << " " << char(siteVectors[i][2]+'A') << "\n";
						cout << "  tempVec=" << tempVec[0] << " " << tempVec[1] << "\n";


						// now find the corresponding siteNumber to this vector from the siteMap
						map<vector<int>, unsigned short>::iterator itMapFind=siteMap.find(tempVec);
						if(itMapFind==siteMap.end()){
							cout << "error: couldn't find site in siteMap!\n";
							return;
						}
						cout << "  actualSiteIndex=" << itMapFind->second << "\n";
						newSites[i]=itMapFind->second;
					}

					// assign the new sites to the spinModel
					tempSpinModel.assignNewSites(newSites);

					// order against sites
					tempSpinModel.orderAgainstSites();

					// push sites to siteCombinations if this tempSpinModel not already exists
					bool existsAlready=false;
					map<vector<unsigned short>,unsigned int>::iterator itSiteComb;
					for(itSiteComb=siteCombinationsToReferenceSpinModel.begin();itSiteComb!=siteCombinationsToReferenceSpinModel.end();itSiteComb++){
						SpinModelClass tempSpinModel2=referenceSpinModels[itSiteComb->second];
						tempSpinModel2.assignNewSites(itSiteComb->first);
						tempSpinModel2.orderAgainstSites();
						if(tempSpinModel==tempSpinModel2){
							existsAlready=true;
							break;
						}
					}
					if(!existsAlready){
						SpinModelClass tempSpinModel3=*itList;
						unsigned int referenceIndex=referenceSpinModels.size();
						// if it not already exists, find out the referenceSpinModel
						for(unsigned int iRef=0;iRef<referenceSpinModels.size();iRef++){
							if(tempSpinModel3==referenceSpinModels[iRef]){
								referenceIndex=iRef;
								break;
							}
						}
						if(referenceIndex==referenceSpinModels.size()){
							referenceSpinModels.push_back(tempSpinModel3);
						}
						siteCombinationsToReferenceSpinModel.insert(pair<vector<unsigned short>,unsigned int>(newSites,referenceIndex));
					}

					//spinModelED+=tempSpinModel/tempSpinModel.numberOfOperators();
				}
			}
		}
	}


	// save siteMap
	stringstream ssSiteMap;
	ssSiteMap << "siteMapEDCluster_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSiteMap;
	fsSiteMap.open(ssSiteMap.str().c_str(),fstream::out);
	for(map<vector<int>,unsigned short>::iterator it1=siteMap.begin();it1!=siteMap.end();it1++){
		for(unsigned int i=0;i<it1->first.size();i++){
			fsSiteMap << it1->first[i] << " ";
		}
		fsSiteMap << "-> " << it1->second << "\n";
	}
	fsSiteMap.close();

	// save siteCombinations and reference SpinModels
	stringstream ssReferenceSpinModel;
	ssReferenceSpinModel << "effSpinModelReferenceTDL_o" << gCUTOrder << ".txt";
	fstream fsReferenceSpinModel;
	fsReferenceSpinModel.open(ssReferenceSpinModel.str().c_str(),fstream::out);
	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		fsReferenceSpinModel << i << "\n";
		fsReferenceSpinModel << referenceSpinModels[i] << "\n";
	}
	fsReferenceSpinModel.close();

	stringstream ssSiteCombinations;
	ssSiteCombinations << "siteCombinationsForReferenceSpinModel_o" << gCUTOrder << ".txt";
	fstream fsSiteCombinations;
	fsSiteCombinations.open(ssSiteCombinations.str().c_str(),fstream::out);
	fsSiteCombinations << "#referenceSpinModel site1 site2 ...\n";
	for(unsigned int j=0;j<referenceSpinModels.size();j++){
		map<vector<unsigned short>,unsigned int>::iterator itSiteComb;
		for(itSiteComb=siteCombinationsToReferenceSpinModel.begin();itSiteComb!=siteCombinationsToReferenceSpinModel.end();itSiteComb++){
			if(itSiteComb->second==j){
				fsSiteCombinations << itSiteComb->second;
				for(unsigned int i=0;i<itSiteComb->first.size();i++){
					fsSiteCombinations << " " << itSiteComb->first[i];
				}
				fsSiteCombinations << "\n";
			}
		}
	}
	fsSiteCombinations.close();

	/*
	// save spinModel
	stringstream ssSpinModelED;
	ssSpinModelED << "effSpinModelED_o" << gCUTOrder << "_" << v1Max << "x" << v2Max << ".txt";
	fstream fsSpinModelED;
	fsSpinModelED.open(ssSpinModelED.str().c_str(),fstream::out);
	fsSpinModelED << spinModelED << "\n";
	fsSpinModelED.close();
	*/
}

void GraphController::performGraphSubtractionsAndEmbeddings(unsigned int const &gCUTOrder, string const &sGraphList, string const &sInitialSiteList, string const &sSubtractionList, string const &sSpinModelFormList, string const &sEffSpinModelList, string const &sMasterClusterLg){
	// 1) read effSpinModels
	// a) read list-files
	vector<string> sGraphs;
	vector<string> sVecSpinModelForm;
	vector<string> sVecEffSpinModel;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	unsigned int x,y;
	string tempString;

	map<unsigned short,string> initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.insert(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();


	char tempChar;
	unsigned int index=0;


	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(fsGraphList>>x){
		fsGraphList>>y;
		fsGraphList>>tempString;
		sGraphs.push_back(tempString);
		graphNumbers.push_back(pair<unsigned int,unsigned int>(x,y));
	}
	fsGraphList.close();

	fstream fsSpinModelFormList;
	fsSpinModelFormList.open(sSpinModelFormList.c_str(),fstream::in);
	while(fsSpinModelFormList>>x){
		fsSpinModelFormList>>y;
		if(pair<unsigned int,unsigned int>(x,y)!=graphNumbers[index]){
			cout << "error! spinModelFormList is not ordered in the same manner as graphList!\n";
		}
		fsSpinModelFormList>>tempString;
		sVecSpinModelForm.push_back(tempString);
		index++;
	}
	fsSpinModelFormList.close();

	index=0;
	fstream fsEffSpinModelList;
	fsEffSpinModelList.open(sEffSpinModelList.c_str(),fstream::in);
	while(fsEffSpinModelList>>x){
		fsEffSpinModelList>>y;
		if(pair<unsigned int,unsigned int>(x,y)!=graphNumbers[index]){
			cout << "error! effSpinModelList is not ordered in the same manner as graphList!\n";
		}
		fsEffSpinModelList>>tempString;
		sVecEffSpinModel.push_back(tempString);
		index++;
	}
	fsEffSpinModelList.close();

	vector<SpinModelClass> vecSpinModels;
	for(unsigned int i=0;i<sVecSpinModelForm.size();i++){
		vector<SpinModelClass> vecSpinModels_singleOperators;
		// b) effSpinModelForm gives the general form of the operators
		// read this and create effSpinModels
		SpinModelClass tempSpinModel;
		fstream fsSpinModelForm;
		fsSpinModelForm.open(sVecSpinModelForm[i].c_str(),fstream::in);
		//first of all skip over the comment lines, and the number of couplings integer
		while(fsSpinModelForm.peek()=='#'){
			getline(fsSpinModelForm,tempString);
		}
		fsSpinModelForm >> x;
		fsSpinModelForm >> noskipws >> tempChar >> skipws;
		while(fsSpinModelForm.peek()=='#'){
			getline(fsSpinModelForm,tempString);
		}
		/////////////////////////////////////////

		while(fsSpinModelForm>>x){
			fsSpinModelForm>>y;
			fsSpinModelForm>>tempSpinModel;
			vecSpinModels_singleOperators.push_back(tempSpinModel);
		}
		fsSpinModelForm.close();

		// c) effSpinModel gives the corresponding sites to the operator
		SpinModelClass fullSpinModel;
		fstream fsEffSpinModel;
		fsEffSpinModel.open(sVecEffSpinModel[i].c_str(),fstream::in);
		while(fsEffSpinModel>>tempChar){
			fsEffSpinModel>>x;

			vector<unsigned short> vecSites;
			unsigned short z;
			for(unsigned int j=0;j<vecSpinModels_singleOperators[x].numberOfOperators();j++){
				fsEffSpinModel >> z;
				vecSites.push_back(z);
			}
			fsEffSpinModel >> noskipws >> tempChar >> skipws;
			pair<SeriesDouble,SeriesDouble> prefactor;
			fsEffSpinModel>>prefactor;
			//fsEffSpinModel.clear();
			SpinModelClass newSpinModel=vecSpinModels_singleOperators[x];
			newSpinModel.assignNewSites(vecSites);
			newSpinModel.orderAgainstSites();
			fullSpinModel+=prefactor*newSpinModel;
		}
		fsEffSpinModel.close();
		vecSpinModels.push_back(fullSpinModel);
	}

	// TEST OUTPUT
	cout << "test output of effSpinModel:\n";
	for(unsigned int i=0;i<vecSpinModels.size();i++){
		cout << graphNumbers[i].first << " " << graphNumbers[i].second << "\n";
		cout << " " << vecSpinModels[i] << "\n\n";
	}


	// load Graphs
	vector<Graph> allGraphs;
	for(unsigned int i=0;i<sGraphs.size();i++){
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphs[i].c_str(),fstream::in);
		Graph g;
		fsGraphFiles >> g;
		allGraphs.push_back(g);
		fsGraphFiles.close();
	}

	// 2) read subtractionList
	vector<vector<int> > vecSubtractions;
	fstream fsSubtractionList;
	fsSubtractionList.open(sSubtractionList.c_str(),fstream::in);
	int c,d,e;
	vector<int> tempVecInt;
	getline(fsSubtractionList,tempString);
	while(fsSubtractionList>>c){
		tempVecInt.push_back(c);
		if(fsSubtractionList.peek()=='\n'){
			vecSubtractions.push_back(tempVecInt);
			tempVecInt.clear();
		}
	}
	fsSubtractionList.close();

	cout << "vecSubtractions.size()=" << vecSubtractions.size() << "\n";

	// 3) calculate subtracted graphs
	//TODO: this should not be done! they should be embedded directly onto the lattice without purifying them!
	cout << "calulate subtracted graphs:\n";
	for(unsigned int i=0;i<vecSubtractions.size();i++){
		// push sites of g1 and g2 into single vectors
		unsigned int numberOfSites=(vecSubtractions[i].size()-5)/2;
		unsigned int order1=vecSubtractions[i][0];
		unsigned int graphNumber1=vecSubtractions[i][1];
		vector<unsigned short> sites1;
		vector<unsigned short> sites2;
		for(unsigned int j=2;j<=numberOfSites+1;j++){
			sites1.push_back(vecSubtractions[i][j]);
			sites2.push_back(vecSubtractions[i][j+numberOfSites+2]);
		}
		unsigned int order2=vecSubtractions[i][2+numberOfSites];
		unsigned int graphNumber2=vecSubtractions[i][3+numberOfSites];

		// create a sorted version of sites2
		// also create a permutation vector
		vector<unsigned short> sortedSites2=sites2;
		vector<unsigned short> permutationVector(sites2.size());
		for(unsigned int j=0;j<permutationVector.size();j++){
			permutationVector[j]=j;
		}
		for(unsigned int j=0;j<sortedSites2.size();j++){
			unsigned int minIndex=j;
			for(unsigned int k=j+1;k<sortedSites2.size();k++){
				if(sortedSites2[k]<sortedSites2[minIndex]){
					minIndex=k;
				}
			}
			if(minIndex!=j){
				unsigned short temp=sortedSites2[j];
				sortedSites2[j]=sortedSites2[minIndex];
				sortedSites2[minIndex]=temp;

				temp=permutationVector[j];
				permutationVector[j]=permutationVector[minIndex];
				permutationVector[minIndex]=temp;
			}
		}

		cout << "sites1=";
		for(unsigned int j=0;j<sites1.size();j++){
			cout << sites1[j] << " ";
		}
		cout << "\n";
		cout << "sites2=";
		for(unsigned int j=0;j<sites2.size();j++){
			cout << sites2[j] << " ";
		}
		cout << "\n";

		cout << "sortedSites2=";
		for(unsigned int j=0;j<sortedSites2.size();j++){
			cout << sortedSites2[j] << " ";
		}
		cout << "\n";
		cout << "permutationVector=";
		for(unsigned int j=0;j<permutationVector.size();j++){
			cout << permutationVector[j] << " ";
		}
		cout << "\n";


		// find index of the spinModels g1 and g2
		unsigned int index1=0;
		unsigned int index2=0;
		for(unsigned int j=0;j<graphNumbers.size();j++){
			if((graphNumbers[j].first==order1) && (graphNumbers[j].second==graphNumber1)){
				index1=j;
			}
			if((graphNumbers[j].first==order2) && (graphNumbers[j].second==graphNumber2)){
				index2=j;
			}
		}

		// get the part of the spinModel of g2 with the sortedSites2
		SpinModelClass sortedSites2SpinModel=vecSpinModels[index2].getOperatorsOnSpecifiedSites(sortedSites2);
		cout << "output of the sortedsites spinModel:\n";
		cout << sortedSites2SpinModel << "\n";

		// use assignNewSites() to apply the corresponding sites of g1 to g2!
		// Important: consider the ordering
		// so create the newSitesVector with considering the ordering:
		vector<unsigned short> newSites(sortedSites2.size());
		for(unsigned int j=0;j<permutationVector.size();j++){
			newSites[j]=sites1[permutationVector[j]];
		}
		sortedSites2SpinModel.assignNewSites(newSites);
		// order the operators against the sites
		sortedSites2SpinModel.orderAgainstSites();
		// use the embeddingFactor to add this spinModel to the spinModel of g1
		cout << "index1=" << index1 << "\n";
		cout << "order1=" << order1 << "\n";
		cout << "graphNumber1=" << graphNumber1 << "\n";
		cout << "index2=" << index2 << "\n";
		cout << "order2=" << order2 << "\n";
		cout << "graphNumber1=" << graphNumber1 << "\n";
		cout << "embeddingFactor=" << vecSubtractions[i][vecSubtractions[i].size()-1] << "\n";
		vecSpinModels[index1]+=vecSubtractions[i][vecSubtractions[i].size()-1]*sortedSites2SpinModel;
	}

	// 4) save subtracted effSpinModels
	for(unsigned int i=0;i<vecSpinModels.size();i++){
		stringstream ssEffSpinModelPure;
		ssEffSpinModelPure << "effSpinModelPure_o" << graphNumbers[i].first << "g" << graphNumbers[i].second << ".txt";
		fstream fsEffSpinModelPure;
		fsEffSpinModelPure.open(ssEffSpinModelPure.str().c_str(), fstream::out);
		fsEffSpinModelPure << vecSpinModels[i];
		fsEffSpinModelPure.close();
	}


	// now this is important for the ground-state energy:
	// up to now the ground state energy was usually NOT per site!
	// so, now divide the groundState energy through the numberOfSites
	// later one must set the site of the groundState Energy to the initialSite
	for(unsigned int i=0;i<vecSpinModels.size();i++){
		vecSpinModels[i].divideGroundStateEnergyBy(graphNumbers[i].first);
	}

	// 5) embed on masterCluster
	// load embeddings
	//TODO: this must be done for all initialSites separately

	vector<SpinModelClass> thermodynamicSpinModels(initialSites.size());
	unsigned int tdlIndex=0;
	map<unsigned short,string>::const_iterator itInitialSites;
	for(itInitialSites=initialSites.begin();itInitialSites!=initialSites.end();itInitialSites++){
		cout << "initialSites=" << itInitialSites->first << " " << itInitialSites->second << "\n";
		vector<pair<unsigned int,vector<unsigned short> > > allEmbeddings;
		fstream fsEmbeddingList;
		fsEmbeddingList.open(itInitialSites->second.c_str(),fstream::in);
		getline(fsEmbeddingList,tempString);
		while(fsEmbeddingList>>c){
			fsEmbeddingList>>d;
			unsigned int graphIndex=0;
			// find the index corresponding to order=c, graphNumber=d
			for(unsigned int i=0;i<graphNumbers.size();i++){
				if((graphNumbers[i].first==c) && (graphNumbers[i].second==d)){
					graphIndex=i;
					break;
				}
			}
			getline(fsEmbeddingList,tempString);
			stringstream ssTemp(tempString);
			vector<unsigned short> tempVecShort;
			while(ssTemp>>e){
				tempVecShort.push_back(e);
			}
			allEmbeddings.push_back(pair<unsigned int,vector<unsigned short> >(graphIndex,tempVecShort));
		}
		fsEmbeddingList.close();

		cout << "embed on masterCluster\n";
		cout << "vecSpinModels.size()=" << vecSpinModels.size() << "\n";
		cout << "graphNumbers.size()=" << graphNumbers.size() << "\n";
		cout << "allGraphs.size()=" << allGraphs.size() << "\n";

		// Now calcualte the thermodynamic spinModel
		for(unsigned int i=0;i<allEmbeddings.size();i++){ // run over all graphs
			unsigned int graphIndex=allEmbeddings[i].first;
			cout << "graphIndex=" << graphIndex << "\n";
			cout << "order=" << graphNumbers[graphIndex].first << " graphNumber=" << graphNumbers[graphIndex].second << "\n";

			// assign sites and add the spinModel
			vector<unsigned short> oldSites=allGraphs[graphIndex].getAllSites();
			SpinModelClass tempSpinModel=vecSpinModels[graphIndex];
			cout << "tempSpinModel=" << tempSpinModel << "\n";
			cout << "oldSites=";
			for(unsigned int j=0;j<oldSites.size();j++){
				cout << oldSites[j] << " ";
			}
			cout << "\n";
			cout << "newSites=";
			for(unsigned int j=0;j<allEmbeddings[i].second.size();j++){
				cout << allEmbeddings[i].second[j] << " ";
			}
			cout << "\n";
			tempSpinModel.assignNewSites(oldSites,allEmbeddings[i].second);
			cout << "tempSpinModel=" << tempSpinModel << "\n";
			tempSpinModel.orderAgainstSites();
			cout << "tempSpinModel=" << tempSpinModel << "\n";
			// now set the site of the groundState energy (the unity) to the initialSite
			tempSpinModel.setGroundStateSiteTo(itInitialSites->first);
			// now kick out all operator-terms which dont include the initialSite!
			tempSpinModel.deleteOperatorsWithoutSite(itInitialSites->first);

			thermodynamicSpinModels[tdlIndex]+=tempSpinModel;
		}

		// 6) save thermodynamic spinModel
		stringstream ssSpinModelTDL;
		ssSpinModelTDL << "effSpinModelTDL_o" << gCUTOrder << "_initialSite" << itInitialSites->first << ".txt";
		fstream fsSpinModelTDL;
		fsSpinModelTDL.open(ssSpinModelTDL.str().c_str(),fstream::out);
		fsSpinModelTDL << thermodynamicSpinModels[tdlIndex] << "\n";
		fsSpinModelTDL.close();
		tdlIndex++;
		cout << "tdlIndex=" << tdlIndex << "\n";
	}


	// 7) load masterCluster and save the spinModel with ascending sites, where only the sites that occur in the spinModel are numbered. So at N sites the occuring sites are numbered from 0 to N-1
	// initialSite is set to 0
	// additionally save the vectors which connect the different sites
	// must be performed for all thermodynamicSpinModels (i.e. for all initialSites)

	LatticeGraph masterClusterLg;

	fstream fsMasterClusterLg;
	fsMasterClusterLg.open(sMasterClusterLg.c_str(),fstream::in);
	fsMasterClusterLg >> masterClusterLg;
	fsMasterClusterLg.close();

	stringstream ssSpinModelTDLFull;
	ssSpinModelTDLFull << "effSpinModelTDLFull_o" << gCUTOrder << ".txt";
	fstream fsSpinModelTDLFull;
	fsSpinModelTDLFull.open(ssSpinModelTDLFull.str().c_str(),fstream::out);
	fsSpinModelTDLFull << "#siteType1 connectionVector1 siteType2 connectionVector2 ...\n";
	fsSpinModelTDLFull.close();

	vector<vector<int> > inverseSiteMap=masterClusterLg.getInverseSiteMap();
	tdlIndex=0;
	for(itInitialSites=initialSites.begin();itInitialSites!=initialSites.end();itInitialSites++){
		vector<int> initialSiteVector=inverseSiteMap[itInitialSites->first];

		cout << "allSiteCombinations\n";
		list<vector<unsigned short> > siteCombinations=thermodynamicSpinModels[tdlIndex].getAllSiteCombinations();

		list<vector<unsigned short> >::iterator itSiteComb;
		cout << "test output of siteCombinations:\n";
		for(itSiteComb=siteCombinations.begin();itSiteComb!=siteCombinations.end();itSiteComb++){
			for(unsigned int i=0;i<itSiteComb->size();i++){
				cout << (*itSiteComb)[i] << " ";
			}
			cout << "\n";
		}

		for(itSiteComb=siteCombinations.begin();itSiteComb!=siteCombinations.end();itSiteComb++){
			SpinModelClass siteCombSpinModel=thermodynamicSpinModels[tdlIndex].getOperatorsOnSpecifiedSites(*itSiteComb);
			siteCombSpinModel.assignNewSites();
			fsSpinModelTDLFull.open(ssSpinModelTDLFull.str().c_str(),fstream::out | fstream::app);
			for(unsigned int i=0;i<itSiteComb->size();i++){
				if(i!=0){
					fsSpinModelTDLFull << " ";
				}
				fsSpinModelTDLFull << char(inverseSiteMap[(*itSiteComb)[i]][2]+'A');
				fsSpinModelTDLFull << " " << inverseSiteMap[(*itSiteComb)[i]][0]-initialSiteVector[0];
				fsSpinModelTDLFull << " " << inverseSiteMap[(*itSiteComb)[i]][1]-initialSiteVector[1];
			}
			fsSpinModelTDLFull << "\n";
			fsSpinModelTDLFull << siteCombSpinModel << "\n";
			fsSpinModelTDLFull.close();
		}
		tdlIndex++;
	}
}

void GraphController::createSiteCombinationsWithEdgeSpins(unsigned int const &gCUTOrder, string const &sGraphList, string const &sMasterCluster, string const &sInitialSiteList){
	// this function creates all possible graphs with edge spins (from the already calculated graphs, embeddings and the masterCluster)
	// in order to have an unique edge spin configuration for each graph only WEAK embeddings are considered (i.e. that to the embedding of a specific graph only graphs are considered where all possible links between sites are drawn)

	// => siteCombination and siteMap files
	// => embedding files
	// => edgeSpinSubtractions file

	unsigned int x,y;
	int c,d,e;
	string tempString;

	// load masterCluster
	Graph gMasterCluster;
	{
	Graph gTemp;
	fstream fsMasterCluster;
	fsMasterCluster.open(sMasterCluster.c_str(),fstream::in);
	fsMasterCluster >> gTemp;
	fsMasterCluster.close();
	gMasterCluster=gTemp.convertToTriangularGraph();
	}

	// find maximum number of couplings
	// now, for each coupling there is a possible edge spin coupling (since the edge spins are stationary, one needs to have an edge spin even for the inverse couplings
	unsigned short numberOfCouplings=couplingIndex.size();
	unsigned short minEdgeSpinCoupling=0;
	map<short,short>::const_iterator itInvCoupl;
	for(itInvCoupl=inverseCouplings.begin();itInvCoupl!=inverseCouplings.end();++itInvCoupl){
		if(itInvCoupl->first<=itInvCoupl->second){
			minEdgeSpinCoupling++;
		}
	}


	// load all graphs (neccessary for creation of siteCombinations and siteMaps)
	vector<string> sGraphs;
	vector<pair<unsigned int,unsigned int> > graphNumbers;

	fstream fsGraphList;
	fsGraphList.open(sGraphList.c_str(),fstream::in);
	while(fsGraphList>>x){
		fsGraphList>>y;
		fsGraphList>>tempString;
		sGraphs.push_back(tempString);
		graphNumbers.push_back(pair<unsigned int,unsigned int>(x,y));
	}
	fsGraphList.close();

	vector<Graph> allGraphs;
	for(unsigned int i=0;i<sGraphs.size();i++){
		fstream fsGraphFiles;
		fsGraphFiles.open(sGraphs[i].c_str(),fstream::in);
		Graph g;
		fsGraphFiles >> g;
		allGraphs.push_back(g.convertToTriangularGraph());
		fsGraphFiles.close();
	}

	// edgeSpinGraphs
	// for each graph save the one edge spin configuration
	vector<vector<vector<unsigned short> > > edgeSpinConf(allGraphs.size()); // vector(Graphs)<vector(all sites of the graph)<vector(number of edge spins for each edgeSpin coupling of this site)> >

	// load initialSiteList
	vector<pair<unsigned short,string> > initialSites;
	fstream fsInitialSiteList;
	fsInitialSiteList.open(sInitialSiteList.c_str(),fstream::in);
	while(fsInitialSiteList>>x){
		fsInitialSiteList>>tempString;
		initialSites.push_back(pair<unsigned short,string>(x,tempString));
	}
	fsInitialSiteList.close();

	// run over all initial sites
	for(unsigned int initialSiteIndex=0;initialSiteIndex<initialSites.size();initialSiteIndex++){
		// load embeddings for this initialSite
		list<pair<unsigned int,vector<unsigned short> > > allEmbeddings;
		fstream fsEmbeddingList;
		fsEmbeddingList.open(initialSites[initialSiteIndex].second.c_str(),fstream::in);
		getline(fsEmbeddingList,tempString);
		while(fsEmbeddingList>>c){
			fsEmbeddingList>>d;

			unsigned int graphIndex=0;
			// find the index corresponding to order=c, graphNumber=d
			for(unsigned int i=0;i<graphNumbers.size();i++){
				if((graphNumbers[i].first==c) && (graphNumbers[i].second==d)){
					graphIndex=i;
					break;
				}
			}

			getline(fsEmbeddingList,tempString);
			stringstream ssTemp(tempString);
			vector<unsigned short> tempVecShort;
			while(ssTemp>>e){
				tempVecShort.push_back(e);
			}
			allEmbeddings.push_back(pair<unsigned int,vector<unsigned short> >(graphIndex,tempVecShort));
		}
		fsEmbeddingList.close();

		// run over all embeddings
		list<pair<unsigned int,vector<unsigned short> > >::iterator itAllEmb=allEmbeddings.begin();
		while(itAllEmb!=allEmbeddings.end()){
			unsigned int actualGraph=itAllEmb->first;
			// check if this embedding is a strong or weak embedding
			Graph assignedGraph;
			allGraphs[actualGraph].assignSites(assignedGraph,itAllEmb->second);
			if(assignedGraph.isStrongSubgraph(gMasterCluster)){
				// if it is a strong embedding, check if the edge spin configuration for this graph was already obtained
				if(edgeSpinConf[actualGraph].size()==0){
					// call method: numberOfEdgeSpins on masterCluster
					edgeSpinConf[actualGraph].resize(itAllEmb->second.size(),vector<unsigned short>(numberOfCouplings,0)); //initialize vector in which the number of edge spins is saved
					gMasterCluster.numberOfEdgeSpins(itAllEmb->second,edgeSpinConf[actualGraph]);
				}
				++itAllEmb;
			}
			else{
				// if it is not a strong embedding, delete this embedding from the list
				itAllEmb=allEmbeddings.erase(itAllEmb);
			}
		}

		// save embeddings with edgeSpins
		stringstream ssEdgeSpinEmbeddings;
		ssEdgeSpinEmbeddings << "edgeSpinEmbeddings_o" << gCUTOrder << "_initialSite" << initialSites[initialSiteIndex].first << ".cfg";
		fstream fsEdgeSpinEmbeddings;
		fsEdgeSpinEmbeddings.open(ssEdgeSpinEmbeddings.str().c_str(),fstream::out);
		fsEdgeSpinEmbeddings << "#order graphNumber sitesOnMasterCluster\n";
		for(itAllEmb=allEmbeddings.begin();itAllEmb!=allEmbeddings.end();++itAllEmb){
			fsEdgeSpinEmbeddings << graphNumbers[itAllEmb->first].first << " " << graphNumbers[itAllEmb->first].second;
			for(unsigned int j=0;j<itAllEmb->second.size();j++){
				fsEdgeSpinEmbeddings << " " << itAllEmb->second[j];
			}
			fsEdgeSpinEmbeddings << "\n";
		}
		fsEdgeSpinEmbeddings.close();
	}

	// calculate the edgeSubtractions (subtraction of subgraphs with edge Spins)
	// for each subgraph, check the edgeSpinConfiguration
	vector<vector<int> > subtractionFactors(allGraphs.size(),vector<int>(allGraphs.size(),0));

	// run over all graphs (which are strong embeddings)
	for(unsigned int i=0;i<allGraphs.size();i++){
		// check if this graph is a strong embedding (i.e. it has a defined edge spin configuration)
		if(edgeSpinConf[i].size()>0){
			// calculate ALL subgraphs, but increase the subtraction factor only if the subgraph is a strong embedding onto the original graph (BUT weak graphs must also be considered on the way, because by removing links these weak graphs can become strong)
			// Step 1: remove links to create subgraphs
			// Step 2: check if created graphs are defined in the graphList
			// Step 2a: if it is defined: add this as a subgraph
			// Step 2b: if it is not defined: remove further links until 2a (or graph is empty, this is just an artificial exit condition)
			// Step 3: proceed with next graph
			vector<Graph> subgraphs=allGraphs[i].removeLinks();
			cout << "allGraphs[" << i << "]:\n";
			cout << allGraphs[i] << "\n";
			cout << "subgraphs.size()=" << subgraphs.size() << "\n";
			for(unsigned int j=0;j<subgraphs.size();j++){
				// check if this is a strong graph
				if(subgraphs[j].isStrongSubgraph(allGraphs[i])){
					// if this is a strong graph, then find the index of this graph
					cout << "subgraphs[" << j << "]:\n";
					cout << subgraphs[j] << "\n";
					// get subGraphIndex
					for(unsigned int subGraphIndex=0;subGraphIndex<allGraphs.size();subGraphIndex++){
						if(subgraphs[j].getCanonicalGraphNumber()==allGraphs[subGraphIndex].getCanonicalGraphNumber()){
							if(subgraphs[j].getLinkGraphNumber()==allGraphs[subGraphIndex].getLinkGraphNumber()){
								// subgraph is found, now increase subtractionFactor
								subtractionFactors[i][subGraphIndex]--;
								break;
							}
						}
					}
				}

				// remove additional links
				vector<Graph> subgraphs2=subgraphs[j].removeLinks();
				for(unsigned int k=0;k<subgraphs2.size();k++){
					// push them back to subgraphs if this graph does not exist already in subgraphs!
					bool foundGraph=false;
					for(unsigned int kk=0;kk<subgraphs.size();kk++){
						if(subgraphs[kk]==subgraphs2[k]){
							foundGraph=true;
							break;
						}
					}
					if(!foundGraph){
						subgraphs.push_back(subgraphs2[k]);
					}
				}
			}
		}
	}

	// subtraction of edge spins and siteCombinations for each graph with edge spins
	// important: the method writeSiteCombinations assumes that each coupling belongs to one instance of an referenceSpinModel

	stringstream ssEdgeSpinSubtractions;
	ssEdgeSpinSubtractions << "edgeSpinSubtractions_o" << gCUTOrder << ".cfg";
	fstream fsEdgeSpinSubtractions;
	fsEdgeSpinSubtractions.open(ssEdgeSpinSubtractions.str().c_str(),fstream::out);
	fsEdgeSpinSubtractions << "#order graphNumber edgeSpinCoupling subtractionFactor\n";
	fsEdgeSpinSubtractions.close();

	for(unsigned int i=0;i<edgeSpinConf.size();i++){
		// now just use the summed number of edgeSpins (not for each site individually) for the edgeSpinSubtractions
		// save also the stuff within siteCombinations

		if(edgeSpinConf[i].size()>0){ //only save strong graphs
			stringstream ssSiteCombinations;
			ssSiteCombinations << "siteCombinations_o" << graphNumbers[i].first << "g" << graphNumbers[i].second << ".cfg";
			fstream fsSiteCombinations;
			fsSiteCombinations.open(ssSiteCombinations.str().c_str(),fstream::out);
			fsSiteCombinations << "#referenceSpinModel site1 site2 ...\n";
			Graph tempKagomeGraph=allGraphs[i].convertToKagomeGraph();

			/*
			cout << "triangle graph:\n";
			cout << allGraphs[i] << "\n";
			cout << "kagome graph:\n";
			cout << tempKagomeGraph << "\n\n";
			*/

			tempKagomeGraph.writeSiteCombinations(fsSiteCombinations);

			vector<int> summedEdgeSpins(numberOfCouplings,0);
			for(unsigned int j=0;j<edgeSpinConf[i].size();j++){
				for(unsigned int k=0;k<numberOfCouplings;k++){
					summedEdgeSpins[k]-=edgeSpinConf[i][j][k];
					for(unsigned int nn=0;nn<edgeSpinConf[i][j][k];nn++){
						fsSiteCombinations << minEdgeSpinCoupling+k << " " << 3*j << " " << 3*j+1 << " " << 3*j+2 << "\n";
					}
				}
			}
			fsSiteCombinations.close();

			fsEdgeSpinSubtractions.open(ssEdgeSpinSubtractions.str().c_str(),fstream::out | fstream::app);
			for(unsigned int k=0;k<numberOfCouplings;k++){
				fsEdgeSpinSubtractions << graphNumbers[i].first << " " << graphNumbers[i].second << " " << minEdgeSpinCoupling+k << " " << summedEdgeSpins[k] << "\n";
			}
			fsEdgeSpinSubtractions.close();
		}
	}

	// save subtraction factors
	stringstream ssEdgeSpinSubgraphEmbeddings;
	ssEdgeSpinSubgraphEmbeddings << "edgeSpinSubgraphEmbeddings_o" << gCUTOrder << ".cfg";
	fstream fsEdgeSpinSubgraphEmbeddings;
	fsEdgeSpinSubgraphEmbeddings.open(ssEdgeSpinSubgraphEmbeddings.str().c_str(),fstream::out);
	fsEdgeSpinSubgraphEmbeddings << "#Order(g1) GraphNumber(g1) Order(g2) GraphNumber(g2) EmbeddingFactor\n";
	for(unsigned int i=0;i<subtractionFactors.size();i++){
		pair<unsigned int,unsigned int> graphNumbers1=graphNumbers[i];
		for(unsigned int j=0;j<subtractionFactors.size();j++){
			if(subtractionFactors[i][j]!=0){
				pair<unsigned int,unsigned int> graphNumbers2=graphNumbers[j];
				fsEdgeSpinSubgraphEmbeddings << graphNumbers1.first << " " << graphNumbers1.second << " " << graphNumbers2.first << " " << graphNumbers2.second << " " << subtractionFactors[i][j] << "\n";
			}
		}
	}
	fsEdgeSpinSubgraphEmbeddings.close();

	// now save the remaining stuff
	// edgeSpinCouplings (which real coupling corresponds to the edgeSpinCoupling)
	stringstream ssEdgeSpinCouplings;
	ssEdgeSpinCouplings << "edgeSpinCouplings.cfg";
	fstream fsEdgeSpinCouplings;
	fsEdgeSpinCouplings.open(ssEdgeSpinCouplings.str().c_str(),fstream::out);
	fsEdgeSpinCouplings << "#edgeSpinCoupling latticeCouplings\n";
	map<short,unsigned short>::iterator itCouplInd;
	for(itCouplInd=couplingIndex.begin();itCouplInd!=couplingIndex.end();++itCouplInd){
		fsEdgeSpinCouplings << minEdgeSpinCoupling+itCouplInd->second << " " << itCouplInd->first << "\n";
	}
	fsEdgeSpinCouplings.close();
}

