/*
 * SpinModelClass.cpp
 *
 *  Created on: Jul 17, 2014
 *      Author: ixert
 */

#include "SpinModelClass.h"
#include "pairFunctions.h"

SpinModelClass::SpinModelClass(){
}

int SpinModelClass::numberOfOperators(){
	// returns the number of operators if the number of operators in each summand is the same, -1 otherwise
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it=spinModel.begin();
	int number=it->second.numberOfOperators();
	it++;
	while(it!=spinModel.end()){
		if(it->second.numberOfOperators()!=number){
			return -1;
		}
		it++;
	}
	return number;
}

void SpinModelClass::assignNewSites(){
	//assigns the new sites 0,1,2,...
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		it->second.assignNewSites();
	}
}
void SpinModelClass::assignNewSites(vector<unsigned short> const &newSites){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		it->second.assignNewSites(newSites);
	}
}
void SpinModelClass::assignNewSites(vector<unsigned short> const &oldSites, vector<unsigned short> const &newSites){
	// in contrast to assignNewSites(vector<unsigned short> const &newSites) this version sets the oldSites specified in oldSites to the newSites specified in newSites!
	// this method also works if the SpinModel consists of for example, two AND four spin operators
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		it->second.assignNewSites(oldSites,newSites);
	}
}
void SpinModelClass::orderAgainstSites(){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		it->second.orderAgainstSites();
	}
}

void SpinModelClass::divideGroundStateEnergyBy(Doub const &n){
	if(n!=0){
		list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
		for(it=spinModel.begin();it!=spinModel.end();it++){
			if(it->second.isUnity()){ // check if this operator is the unity, i.e. the gsEnergy Operator.
				it->first.first/=n;
				it->first.second/=n;
			}
		}
	}
	else{
		cerr << "Divison by zero!\n";
	}
}

void SpinModelClass::setGroundStateSiteTo(unsigned short const &initialSite){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		if(it->second.isUnity()){ // check if this operator is the unity, i.e. the gsEnergy Operator.
			vector<unsigned short> vus(1,initialSite);
			it->second.assignNewSites(vus);
		}
	}
}

list<vector<unsigned short> > SpinModelClass::getAllSiteCombinations(){
	list<vector<unsigned short> > siteCombinations;
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	list<vector<unsigned short> >::iterator itSiteComb;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		bool foundComb=false;
		vector<unsigned short> tempVec=it->second.getSites();
		for(itSiteComb=siteCombinations.begin();itSiteComb!=siteCombinations.end();itSiteComb++){
			if((*itSiteComb)==tempVec){
				foundComb=true;
				break;
			}
		}
		if(!foundComb){
			siteCombinations.push_back(tempVec);
		}
	}

	return siteCombinations;
}
SpinModelClass SpinModelClass::getOperatorsOnSpecifiedSites(vector<unsigned short> const &newSites){
	SpinModelClass newSpinModel;
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		if(it->second.areSameSites(newSites)){
			newSpinModel.spinModel.push_back(*it);
		}
	}
	return newSpinModel;
}
SpinModelClass SpinModelClass::getOperatorsWhichContainSite(unsigned short const &site){
	SpinModelClass newSpinModel;
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it;
	for(it=spinModel.begin();it!=spinModel.end();it++){
		if(it->second.containsSite(site)){
			newSpinModel.spinModel.push_back(*it);
		}
	}
	return newSpinModel;
}
void SpinModelClass::deleteOperatorsWithoutSite(unsigned short const &site){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it=spinModel.begin();
	while(it!=spinModel.end()){
		if(!(it->second.containsSite(site))){
			it=spinModel.erase(it);
		}
		else{
			it++;
		}
	}
}

SpinModelClass& SpinModelClass::operator+=(SpinModelClass const& rhs){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::const_iterator it1;
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it2;

	for(it1=rhs.spinModel.begin();it1!=rhs.spinModel.end();it1++){
		bool foundOperators=false;;
		for(it2=spinModel.begin();it2!=spinModel.end();it2++){
			if(it1->second==it2->second){
				it2->first+=it1->first;
				if(it2->first==pair<SeriesDouble,SeriesDouble>(SeriesDouble(0),SeriesDouble(0))){
					spinModel.erase(it2);
				}
				foundOperators=true;
				break;
			}
		}
		if(!foundOperators){
			spinModel.push_back(*it1);
		}
	}

	return *this;
}
SpinModelClass& SpinModelClass::operator+=(SpinOperatorsOnSites<Complex> const& rhs){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it2;

	bool foundOperators=false;;
	for(it2=spinModel.begin();it2!=spinModel.end();it2++){
		if(it2->second==rhs){
			it2->first.first+=SeriesDouble(1);
			if(it2->first==pair<SeriesDouble,SeriesDouble>(SeriesDouble(0),SeriesDouble(0))){
				spinModel.erase(it2);
			}
			foundOperators=true;
			break;
		}
	}
	if(!foundOperators){
		spinModel.push_back(pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> >(pair<SeriesDouble,SeriesDouble>(SeriesDouble(1),SeriesDouble(0)),rhs));
	}

	return *this;
}
SpinModelClass& SpinModelClass::operator-=(SpinModelClass const& rhs){
	*this+=Doub(-1)*rhs;
	return *this;
}
SpinModelClass& SpinModelClass::operator*=(Doub const& rhs){
	if(rhs==0){
		spinModel.clear();
	}
	else{
		list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it1;
		for(it1=spinModel.begin();it1!=spinModel.end();it1++){
			it1->first*=rhs;
		}
	}
	return *this;
}
SpinModelClass& SpinModelClass::operator*=(pair<SeriesDouble,SeriesDouble> const& rhs){
	if(rhs==pair<SeriesDouble,SeriesDouble>(SeriesDouble(0),SeriesDouble(0))){
		spinModel.clear();
	}
	else{
		list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it1;
		for(it1=spinModel.begin();it1!=spinModel.end();it1++){
			it1->first*=rhs;
		}
	}
	return *this;
}
SpinModelClass& SpinModelClass::operator/=(Doub const& rhs){
	if(rhs==0){
		cerr << "Error: Division by zero is not allowed!\n";
	}
	else{
		*this*=1/rhs;
	}
	return *this;
}

const SpinModelClass operator+(SpinModelClass const& lhs, SpinModelClass const& rhs){
	SpinModelClass smc=lhs;
	smc+=rhs;
	return smc;
}
const SpinModelClass operator+(SpinModelClass const& lhs, SpinOperatorsOnSites<Complex> const& rhs){
	SpinModelClass smc=lhs;
	smc+=rhs;
	return smc;
}
const SpinModelClass operator+(SpinOperatorsOnSites<Complex> const& lhs, SpinModelClass const& rhs){
	SpinModelClass smc=rhs;
	smc+=lhs;
	return smc;
}
const SpinModelClass operator-(SpinModelClass const& lhs, SpinModelClass const& rhs){
	SpinModelClass smc=lhs;
	smc-=rhs;
	return smc;
}
const SpinModelClass operator*(SpinModelClass const& lhs, Doub const& rhs){
	SpinModelClass smc=lhs;
	smc*=rhs;
	return smc;
}
const SpinModelClass operator*(Doub const& lhs, SpinModelClass const& rhs){
	SpinModelClass smc=rhs;
	smc*=lhs;
	return smc;
}
const SpinModelClass operator*(SpinModelClass const& lhs, pair<SeriesDouble,SeriesDouble> const& rhs){
	SpinModelClass smc=lhs;
	smc*=rhs;
	return smc;
}
const SpinModelClass operator*(pair<SeriesDouble,SeriesDouble> const& lhs, SpinModelClass const& rhs){
	SpinModelClass smc=rhs;
	smc*=lhs;
	return smc;
}
const SpinModelClass operator/(SpinModelClass const& lhs, Doub const& rhs){
	SpinModelClass smc=lhs;
	smc/=rhs;
	return smc;
}

istream& operator>>(istream &o, SpinModelClass &s){
	char tempChar;
	tempChar=o.peek();
	if(tempChar=='\n'){
		o>>noskipws>>tempChar>>skipws;
	}

	// get complete line of stream
	string tempstring;
	getline(o,tempstring);
	istringstream iss(tempstring);

	s.spinModel.clear();

	SpinOperatorsOnSites<Complex> tempSpinOperator;

	pair<SeriesDouble,SeriesDouble> unitSeries(SeriesDouble(1.0),SeriesDouble(0));

	while(iss>>tempSpinOperator){
		s.spinModel.push_back(pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> >(unitSeries,tempSpinOperator));
	}

	if(s.spinModel.size()==0){
		o.setstate(ios_base::failbit);
		//cout << "Leaving with failbit ..." << endl;
	}
	else{
		o.clear();
	}

	//cout << "streamIsGood=" << o.good() << "\n";

	return o;
}

ostream& operator<<(ostream &o, SpinModelClass &s){
	list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::iterator it1;
	for(it1=s.spinModel.begin();it1!=s.spinModel.end();it1++){
		o << "+" << it1->first << "*" << it1->second << "\n";
	}
	return o;
}

bool operator==(SpinModelClass const &lhs, SpinModelClass const &rhs){
	if(lhs.spinModel.size()==rhs.spinModel.size()){
		// run over all elments of lhs and find them in rhs
		list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::const_iterator itLhs;
		for(itLhs=lhs.spinModel.begin();itLhs!=lhs.spinModel.end();itLhs++){
			bool foundElement=false;
			list<pair<pair<SeriesDouble,SeriesDouble>,SpinOperatorsOnSites<Complex> > >::const_iterator itRhs;
			for(itRhs=rhs.spinModel.begin();itRhs!=rhs.spinModel.end();itRhs++){
				if(*itRhs==*itLhs){
					foundElement=true;
					break;
				}
			}
			if(!foundElement){
				return false;
			}
		}
		// if all elements are found both spinModels are equal
		return true;
	}
	return false;
}

bool operator!=(SpinModelClass const &lhs, SpinModelClass const &rhs){
	return !(lhs==rhs);
}
