/*
 * main.cpp
 *
 *  Created on: May 2, 2014
 *      Author: ixert
 */

#include<iostream>
#include<sstream>
#include<fstream>
#include "LatticeGraph.h"
#include "GraphController.h"

using namespace std;

int main(int argc, char *argv[]){
	// argument-list:
	// 4 arguments (argc=5):
	// calculationType=1, gCUTOrder, inverseCouplingsFile, latticeFile
	// => graphCreation of graphs on the lattice specified in latticeFile
	// => leads to graphList

	// 4 arguments (argc=5):
	// calculationType=2, pCUTOrder, inverseCouplingsFile, latticeFile
	// => graphCreation of graphs on the lattice specified in latticeFile
	// => leads to graphList, but kicks out all non pCUT Graphs

	// 4 arguments (argc=5):
	// calculationType=3, gCUTOrder, inverseCouplingsFile, graphList
	// => graphSubtraction
	// => leads to subtractionList

	// 6 arguments (argc=7):
	// calculationType=4, gCUTOrder, inverseCouplingsFile, graphList, masterCluster, initialSiteList
	// => graphEmbedding onto the masterCluster (always including the initialSite)
	// => leads to embeddingList

	// 9 arguments (argc=10):
	// calculationType=5, gCUTOrder, inverseCouplingsFile, graphList, initialSiteList, subtractionList, spinModelFormList, effSpinModelList, masterClusterLg
	// => actual subtraction/embedding for the effective Spin-Models specified in effSpinModelList
	// => leads to thermodynamic spin model

	// 9 arguments (argc=10):
	// calculationType=6, inverseCouplingsFile, latticeFile, xMax, yMax, n, masterClusterFilename, gnuplotFilename, plotPointsFilename
	// => createMasterCluster
	// => leads to MasterCluster

	// 7 arguments (argc=8)
	// calculationType=7, gCUTOrder, interUnitCellVectors, intraUnitCellVectors, xMax, yMax, spinModelTDL
	// => createEDCluster
	// => creates an EDCluster with the SpinHamiltonian from spinModelTDL

    // 7 arguments (argc=8)
    // calculationType=11, gCUTOrder, inverseCouplingsFile, latticeFile, graphList, masterCluster, initialSiteList
    // => creates all different edge spin configurations on graphs
    // => leads to siteCombinations and new embedding files


	// read parameters
	if(argc<2){
		cout << "no arguments!\n";
		return 1;
	}

	unsigned int calculationType=0;
	unsigned int gCUTOrder=2;
	string sInverseCouplingsFile="";
	string sLatticeFile="";
	string sGraphList="";
	string sMasterCluster="";
	string sMasterClusterLg="";
	string sInitialSiteList="";
	string sCouplingList="";
	string sSubtractionList="";
	string sSpinModelFormList="";
	string sEffSpinModelList="";
	string sGnuplot="";
	string sPlotPoints="";
	string sEffSpinModelTDL="";
	string sInterUnitCellVectors="";
	string sIntraUnitCellVectors="";

	unsigned int yMax=0;
	unsigned int xMax=0;
	unsigned int n=100; // this is the samples parameter for the creation of the master cluster. larger value means longer calculation time for the master cluster. Should be increased if the masterCluster gets bigger!

	stringstream ssCalculationType(argv[1]);
	ssCalculationType >> calculationType;

	switch(calculationType){
	case 1:{
		if(argc!=5){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[4]);
		ssLatticeFile >> sLatticeFile;
		break;
	}
	case 2:{
		if(argc!=5){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[4]);
		ssLatticeFile >> sLatticeFile;
		break;
	}
	case 3:{
		if(argc!=5){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssGraphList(argv[4]);
		ssGraphList >> sGraphList;
		break;
	}
	case 4:{
		if(argc!=7){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssGraphList(argv[4]);
		ssGraphList >> sGraphList;
		stringstream ssMasterCluster(argv[5]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssInitialSite(argv[6]);
		ssInitialSite >> sInitialSiteList;
		break;
	}
	case 5:{
		if(argc!=10){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInverseCouplingsFile(argv[3]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssGraphList(argv[4]);
		ssGraphList >> sGraphList;
		stringstream ssInitialSite(argv[5]);
		ssInitialSite >> sInitialSiteList;
		stringstream ssSubtractionList(argv[6]);
		ssSubtractionList >> sSubtractionList;
		stringstream ssSpinModelFormList(argv[7]);
		ssSpinModelFormList >> sSpinModelFormList;
		stringstream ssEffSpinModelList(argv[8]);
		ssEffSpinModelList >> sEffSpinModelList;
		stringstream ssMasterClusterLg(argv[9]);
		ssMasterClusterLg >> sMasterClusterLg;
		break;
	}
	case 6:{
		if(argc!=10){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssInverseCouplingsFile(argv[2]);
		ssInverseCouplingsFile >> sInverseCouplingsFile;
		stringstream ssLatticeFile(argv[3]);
		ssLatticeFile >> sLatticeFile;
		stringstream ssXMax(argv[4]);
		ssXMax >> xMax;
		stringstream ssYMax(argv[5]);
		ssYMax >> yMax;
		stringstream ssN(argv[6]);
		ssN >> n;
		stringstream ssMasterCluster(argv[7]);
		ssMasterCluster >> sMasterCluster;
		stringstream ssGnuplot(argv[8]);
		ssGnuplot >> sGnuplot;
		stringstream ssPlotPoints(argv[9]);
		ssPlotPoints >> sPlotPoints;
		break;
	}
	case 7:{
		if(argc!=8){
			cout << "wrong number of arguments!\n";
			return 1;
		}
		stringstream ssgCUTOrder(argv[2]);
		ssgCUTOrder >> gCUTOrder;
		stringstream ssInterUnitCellVectors(argv[3]);
		ssInterUnitCellVectors >> sInterUnitCellVectors;
		stringstream ssIntraUnitCellVectors(argv[4]);
		ssIntraUnitCellVectors >> sIntraUnitCellVectors;
		stringstream ssXMax(argv[5]);
		ssXMax >> xMax;
		stringstream ssYMax(argv[6]);
		ssYMax >> yMax;
		stringstream ssSpinModelTDL(argv[7]);
		ssSpinModelTDL >> sEffSpinModelTDL;
		break;
	}
    case 11:{
        if(argc!=8){
            cout << "wrong number of arguments!\n";
            return 1;
        }
        stringstream ssgCUTOrder(argv[2]);
        ssgCUTOrder >> gCUTOrder;
        stringstream ssInverseCouplingsFile(argv[3]);
        ssInverseCouplingsFile >> sInverseCouplingsFile;
        stringstream ssLatticeFile(argv[4]);
        ssLatticeFile >> sLatticeFile;
        stringstream ssGraphList(argv[5]);
        ssGraphList >> sGraphList;
        stringstream ssMasterCluster(argv[6]);
        ssMasterCluster >> sMasterCluster;
        stringstream ssInitialSite(argv[7]);
        ssInitialSite >> sInitialSiteList;
        break;
    }
	}

	if(gCUTOrder<2){
		cout << "gCUTOrder must be at least 2!\n";
		return 1;
	}

	cout << "calculationType=" << calculationType << "\n";
	cout << "gCUTOrder=" << gCUTOrder << "\n";
	cout << "sInverseCouplingsFile=" << sInverseCouplingsFile << "\n";
	cout << "sLatticeFile=" << sLatticeFile << "\n";
	cout << "sGraphList=" << sGraphList << "\n";
	cout << "sMasterCluster=" << sMasterCluster << "\n";
	cout << "sMasterClusterLg=" << sMasterClusterLg << "\n";
	cout << "sInitialSiteList=" << sInitialSiteList << "\n";
	cout << "sCouplingList=" << sCouplingList << "\n";
	cout << "sSubtractionList=" << sSubtractionList << "\n";
	cout << "sSpinModelFormList=" << sSpinModelFormList << "\n";
	cout << "sEffSpinModelList=" << sEffSpinModelList << "\n";
	cout << "sGnuplot=" << sGnuplot << "\n";
	cout << "sPlotPoints=" << sPlotPoints << "\n";
	cout << "yMax=" << yMax << "\n";
	cout << "xMax=" << xMax << "\n";
	cout << "n=" << n << "\n";
	cout << "sEffSpinModelTDL=" << sEffSpinModelTDL << "\n";
	cout << "sInterUnitCellVectors=" << sInterUnitCellVectors << "\n";
	cout << "sIntraUnitCellVectors=" << sIntraUnitCellVectors << "\n";

	// set inverseCouplings (specifically for the KMH-Model on the honeycomb lattice)

	// calculation
	GraphController gc(sInverseCouplingsFile,sLatticeFile);
	switch(calculationType){
		case 1:{
			gc.createGraphs(gCUTOrder,false);
			break;
		}
		case 2:{
			gc.createGraphs(gCUTOrder,true);
			break;
		}
		case 3:{
			gc.calculateGraphSubtractions(gCUTOrder,sGraphList);
			break;
		}
		case 4:{
			gc.calculateGraphEmbeddings(gCUTOrder,sGraphList,sMasterCluster,sInitialSiteList);
			break;
		}
		case 5:{
			gc.performGraphSubtractionsAndEmbeddings(gCUTOrder,sGraphList,sInitialSiteList,sSubtractionList,sSpinModelFormList,sEffSpinModelList,sMasterClusterLg);
			break;
		}
		case 6:{
			gc.createMasterCluster(xMax,yMax,n,sMasterCluster,sGnuplot,sPlotPoints);
			break;
		}
		case 7:{
			gc.createEDCluster(gCUTOrder,sInterUnitCellVectors,sIntraUnitCellVectors,xMax,yMax,sEffSpinModelTDL);
			break;
		}
        case 11:{
            gc.createSiteCombinationsWithEdgeSpins(gCUTOrder,sGraphList,sMasterCluster,sInitialSiteList);
            break;
        }
	}


	return 0;
}
