/*
 * SpinDerivatorController.cpp
 *
 *  Created on: Feb 3, 2014
 *      Author: ixert
 */

#include "SpinDerivatorController.h"
#include "SeriesTemplate.h"
#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include "svd.h"
#include "functions.h"
#include "StateTemplateSimple.h"

using namespace std;

SpinDerivatorController::SpinDerivatorController(unsigned int const &Nvalue, string &sStates) {
	// this constructor reads the states

	N=Nvalue;
	State::setN(N);
	StateDouble::setN(N);

	vector<string> sStateFiles;

	stringstream inputFormatFull;
	inputFormatFull << "Left input state (bra) :";

	stringstream outputFormat;
	outputFormat << "state";
	stringstream outputFormatFull;
	outputFormatFull<< "Output " << outputFormat.str() << " :";

	// load stateFileNames
	string tempString;
	fstream fsStateFiles;
	fsStateFiles.open(sStates.c_str(),fstream::in);
	getline(fsStateFiles,tempString);
	while(fsStateFiles>>tempString){
		sStateFiles.push_back(tempString);
	}
	fsStateFiles.close();

	cout << "sStateFiles.size()=" << sStateFiles.size() << "\n";

	for(unsigned int i=0;i<sStateFiles.size();i++){
		fstream fsState;
		fsState.open(sStateFiles[i].c_str(),fstream::in);;
		while(fsState.good()){
			unsigned int stateSize;
			string tempString;
			getline(fsState,tempString);
			if(tempString==inputFormatFull.str()){
				SpinState tempState;
				fsState >> tempState;
				//SpinState tempSpinState=tempState.convertToSpinState();
				//inputStates.push_back(tempSpinState);
				inputStates.push_back(tempState);
				stateSize=tempState.size();
			}
			if(tempString==outputFormatFull.str()){
				if(outputFormat.str()=="state"){
					SpinStateTemplate<SeriesTemplate<mpq_class,ExponentsTemplate<int> > > tempState_mpqClass(stateSize);
					fsState>>tempState_mpqClass;
					SpinStateTemplate<SeriesTemplate<mpq_class,ExponentsTemplate<int> > > newState_mpqClass=tempState_mpqClass;
					while(fsState>>tempState_mpqClass){
						newState_mpqClass+=tempState_mpqClass;
					}
					// convert mpq_class state to SpinState (Double)

					/*
						SpinState tempState(stateSize);
						fsState>>tempState;
						SpinState newState=tempState;
						while(fsState>>tempState){
							newState+=tempState;
						}
					 */

					vector<vector<bool> > singleStates_mpqClass=newState_mpqClass.getSingleStates();
					vector<SeriesTemplate<mpq_class,ExponentsTemplate<int> > > amplitudes_mpqClass=newState_mpqClass.getAmplitudes();

					SpinState newState(stateSize);
					if(singleStates_mpqClass.size()>0){
						unsigned int numberOfSites=newState.size()/N;
						for(unsigned int singleIndex=0;singleIndex<singleStates_mpqClass.size();singleIndex++){
							SeriesTemplate<Doub,ExponentsTemplate<int> > tempDoubSeries=convertMpqToDoub<ExponentsTemplate<int> >(amplitudes_mpqClass[singleIndex]);
							//tempDoubSeries=tempDoubSeries.getPureContribution(numberOfSites); //TODO: getPureContributions must know the numberOfBonds involved in the pCUT calculation!
							if(myAbs(tempDoubSeries)>zeroEPS){
								newState+=tempDoubSeries*SpinState(singleStates_mpqClass[singleIndex]);
							}
						}
					}
					outputStates.push_back(newState);
				}
				break;
			}
		}
		fsState.close();
	}

	// save inputStates in complex form
	vector<Doub> evalVector;
	evalVector.push_back(1);
	for(unsigned int i=0;i<inputStates.size();i++){
		vector<vector<bool> > singleStates=inputStates[i].getSingleStates();
		/*
		 usually inputStates[i] is just a single state with amplitude 1! => if obtained from Solver! => therefore just convert it directly
		vector<SeriesTemplate<Doub,ExponentsTemplate<int> > > amplitudes=inputStates[i].getAmplitudes();
		*/
		SpinStateTemplate<Doub> tempComplexSpinState(singleStates[0]);
		/*
		tempComplexSpinState*=amplitudes[0].evaluate(evalVector);
		for(unsigned int j=1;j<singleStates.size();j++){
			tempComplexSpinState+=amplitudes[j].evaluate(evalVector)*SpinStateTemplate<Doub>(singleStates[j]);
		}
		*/
		inputStatesComplex.push_back(tempComplexSpinState);
	}
}

bool SpinDerivatorController::calculateEffSpinModel(){
	// calculates the couplings of the effSpinModel which belong to the given spinModelForm
	// to achieve this a SVD is used
	// As in general the amplitudes are complex the real and imaginary parts of the couplings are splitted so the equation System is twice in size

	// first of all: check if all amplitudes are real and determine vector/matrix sizes
	unsigned int numberOfSites=inputStates[0].size()/N;
	unsigned int numberOfStates=inputStates.size();
	unsigned int numberOfStateCombinations=(numberOfStates*(numberOfStates+1))/2;
	unsigned int numberOfAllOperators=0;


	//// 1) create all permutation operator sequences ////
	// create all single permutation operators
	{
	list<pair<unsigned int,unsigned int> > singlePermutationOperators;
	for(unsigned int i=0;i<numberOfSites;i++){
		for(unsigned int j=i+1;j<numberOfSites;j++){
			singlePermutationOperators.push_back(pair<unsigned int,unsigned int>(i,j));
			list<pair<unsigned int,unsigned int> > tempSingleList;
			tempSingleList.push_back(pair<unsigned int,unsigned int>(i,j));
			permutationOperators.push_back(tempSingleList);
		}
	}

	// create all operators
	list<list<pair<unsigned int,unsigned int> > > tempList=permutationOperators;
	for(unsigned int i=1;i<numberOfSites-1;i++){
		cout << "\n";
		list<list<pair<unsigned int,unsigned int> > > tempList2;
		list<list<pair<unsigned int,unsigned int> > >::iterator itPermOp;
		for(itPermOp=tempList.begin();itPermOp!=tempList.end();++itPermOp){
			cout << "actualOperator=";
			for(list<pair<unsigned int,unsigned int> >::iterator itBla=itPermOp->begin();itBla!=itPermOp->end();++itBla){
				cout << "P" << itBla->first << itBla->second;
			}
			cout << "\n";
			list<pair<unsigned int,unsigned int> >::iterator itSinglePermOp;
			for(itSinglePermOp=singlePermutationOperators.begin();itSinglePermOp!=singlePermutationOperators.end();++itSinglePermOp){
				cout << "insert P" << itSinglePermOp->first << itSinglePermOp->second << "\n";
				// check if this single permutation operator is already contained within this sequence
				list<pair<unsigned int,unsigned int> >::iterator itPermOpSeq;
				for(itPermOpSeq=itPermOp->begin();itPermOpSeq!=itPermOp->end();++itPermOpSeq){
					if((*itPermOpSeq)==(*itSinglePermOp)){
						break;
					}
				}
				if(itPermOpSeq==itPermOp->end()){
					//// this is the tricky part! find correct position of new operator!
					list<pair<unsigned int,unsigned int> > tempListSeq=(*itPermOp);
					list<pair<unsigned int,unsigned int> >::iterator itTempListSeq;
					//a) find position of first "blockation"
					list<pair<unsigned int,unsigned int> >::iterator itBlock=tempListSeq.end();
					for(itTempListSeq=tempListSeq.begin();itTempListSeq!=tempListSeq.end();++itTempListSeq){
						if((itTempListSeq->first==itSinglePermOp->first) || (itTempListSeq->first==itSinglePermOp->second) || (itTempListSeq->second==itSinglePermOp->first) || (itTempListSeq->second==itSinglePermOp->second)){
							itBlock=itTempListSeq;
						}
					}
					//b) find position to insert
					for(itTempListSeq=tempListSeq.begin();itTempListSeq!=itBlock;++itTempListSeq){
						if((*itTempListSeq)<(*itSinglePermOp)){
							break;
						}
					}
					tempListSeq.insert(itTempListSeq,*itSinglePermOp);
					cout << "tempListSeq=";
					for(list<pair<unsigned int,unsigned int> >::iterator itBla=tempListSeq.begin();itBla!=tempListSeq.end();++itBla){
							cout << "P" << itBla->first << itBla->second;
					}
					cout << "\n";
					//////////////////////////////////////////////////////////////////////

					// insert into tempList2 if not already contained
					list<list<pair<unsigned int,unsigned int> > >::iterator itTempList2;
					for(itTempList2=tempList2.begin();itTempList2!=tempList2.end();++itTempList2){
						if(itTempList2->size()==tempListSeq.size()){
							list<pair<unsigned int,unsigned int> >::iterator itTempList2Seq;
							list<pair<unsigned int,unsigned int> >::iterator itDuplicate=tempListSeq.begin();
							for(itTempList2Seq=itTempList2->begin();itTempList2Seq!=itTempList2->end();++itTempList2Seq){
								if((*itTempList2Seq)!=(*itDuplicate)){
									break;
								}
								++itDuplicate;
							}
							if(itTempList2Seq==itTempList2->end()){
								break;
							}
						}
					}
					if(itTempList2==tempList2.end()){
						cout << "insert!\n";
						tempList2.push_back(tempListSeq);
					}
				}
			}
		}
		// push all operators into permutationOperators
		list<list<pair<unsigned int,unsigned int> > >::iterator itTempList2;
		for(itTempList2=tempList2.begin();itTempList2!=tempList2.end();++itTempList2){
			permutationOperators.push_back(*itTempList2);
		}
		tempList.swap(tempList2);
	}

	// push back a permutation operator which is the identity => P00
	pair<unsigned int,unsigned int> P00;
	list<pair<unsigned int,unsigned int> > P00list;
	P00list.push_back(P00);
	permutationOperators.push_back(P00list);
	}


	// test output
	list<list<pair<unsigned int,unsigned int> > >::iterator it;
	for(it=permutationOperators.begin();it!=permutationOperators.end();++it){
		list<pair<unsigned int,unsigned int> >::reverse_iterator rit;
		for(rit=it->rbegin();rit!=it->rend();++rit){
			cout << "P" << rit->first << rit->second;
		}
		cout << "\n";
	}

	//// 2) create the LGS

	numberOfAllOperators=permutationOperators.size();

	unsigned int JSize=numberOfAllOperators;
	cout << "numberOfAllOperators=" << numberOfAllOperators << "\n";
	unsigned int rowSize=numberOfStateCombinations;
	cout << "numberOfStateCombinations=" << numberOfStateCombinations << "\n";

	MatDoub A(numberOfStateCombinations,numberOfAllOperators,0.0);
	VecDoubSeries b(rowSize);

	// calculate b-vector (<inputState|outputState>) which are the matrix elements after applying Takahashi Perturbation Theory
	unsigned int k=0;
	for(unsigned int i=0;i<numberOfStates;i++){
		for(unsigned int j=i;j<numberOfStates;j++){
			b[k]=inputStates[j]*outputStates[i];
			cout << "b[" << k << "]=" << b[k] << "\n";
			k++;
		}
	}

	// create A-Matrix
	unsigned int kModel=0;
	list<list<pair<unsigned int,unsigned int> > >::iterator itPermOp;
	for(itPermOp=permutationOperators.begin();itPermOp!=permutationOperators.end();++itPermOp){
		unsigned int kState=0;
		for(unsigned int i=0;i<numberOfStates;i++){
			SpinStateTemplate<Doub> operatorOnState=inputStatesComplex[i].applyPermutationOperators(*itPermOp);
			cout << inputStatesComplex[i] << " -> " << operatorOnState << "\n";
			for(unsigned int j=i;j<numberOfStates;j++){
				Doub scalarProd=inputStatesComplex[j]*operatorOnState;
				A[kState][kModel]=A[kState][kModel]+scalarProd;
				kState++;
			}
		}
		kModel++;
	}

	VecDoubSeries J(JSize);


	for(unsigned int i=0;i<A.nrows();i++){
		for(unsigned int j=0;j<A.ncols();j++){
			cout << A[i][j] << " ";
		}
		cout << "\n";
	}
	/*
	for(unsigned int i=0;i<b.size();i++){
		cout << "b["<< i<< "]=" << b[i] << "\n";
	}
	*/


	// 3) solve LGS
	SVD svdSystem(A);
	cout << "rank=" << svdSystem.rank() << "\n";
	svdSystem.solve(b,J);

	// now check if equation system is solved (A*J=b?)
	if(!testNRSolution(A,J,b)){
		cerr << "Spin model could not be derived!\n";
		return false;
	}

	for(unsigned int i=0;i<numberOfAllOperators;i++){
		effCouplings.push_back(SeriesDouble(J[i]));
	}

	return true;
}

void SpinDerivatorController::saveEffSpinModel(string &sSpinModel){
	// save the permutation operator model
	fstream fsSpinModel;
	fsSpinModel.open(sSpinModel.c_str(),fstream::out);
	list<list<pair<unsigned int,unsigned int> > >::iterator itPermOp;
	list<SeriesDouble>::iterator itCoupl=effCouplings.begin();
	cout << "effCouplings.size()=" << effCouplings.size() << "\n";
	cout << "permutationOperators.size()=" << permutationOperators.size() << "\n";
	for(itPermOp=permutationOperators.begin();itPermOp!=permutationOperators.end();++itPermOp){
		list<pair<unsigned int,unsigned int> >::iterator itPermOpSeq;
		for(itPermOpSeq=itPermOp->begin();itPermOpSeq!=itPermOp->end();++itPermOpSeq){
			fsSpinModel << "P " << itPermOpSeq->first << " " << itPermOpSeq->second << " ";
		}
		fsSpinModel << *itCoupl << "\n";
		itCoupl++;
	}
	fsSpinModel.close();
}
