/*
 * StateTemplate.h
 *
 *  Created on: 19.12.2011
 *      Author: ixert
 */

#ifndef STATETEMPLATESIMPLE_H_
#define STATETEMPLATESIMPLE_H_

#include <iostream>
#include <list>
#include <map>
#include <vector>
#include <math.h>
#include <sstream>
#include <set>
#include "globalVariables.h"
#include "SeriesTemplate.h"
#include "ExponentsTemplate.h"
#include "mathFunctions.h"

using namespace std;

template<class T>
class StateTemplateSimple {
template<class S>
friend ostream& operator<<(ostream &o, StateTemplateSimple<S> const &s);
template<class S>
friend istream& operator>>(istream &o, StateTemplateSimple<S> &s);
template<class S>
friend S operator*(StateTemplateSimple<S> const &s1, StateTemplateSimple<S> const &s2);
template<class S>
friend StateTemplateSimple<S> productState(StateTemplateSimple<S> const &s1, StateTemplateSimple<S> const &s2);
template<class S>
friend StateTemplateSimple<S> insertState(StateTemplateSimple<S> const &s1, StateTemplateSimple<S> const &s2, unsigned int *pos);
template<class S>
friend StateTemplateSimple<S> insertState(StateTemplateSimple<S> const &s1, StateTemplateSimple<S> const &s2, vector<unsigned int> const &pos);

protected:
	static unsigned int N;
	static unsigned int spinBitsPerSite;
	unsigned int n; // number of bits!
	map<vector<bool>,T> stateCont;

public:
	StateTemplateSimple();
	StateTemplateSimple(vector<bool> &v);
	StateTemplateSimple(vector<bool> &v, T const &x);
	StateTemplateSimple(vector<bool> const &v, vector<unsigned int> const &pos);
	StateTemplateSimple(unsigned int nVal);
	StateTemplateSimple(unsigned int nVal,string c);

	unsigned int size();
	vector<T> getAmplitudes() const;
	vector<vector<bool> > getSingleStates() const;
	vector<StateTemplateSimple<T> > splitSuperposition() const;
	bool isEmpty() const;

	int QPposition(unsigned int const &bitNumber=2) const;

    StateTemplateSimple<T>& operator+=(StateTemplateSimple<T> const &s);
    StateTemplateSimple<T>& operator-=(StateTemplateSimple<T> const &s);
    StateTemplateSimple<T>& operator*=(T const &x);
    StateTemplateSimple<T>& operator/=(T const &x);

    static void setN(unsigned int const &Nvalue);
};

template<class T>
unsigned int StateTemplateSimple<T>::N=0;
template<class T>
unsigned int StateTemplateSimple<T>::spinBitsPerSite=0;
template<class T>
void StateTemplateSimple<T>::setN(unsigned int const &Nvalue){
	N=Nvalue;
	//spinBitsPerSite=N/2+N%2;
	spinBitsPerSite=N;
}

//// friend functions declaration ////
template<class T>
ostream& operator<<(ostream &o, StateTemplateSimple<T> const &s);
template<class T>
istream& operator>>(istream &o, StateTemplateSimple<T> &s);
template<class T>
T operator*(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2);
template<class T>
StateTemplateSimple<T> productState(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2);
template<class T>
StateTemplateSimple<T> insertState(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2, unsigned int *pos);

//// non class/friend functions declaration ////
template<class T>
const StateTemplateSimple<T> operator+(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2);
template<class T>
const StateTemplateSimple<T> operator-(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2);
template<class T>
const StateTemplateSimple<T> operator*(StateTemplateSimple<T> const &s, T const &x);
template<class T>
const StateTemplateSimple<T> operator*(T const &x, StateTemplateSimple<T> const &s);
template<class T>
const StateTemplateSimple<T> operator/(StateTemplateSimple<T> const &s, T const &x);


//// implementation ////
//// class functions ////
template<class T>
StateTemplateSimple<T>::StateTemplateSimple():n(0){
}
template<class T>
StateTemplateSimple<T>::StateTemplateSimple(vector<bool> &v){
	n=v.size();
	if(n>0){
		stateCont.insert(pair<vector<bool>,T>(v,1));
	}
}
template<class T>
StateTemplateSimple<T>::StateTemplateSimple(vector<bool> &v, T const &x){
	n=v.size();
	if(n>0){
		stateCont.insert(pair<vector<bool>,T>(v,x));
	}
}
template<class T>
StateTemplateSimple<T>::StateTemplateSimple(vector<bool> const &v, vector<unsigned int> const &pos){
	n=pos.size();
	if(n>0){
		vector<bool> vv(n);
		for(unsigned int i=0;i<n;i++){
			vv[i]=*(v.begin()+pos[i]);
		}
		stateCont.insert(pair<vector<bool>,T>(vv,T(1)));
	}
}
template<class T>
StateTemplateSimple<T>::StateTemplateSimple(unsigned int nVal):n(nVal){
}
template<class T>
StateTemplateSimple<T>::StateTemplateSimple(unsigned int nVal,string c):n(nVal){
	vector<bool> bs;
	for(unsigned int i=0;i<n;i++){
		bs.push_back(c[i]-'0');
	}
	if(n>0){
		stateCont.insert(pair<vector<bool>,T>(bs,1));
	}
}

template<class T>
unsigned int StateTemplateSimple<T>::size(){
	return n;
}

template<class T>
vector<T> StateTemplateSimple<T>::getAmplitudes() const{
	vector<T> amplitudes;
	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=stateCont.begin(); it!=stateCont.end();it++){
		amplitudes.push_back(it->second);
	}
	return amplitudes;
}
template<class T>
vector<vector<bool> > StateTemplateSimple<T>::getSingleStates() const{
	vector<vector<bool> > singleStates;
	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=stateCont.begin(); it!=stateCont.end();it++){
		singleStates.push_back(it->first);
	}
	return singleStates;
}

template<class T>
vector<StateTemplateSimple<T> > StateTemplateSimple<T>::splitSuperposition() const{
	vector<StateTemplateSimple<T> > singleStates;
	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=stateCont.begin(); it!=stateCont.end();it++){
		StateTemplateSimple<T> s(it->first,it->second);
		singleStates.push_back(s);
	}
	return singleStates;
}

template<class T>
bool StateTemplateSimple<T>::isEmpty() const{
	if(stateCont.size()==0){
		return true;
	}
	else{
		return false;
	}
}

template<class T>
int StateTemplateSimple<T>::QPposition(unsigned int const &bitNumber) const{
	int pos=-1;;
	if(stateCont.size()==1){
		typename std::map<vector<bool>,T>::const_iterator it;
		it=stateCont.begin();
		for(unsigned int i=0;i<it->first.size();i+=bitNumber){
			bool foundQP=false;
			for(unsigned int j=0;j<bitNumber;j++){
				if(it->first[i+j]){
					foundQP=true;
					break;
				}
			}
			if(foundQP){
				if(pos<0){
					pos=i/bitNumber;
				}
				else{
					return -1;
				}
			}
		}
	}
	return pos;
}

template<class T>
StateTemplateSimple<T>& StateTemplateSimple<T>::operator+=(StateTemplateSimple<T> const &s){
	if (n == s.n) {
		typename std::map<vector<bool>,T>::const_iterator it2;
		for (it2 = s.stateCont.begin(); it2 != s.stateCont.end(); ++it2) {
			pair<typename std::map<vector<bool>,T>::iterator,bool> ret=stateCont.insert(*it2);
			if(!(ret.second)){
				ret.first->second+=it2->second;
				if(myAbs(ret.first->second)<=zeroEPS){
					stateCont.erase(ret.first);
				}
			}
		}
	}
	else {
		cerr << "State& State::operator+=(State const &s): States not same length\n";
	}
    return *this;
}
template<class T>
StateTemplateSimple<T>& StateTemplateSimple<T>::operator-=(StateTemplateSimple<T> const &s){
	if (n == s.n) {
		typename std::map<vector<bool>,T>::const_iterator it2;
		for (it2 = s.stateCont.begin(); it2 != s.stateCont.end(); ++it2) {
			pair<typename std::map<vector<bool>,T>::iterator,bool> ret=stateCont.insert(pair<vector<bool>,T>(it2->first,-it2->second));
			if(!(ret.second)){
				ret.first->second-=it2->second;
				if(myAbs(ret.first->second)<=zeroEPS){
					stateCont.erase(ret.first);
				}
			}
		}
	}
	else {
		cerr << "State& State::operator-=(State const &s): States not same length\n";
	}
    return *this;
}
template<class T>
StateTemplateSimple<T>& StateTemplateSimple<T>::operator*=(T const &x){
	typename std::map<vector<bool>,T>::iterator it=stateCont.begin();
	typename std::map<vector<bool>,T>::iterator tempIt;
    while(it!=stateCont.end()){
    	it->second*=x;
    	if(myAbs(it->second)<=zeroEPS){
    		tempIt=it;
    		it++;
    		stateCont.erase(tempIt);
    	}
    	else{
    		it++;
    	}
    }
    return *this;
}
template<class T>
StateTemplateSimple<T>& StateTemplateSimple<T>::operator/=(T const &x){
	typename std::map<vector<bool>,T>::iterator it=stateCont.begin();
	typename std::map<vector<bool>,T>::iterator tempIt;
    while(it!=stateCont.end()){
    	it->second/=x;
    	if(myAbs(it->second)<=zeroEPS){
    		tempIt=it;
    		it++;
    		stateCont.erase(tempIt);
    	}
    	else{
    		it++;
    	}
    }
    return *this;
}

template<class T>
T operator*(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2){
	if(s1.n>0){
		T x=T(0);
		//T x;
		typename std::map<vector<bool>,T>::const_iterator it1;
		typename std::map<vector<bool>,T>::const_iterator it2;
		if (s1.stateCont.size() < s2.stateCont.size()) {
			for (it1 = s1.stateCont.begin(); it1 != s1.stateCont.end(); ++it1) {
				it2 = s2.stateCont.find(it1->first);
				if (it2 != s2.stateCont.end()) {
					x += ::conj(it1->second) * (it2->second);
				}
			}
		}
		else {
			for (it1 = s2.stateCont.begin(); it1 != s2.stateCont.end(); ++it1) {
				it2 = s1.stateCont.find(it1->first);
				if (it2 != s1.stateCont.end()) {
					x += ::conj(it1->second) * (it2->second);
				}
			}
		}
		return x;
	}
	else{
		return T(1);
	}
}

/*
template<>
Doub operator*(StateTemplateSimple<Doub> const &s1, StateTemplateSimple<Doub> const &s2){
	if(s1.n>0){
		Doub x=0;
		//T x;
		typename std::map<vector<bool>,Doub>::const_iterator it1;
		typename std::map<vector<bool>,Doub>::const_iterator it2;
		if (s1.stateCont.size() < s2.stateCont.size()) {
			for (it1 = s1.stateCont.begin(); it1 != s1.stateCont.end(); ++it1) {
				it2 = s2.stateCont.find(it1->first);
				if (it2 != s2.stateCont.end()) {
					x += it1->second * (it2->second);
				}
			}
		}
		else {
			for (it1 = s2.stateCont.begin(); it1 != s2.stateCont.end(); ++it1) {
				it2 = s1.stateCont.find(it1->first);
				if (it2 != s1.stateCont.end()) {
					x += it1->second * (it2->second);
				}
			}
		}
		return x;
	}
	else{
		return 1;
	}
}
*/


/*template<>
SeriesTemplate<Complex, ExponentsTemplate<int> > operator*(StateTemplateSimple<SeriesTemplate<Complex, ExponentsTemplate<int> > > const &s1,StateTemplateSimple<SeriesTemplate<Complex, ExponentsTemplate<int> > > const &s2){
	if(s1.n>0){
		SeriesTemplate<Complex, ExponentsTemplate<int> > x;
		std::map<vector<bool>,SeriesTemplate<Complex, ExponentsTemplate<int> > >::const_iterator it1;
		std::map<vector<bool>,SeriesTemplate<Complex, ExponentsTemplate<int> > >::const_iterator it2;
		if (s1.stateCont.size() < s2.stateCont.size()) {
			for (it1 = s1.stateCont.begin(); it1 != s1.stateCont.end(); ++it1) {
				it2 = s2.stateCont.find(it1->first);
				if (it2 != s2.stateCont.end()) {
					x += (it1->second) * (it2->second);
				}
			}
		}
		else {
			for (it1 = s2.stateCont.begin(); it1 != s2.stateCont.end(); ++it1) {
				it2 = s1.stateCont.find(it1->first);
				if (it2 != s1.stateCont.end()) {
					x += (it1->second) * (it2->second);
				}
			}
		}
		return x;
	}
	else{
		return SeriesTemplate<Complex, ExponentsTemplate<int> >();
	}
}*/


//// friend functions implementations ////
template<class T>
ostream& operator<<(ostream &o, StateTemplateSimple<T> const &s){
	bool showposset=false;
	ios::fmtflags f;
	f = cout.flags();
	if(f & ios::showpos){
		showposset=true;
	}

	typename std::map<vector<bool>,T>::const_iterator it;
	for(it=s.stateCont.begin(); it!=s.stateCont.end();++it){
		o << showpos;
		o << (it->second) << "|" << noshowpos;
		for(unsigned int i=0;i<s.n;i++){
			o << it->first[i];
		}
		o << ">";
	}

	if(showposset){
		o << showpos;
	}

	return o;
}
template<class T>
istream& operator>>(istream &o, StateTemplateSimple<T> &s){
	char tempChar;
	tempChar=o.peek();
	if(tempChar=='\n'){
		o>>noskipws>>tempChar>>skipws;
	}
	string tempstring;
	getline(o,tempstring);
	istringstream iss(tempstring);

	//cout << "iss=" << iss.str() << "\n";

	s.stateCont.clear();

	T x;
	char temp;
	//iss>>temp;
	T vz=T(1);
	//cout << "vz=" << vz << "\n";
	/*
	if(iss.peek()=='+'){
		iss>>temp;
	}
	else{
		if(iss.peek()=='-'){
			iss>>temp;
			vz=T(-1);
		}
	}
	*/
	while(iss>>x){
		//iss>>x;
		iss>>temp;
		vector<bool> vb;
		iss>>temp;
		while(temp!='>'){
			if(temp=='0'){
				vb.push_back(false);
			}
			else{
				if(temp=='1'){
					vb.push_back(true);
				}
			}
			iss>>temp;
		}
		//cout << x << "\n";
		s.n=vb.size();
		s.stateCont.insert(pair<vector<bool>,T>(vb,vz*x));

		/*
		cout << "|";
		for(unsigned int vbIndex=0;vbIndex<vb.size();vbIndex++){
			cout << vb[vbIndex];
		}
		cout << ">\n";
		*/
		/*
		vz=T(1);
		
		if(iss.peek()=='+'){
			iss>>temp;
		}
		else{
			if(iss.peek()=='-'){
				iss>>temp;
				vz=T(-1);
			}
		}
		*/
	}
	if(s.stateCont.size()==0){
		o.setstate(ios_base::failbit);
	}
	else{
		o.clear();
	}

	return o;
}

//// non class/friend functions implementations ////
template<class T>
const StateTemplateSimple<T> operator+(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2){
    StateTemplateSimple<T> s=s1;
    s+=s2;
    return s;
}
template<class T>
const StateTemplateSimple<T> operator-(StateTemplateSimple<T> const &s1, StateTemplateSimple<T> const &s2){
    StateTemplateSimple<T> s=s1;
    s-=s2;
    return s;
}

template<class T>
const StateTemplateSimple<T> operator*(StateTemplateSimple<T> const &s, T const &x){
    StateTemplateSimple<T> s2=s;
    s2*=x;
    return s2;
}
template<class T>
const StateTemplateSimple<T> operator*(T const &x, StateTemplateSimple<T> const &s){
    StateTemplateSimple<T> s2=s;
    s2*=x;
    return s2;
}
template<class T>
const StateTemplateSimple<T> operator/(StateTemplateSimple<T> const &s, T const &x){
    StateTemplateSimple<T> s2=s;
    s2/=x;
    return s2;
}


typedef StateTemplateSimple<Doub> StateDouble;
typedef StateTemplateSimple<SeriesTemplate<Complex,ExponentsTemplate<int> > > StateComplex;
typedef StateTemplateSimple<SeriesTemplate<Doub,ExponentsTemplate<int> > > State;

#endif /* STATETEMPLATE_H_ */
