/*
 * Modules.cpp
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 */

#include <Framework/Modules.h>
#include <Framework/Solver.h>

#include <Modules/ModelProvider/TemplateModelProvider/TemplateModelProvider.h>
#include <Modules/ModelProvider/GeneralLatticeModel/GeneralLatticeModel.h>
#include <Modules/ModelProvider/LWLowField/LWLowField.h>
#include <Modules/ModelProvider/ExampleSpecialLatticeModel/ExampleSpecialLatticeModel.h>
#include <Modules/ModelProvider/GeneralGraphModel/GeneralGraphModel.h>
#include <Modules/ModelProvider/ToricCode/ToricCode.h>
#include <Modules/ModelProvider/FermionicHubbard/FermionicHubbard.h>

#include <Modules/LatticeProvider/TemplateLatticeProvider/TemplateLatticeProvider.h>
#include <Modules/LatticeProvider/RotatedClusterDispersion/RotatedClusterDispersion.h>
#include <Modules/LatticeProvider/ClusterInFieldDispersion/ClusterInFieldDispersion.h>
#include <Modules/LatticeProvider/LatticeFileLoader/LatticeFileLoader.h>
#include <Modules/LatticeProvider/IsingChainGroundEnergyClusterDecomposition/IsingChainGroundEnergyClusterDecomposition.h>
#include <Modules/LatticeProvider/RotatedClusterGapAndGroundState/RotatedClusterGapAndGroundState.h>
#include <Modules/LatticeProvider/ClusterInFieldGapAndGroundState/ClusterInFieldGapAndGroundState.h>

#include <Modules/DataProcessor/TemplateDataProcessor/TemplateDataProcessor.h>
#include <Modules/DataProcessor/Loewdin/DirectSolver/DirectSolver.h>
#include <Modules/DataProcessor/Loewdin/TwoSideSolver/TwoSideSolver.h>
#include <Modules/DataProcessor/Loewdin/TwoSideCommonSolver/TwoSideCommonSolver.h>
#include <Modules/DataProcessor/Loewdin/TwoSideFirstApplicationSolver/TwoSideFirstApplicationSolver.h>
#include <Modules/DataProcessor/Takahashi/DirectSolver/DirectSolver.h>
#include <Modules/DataProcessor/Takahashi/DirectFirstApplicationSolver/DirectFirstApplicationSolver.h>
#include <Modules/DataProcessor/Takahashi/TreeSolver/TreeSolver.h>
#include <Modules/DataProcessor/Takahashi/TwoSideSolver/TwoSideSolver.h>
#include <Modules/DataProcessor/Takahashi/TwoSideFirstApplicationSolver/TwoSideFirstApplicationSolver.h>
#include <Modules/DataProcessor/PCUT/DirectSolver/DirectSolver.h>
#include <Modules/DataProcessor/PCUT/TreeSolver/TreeSolver.h>
#include <Modules/DataProcessor/PCUT/TwoSideSolver/TwoSideSolver.h>
#include <Modules/DataProcessor/BondfilePlotter/BondfilePlotter.h>



#include <Modules/DataWriter/TemplateDataWriter/TemplateDataWriter.h>
//#include <Modules/DataWriter/ProjectorWriter/ProjectorWriter.h>
#include <Modules/DataWriter/DefaultWriter/DefaultWriter.h>
#include <Modules/DataWriter/TCHFWriter/TCHFWriter.h>
#include <Modules/DataWriter/TCLFWriter/TCLFWriter.h>

#include <Modules/TemplateModuleType/TemplateModuleImplementation/TemplateModuleImplementation.h>

// Called by the constructor of the Solver class. Create the instances of the selected modules.
void Modules::createInstances() {

#define MODULE(TYPE,NAME) \
	if (!nameOf##TYPE.compare(#NAME)) { \
		if (solver.globalConfiguration.verbosityLevel > 0) \
			std::cerr << "Loading " #TYPE " : "  #NAME ".\n"; \
		pointerTo##TYPE = new NAME(); \
	} \
    else if (!nameOf##TYPE.compare("Disabled")) \
           pointerTo##TYPE = 0;

	// You must add a line for every module implementation here.
	// MODULE(MODULE_TYPE, MODULE_IMPLEMENTATION)

	// The ModelProviders
	MODULE(ModelProvider,TemplateModelProvider)
	MODULE(ModelProvider,GeneralLatticeModel)
	MODULE(ModelProvider,LWLowField)
	MODULE(ModelProvider,ExampleSpecialLatticeModel)
	MODULE(ModelProvider,GeneralGraphModel)
	MODULE(ModelProvider,ToricCode)
	MODULE(ModelProvider,FermionicHubbard)

	// Test, if the configuration of the Exponents class templates (Interface/Interface.h)
	// matches the model properties provided by the model provider.


	if (solver.modelProperties.numberOfExpansionParameters > sizeof(Exponents)) {
		std::cerr
				<< "Error! The number of expansion parameters of the model is "
				<< solver.modelProperties.numberOfExpansionParameters
				<< ", but the Exponents class only supports "
				<< sizeof(Exponents) << " expansion parameters.\n"
				<< "Please change the Exponents typedef in the Interface/Interface.h file and recompile.\n\n";
		exit(-1);
	} else if (solver.globalConfiguration.verbosityLevel > 0
			&& solver.modelProperties.numberOfExpansionParameters
					!= sizeof(Exponents)) {
		std::cerr
				<< "Warning! The number of expansion parameters of the model is "
				<< solver.modelProperties.numberOfExpansionParameters
				<< ", but the Exponents class is configured to "
				<< sizeof(Exponents) << " expansion parameters.\n"
				<< "It is recommended to change the Exponents typedef in the Interface/Interface.h file and recompile.\n\n";

	}

	// You must add a line for every module implementation here.
	// MODULE(MODULE_TYPE, MODULE_IMPLEMENTATION)

	// The LatticeProvider
	MODULE(LatticeProvider, TemplateLatticeProvider)
	MODULE(LatticeProvider, LatticeFileLoader)
	MODULE(LatticeProvider, RotatedClusterDispersion)
	MODULE(LatticeProvider, ClusterInFieldDispersion)
	MODULE(LatticeProvider, IsingChainGroundEnergyClusterDecomposition)
	MODULE(LatticeProvider, RotatedClusterGapAndGroundState)
	MODULE(LatticeProvider, ClusterInFieldGapAndGroundState)

	// The DataProcessors
	MODULE(DataProcessor,TemplateDataProcessor)
	MODULE(DataProcessor,BondfilePlotter)
	MODULE(DataProcessor,Loewdin::DirectSolver)
	MODULE(DataProcessor,Loewdin::TwoSideSolver)
	MODULE(DataProcessor,Loewdin::TwoSideCommonSolver)
	MODULE(DataProcessor,Loewdin::TwoSideFirstApplicationSolver)
	MODULE(DataProcessor,Takahashi::DirectSolver)
	MODULE(DataProcessor,Takahashi::DirectFirstApplicationSolver)
	MODULE(DataProcessor,Takahashi::TreeSolver)
	MODULE(DataProcessor,Takahashi::TwoSideSolver)
	MODULE(DataProcessor,Takahashi::TwoSideFirstApplicationSolver)
	MODULE(DataProcessor,PCUT::DirectSolver)
	MODULE(DataProcessor,PCUT::TreeSolver)
	MODULE(DataProcessor,PCUT::TwoSideSolver)

	// The DataWriters
	MODULE(DataWriter,TemplateDataWriter)
	  //MODULE(DataWriter,ProjectorWriter)
	MODULE(DataWriter,DefaultWriter)
	MODULE(DataWriter,TCHFWriter)
	MODULE(DataWriter,TCLFWriter)

	// The TemplateModuleTypes
	MODULE(TemplateModuleType,TemplateModuleImplementation)

}

// Methods to stream from the Config/Modules.cfg file.
// There must be an entry in the modules config file for every module type.
void Modules::streamOutExample(std::ostream& stream) {

	stream << "# The Module configuration\n\n" << "# ModelProvider\n"
			<< "Disabled\n\n" << "# BondListProvider\n" << "Disabled\n\n"
			<< "# DataProcessor\n" << "Disabled\n\n" << "# DataWriter\n"
			<< "Disabled\n\n\n" << "# TemplateModuleType\n" << "Disabled\n";

}

void Modules::streamIn(std::istream& stream) {

	stream >> nameOfModelProvider >> nameOfLatticeProvider
			>> nameOfDataProcessor >> nameOfDataWriter
			>> nameOfTemplateModuleType;

}

