/*
 * Modules.h
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 */

#ifndef MODULES_H_
#define MODULES_H_

#include <string>
#include <Modules/TemplateModuleType/TemplateModuleType.h>
#include <Modules/ModelProvider/ModelProvider.h>
#include <Modules/LatticeProvider/LatticeProvider.h>
#include <Modules/DataProcessor/DataProcessor.h>
#include <Modules/DataWriter/DataWriter.h>

/**
 The class which holds pointers to all currently active modules.
 If you want to create your own module or module type, you need to register them in Modules.h
 and in Modules.cpp (see TemplateModuleType.h and TemplateModuleImplementation.h for details).
 */
class Modules: public Utilities::ParameterSet {

public:

	void streamOutExample(std::ostream& stream);

	void streamIn(std::istream& stream);

	/** Called by the constructor of the Solver class. Create the instances of the selected modules.*/
	void createInstances();

#define MODULE_TYPE(NAME) \
  NAME* pointerTo##NAME; \
  std::string nameOf##NAME;

	// You must add a line for every module type here.

	MODULE_TYPE(ModelProvider)
	MODULE_TYPE(LatticeProvider)
	MODULE_TYPE(DataProcessor)
	MODULE_TYPE(DataWriter)

	MODULE_TYPE(TemplateModuleType)

};

#endif /* MODULES_H_ */
