/*
 * Solver.h
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 */

#ifndef SOLVER_H_
#define SOLVER_H_

#include <cstdlib>
#include <Interface/StatePair.h>
#include <Framework/Modules.h>
#include <Interface/GlobalConfiguration.h>
#include <Interface/LatticeProperties.h>
#include <Interface/GlobalProperties.h>

/** The central class of the solver. There is exactly one instance of this class (::solver), created by Main.cpp.
 It holds all data and calls the modules.*/
class Solver {

public:

	/** To load global configuration files and the module configuration.*/
	Solver();

	/** Calls the modules in the appropriate order.*/
	int run();

	/** The time when the Solver object was created.
	 * Used to calculate the processing time of the program. */
	time_t startingTime;

	/** The sub directory where to search for the configuration files.*/
	std::string configurationDirectory;

	/** Contains the pointers to the active modules.*/
	Modules modules;

	/** The global problem and solver independent configurations,
	 like the order to which the calculations should be done*/
	GlobalConfiguration globalConfiguration;

	/** Properties of the model.*/
	ModelProperties modelProperties;

	/** Properties of the lattice.*/
	LatticeProperties latticeProperties;

	/** Properties of the system, dependent of the model and lattice.*/
	GlobalProperties globalProperties;

	/** The input states that should be processed by the DataProcessor.*/
	StatePair inputStates;

	/** The output state calculated by the DataProcessor.*/
	State outputState;

	/** For example matrix elements or eigen values calculated by the DataProcessor. */
	stateAmplitude outputAmplitude;

};

/** The reference to the global solver instance to allow the modules access to
 the global parameters.*/
extern Solver solver;

#endif /* SOLVER_H_ */
