/*
 * Bond.h
 *
 *  Created on: 21.09.2011
 *      Author: klagges
 */

#ifndef BOND_H_
#define BOND_H_

#include <list>
#include <sstream>

/** Class to represent a bond of the lattice.
 * A lattice bond is one term of the \f$ V \f$ Operator, acting on a given set of lattice sites.*/
class Bond {
public:

	/** The number of the perturbation operator this bond is associated with.*/
	unsigned int operatorNumber;

	/** The lattice sites on which the perturbation operator acts as a list.*/
	std::list<unsigned int> sites;

	/** An index to identify this bond in a list of bonds. Should be unique for each bond in a bond list.*/
	unsigned int index;

};

/** Stream in from a string like 0  0 1; (OperatorNumber ListOfSites)*/
std::istream& operator>>(std::istream& i, Bond& b);

/** Stream out as a string like 2  0  0 1; (Index OperatorNumber ListOfSites)*/
std::ostream& operator<<(std::ostream& o, const Bond& b);

#endif /* BOND_H_ */
