/*
 * ExpandedAmplitudeIO.h
 *
 *  Created on: 07.01.2011
 *      Author: klagges
 */

#ifndef EXPANDEDAMPLITUDEIO_H_
#define EXPANDEDAMPLITUDEIO_H_

#include <Interface/ExponentsIO.h>
#include <Utilities/IsPositive.h>

/** Stream out as a string like 2+1/2*h1^2*h2*h3^3-4*h2*h3^3. */
template<typename cont, typename exp, typename amp>
std::ostream& operator<<(std::ostream& o,
		const ExpandedAmplitudeTemplate<cont, exp, amp>& e) {

	for (typename cont::const_iterator it = e.begin(); it != e.end(); it++) {

		if (Utilities::isPositive(it->second))
			o << "+";

		o << it->second;

		if (!(it->first.equalsOne())) {

			o << "*" << it->first;
		}

	}

	return o;
}

/** Stream in from a string like 2 + 1/2*h1^2 h2 h3^3 -4*h2 h3^3. */
template<typename cont, typename exp, typename amp>
std::istream& operator>>(std::istream& i,
		ExpandedAmplitudeTemplate<cont, exp, amp>& e) {

	e.clear();

	std::string s;

	while (i.good()) {

		char peek = i.peek();

		if (peek == '|' || peek == ';')
			break;

		if (peek == EOF) {
			i.get();
			break;
		}

		if (peek == ' ' || peek == '\t' || peek == '\n' || peek == '*'
				|| peek == '_') {

			i.get();
			continue;

		}

		s.push_back(i.get());

	}

	if (s.size() == 0 || (s.size() == 1 && s[0] == '+')) {
		e = ExpandedAmplitudeTemplate<cont, exp, amp>(amplitude(1));
		return i;
	}

	if (s.size() == 1 && s[0] == '-') {
		e = ExpandedAmplitudeTemplate<cont, exp, amp>(amplitude(-1));
		return i;
	}

	std::stringstream ss(s);

	while (ss.good()) {

		amplitude r;
		Exponents ex;

		bool negative = false;

		char peek = ss.peek();
		if (peek == '+') {
			ss.get();
			peek = ss.peek();
		} else {
			if (peek == '-') {
				negative = true;

				ss.get();
				peek = ss.peek();

			}
		}

		if (peek == 'h')
			r = 1;
		else {
			ss >> r;

		}
		if (negative)
			r *= -1;

		ss >> ex;

		e.add(r, ex);

	}

	return i;

}

#endif /* EXPANDEDAMPLITUDEIO_H_ */
