/*
 * ExponentsIO.h
 *
 *  Created on: 10.01.2011
 *      Author: klagges
 */

#ifndef EXPONENTSIO_H_
#define EXPONENTSIO_H_

#include <Framework/Solver.h>
#include <sstream>

/** Stream out as a string like h_0*h_1^3*h_2^2.*/
template<unsigned int n>
std::ostream& operator<<(std::ostream& o, const ExponentsTemplate<n>& ex) {

	bool first = false;

	for (unsigned int index = 0; index < n; index++) {
		if (ex.e[index] == 0)
			continue;
		if (first)
			o << "*";
		first = true;
		o << "h" << index;
		if (ex.e[index] != 1)
			o << "^" << (int) ex.e[index];
	}

	return o;
}

/** Stream in from a string like h_0 h_1^3h_2^2.*/
template<unsigned int n>
std::istream& operator>>(std::istream& i, ExponentsTemplate<n>& ex) {

	std::string factorString;

	char lastChar = 0;

	while (i.good()) {

		char peek = i.peek();

		if (peek == EOF) {
			i.get();
			break;
		}

		if (peek == '|' || peek == ';')
			break;

		if (peek == ' ' || peek == '\t' || peek == '\n' || peek == '*'
				|| peek == '_') {

			i.get();
			continue;

		}

		if (peek == '+' || peek == '-') {

			if (lastChar != '^')
				break;

		}

		factorString.push_back(i.get());
		lastChar = peek;

	}

	for (std::string::size_type hpos = factorString.find('h');
			hpos != std::string::npos;
			hpos = factorString.find('h', hpos + 1)) {

		std::string::size_type nexth = factorString.find('h', hpos + 1);

		std::string hString = factorString.substr(hpos, nexth - hpos);

		std::string exponentString;
		std::string parameterString;

		std::string::size_type hatpos = hString.find('^');

		if (hatpos != std::string::npos) {
			parameterString = hString.substr(1, hatpos - 1);
			exponentString = hString.substr(hatpos + 1, std::string::npos);

		} else {
			parameterString = hString.substr(1, std::string::npos);
			exponentString = "1";
		}

		int parameter;
		std::stringstream ss(parameterString);
		ss >> parameter;
		int exponent;
		std::stringstream ss2(exponentString);
		ss2 >> exponent;

		ex.e[parameter] += exponent;

	}

	return i;

}

#endif /* EXPONENTSIO_H_ */
