/*
 * GlobalConfiguration.h
 *
 *  Created on: 01.12.2010
 *      Author: klagges
 */

#ifndef GLOBALCONFIGURATION_H_
#define GLOBALCONFIGURATION_H_

#include <Utilities/ParameterSet.h>

/** Class to represent the global model, lattice and data processor independent configurations,
 like the order to which the calculations should be done*/
class GlobalConfiguration: public Utilities::ParameterSet {

public:

	void streamOutExample(std::ostream& stream);

	void streamIn(std::istream& stream);

	/** The order to which the calculations should be done*/
	unsigned int order;

	/** The order from which (the minimal order) the calculations should be done.*/
	unsigned int minOrder;

	/** During a calculation this value should be the order of the current calculation.
	 * Usage is optional. The exact meaning depends on the used modules.*/
	unsigned int currentOrder;

	/** The maximal number of threads that should be created to perform calculations in parallel. */
	unsigned int numberOfThreads;

	/** The amount of status outputs the program should do overt the cerr pipe.
	 *
	 * 0: No output
	 * 1: Some output of what the program is doing (Loaded, executed modules...)
	 * 2: 2: more output of what is happening (processed terms...)
	 * 3: Detailed output of what exactly is going on (processed bonds...)
	 *
	 * */
	unsigned int verbosityLevel;

};

#endif /* GLOBALCONFIGURATION_H_ */
