/*
 * ModelProperties.h
 *
 *  Created on: 02.12.2010
 *      Author: klagges
 */

#ifndef MODELPROPERTIES_H_
#define MODELPROPERTIES_H_

#include <Utilities/ParameterSet.h>

/** Representation of problem specific properties, that are of global interest.
 There is a global instance of this class in the Solver class.*/
class ModelProperties: public Utilities::ParameterSet {

public:

	void streamOutExample(std::ostream& stream);

	void streamIn(std::istream& stream);

	/** The number of bits used to represent the hilbert space of a single lattice site.*/
	unsigned int bitsPerLatticeSite;

	/** The number of expansion parameters*/
	unsigned int numberOfExpansionParameters;

	/** The smallest Number N, so that all perturbation operators can be written as the sum of the
	 \f$ 2N+1 \f$ T operators.

	 That means:
	 \f[
	 V = T_0 + T_1 + T_{-1} + ... + T_N + T_{-N}
	 \f]
	 Where \f$ T_i \f$ is an operator transferring a state in the n-th \f$ H_0 \f$ eigen space into a
	 state in the (n+i)-th \f$ H_0 \f$ eigen space.
	 */
	int Nmax;

	/** The distance between two neighboring energy levels of the unperturbed \f$ H_0 \f$ operator.
	 * If the \f$ H_0 \f$ spectrum is not equidistant, you should give the maximum distance of
	 * two neighboring energy levels. It is used to calculate the energy boarders to truncate
	 * the Hilbert space during the application of operators from not contributing states.*/
	double energyDelta;

};

#endif /* MODELPROPERTIES_H_ */
