/*
 * BonfilePlotter.cpp
 *
 *  Created on: 08.12.2011
 *      Author: kalis
 *
 */

#include <Modules/DataProcessor/BondfilePlotter/BondfilePlotter.h>
#include <Interface/StateIO.h>

// The constructor should initialize all data needed for the implementation.
// (i.e. the coefficients file)
BondfilePlotter::BondfilePlotter() {
	configuration.readFromConfigFile("BondfilePlotter.cfg");
	GRAPHcounter = 0;
}

// The method that is called by the solver class.
// In the module implementation this method is no longer virtual.
void BondfilePlotter::execute() {
	unsigned int sites = (sizeof(BaseState) * 8);
	std::list<unsigned int> KetSetToOne, BraSetToOne;
	for (unsigned int i=0;i<sites;i++){
		if(solver.inputStates.left.begin()->first.get(i)){
			BraSetToOne.push_back(i);
		}
		if(solver.inputStates.right.begin()->first.get(i)){
			KetSetToOne.push_back(i);
		}
	}
	std::list<graphObject>::iterator GRAPHITERATOR =
			configuration.allGraphs.begin();
	std::cout << solver.latticeProperties << std::endl;

	/** main routine to plot all bond-types */
	for (unsigned int i = 0; i < configuration.numberOfPerturbationOperators; i++) {
		/** creating instance of dot class to handle out */
		dot DOT;

		/** care about the file name here */
		std::string dotScript = "h";
		std::string tmp = convertInt((*GRAPHITERATOR).perturbationOperator);
		dotScript += tmp;
		tmp = convertInt(GRAPHcounter);
		GRAPHcounter++;
		dotScript += ("_" + tmp);
		DOT.createDotScript(dotScript);

		/** adjustments for directed and non-directed graphs */
		if ((*GRAPHITERATOR).directed) {
			std::string add = "";
			DOT.addCommandLineToScript("digraph g { \n {node [style=filled,color=red]");
			std::list<unsigned int>::iterator PARTICLEPOS = KetSetToOne.begin();
			unsigned int size = KetSetToOne.size();
			for(unsigned int j=0;j<size;j++){
				add += convertInt((*PARTICLEPOS));
				add += " ";
				PARTICLEPOS++;
			}
			DOT.addCommandLineToScript(add);
			DOT.addCommandLineToScript("} \n {node [style=filled,color=green]");
			PARTICLEPOS = BraSetToOne.begin();
			size = BraSetToOne.size();
			add = "";
			for(unsigned int j=0;j<size;j++){
				add += convertInt((*PARTICLEPOS));
				add += " ";
				PARTICLEPOS++;
			}
			DOT.addCommandLineToScript(add);
			DOT.addCommandLineToScript("} \n overlap=scale \n splines=true");
		} else {
			std::string add = "";
			DOT.addCommandLineToScript("graph g { \n {node [style=filled,color=red]");
			std::list<unsigned int>::iterator PARTICLEPOS = KetSetToOne.begin();
			unsigned int size = KetSetToOne.size();
			for(unsigned int j=0;j<size;j++){
				add += convertInt((*PARTICLEPOS));
				add += " ";
				PARTICLEPOS++;
			}
			DOT.addCommandLineToScript(add);
			DOT.addCommandLineToScript("} \n {node [style=filled,color=green]");
			PARTICLEPOS = BraSetToOne.begin();
			size = BraSetToOne.size();
			add = "";
			for(unsigned int j=0;j<size;j++){
				add += convertInt((*PARTICLEPOS));
				add += " ";
				PARTICLEPOS++;
			}
			DOT.addCommandLineToScript(add);
			DOT.addCommandLineToScript("} \n overlap=scale \n splines=true");
		}

		/** begin with first bond given by LatticeProvider */
		std::list<Bond>::iterator BONDITERATOR =
				solver.latticeProperties.bonds.begin();

		/** iterate over all bonds */
		while (BONDITERATOR != solver.latticeProperties.bonds.end()) {
			/** if we find the desired pertubation operator we take the line and ... */
			if (((*GRAPHITERATOR).perturbationOperator)
					== ((*BONDITERATOR).operatorNumber)) {
				std::list<unsigned int>::iterator SITEITERATOR =
						(*BONDITERATOR).sites.begin();
				std::multimap<unsigned int, unsigned int>::iterator
						INTERACTINGSITESITERATOR =
								(*GRAPHITERATOR).interactingSites.begin();
				for (unsigned int i = 0; i
						< (*GRAPHITERATOR).numberOfInteractions; i++) {
					/** ... write the line into dot/neato script, with order given by BondfilePlotter.cfg */
					std::string K, V, com;
					unsigned int k, v;
					advance(SITEITERATOR,
							(unsigned int) (*INTERACTINGSITESITERATOR).first);
					k = (*SITEITERATOR);
					SITEITERATOR = (*BONDITERATOR).sites.begin();
					advance(SITEITERATOR,
							(unsigned int) (*INTERACTINGSITESITERATOR).second);
					v = (*SITEITERATOR);
					K = convertInt(k);
					V = convertInt(v);
					/** make the script look pretty */
					if ((*GRAPHITERATOR).directed) {
						com = K + "->" + V + ";";

					} else {
						com = K + "--" + V + ";";
					}
					DOT.addCommandLineToScript(com);
					SITEITERATOR = (*BONDITERATOR).sites.begin();
					INTERACTINGSITESITERATOR++;
				}
			}
			BONDITERATOR++;
		}
		DOT.addCommandLineToScript("}");
		DOT.executeDotScript((*GRAPHITERATOR).directed, dotScript);
		DOT.deleteDotScript();
		GRAPHITERATOR++;
	}
}
;

std::string BondfilePlotter::convertInt(unsigned int number) {
	std::stringstream ss; //create a stringstream
	ss << number; //add number to the stream
	return ss.str(); //return a string with the contents of the stream
}
