/*
 * Bondfileplotter
 *
 *  Created on: 24.12.2010
 *      Author: kalis
 */

#ifndef BONDFILEPLOTTER_H_
#define BONDFILEPLOTTER_H_

#include <Modules/DataProcessor/DataProcessor.h>
#include <Utilities/ParameterSet.h>
#include <iostream>
#include <map>
#include <list>
#include <string>
#include <sstream>
#include <fstream>

class BondfilePlotter: public DataProcessor {

public:

	/** The constructor should initialize all data needed for the implementation.
	 (i.e. the coefficients file)*/
	BondfilePlotter();

	/** The virtual method that is called by the solver class.
	 In the module implementation this method is overloaded.*/
	void execute();

	std::string convertInt(unsigned int number);

	unsigned int GRAPHcounter;

private:

	/** Class to define a Graph */
	class graphObject{

	public:
		unsigned int perturbationOperator;
		unsigned int numberOfInteractions;
		std::multimap<unsigned int, unsigned int> interactingSites;
		bool directed;
	};

	/** Configuration Class, inherits all information read from cfg file */
	class Configuration: public Utilities::ParameterSet {

	public:

		void streamOutExample(std::ostream& stream);

		void streamIn(std::istream& stream);

		std::multimap<unsigned int, unsigned int> splitInteractingSites(std::string& a, BondfilePlotter::graphObject& G);
		bool isDirected(std::string a);
		void addElementToMapAndCropUnitlNext(std::string& a, size_t pos,BondfilePlotter::graphObject G, std::multimap<unsigned int, unsigned int>& RETURN);
		std::string cropUntilNextOperator(std::string& a);
		unsigned int extractNextOperator(std::string& a);

		/** Identifies how many Lattices to plot */
		unsigned int numberOfPerturbationOperators;

		/** boundary conditions */
		bool periodic;

		/** List of all Graphs to plot */
		std::list<graphObject> allGraphs;

	};

	/** The configuration */
	Configuration configuration;

	class dot {

	public:

		std::string nameOfDotScript;

		void createDotScript(std::string name);
		void addCommandLineToScript(std::string command);
		void executeDotScript(bool directed, std::string imageName);
		void deleteDotScript();

	};
};

#endif /* BONDFILEPLOTTER_H_ */
