/*
 * Configuration.cpp
 *
 *  Created on: 08.12.2011
 *      Author: kalis
 */

#include <Modules/DataProcessor/BondfilePlotter/BondfilePlotter.h>

void BondfilePlotter::Configuration::streamOutExample(std::ostream& stream) {

	stream << "# boundary conditions (1 = periodic) (0 = open)\n" << "\n \n "
			<< "# number of different bond indices (PerturbationOperatorNumbers)\n"
			<< "# each index will result in a spereate plot\n" << "\n \n"
			<< "# define structure of interacting sites\n"
			<< "# e.g: Bond entry:\n" << "# Format:\n"
			<< "# PerturbationOperatorNumber 	LaticeSiteNumbers;\n"
			<< "# 0 \t \t \t \t \t \t \t 12 13 14 15 16;\n"
			<< "# \t \t \t \t \t \t \t (0  1  2  3  4);\n"
			<< "# 1  \t \t \t \t \t \t \t 17 18;\n"
			<< "# \t \t \t \t \t \t \t (0  1);\n"
			<< "# then we like to create a star-like interaction for operator 0,\n"
			<< "# and a two site interaction (not directed) for operator 1:\n"
			<< "# 0:\n" << "# 2 -> 0;\n" << "# 2 -> 1;\n" << "# 2 -> 3;\n"
			<< "# 2 -> 4;\n" << "#\n" << "# 1:\n" << "# 0 -- 1;\n" << "#\n"
			<< "# these numbers should start with 0 and increase by +1 for every following site\n"
			<< "# since this structure will be used for every line in the bondfile\n"
			<< "# you can choose between directed graphs \"->\" or undirected graphs \"--\"\n";
}

void BondfilePlotter::Configuration::streamIn(std::istream& stream) {
	std::string input, res, seq;
	stream >> periodic >> numberOfPerturbationOperators;
	while (stream.good()) {
		getline(stream, input, '\n');
		res += input;
	}
	input = res;
	for (unsigned int i = 0; i < numberOfPerturbationOperators; i++) {
		graphObject G;
		G.perturbationOperator = extractNextOperator(input);
		seq = cropUntilNextOperator(input);
		G.interactingSites = splitInteractingSites(seq, G);
		G.numberOfInteractions = (unsigned int) G.interactingSites.size();
		allGraphs.push_back(G);
	}
}

std::multimap<unsigned int, unsigned int> BondfilePlotter::Configuration::splitInteractingSites(
		std::string& a, BondfilePlotter::graphObject& G) {
	std::multimap<unsigned int, unsigned int> RETURN;
	std::stringstream ss(std::stringstream::in | std::stringstream::out);
	size_t f;
	G.directed = isDirected(a);
	f = a.find_first_of(";");
	while (f < std::string::npos) {
		addElementToMapAndCropUnitlNext(a, f, G, RETURN);
		f = a.find_first_of(";", f);
	}
	return RETURN;
}

bool BondfilePlotter::Configuration::isDirected(std::string a) {
	size_t f;
	f = a.find(">");
	if (f != std::string::npos) {
		return true;
	}
	return false;
}

void BondfilePlotter::Configuration::addElementToMapAndCropUnitlNext(
		std::string& a, size_t pos, BondfilePlotter::graphObject G,
		std::multimap<unsigned int, unsigned int>& RETURN) {
	size_t f, f2;
	std::string key, value;
	unsigned int KEY = 0, VALUE = 0;
	std::stringstream ss(std::stringstream::in | std::stringstream::out);
	std::stringstream ss2(std::stringstream::in | std::stringstream::out);
	f = a.find_first_of("-");
	f2 = a.find_first_not_of("-", 0);
	key = a.substr(f2, f2 + f);
	ss << key;
	ss >> KEY;
	if (G.directed) {
		f = a.find_last_of(">", pos);
	} else {
		f = a.find_last_of("-", pos);
	}
	value = a.substr(f + 1, pos - f - 1);
	a.erase(0, pos + 1);
	ss2 << value;
	ss2 >> VALUE;
	RETURN.insert(std::pair<unsigned int, unsigned int>(KEY, VALUE));
}

std::string BondfilePlotter::Configuration::cropUntilNextOperator(
		std::string& a) {
	std::string seq;
	size_t f1, f2, f3;
	f1 = a.find_first_not_of(":");
	f2 = a.find_first_of(":");
	f3 = a.find_last_of(";", f2);
	seq = a.substr((int) f1, (int) f3 + 1);
	a.erase((int) f1, (int) f3 + 1);
	return seq;
}

unsigned int BondfilePlotter::Configuration::extractNextOperator(std::string& a) {
	unsigned int OP;
	std::stringstream ss(std::stringstream::in | std::stringstream::out);
	size_t f;
	std::string op;
	f = a.find_first_of(":");
	op = a.substr(0, (int) f);
	a.erase(0, (int) f + 1);
	ss << op;
	ss >> OP;
	return OP;
}

