/*
 * Coefficients.cpp
 *
 *  Created on: 22.11.2010
 *      Author: klagges
 */

#include <Modules/DataProcessor/Loewdin/DirectSolver/DirectSolver.h>
#include <Framework/Solver.h>

void Loewdin::DirectSolver::Coefficients::streamOutExample(std::ostream& stream) {

	stream << "# Here should be a coefficients file\n\n"
			<< "# No order present\n0";

}

void Loewdin::DirectSolver::Coefficients::streamIn(std::istream& stream) {

	unsigned int order;

	stream >> order;

	unsigned int minOrder = std::min(order, solver.globalConfiguration.order);

	if (minOrder < solver.globalConfiguration.order) {
		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Warning! Calculation up to order "
					<< solver.globalConfiguration.order
					<< " is requested, but only coefficients up to order "
					<< order << " are present!\n";
	}

	c.resize(minOrder);

	// Stream in the coefficients.
	Term t;

	stream >> t;

	while (stream.good()) {

		// We only read in the coefficients up to the desired order.
		if (t.order > minOrder)
			break;

		// Store the term directly.
		c[t.order - 1].push_back(t);

		if (operatorResults.find(t.operators) == operatorResults.end()) {

			// We have not seen a term with the same operator part before.
			// We parse it to transfer it into a list of operator descriptors.

			ListedOperator operatorResult;

			unsigned int vCounter = t.operatorOrder;
			unsigned int sCounter = 0;

			for (size_t i = t.operators.length() - 3; i > 0; i--) {

				// Count the number of S operators...
				switch (t.operators[i]) {
				case 'S':
					sCounter++;
					break;
					// between two V operators.
				case 'V':
					vCounter--;
					operatorResult.listedOperators.push_back(
							Utilities::OperatorDescription(true, vCounter,
									sCounter));
					sCounter = 0;
					break;
				}

			}

			// This is the final PV operator.
			operatorResult.listedOperators.push_back(
					Utilities::OperatorDescription(true, 0, 0));

			operatorResults.insert(
					std::map<std::string, ListedOperator>::value_type(
							t.operators, operatorResult));

		}

		stream >> t;

	}

}

