/*
 * DirectSolver.h
 *
 *  Created on: 01.12.2010
 *      Author: klagges
 */

#ifndef LOEWDINDIRECTSOLVER_H_
#define LOEWDINDIRECTSOLVER_H_

#include <Modules/DataProcessor/DataProcessor.h>
#include <Modules/DataProcessor/Loewdin/Common/Term.h>
#include <Modules/DataProcessor/Loewdin/Common/Configuration.h>
#include <Utilities/OperatorDescription.h>

namespace Loewdin {

/** Calculates the energy corrections according to the degenerated perturbation
 method in the projector operator formalism by Loewdin.

 The input states must be eigen states of the terms of the effective Hamiltonian.

 This module reads the Solver::inputStates.right member and writes the eigen energies
 into the Solver::outputAmplitude member of the solver object.

 In this implementation the eigen values of the operator parts of the coefficient terms
 are calculated by directly applying them to the input states and comparing the
 results to the input states. The energy correction of every order is calculated successively
 from this eigen values and the energy corrections of lower orders.
 */
class DirectSolver: public DataProcessor {

public:

	DirectSolver();

	void execute();

private:

	/**
	 * Represents an operator part of a term (like PVSVSSVP) and its eigenvalue.
	 */
	class ListedOperator {
	public:

		/** The eigen value of the operator represented by this object. */
		stateAmplitude eigenValue;

		/** The operator as a list of OperatorDescription objects. */
		std::list<Utilities::OperatorDescription> listedOperators;

	};

	/**
	 * Class to store and stream in Loewdin coefficients.
	 * Stores every term separately. Additionally all occurring
	 * operators are stored in a map of ListedOperator objects.
	 */
	class Coefficients: public Utilities::ParameterSet {

	public:

		void streamOutExample(std::ostream& stream);

		void streamIn(std::istream& stream);

		/** All coefficients as c[order][term of order]*/
		std::vector<std::list<Term> > c;

		/** To store the operator parts in a listed form. */
		std::map<std::string, Loewdin::DirectSolver::ListedOperator> operatorResults;

	};

	/** Configuration to chose from different coefficient files.*/
	Configuration configuration;

	/** The coefficients and some results.*/
	Coefficients coefficients;

};

}

#endif /* LOEWDINDIRECTSOLVER_H_ */
