/*
 * Coefficients.cpp
 *
 *  Created on: 19.02.2011
 *      Author: klagges
 */

#include <Modules/DataProcessor/Loewdin/TwoSideCommonSolver/TwoSideCommonSolver.h>
#include <Framework/Solver.h>

void Loewdin::TwoSideCommonSolver::Coefficients::streamOutExample(
		std::ostream& stream) {

	stream << "# Here should be a coefficients file\n\n"
			<< "# No order present\n0";

}

void Loewdin::TwoSideCommonSolver::Coefficients::streamIn(std::istream& stream) {

	unsigned int order;

	stream >> order;

	unsigned int minOrder = std::min(order, solver.globalConfiguration.order);

	if (minOrder < solver.globalConfiguration.order) {
		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Warning! Calculation up to order "
					<< solver.globalConfiguration.order
					<< " is requested, but only coefficients up to order "
					<< order << " are present!\n";
	}

	c.resize(minOrder);

	// Stream in the coefficients.
	Term t;

	stream >> t;

	while (stream.good()) {

		// We only read in the coefficients up to the desired order.
		if (t.order > minOrder)
			break;

		// Store the term directly.
		c[t.order - 1].push_back(t);

		if (operatorResults.find(t.operators) == operatorResults.end()) {

			// We have not seen a term with the same operator part before.
			// We parse it to transfer it into lists of operator descriptors.

			ListedOperator operatorResult;

			unsigned int vCounter = t.operatorOrder;
			unsigned int sCounter = 0;

			for (size_t i = t.operators.length() - 3; i > 0; i--) {

				switch (t.operators[i]) {
				case 'S':
					// We count the number of S operators...
					sCounter++;
					break;
				case 'V':
					// between two V operators.
					vCounter--;
					operatorResult.rightSideOperators.push_back(
							Utilities::OperatorDescription(true, vCounter,
									sCounter));
					sCounter = 0;
					break;
				}

			}

			// The operator list seen from the right side is the same as seen from the left side,
			operatorResult.leftSideOperators
					= operatorResult.rightSideOperators;

			// But with reverse order.
			operatorResult.leftSideOperators.reverse();

			// Additionally we have to mirror the remainingV property of the OperatorDescription
			for (std::list<Utilities::OperatorDescription>::iterator it =
					operatorResult.leftSideOperators.begin(); it
					!= operatorResult.leftSideOperators.end(); it++) {

				it->remainingV = t.operatorOrder - it->remainingV;

			}

			// Now we add the final PV (or VP) operator to the two lists.
			operatorResult.rightSideOperators.push_back(
					Utilities::OperatorDescription(true, 0, 0));
			operatorResult.leftSideOperators.push_back(
					Utilities::OperatorDescription(true, 0, 0));

			operatorResult.order = t.operatorOrder;


			//std::cout << t.operators << "\n" << operatorResult;

			operatorResults.insert(
					std::map<std::string, ListedOperator>::value_type(
							t.operators, operatorResult));

		}

		stream >> t;

	}

}

