/*
 * Term.cpp
 *
 *  Created on: 15.04.2011
 *      Author: kamfor
 */

#include <Modules/DataProcessor/PCUT/Common/Term.h>
#include <Framework/Solver.h>
#include <Utilities/Utilities.h>
#include <Utilities/TypeCast.h>

void PCUT::Term::apply(State &s) {

	solver.globalConfiguration.currentOrder = 0;

	for (std::list<char>::iterator it = operators.begin(); it
			!= operators.end(); it++) {

		solver.globalConfiguration.currentOrder++;

		State temp;
		solver.modules.pointerToModelProvider->applyT(s, temp, *it);
		s.swap(temp);

	}
}

// Stream out as a string like 3 T2T0....T-1 1/42
std::ostream& PCUT::operator<<(std::ostream& o, const PCUT::Term& d) {

	o << d.operators.size() << " ";

	for (std::list<char>::const_iterator it = d.operators.begin(); it
			!= d.operators.end(); it++)
		o << (int) *it << " ";

	o << d.factor;

	return o;

}

// Stream in from a string like 3 T2T0....T-1 1/42
std::istream& PCUT::operator>>(std::istream& i, PCUT::Term& d) {

	d.operators.clear();
	int temporder;
	d.lowestQPSectorReached = 0;
	short currentQPSector = 0;

	i >> temporder;

	if (!i.good())
		return i;

	for (int m = 1; m <= temporder; m++) {

		int tempoperator;
		i >> tempoperator;
		currentQPSector += tempoperator;
		if (currentQPSector < d.lowestQPSectorReached)
			d.lowestQPSectorReached = currentQPSector;
		d.operators.push_back((char) tempoperator);
	}

//	std::cout<<"\n";
//	for(std::list<char>::iterator it=d.operators.begin(); it!=d.operators.end(); it++ )std::cout<<(int)(*it)<<" ";
//	std::cout<<"\n";
//	std::cout<<" --> "<<d.lowestQPSectorReached<<"\n";

	mpz_class numerator;
	mpz_class denominator;

	i >> numerator;
	i >> denominator;
	if (denominator < 0) {
		std::cerr << "ERROR in coefficient file: denominator<0";
		exit(1);
	}

	rational factor(numerator, denominator);

	Utilities::typeCast(factor, d.factor);

	return i;
}

