/*
 * DirectSolver.cpp
 *
 *  Created on: 15.04.2010
 *      Author: kamfor
 *
 */

#include <Modules/DataProcessor/PCUT/DirectSolver/DirectSolver.h>
#include <Framework/Solver.h>

PCUT::DirectSolver::DirectSolver() {

	// First read the pcut configuration
	pcutConfiguration.readFromConfigFile("PCUTConfiguration.cfg");

	coefficients.numberOfQuasiParticles
			= pcutConfiguration.numberOfQuasiParticles;

	coefficients.readFromCoefficientsFile(
			pcutConfiguration.nameOfCoefficientsFile);

	numberOfBitsToDrop.readFromConfigFile("BitsToDrop.cfg");

	// 	std::cout << coefficients;
}

void PCUT::DirectSolver::execute() {

	for (std::list<Term>::iterator termIt = coefficients.c.begin(); termIt
			!= coefficients.c.end(); termIt++) {

		// We iterate over every term (of every order)

		if (solver.globalConfiguration.verbosityLevel > 1)
			std::cerr << "Processing term: " << *termIt << "\n";

		// Directly apply the operators of this term to the input state.
		State termOutputState(solver.inputStates.right);

		termIt->apply(termOutputState);

		// If numberOfBitsToDrop!=0, perform bit-truncation (see comments in "BitsToDrop.cfg")
		// Otherwise proceed as usual
		if (numberOfBitsToDrop.value) {
			State filteredTermOutputState;

			for (State::const_iterator it = termOutputState.begin(); it
					!= termOutputState.end(); it++) {

				BaseState truncatedBaseState = it->first;

				for (unsigned int index =
						(solver.globalProperties.bitsPerBaseState
								- numberOfBitsToDrop.value); index
						< solver.globalProperties.bitsPerBaseState; index++) {
					truncatedBaseState.unSet(index);

				}

				filteredTermOutputState.add(truncatedBaseState, it->second);
			}

			// Sum over all output states.
			solver.outputState.addMultiple(filteredTermOutputState,
					termIt->factor);
		} else {

			// Sum over all output states.
			solver.outputState.addMultiple(termOutputState, termIt->factor);
		}

	}

	if (solver.globalConfiguration.verbosityLevel > 1)
		std::cerr << "\n";

}
