/*
 * DirectSolver.h
 *
 *  Created on: 15.04.2010
 *      Author: kamfor
 *
 */

#ifndef PCUTDRECTSOLVER_H_
#define PCUTDRECTSOLVER_H_

#include <Modules/DataProcessor/PCUT/Common/Configuration.h>
#include <Modules/DataProcessor/DataProcessor.h>
#include <Modules/DataProcessor/PCUT/Common/Term.h>
#include <Utilities/OperatorDescription.h>
#include <Utilities/StreamableBaseType.h>
#include <set>

namespace PCUT {

/** Applies an effective Hamiltonian according to the degenerated perturbation
 method by Takahashi.

 This module reads the Solver::inputStates.right member and writes the states after
 the application of the effective Hamiltonian to the Solver::outputState member
 of the Solver object.

 In this implementation the output states are calculated by directly applying
 every coefficient term to the input state and summing over the results.
 */
class DirectSolver: public DataProcessor {

public:

	DirectSolver();

	void execute();

private:

	/** Class to store and stream in Takahashi coefficients.
	 * Stores every term in a set, with its operator part in a listed form.*/
	class Coefficients: public Utilities::ParameterSet {
	public:

		void streamOutExample(std::ostream& stream);

		void streamIn(std::istream& stream);

		/** The coefficients.*/
		std::list<Term> c;

		short numberOfQuasiParticles;
	};

	friend std::ostream& operator<<(std::ostream& o, DirectSolver::Coefficients& c);	

	/** PCUT initial configuration: coefficients-file, qp-number. */
	Configuration pcutConfiguration;

	/** The coefficients.*/
	Coefficients coefficients;

//	/** Number of quasi-particles in the current qp-sector. The default value is -1, meaning the qp-sector is not specified. */
//	short numberOfQuasiParticles;

	/** Number of bits to drop (spin background in the toric code). */
	Utilities::StreamableBaseType<unsigned int> numberOfBitsToDrop;
};

std::ostream& operator<<(std::ostream& o, DirectSolver::Coefficients& c);

}
#endif /* PCUTDRECTSOLVER_H_ */
