/*
 * TreeNode.cpp
 *
 *  Created on: 18.04.2011
 *      Author: klagges
 */

#include <Modules/DataProcessor/PCUT/TreeSolver/TreeSolver.h>
#include <Framework/Solver.h>

PCUT::TreeSolver::TreeNode::TreeNode() :
	op(-128), factor(0), currentOrder(1) {
}

PCUT::TreeSolver::TreeNode::TreeNode(const char c, const unsigned int currentOr) :
	op(c), factor(0), currentOrder(currentOr) {
}

void PCUT::TreeSolver::TreeNode::insertTerm(Term& term) {

	if (term.operators.size() == 0) {

		factor = term.factor;

		return;

	}

	std::list<PCUT::TreeSolver::TreeNode>::iterator childrenIt =
			children.begin();

	while (childrenIt != children.end() && childrenIt->op
			!= *term.operators.begin())
		childrenIt++;

	if (childrenIt == children.end()) {

		if (op == -128)

			childrenIt = children.insert(childrenIt, TreeNode(
					*term.operators.begin(), currentOrder));
		else
			childrenIt = children.insert(childrenIt, TreeNode(
					*term.operators.begin(), currentOrder + 1));

	}

	term.operators.pop_front();

	childrenIt->insertTerm(term);

}

void PCUT::TreeSolver::TreeNode::streamOutRecursively(Term t, std::ostream& o) {

	if (op != -128)
		t.operators.push_back(op);

	if (!Utilities::equalsZero(factor)) {
		t.factor = factor;
		o << t << "\n";
	}

	for (std::list<TreeNode>::iterator childrenIt = children.begin(); childrenIt
			!= children.end(); childrenIt++) {

		childrenIt->streamOutRecursively(t, o);

	}

}

void PCUT::TreeSolver::TreeNode::streamOutRecursivelyAsTree(std::list<char> s,
		std::ostream& o) {

	for (std::list<char>::const_iterator sit = s.begin(); sit != s.end(); sit++) {

		o << (int) *sit << " ";

	}

	o << " Node: " << (int) op << " currentOrder:" << currentOrder;
	if (!Utilities::equalsZero(factor))
		o << " Factor: " << factor;
	o << "\n";

	if (op != -128)
		s.push_back(op);

	for (std::list<TreeNode>::iterator childrenIt = children.begin(); childrenIt
			!= children.end(); childrenIt++) {

		childrenIt->streamOutRecursivelyAsTree(s, o);

	}
}

void PCUT::TreeSolver::TreeNode::sortRecursively() {

	children.sort();

	for (std::list<TreeNode>::iterator childrenIt = children.begin(); childrenIt
			!= children.end(); childrenIt++) {

		childrenIt->sortRecursively();

	}

}

void PCUT::TreeSolver::TreeNode::calculateOutputStateRecursively(
		State & inputState, State & outputState, Term t) {

	solver.globalConfiguration.currentOrder = currentOrder;

	if (op == -128) {
		// This is for the root node only. No operator is associated with it and we simply call all children.

		for (std::list<TreeNode>::iterator childrenIt = children.begin(); childrenIt
				!= children.end(); childrenIt++) {

			childrenIt->calculateOutputStateRecursively(inputState,
					outputState, t);

		}

		return;

	}

	t.operators.push_back(op);

	// Apply the associated operator.
	solver.modules.pointerToModelProvider->applyT(inputState,
			intermediateState, op);

	if (!Utilities::equalsZero(factor)) {
		// And if a term is assoziated with this node add it to the output state.

		t.factor = factor;

		solver.outputState.addMultiple(intermediateState, factor);

		if (solver.globalConfiguration.verbosityLevel > 1)
			std::cerr << "Applying term: " << t << "\n";

	}

	// Recursive call of all children.
	for (std::list<TreeNode>::iterator childrenIt = children.begin(); childrenIt
			!= children.end(); childrenIt++) {

		childrenIt->calculateOutputStateRecursively(intermediateState,
				outputState, t);

	}

	intermediateState.clear();

}

bool PCUT::operator<(const TreeSolver::TreeNode& l,
		const TreeSolver::TreeNode& r) {

	// Sort the children of a TreeNode so, that T_c with a high c are processed first.
	if (l.op > r.op)
		return true;

	return false;

}
