/*
 * Coefficients.cpp
 *
 *  Created on: 04.05.2011
 *      Author: klagges
 */

#include <Modules/DataProcessor/PCUT/TwoSideSolver/TwoSideSolver.h>
#include <Framework/Solver.h>

void PCUT::TwoSideSolver::Coefficients::streamOutExample(std::ostream& stream) {

	stream << "# Here should be a coefficients file.\n";

}

void PCUT::TwoSideSolver::Coefficients::streamIn(std::istream& stream) {

	// Stream in the coefficients.
	Term t;

	stream >> t;

	while (stream.good()) {

		// We only read in the coefficients up to the desired order.
		if (t.operators.size() > solver.globalConfiguration.order)
			break;

//		// And skip the coefficients with too low order.
//		if (t.operators.size() >= solver.globalConfiguration.minOrder)
//			c.push_front(t);

		// And skip the coefficients with too low order.
		if (t.operators.size() >= solver.globalConfiguration.minOrder && (numberOfQuasiParticles == -1 || numberOfQuasiParticles
				+ t.lowestQPSectorReached >= 0))
			c.push_front(t);

		stream >> t;
	}

	if (c.begin()->operators.size() < solver.globalConfiguration.order) {
		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Warning! Calculation up to order "
					<< solver.globalConfiguration.order
					<< " is requested, but only coefficients up to order "
					<< c.begin()->operators.size() << " are present!\n";
	}
}

std::ostream& PCUT::operator<<(std::ostream& o,
		PCUT::TwoSideSolver::Coefficients& c) {

	for (std::list<PCUT::Term>::reverse_iterator it = c.c.rbegin(); it
			!= c.c.rend(); it++) {

		o << *it << std::endl;
	}

	return o;
}
