/*
 * Coefficients.cpp
 *
 *  Created on: 06.02.2011
 *      Author: klagges
 */

#include <Modules/DataProcessor/Takahashi/DirectFirstApplicationSolver/DirectFirstApplicationSolver.h>
#include <Framework/Solver.h>

void Takahashi::DirectFirstApplicationSolver::Coefficients::streamOutExample(
		std::ostream& stream) {

	stream << "# Here should be a coefficients file\n\n"
			<< "# No order present\n0";

}

void Takahashi::DirectFirstApplicationSolver::Coefficients::streamIn(
		std::istream& stream) {

	unsigned int order;

	stream >> order;

	unsigned int minOrder = std::min(order, solver.globalConfiguration.order);

	if (minOrder < solver.globalConfiguration.order) {

		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Warning! Calculation up to order "
					<< solver.globalConfiguration.order
					<< " is requested, but only coefficients up to order "
					<< order << " are present!\n";
	}

	// Stream in the coefficients.
	Term t;

	stream >> t;

	while (stream.good()) {

		// We only read in the coefficients up to the desired order.
		if (t.order > minOrder)
			break;

		// And skip terms with too low order.
		if (t.order >= solver.globalConfiguration.minOrder) {

			ListedTerm listedTerm;

			int globalVCounter = t.order;
			int i = t.operators.length() - 2;

			// We want to create an OperatorDescription object for ever V operator in the term.
			while (globalVCounter > 0) {

				unsigned int localVCounter = 0;

				// We search for the next P operator in the term.
				size_t lastPposition = t.operators.find_last_of('P', i - 1);
				int lastPposition_int;

				if (lastPposition == std::string::npos)
					lastPposition_int = -1;
				else
					lastPposition_int = lastPposition;

				// And count the number of V operators until this P operator.
				// This is needed for the OperatorDescription::remainingV entries.
				for (int j = lastPposition_int + 1; j <= i - 1; j++) {
					if (t.operators[j] == 'V')
						localVCounter++;
				}

				while (localVCounter > 0) {
					// For every V operator

					// we count the number of S operators that need to be applied
					size_t lastVposition = t.operators.find_last_of('V', i - 1);

					unsigned int SCounter = i - lastVposition - 1;

					i = lastVposition;

					// and create the according OperatorDescription object.
					if (lastPposition_int != -1)
						listedTerm.listedOperators.push_back(
								Utilities::OperatorDescription(true,
										localVCounter, SCounter));
					else
						listedTerm.listedOperators.push_back(
								Utilities::OperatorDescription(true, 10000,
										SCounter));

					globalVCounter--;
					localVCounter--;

				}

				unsigned int SCounter = i - lastPposition_int - 1;

				if (lastPposition_int != -1)
					listedTerm.listedOperators.push_back(
							Utilities::OperatorDescription(true, localVCounter,
									SCounter));
				else
					listedTerm.listedOperators.push_back(
							Utilities::OperatorDescription(true, 10000,
									SCounter));

				i = lastPposition_int - 1;
				globalVCounter--;
				localVCounter--;

			}

			if (listedTerm.listedOperators.size() > 0) {

				listedTerm.listedOperators.begin()->applyV = false;

				listedTerm.term = t;

				c.insert(listedTerm);

			}

		}

		stream >> t;

	}

}
