/*
 * TakahashiDrectSolver.cpp
 *
 *  Created on: 17.11.2010
 *      Author: klagges
 *
 */

#include <Modules/DataProcessor/Takahashi/DirectFirstApplicationSolver/DirectFirstApplicationSolver.h>
#include <Framework/Solver.h>

Takahashi::DirectFirstApplicationSolver::DirectFirstApplicationSolver() {

	nameOfCoefficientsFile.readFromConfigFile("TakahashiConfiguration.cfg");

	coefficients.readFromCoefficientsFile(nameOfCoefficientsFile.value);

}

void Takahashi::DirectFirstApplicationSolver::execute() {

	for (std::set<ListedTerm>::reverse_iterator termIt =
			coefficients.c.rbegin(); termIt != coefficients.c.rend(); termIt++) {

		// We iterate over every term (of every order)

		if (solver.globalConfiguration.verbosityLevel > 1)
			std::cerr << "Processing term: " << termIt->term.factor << " "
					<< termIt->term.operators << "\n";

		// Directly apply the operators of this term to the input state.
		State termOutputState(solver.inputStates.right);

		for (std::list<Utilities::OperatorDescription>::const_iterator
				operatorIt = termIt->listedOperators.begin(); operatorIt
				!= termIt->listedOperators.end(); operatorIt++) {

			operatorIt->apply(termOutputState);

		}

		// Sum over all output states.
		solver.outputState.addMultiple(termOutputState, termIt->term.factor);

		if (solver.globalConfiguration.verbosityLevel > 2)
			std::cerr << "\n";

	}

	if (solver.globalConfiguration.verbosityLevel > 1)
		std::cerr << "\n";

}
