/*
 * DirectSolver.h
 *
 *  Created on: 17.11.2010
 *      Author: klagges
 *
 */

#ifndef TAKAHASHIDRECTSOLVER_H_
#define TAKAHASHIDRECTSOLVER_H_

#include <Modules/DataProcessor/DataProcessor.h>
#include <Modules/DataProcessor/Takahashi/Common/Term.h>
#include <Utilities/OperatorDescription.h>
#include <Utilities/StreamableBaseType.h>
#include <set>

namespace Takahashi {

/** Applies an effective Hamiltonian according to the degenerated perturbation
 method by Takahashi.

 This module reads the Solver::inputStates.right member and writes the states after
 the application of the effective Hamiltonian to the Solver::outputState member
 of the Solver object.

 In this implementation the output states are calculated by directly applying
 every coefficient term to the input state and summing over the results.
 */
class DirectSolver: public DataProcessor {

public:

	DirectSolver();

	void execute();

private:

	/**
	 * Represents a term of the Takahashi coefficient file. The operator part
	 * is additionally stored in a listed form.
	 */
	class ListedTerm {
	public:

		/** The term represented by this object */
		Term term;

		/** The operator part of this term as a list of OperatorDescription objects.*/
		std::list<Utilities::OperatorDescription> listedOperators;

	};
	friend bool operator<(const DirectSolver::ListedTerm& l,
			const DirectSolver::ListedTerm& r);
	friend std::ostream& operator<<(std::ostream& o, DirectSolver::ListedTerm& d);

	/** Class to store and stream in Takahashi coefficients.
	 * Stores every term in a set, with its operator part in a listed form.*/
	class Coefficients: public Utilities::ParameterSet {
	public:

		void streamOutExample(std::ostream& stream);

		void streamIn(std::istream& stream);

		/** The coefficients.*/
		std::set<ListedTerm> c;

	};

	/** Configuration to choose from different configuration files.*/
	Utilities::StreamableBaseType<std::string> nameOfCoefficientsFile;

	/** The coefficients.*/
	Coefficients coefficients;

};

/** Used to sort the coefficients from higher to lower order.
 */
bool operator<(const DirectSolver::ListedTerm& l,
		const DirectSolver::ListedTerm& r);

/** Stream out as a string.*/
std::ostream& operator<<(std::ostream& o, DirectSolver::ListedTerm& d);

}
#endif /* TAKAHASHIDRECTSOLVER_H_ */
