/*
 * Coefficients.cpp
 *
 *  Created on: 06.02.2011
 *      Author: klagges
 */

#include <Modules/DataProcessor/Takahashi/TreeSolver/TreeSolver.h>
#include <Modules/DataProcessor/Takahashi/Common/Term.h>
#include <Framework/Solver.h>

void Takahashi::TreeSolver::Coefficients::streamOutExample(std::ostream& stream) {

	stream << "# Here should be a coefficients file\n\n"
			<< "# No order present\n0";

}

void Takahashi::TreeSolver::Coefficients::streamIn(std::istream& stream) {

	unsigned int order;

	stream >> order;

	unsigned int minOrder = std::min(order, solver.globalConfiguration.order);

	if (minOrder < solver.globalConfiguration.order) {

		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Warning! Calculation up to order "
					<< solver.globalConfiguration.order
					<< " is requested, but only coefficients up to order "
					<< order << " are present!\n";
	}

	// The root node is always a P operator node.
	root.op = Utilities::OperatorDescription('P');

	// Stream in the coefficients.
	Term t;

	stream >> t;

	while (stream.good()) {

		// We only read in the coefficients up to the desired order.
		if (t.order > minOrder)
			break;

		// And skip terms with too low order.
		if (t.order >= solver.globalConfiguration.minOrder) {

			// Cut off the initial P operator.
			t.operators.resize(t.operators.size() - 1);

			// And insert the term into the tree.
			root.insertTerm(t);

		}

		stream >> t;

	}

}
