/*
 * Coefficients.cpp
 *
 *  Created on: 22.02.2011
 *      Author: klagges
 */

#include <Modules/DataProcessor/Takahashi/TwoSideSolver/TwoSideSolver.h>
#include <Framework/Solver.h>

void Takahashi::TwoSideSolver::Coefficients::streamOutExample(
		std::ostream& stream) {

	stream << "# Here should be a coefficients file\n\n"
			<< "# No order present\n0";

}

void Takahashi::TwoSideSolver::Coefficients::streamIn(std::istream& stream) {

	unsigned int order;

	stream >> order;

	unsigned int minOrder = std::min(order, solver.globalConfiguration.order);

	if (minOrder < solver.globalConfiguration.order) {

		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Warning! Calculation up to order "
					<< solver.globalConfiguration.order
					<< " is requested, but only coefficients up to order "
					<< order << " are present!\n";
	}

	// Stream in the coefficients.
	Term t;

	stream >> t;

	while (stream.good()) {

		// We only read in the coefficients up to the desired order.
		if (t.order > minOrder)
			break;

		// And skip terms with too low order.
		if (t.order >= solver.globalConfiguration.minOrder) {

			// We want to create an OperatorDescription object for ever V operator in the term.
			ListedTerm operatorResult;

			int globalVCounter = t.order - 1;
			int i = t.operators.length() - 2;

			while (globalVCounter >= 0) {

				unsigned int localVCounter = 0;

				// We search for the next P operator in the term.
				size_t lastPposition = t.operators.find_last_of('P', i - 1);

				// And count the number of V operators until this P operator.
				// This is needed for the OperatorDescription::remainingV entries.
				for (int j = lastPposition + 1; j < i - 1; j++) {
					if (t.operators[j] == 'V')
						localVCounter++;
				}

				while (localVCounter > 0) {

					// For every V operator

					// we count the number of S operators that need to be applied
					size_t lastVposition = t.operators.find_last_of('V', i - 1);

					unsigned int SCounter = i - lastVposition - 1;

					i = lastVposition;

					// and create the according OperatorDescription object.
					operatorResult.rightSideOperators.push_back(
							Utilities::OperatorDescription(true, localVCounter,
									SCounter));

					globalVCounter--;
					localVCounter--;

				}

				// This is the final PV operator.
				operatorResult.rightSideOperators.push_back(
						Utilities::OperatorDescription(true, 0, 0));

				globalVCounter--;
				localVCounter--;
				i = i - 2;

			}

			// Now we repeat the whole procedure beginning from the left side.

			globalVCounter = t.order - 1;
			i = 1;

			while (globalVCounter >= 0) {

				unsigned int localVCounter = 0;

				// We search for the next P operator in the term.
				size_t firstPposition = t.operators.find_first_of('P', i + 1);

				// And count the number of V operators until this P operator.
				// This is needed for the OperatorDescription::remainingV entries.
				for (unsigned int j = i + 1; j < firstPposition; j++) {
					if (t.operators[j] == 'V')
						localVCounter++;
				}

				while (localVCounter > 0) {
					// For every V operator

					// we count the number of S operators that need to be applied
					size_t firstVposition = t.operators.find_first_of('V',
							i + 1);

					unsigned int SCounter = firstVposition - i - 1;

					i = firstVposition;

					// and create the according OperatorDescription object.
					operatorResult.leftSideOperators.push_back(
							Utilities::OperatorDescription(true, localVCounter,
									SCounter));

					globalVCounter--;
					localVCounter--;

				}

				// This is the final VP operator.
				operatorResult.leftSideOperators.push_back(
						Utilities::OperatorDescription(true, 0, 0));

				globalVCounter--;
				localVCounter--;
				i = i + 2;

			}

			operatorResult.term = t;

			c.insert(operatorResult);

		}

		stream >> t;

	}

}

