/*
 * TemplateDataProcessor.h
 *
 *  Created on: 17.11.2010
 *      Author: klagges
 *
 * A template for implementation of a DataProcessor. To create your own DataProcessors:
 *
 * - Create a folder for your module. Copy and rename TemplateDataProcessor.h
 *   and TemplateDataProcessor.cpp to it. Adapt the class name.
 *
 * - Add your .cpp file to the Makefile to make sure it will be compiled.
 *
 * - Register the new module in the Framework/Module.cpp file, by including the
 *   new header file and adding a new MODULE(...) line.
 *
 * - Be aware, that module implementations are only instantiated and called, if
 *   they are entered in the Config/Modules.cfg file.
 *
 */

#ifndef TEMPLATEDATAPROCESSOR_H_
#define TEMPLATEDATAPROCESSOR_H_

#include <Modules/DataProcessor/DataProcessor.h>
#include <iostream>
//#include <Framework/TemplateParameterSet.h>

/** A template for a DataProcessor.
 * You find an instruction how to create new a new DataProcessor in
 * Modules/DataProcessor/TemplateDataProcessor/TemplateDataProcessor.h.*/
class TemplateDataProcessor: public DataProcessor {

public:

	/** The constructor should initialize all data needed for the implementation.
	 (i.e. the coefficients file)*/
	TemplateDataProcessor();

	/** The virtual method that is called by the solver class.
	 In the module implementation this method is overloaded.*/
	void execute();

private:

	// Add members and methods you need for the calculations of your module here.

	//TemplateParameterSet templateParameterSet;


};

#endif /* TEMPLATEDATAPROCESSOR_H_ */
