/*
 * DefaultWriter.cpp
 *
 *  Created on: 03.12.2010
 *      Author: klagges
 */

#include <Modules/DataWriter/DefaultWriter/DefaultWriter.h>
#include <Interface/StateIO.h>
#include <iomanip>
#include <fstream>

DefaultWriter::DefaultWriter() {

	configuration.readFromConfigFile("DefaultWriter.cfg");

}

void DefaultWriter::execute() {

	if (configuration.outputInFile) {

		std::stringstream fileName;

		fileName << solver.configurationDirectory << "_"
				<< solver.modules.nameOfModelProvider << "_"
				<< solver.modules.nameOfLatticeProvider << "_"
				<< solver.modules.nameOfDataProcessor << "_Order"
				<< solver.globalConfiguration.order << ".txt";

		std::ofstream ofs(fileName.str().c_str(),
				std::ios_base::out | std::ios_base::trunc);

		if (ofs.is_open()) {

			if (solver.globalConfiguration.verbosityLevel > 0)
				std::cerr << "Writing result to file :" << fileName.str()
						<< "\n";



			ofs.precision(configuration.digits);

			// Give out the input states:

			if (configuration.leftInputState) {
				if (configuration.outputExplanations)
					ofs << "Left input state (bra) :\n";
				ofs << solver.inputStates.left << "\n";

			}

			if (configuration.rightInputState) {
				if (configuration.outputExplanations)
					ofs << "Right input state (ket) :\n";
				ofs << solver.inputStates.right << "\n";

			}

			// Give out the output state.
			if (configuration.outputState) {
				if (configuration.outputExplanations)
					ofs << "Output state :\n";
				ofs << solver.outputState << "\n";

			}

			// And the output amplitude:
			if (configuration.outputAmplitude) {
				if (configuration.outputExplanations)
					ofs << "Output amplitude :\n";
				ofs << solver.outputAmplitude << "\n";

			}

			return;

		}

	}

	std::cout.precision(configuration.digits);

	// Give out the input states:

	if (configuration.leftInputState) {
		if (configuration.outputExplanations)
			std::cout << "Left input state (bra) :\n";
		std::cout << solver.inputStates.left << "\n";

	}

	if (configuration.rightInputState) {
		if (configuration.outputExplanations)
			std::cout << "Right input state (ket) :\n";
		std::cout << solver.inputStates.right << "\n";

	}

	// Give out the output state.
	if (configuration.outputState) {
		if (configuration.outputExplanations)
			std::cout << "Output state :\n";
		std::cout << solver.outputState << "\n";

	}

	// And the output amplitude:
	if (configuration.outputAmplitude) {
		if (configuration.outputExplanations)
			std::cout << "Output amplitude :\n";
		std::cout << solver.outputAmplitude << "\n";

	}

}
