/*
 * ProjectorWriter.cpp
 *
 *  Created on: 03.12.2010
 *      Author: klagges
 */
#include <Modules/ModelProvider/GeneralLatticeModel/GeneralLatticeModel.h>
#include <Modules/DataWriter/ProjectorWriter/ProjectorWriter.h>
#include <Interface/StateIO.h>
#include <Interface/Interface.h>
#include <Interface/BaseState.h>
#include <Interface/BaseStateIO.h>
#include <Interface/ExpandedAmplitudeIO.h>
#include <Utilities/Utilities.h>
#include <iomanip>
#include <fstream>
#include <string>
#include <gmpxx.h>
#include <stdio.h>

#define DEFAULT_MPF_CLASS_PRECISION 128

ProjectorWriter::ProjectorWriter() {

	configuration.readFromConfigFile("ProjectorWriter.cfg");

}

void ProjectorWriter::execute() {

 

	if (configuration.outputInFile) {

 


		std::stringstream fileName;

		fileName << solver.configurationDirectory << "_"
				<< solver.modules.nameOfModelProvider << "_"
				<< solver.modules.nameOfLatticeProvider << "_"
				<< solver.modules.nameOfDataProcessor << "_Order"
				<< solver.globalConfiguration.order << ".txt";

		std::ofstream ofs(fileName.str().c_str(),
				std::ios_base::out | std::ios_base::trunc);

		if (ofs.is_open()) {

			if (solver.globalConfiguration.verbosityLevel > 0)
				std::cerr << "Writing result to file :" << fileName.str()
						<< "\n";



			ofs.precision(configuration.digits);

			// Give out the input states:

			if (configuration.leftInputState) {
				if (configuration.outputExplanations)
					ofs << "Left input state (bra) :\n";
				ofs << solver.inputStates.left << "\n";

			}

			if (configuration.rightInputState) {
				if (configuration.outputExplanations)
					ofs << "Right input state (ket) :\n";
				ofs << solver.inputStates.right << "\n";

			}

			// Give out the output state.
			if (configuration.outputState) {
				if (configuration.outputExplanations)
					ofs << "Output state :\n";
				ofs << solver.outputState << "\n";

			}

		

			// And the output amplitude:
			if (configuration.outputAmplitude) {
				if (configuration.outputExplanations)
					ofs << "Output amplitude :\n";
				ofs << solver.outputAmplitude << "\n";
				
			}

			return;

		}

	}

	std::cout.precision(configuration.digits);

	// Give out the input states:

	if (configuration.leftInputState) {
		if (configuration.outputExplanations)
			std::cout << "Left input state (bra) :\n";
		std::cout << solver.inputStates.left << "\n";

	}

	if (configuration.rightInputState) {
		if (configuration.outputExplanations)
			std::cout << "Right input state (ket) :\n";
		std::cout << solver.inputStates.right << "\n";

	}

	// Give out the output state.
	if (configuration.outputState) {
		if (configuration.outputExplanations)
			std::cout << "Output state :\n";
		std::cout << solver.outputState << "\n";

	}

	// And the output amplitude:
	if (configuration.outputAmplitude) {
		if (configuration.outputExplanations)
			std::cout << "Output amplitude :\n";
		std::cout << solver.outputAmplitude << "\n";

	}

	//New!
	std::cout<<"Ab hier mal was eigenes: "<<std::endl;
	if (solver.outputState.size() != 0) {	
	  for (State::const_iterator it = solver.outputState.begin(); it
		 != solver.outputState.end(); it++) {
	   
	    std::string stringOfBits;
	    std::stringstream tmpStream;

	    tmpStream << (it->first);
	    stringOfBits = tmpStream.str();

	    stringOfBits.erase(0, stringOfBits.find_first_of("|") + 1);
	    stringOfBits.erase(stringOfBits.find_first_of(">"), stringOfBits.size()- stringOfBits.find_first_of(">"));
	    
	    unsigned int counter = 0;
	    unsigned int gate = 0;

	    for (std::string::const_iterator str_it = stringOfBits.begin(); str_it
		   < stringOfBits.end(); str_it++) {

	      if (((*str_it) == '1')&&(gate<(stringOfBits.size()-solver.latticeProperties.bonds.size()+1))) {
		counter += 1;
	      }

	      gate += 1;
	    }
	    
	    //This is the relevant contribution for reference state
	    if (counter==0){
	      std::cout<<"Relevant contribution: "<<(it->second)<<std::endl;
	      std::cout<<"Number of plaquettes: "<<solver.latticeProperties.bonds.size()<<std::endl;
	      std::cout<<"Number of terms in amplitude: "<<(it->second).size()<<std::endl;
	  
	      //Put h0->-h0, Multiply with normalization factor (1+\phi)^#plaquettes
	     
	      std::string s="1.6180339887498948482045868343656381177203091798058";
	      std::string s2="1.0";

	      mpf_class phi(s,500);
	      mpf_class one(s2,500);

	      mpf_class N=one+phi*phi;
	      for (int j=1;j<solver.latticeProperties.bonds.size();j++){
		N=N*(one+phi*phi);
	      }
	    
	      std::cout<<"Normalization: "<<N<<std::endl;

	      std::cout<<"Final contribution: "<<std::endl;
	      //std::cout<<"e[0]:=";
	      for (Utilities::associative_list<Exponents, amplitude>::iterator it2= (it->second).begin(); it2!= (it->second).end(); it2++) {
		if (((int)(it2->first).e[0])%2==0) {
		  if (abs(it2->second)>0.0000000000000000000000000000001) {
		    it2->second=N*it2->second;
		    if (it2->second<0) {
		      std::cout<<it2->second<<"*";
		      std::cout<<it2->first;
		    }
		    else {
		      std::cout<<"+"<<it2->second<<"*";
		      std::cout<<it2->first;
		    }
		  }
		}
		else {
		   if (abs(it2->second)>0.0000000000000000000000000000001) {
		     it2->second=-1.0*N*it2->second;
		      if (it2->second<0) {
			std::cout<<it2->second<<"*";
			std::cout<<it2->first;
		      }
		      else {
			std::cout<<"+"<<it2->second<<"*";
			std::cout<<it2->first;
		      }
		   }
		}
	      }    
	  
	      //std::cout<<";"<<std::endl;
	    }
	  }
	}

}
