/*
 * DefaultWriter.h
 *
 *  Created on: 03.12.2010
 *      Author: klagges
 */

#ifndef PROJECTORWRITER_H_
#define PROJECTORWRITER_H_

#include <Modules/DataWriter/DataWriter.h>
#include <Utilities/ParameterSet.h>
#include <iostream>

/** A DataWriter to write all relevant members of the Solver object to std::cout or into a file.
 * It can be configured via a configuration file.*/
class ProjectorWriter: public DataWriter {

public:

	ProjectorWriter();

	void execute();

private:

	/** Configuration for the ProjectorWriter.*/
	class Configuration: public Utilities::ParameterSet {

	public:

		void streamOutExample(std::ostream& stream);

		void streamIn(std::istream& stream);

		/** 0 = write to std::cout, 1 = write to file.*/
		bool outputInFile;

		/** 0 = only give out the data, 1 = additionally give out the explanation of the data.*/
		bool outputExplanations;

		/** Give out the left input state.*/
		bool leftInputState;

		/** Give out the right input state.*/
		bool rightInputState;

		/** Give out the output state.*/
		bool outputState;

		/** Give out the output amplitude.*/
		bool outputAmplitude;

		/** The number of digits the should be given out for floating point values.*/
		unsigned int digits;

	};

	/** The configuration of the ProjectorWriter.*/
	Configuration configuration;

};

#endif /* PROJECTORWRITER_H_ */
