/*
 * ClusterInFieldDispersion.h
 *
 *  Created on: 26.10.2010
 *      Author: kalis
 *
 *
 */

#ifndef CLUSTERINFIELDDISPERSION_H_
#define CLUSTERINFIELDDISPERSION_H_

#include <Modules/LatticeProvider/RotatedClusterDispersion/RotatedClusterDispersion.h>
#include <Modules/LatticeProvider/LatticeProvider.h>
#include <Utilities/ParameterSet.h>
#include <Interface/Interface.h>
#include <Interface/Bond.h>

/** Lattice Provider for Rotated Cluster in Field
 * Rotated Cluster in Field is the more general model
 * because it is only limited by the Linked-Cluster-Theorem.
 * Cluster in Field LatticeProvider should contain the same
 * functions. We could only use some more symmetries and limitations
 * of the model, therefore specified functions are redeclared here.
 *  */

class ClusterInFieldDispersion: public RotatedClusterDispersion {

public:
	/** The constructor should initialize all data needed for the implementation.*/
	ClusterInFieldDispersion();

	/** In the module implementations this method needs to be overloaded.*/
	bool execute();

	/** functions for manipulating hopping list e.g. for calculating just one hopping */
	void initializeCompleteHoppingList();

	/** creating optimzed bondfile for the specific problem */
	void createOptimzedBondfile(
			std::list<std::vector<unsigned int>>::iterator it_erase);

	/** rename or recount marked bonds */
	std::list<Bond> renameBondsAndStates(unsigned int bonds[300][300],
			unsigned int sx, unsigned int sy, unsigned int fx, unsigned int fy,
			BaseState& initial, BaseState& final);

	/** creation of specified bond files for Cluster in Field */
	std::list<Bond> createPeriodicSquareCluster(unsigned int size,
			unsigned int Dx, unsigned int Dy, BaseState& initial,
			BaseState& final);
	std::list<Bond> createPeriodicRectangleCluster(unsigned int nx,
			unsigned int ny, unsigned int Dx, unsigned int Dy,
			unsigned int periodicRectangleSize, BaseState& initial,
			BaseState& final);
	std::list<Bond> createPeriodicDiamondCluster(unsigned int Dx,
			unsigned int Dy, unsigned int periodicDiamondSize,
			BaseState& initial, BaseState& final);

	/** function compares bond sizes and insert smaller bond-setting into solver.latticeProperties.bonds */
	void comparePeriodicRectangleAndOpenBoundary(unsigned int nx,
			unsigned int ny, unsigned int x, unsigned int y,
			unsigned int periodicRectangleSize, BaseState& initial,
			BaseState& final);
	void comparePeriodicDiamondAndOthers(unsigned int nx, unsigned int ny,
			unsigned int x, unsigned int y, unsigned int periodicDiamondSize,
			BaseState& initial, BaseState& final);

	/** output functions; should be redefined due to sorting options for maple command :) */
	void consoleOutput();
	void fileOutput();
	void consoleOutputWithDispersion();
	void fileOutputWithDispersion();
	void output();

private:

	/** Configuration class needs to be implementend for streiming in config files */
	class Configuration: public Utilities::ParameterSet {

	public:

		void streamOutExample(std::ostream& stream);
		void streamIn(std::istream& stream);

		std::list<std::vector<unsigned int> > dist;
		std::list<std::vector<unsigned int> > dist_copy;

		/** parameters to control caclulation and output */
		bool startWith1QP;
		bool outputControlQP;
		bool generate_dispersion;
		bool file_out;
		bool console_out;
		bool executed;
		bool fullHopping;

	};

	/** list of amlitudes for specific hopping element */
	std::list<stateAmplitude> results;
	std::list<stateAmplitude> results_copy;

	/** The configuration */
	Configuration configuration;

};

#endif /* CLUSTERINFIELDDISPERSION_H_ */
