/*
 * RotatedClusterGapAndGroundState.cpp
 *
 *  Created on: 21.09.2011
 *      Author: klagges
 */

#include <Modules/LatticeProvider/ClusterInFieldGapAndGroundState/ClusterInFieldGapAndGroundState.h>
#include <Framework/Solver.h>
#include <Interface/StateIO.h>

ClusterInFieldGapAndGroundState::ClusterInFieldGapAndGroundState() {

	progress = Progress::initial;

	std::cout.precision(16);
	newBond.index = 0;
	newBond.operatorNumber = 1;
	newhzBond.index = 0;
	newhzBond.operatorNumber = 0;
	newhzBond.sites.push_back(0);
	if (solver.globalConfiguration.order < 8) {
		newBond.sites.push_back(56);
		newBond.sites.push_back(7);
		newBond.sites.push_back(0);
		newBond.sites.push_back(1);
		newBond.sites.push_back(8);
	} else {
		newBond.sites.push_back(90);
		newBond.sites.push_back(9);
		newBond.sites.push_back(0);
		newBond.sites.push_back(1);
		newBond.sites.push_back(10);
	}

}

bool ClusterInFieldGapAndGroundState::execute() {

	switch (progress) {

	case (Progress::initial):

		if (solver.globalConfiguration.order < 8) {

			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterInField/BondFile8x8.cfg");
			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterInField/InputStates8x8Superposition.cfg");
			solver.globalProperties.bitsPerBaseState = 64;
			solver.globalProperties.eigenEnergy
					= solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);

		} else {
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterInField/BondFile10x10.cfg");
			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterInField/InputStates10x10Superposition.cfg");
			solver.globalProperties.bitsPerBaseState = 100;
			solver.globalProperties.eigenEnergy
					= solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);
		}

		solver.inputStates.left.clear();
		solver.latticeProperties.bonds.clear();
		solver.latticeProperties.bonds.push_back(newhzBond);
		solver.latticeProperties.bonds.push_back(newBond);

		solver.modules.pointerToModelProvider->applyV(solver.inputStates.right,
				solver.inputStates.left, 100);

		solver.latticeProperties.bonds.clear();

		if (solver.globalConfiguration.order < 8)
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterInField/BondFile8x8.cfg");
		else
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterInField/BondFile10x10.cfg");

		std::cout << "ExitedEnergyPerSite" << solver.configurationDirectory
				<< ":=";
		progress = Progress::calculateExitedEnergy;

		return true;
		break;
	case (Progress::calculateExitedEnergy):
		std::cout << solver.outputAmplitude;
		std::cout << ";\n";

		solver.inputStates.left.clear();
		solver.inputStates.right.clear();

		if (solver.globalConfiguration.order < 8) {

			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterInField/InputStates8x8Groundstate.cfg");
			solver.globalProperties.eigenEnergy
					= solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);

		} else {

			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterInField/InputStates10x10Groundstate.cfg");
			solver.globalProperties.eigenEnergy
					= solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);
		}

		solver.inputStates.left.clear();
		solver.latticeProperties.bonds.clear();
		solver.latticeProperties.bonds.push_back(newhzBond);
		solver.latticeProperties.bonds.push_back(newBond);
		newhzBond.sites.clear();
		newBond.sites.clear();

		solver.modules.pointerToModelProvider->applyV(solver.inputStates.right,
				solver.inputStates.left, 100);

		solver.latticeProperties.bonds.clear();

		if (solver.globalConfiguration.order < 8)
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterInField/BondFile8x8.cfg");
		else
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterInField/BondFile10x10.cfg");

		std::cout << "GroundStateEnergyPerSite"
				<< solver.configurationDirectory << ":=";

		progress = Progress::calculateGroundEnergy;

		return true;
		break;
	case (Progress::calculateGroundEnergy):
		std::cout << solver.outputAmplitude;
		std::cout << ";\n";

		std::cout << "Gap" << solver.configurationDirectory
				<< ":=(ExitedEnergyPerSite" << solver.configurationDirectory
				<< "-GroundStateEnergyPerSite" << solver.configurationDirectory
				<< ")*" << solver.latticeProperties.numberOfLatticeSites
				<< ";\n";

		return false;
		break;

	}

	return false;

}
