/*
 * IsingChainGroundEnergyClusterDecomposition.cpp
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 */

#include <Modules/LatticeProvider/IsingChainGroundEnergyClusterDecomposition/IsingChainGroundEnergyClusterDecomposition.h>
#include <Framework/Solver.h>
#include <Interface/StateIO.h>

IsingChainGroundEnergyClusterDecomposition::IsingChainGroundEnergyClusterDecomposition() :
		result() {

	if (solver.globalConfiguration.order % 2 == 1) {
		std::cerr
				<< "Error: for the IsingChainGroundEnergyClusterDecomposition it only makes sense to calculate even orders, as all uneven orders vanish.\n";
		exit(-1);
	}

	// We start with the graph consisting of one bond.
	graphCounter = 1;

	// The input state is the state with '0' everywhere.
	solver.inputStates.right.add(BaseState(), stateAmplitude(amplitude(1)));

}

bool IsingChainGroundEnergyClusterDecomposition::execute() {

	if (solver.outputState.begin() != solver.outputState.end()) {

		// We are interested in the ground state energy per lattice site so we have to
		// by the number of lattice sites. As also the embedding factor of a graph is
		// equal to the number of lattice sites, we can just add up the results of the graph
		// calculations.
		result += solver.outputState.begin()->second;

	}

	// We only must consider graphs of size order/2 as larger graphs cannot contribute
	// (it is not possible to touch every bond twice).
	if (graphCounter > solver.globalConfiguration.order / 2) {
		solver.outputAmplitude = result;
		return false;
	}

	// Now we set up the new graph by adding another lattice site into the chain.

	solver.latticeProperties.numberOfLatticeSites = graphCounter + 1;

	unsigned int bytesForStates = (solver.latticeProperties.numberOfLatticeSites
			* solver.modelProperties.bitsPerLatticeSite + 7) / 8;

	unsigned int numberOfExtraBitsForSites = bytesForStates * 8
			- (solver.latticeProperties.numberOfLatticeSites
					* solver.modelProperties.bitsPerLatticeSite);

	// We need to reserve some extra bits as the GeneralGraphModel needs them to keep
	// track of which bond was already touched.

	unsigned int bytesForExtraInfo =
			(solver.latticeProperties.numberOfLatticeSites + 7) / 8;

	solver.latticeProperties.numberOfExtraBits = numberOfExtraBitsForSites
			+ bytesForExtraInfo * 8;

	// Now add a new bond to connect the new lattice site to the chain.
	Bond newBond;
	newBond.index = graphCounter - 1;
	newBond.operatorNumber = 0;
	newBond.sites.push_back(graphCounter - 1);
	newBond.sites.push_back(graphCounter);

	solver.latticeProperties.bonds.push_back(newBond);

	graphCounter++;

	return true;

}
