/*
 * IsingChainGroundEnergyClusterDecomposition.h
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 */

#ifndef ISINGCHAINGROUNDENERGYCLUSTERDECOMPOSITION_H_
#define ISINGCHAINGROUNDENERGYCLUSTERDECOMPOSITION_H_

#include <Modules/LatticeProvider/LatticeProvider.h>
#include <Interface/Interface.h>

/** A LatticeProvider intended to calculate the ground state energy per lattice site of an "Ising like" model on a chain.
 *
 * "Ising like" means, that the ground state is unique, that there is one type of
 * perturbation operator that couples two lattice sites and that during
 * the calculations all sites must be touched an even number of times.
 *
 * The calculation is done by doing a graph decomposition of the chain.
 *
 * It is intended to be used with one of the DirectSolvers or TreeSolvers DataProcessors
 * and with the GeneralGraphModell ModelProvider to make sure that during the calculation
 * of a graph only contributions are considered where each bond is touched at least once.
 *
 * It is assumed that only one (even) order is calculated (solver.globalConfiguration.minOrder = solver.globalConfiguration.order)
 *
 * */
class IsingChainGroundEnergyClusterDecomposition: public LatticeProvider {

public:

	IsingChainGroundEnergyClusterDecomposition();

	bool execute();

private:

	/** Counter to keep track of which graph is currently calculated. */
	unsigned int graphCounter;

	/** To store the result of the calculation.*/
	stateAmplitude result;

};

#endif /* ISINGCHAINGROUNDENERGYCLUSTERDECOMPOSITION_H_ */
