/*
 * RotatedClusterGapAndGroundState.cpp
 *
 *  Created on: 21.09.2011
 *      Author: klagges
 */

#include <Modules/LatticeProvider/RotatedClusterGapAndGroundState/RotatedClusterGapAndGroundState.h>
#include <Framework/Solver.h>
#include <Interface/StateIO.h>

RotatedClusterGapAndGroundState::RotatedClusterGapAndGroundState() {

	progress = Progress::initial;

	std::cout.precision(16);

}

bool RotatedClusterGapAndGroundState::execute() {

	switch (progress) {

	case (Progress::initial):

		if (solver.globalConfiguration.order <= 5) {

			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterState/BondFile12x6Brickwall.cfg");
			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterState/InputStates12x6Superposition.cfg");
			solver.globalProperties.bitsPerBaseState = 72;
			solver.globalProperties.eigenEnergy =
					solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);

		} else {
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterState/BondFile14x7Brickwall.cfg");
			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterState/InputStates14x7Superposition.cfg");
			solver.globalProperties.bitsPerBaseState = 98;
			solver.globalProperties.eigenEnergy =
					solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);
		}

		//clear left input state -> want to solve with first application solver
		solver.inputStates.left.clear();
		// only calculate on one bond, because system is translation invariant
		solver.latticeProperties.bonds.resize(1);

		//create input state with application if one V-Operator on ONE bond
		solver.modules.pointerToModelProvider->applyV(solver.inputStates.right,
				solver.inputStates.left, 100);

		//clear bonds, we want to calcualte the rest on the regular bondfile
		solver.latticeProperties.bonds.clear();

		if (solver.globalConfiguration.order <= 5)
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterState/BondFile12x6Brickwall.cfg");
		else
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterState/BondFile14x7Brickwall.cfg");

		std::cout
				<< "ExitedEnergyPerSite"
				<< solver.configurationDirectory
				<< ":="
				//<< solver.globalProperties.eigenEnergy
				/// solver.latticeProperties.numberOfLatticeSites;
				;
		progress = Progress::calculateExitedEnergy;

		return true;
		break;
	case (Progress::calculateExitedEnergy):
		std::cout << solver.outputAmplitude;
		std::cout << ";\n";

		solver.inputStates.left.clear();
		solver.inputStates.right.clear();

		if (solver.globalConfiguration.order <= 5) {

			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterState/InputStates12x6GroundState.cfg");
			solver.globalProperties.eigenEnergy =
					solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);

		} else {

			solver.inputStates.readFromConfigFile(
					"BondFiles/ClusterState/InputStates14x7GroundState.cfg");
			solver.globalProperties.eigenEnergy =
					solver.modules.pointerToModelProvider->getH_0Eigenvalue(
							solver.inputStates.right.begin()->first);
		}

		solver.inputStates.left.clear();
		solver.latticeProperties.bonds.resize(1);

		solver.modules.pointerToModelProvider->applyV(solver.inputStates.right,
				solver.inputStates.left, 100);

		solver.latticeProperties.bonds.clear();

		if (solver.globalConfiguration.order <= 5)
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterState/BondFile12x6Brickwall.cfg");
		else
			solver.latticeProperties.readFromConfigFile(
					"BondFiles/ClusterState/BondFile14x7Brickwall.cfg");

		std::cout
				<< "GroundStateEnergyPerSite"
				<< solver.configurationDirectory
				<< ":="
				//<< solver.globalProperties.eigenEnergy
				//		/ solver.latticeProperties.numberOfLatticeSites;
		;

		progress = Progress::calculateGroundEnergy;

		return true;
		break;
	case (Progress::calculateGroundEnergy):

		std::cout << solver.outputAmplitude;
		std::cout << ";\n";

		std::cout << "Gap" << solver.configurationDirectory
				<< ":=(ExitedEnergyPerSite" << solver.configurationDirectory
				<< "-GroundStateEnergyPerSite" << solver.configurationDirectory
				<< ")*" << solver.latticeProperties.numberOfLatticeSites
				<< ";\n";

		return false;
		break;

	}

	return false;

}
