/*
 * ExampleSpecialLatticeModel.cpp
 *
 *  Created on: 04.04.2011
 *      Author: klagges
 */

#include <Modules/ModelProvider/ExampleSpecialLatticeModel/ExampleSpecialLatticeModel.h>
#include <Framework/Solver.h>

bool ExampleSpecialLatticeModel::acceptInputState(const BaseState& inputState,
		const stateAmplitude& inputAmplitude, const Bond& bond) {

	if (sourceState->type == State::ket) {

		// For ket vectors we only accept bonds that where not yet touched
		if (inputState.get(
				*bond.sites.begin() + solver.latticeProperties.numberOfLatticeSites
						* latticeModelConfiguration.bitsPerLatticeSite) == 0)
			return true;
		else
			return false;

	} else {

		// and for bra vectors we only accept bonds that where already touched.
		if (inputState.get(
				*bond.sites.begin() + solver.latticeProperties.numberOfLatticeSites
						* latticeModelConfiguration.bitsPerLatticeSite) == 1)
			return true;
		else
			return false;

	}

}

bool ExampleSpecialLatticeModel::updateExtraInformation(BaseState& outputState,
		const BaseState& inputState, const stateAmplitude& inputAmplitude,
		const Bond& bond) {

	if (sourceState->type == State::ket) {
		// For ket vectors we set the bit associated to this bond to 1
		outputState.set(
				*bond.sites.begin() + solver.latticeProperties.numberOfLatticeSites
						* latticeModelConfiguration.bitsPerLatticeSite);
	} else {
		//		// and for bra vectors we set it to 0.
		outputState.unSet(
				*bond.sites.begin() + solver.latticeProperties.numberOfLatticeSites
						* latticeModelConfiguration.bitsPerLatticeSite);
	}

	// Discard no result.
	return true;

}
