/*
 * GeneralGraphModel.cpp
 *
 *  Created on: 04.04.2011
 *      Author: klagges
 */

#include <Modules/ModelProvider/GeneralGraphModel/GeneralGraphModel.h>
#include <Framework/Solver.h>

bool GeneralGraphModel::acceptInputState(const BaseState& inputState,
		const stateAmplitude& inputAmplitude, const Bond& bond) {

	unsigned int bytesForState = (solver.globalProperties.bitsPerBaseState
			- solver.latticeProperties.numberOfExtraBits + 7) / 8;

	// The bond was not touched before so we can accept it.
	if (!inputState.get(bond.index + bytesForState * 8))
		return true;

	// The bond was touched before so we must find out if after this bond
	// is touched again, there are still enough orders left to touch
	// all untouched bonds
	unsigned int numberOfTouchedBonds = inputState.count(bytesForState,
			sizeof(BaseState) - 1);

	if (solver.globalConfiguration.order
			- solver.globalConfiguration.currentOrder
			>= solver.latticeProperties.bonds.size() - numberOfTouchedBonds) {

		return true; // if so we can accept
	} else
		return false;// otherwise we have to decline.

}

bool GeneralGraphModel::updateExtraInformation(BaseState& outputState,
		const BaseState& inputState, const stateAmplitude& inputAmplitude,
		const Bond& bond) {

	unsigned int bytesForState = (solver.globalProperties.bitsPerBaseState
			- solver.latticeProperties.numberOfExtraBits + 7) / 8;

	// mark the touched bond.
	outputState.set(bond.index + bytesForState * 8);

	// Discard no result.
	return true;

}
