/*
 * GeneralGraphModel.h
 *
 *  Created on: 04.04.2011
 *      Author: klagges
 */

#ifndef CLUSTERMODEL_H_
#define CLUSTERMODEL_H_

#include <Modules/ModelProvider/GeneralLatticeModel/GeneralLatticeModel.h>

/** A ModelProvider that does the same as the GeneralLatticeModel, but makes
 * sure only contributions are considered during the calculation
 * where all bonds are touched at least once.
 *
 * This is done by storing extra information within an base state. In every base
 * state it is stored which bond have been touched before. This information
 * is stored by setting a bit to 0 (not touched yet) or to 1 (touched before).
 * The bit to store this information for the first bond has a number wich is a
 * multiple of 8. So if for example 19 bits are used to store the base state itselfe,
 * the bit to store if the first bond is touched or not is the bit number 24
 * (rounding 19 up to multiples of 8).
 *
 * Please make sure that the bond indexes in the solver.latticeProperties.bonds list
 * are correctly set as the index is used to identify the bonds.
 *
 * */
class GeneralGraphModel: public GeneralLatticeModel {

	bool acceptInputState(const BaseState& inputState,
			const stateAmplitude& inputAmplitude, const Bond& bond);

	bool updateExtraInformation(BaseState& outputState,
			const BaseState& inputState,
			const stateAmplitude& inputAmplitude, const Bond& bond);

};

#endif /* CLUSTERMODEL_H_ */
