/*
 * TemplateModuleImplementation.h
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 * A template for implementation of modules. To create your own implementations:
 *
 * - Create a folder for your module. Copy and rename TemplateModuleImplementation.h
 *   and TemplateModuleImplementation.cpp to it. Adapt the class name.
 *
 * - Add your .cpp file to the Makefile to make sure it will be compiled.
 *
 * - Register the new module in the Framework/Module.cpp file, by including the
 *   new header file and adding a new MODULE(...) line.
 *
 * - Be aware, that module implementations are only instantiated and called, if
 *   they are entered in the Config/Modules.cfg file.
 *
 */

#ifndef TEMPLATEMODULEIMPLEMENTATION_H_
#define TEMPLATEMODULEIMPLEMENTATION_H_

#include <Modules/TemplateModuleType/TemplateModuleType.h>
#include <Utilities/TemplateParameterSet.h>

/** A template for a module implementation.
 * You find an instruction how to create new module implementations in
 * Modules/TemplateModuleType/TemplateModuleImplementation/TemplateModuleImplementation.h. */
class TemplateModuleImplementation: public TemplateModuleType {

public:

	/** The constructor should initialize all data needed for the implementation.*/
	TemplateModuleImplementation();

	/** In the module implementations this method needs to be overloaded.*/
	void execute();

private:

	// Add members and methods you need for the calculations of your module here.

	/** Data that might be used by the implementation.*/
	Utilities::TemplateParameterSet templateParameterSet;

};

#endif /* TEMPLATEMODULEIMPLEMENTATION_H_ */
