/*
 * TemplateModuleType.h
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 * A template for module types. To create your own module types:
 *
 * - Create a folder for your module type and copy and rename the TemplateModuleType.h
 *   and TemplateModuleType.cpp to it. Adapt the class name.
 *
 * - Add your .cpp file to the Makefile to make sure it will be compiled.
 *
 * - Register the module type in the Framework/Modules.h file, by including the
 *   new header file and adding a new MODULE_TYPE(...) line.
 *
 * - Add an entry to the Config/Modules.cfg file and change the streamIn() and
 *   streamOutExample() functions in Framework/Modules.cpp accordingly.
 *
 * - Modify the run() function in the Framework/Solver.cpp file to call your module
 *   if necessary.
 *
 * - You maybe want to create a template module implementation for your module type.
 *   (see TemplateModuleImplementation/TemplateModuleImplementation.h)
 *
 */

#ifndef TEMPLATEMODULETYPE_H_
#define TEMPLATEMODULETYPE_H_

/** A template for a module type.
 * You find an instruction how to create new module types in
 * Modules/TemplateModuleType/TemplateModuleType.h.*/
class TemplateModuleType {

public:

	/** Add the virtual methods that need to be overloaded by the module implementations here.
	 Of course you can provide default implementations for some or all of them.*/
	virtual void execute() = 0;

};

#endif /* TEMPLATEMODULETYPE_H_ */
