/*
 * DiagonalOperator.cpp
 *
 *  Created on: 02.02.2011
 *      Author: klagges
 */

#include <Utilities/DiagonalOperator.h>
#include <Interface/BaseStateIO.h>
#include <math.h>
#include <set>

Utilities::DiagonalOperator::DiagonalOperator() :
	std::vector<double>() {

}

// Copy constructor
Utilities::DiagonalOperator::DiagonalOperator(
		const Utilities::DiagonalOperator& s) :
	std::vector<double>(s) {

}

// Assignment operator
Utilities::DiagonalOperator& Utilities::DiagonalOperator::operator=(
		const Utilities::DiagonalOperator& s) {

	std::vector<double>::operator=(s);

	return *this;

}

void Utilities::DiagonalOperator::streamOutExample(std::ostream& stream) {

	stream << "# The representation of a diagonal operator\n\n"
			<< "# Format:\n"
			<< "# Base states are represented by strings (i.e. |0101>). The eigen energies are floating point numbers.\n\n"
			<< "|0101> ->  1\n" << "|0111> -> -1 \n\n"
			<< "# Means, that the state |0101> has the energy 1, |0111> the energy -1\n"
			<< "# and all other base states zero energy.\n";

}

void Utilities::DiagonalOperator::streamIn(std::istream& stream) {

	BaseState sourceState(0);

	std::string s;

	while (stream.good()) {

		char peek = stream.get();

		if (peek == '>' || peek == EOF)
			break;

		if (peek == '0' || peek == '1')
			s.push_back(peek);

	}

	unsigned int numberOfBits = 0;

	for (std::string::const_iterator stringIt = s.begin(); stringIt != s.end(); stringIt++) {

		switch (*stringIt) {

		case '0':
			numberOfBits++;
			break;
		case '1':
			sourceState.set(numberOfBits);
			numberOfBits++;
			break;

		}

	}

	resize(1 << numberOfBits, 0);

	while (stream.good()) {

		stream.ignore(2048, '>');

		double targetEnergy;

		stream >> targetEnergy;

		if (!stream.good())
			break;

		operator [](sourceState.toInteger()) = targetEnergy;

		stream >> sourceState;

	}

}

std::ostream& Utilities::operator<<(std::ostream& o,
		const Utilities::DiagonalOperator& d) {

	BaseState outputBaseState(log2(d.size()));

	for (unsigned int i = 0; i < d.size(); i++) {

		outputBaseState = i;

		o << outputBaseState << " -> ";

		o << d[i] << "\n";

	}

	return o;

}

double Utilities::DiagonalOperator::getMaximalDelta() {

	std::set<double> sortetEnergies;

	for (DiagonalOperator::const_iterator it = begin(); it != end(); it++) {

		sortetEnergies.insert(*it);

	}

	double maximumDelta = 0;

	std::set<double>::const_iterator it1 = sortetEnergies.begin();

	if (it1 != sortetEnergies.end()) {

		std::set<double>::const_iterator it2 = ++sortetEnergies.begin();

		while (it2 != sortetEnergies.end()) {

			if ((*it2 - *it1) > maximumDelta)
				maximumDelta = *it2 - *it1;

			it1 = it2;
			it2++;

		}

	}

	return maximumDelta;

}

double Utilities::DiagonalOperator::getOverallDelta() {

	DiagonalOperator::const_iterator it = begin();

	if (it == end())
		return 0;

	double min = *it;
	double max = *it;

	it++;

	while (it != end()) {

		if (*it > max)
			max = *it;
		else if (*it < min)
			min = *it;
		it++;

	}

	return max - min;


}

