/*
 * DiagonalOperator.h
 *
 *  Created on: 02.02.2011
 *      Author: klagges
 */

#ifndef DIAGONALOPERATOR_H_
#define DIAGONALOPERATOR_H_

#include <Utilities/ParameterSet.h>
#include <vector>

namespace Utilities {

/** Represents a diagonal operator.
 *
 * The eigen value of each base state is stored in a vector.
 * The key for a base state should be the value returned by BaseState::getInt();
 * Can be streamed in from configuration files.
 * */
class DiagonalOperator: public Utilities::ParameterSet, public std::vector<
		double> {
public:

	void streamOutExample(std::ostream& stream);

	void streamIn(std::istream& stream);

	/** Default constructor.*/
	DiagonalOperator();

	/** Copy constructor*/
	DiagonalOperator(const DiagonalOperator& s);

	/** Assignment operator*/
	DiagonalOperator& operator=(const DiagonalOperator& s);

	/** Get the maximum difference between two neighboring energy levels of this operator. */
	double getMaximalDelta();

	/** Return the difference between the highest and the lowest eigenvalue of this operator.*/
	double getOverallDelta();

};

/** Stream out as a string like:
 \code
 |0111> -> 2
 |0101> -> -1
 \endcode */
std::ostream& operator<<(std::ostream& o, const Utilities::DiagonalOperator& d);

}

#endif /* DIAGONALOPERATOR_H_ */
