/*
 * Operator.cpp
 *
 *  Created on: 23.11.2010
 *      Author: klagges
 */

#include <Utilities/Operator.h>
#include <Framework/Solver.h>
#include <Interface/StateIO.h>
#include <Utilities/TypeCast.h>

Utilities::Operator::Operator() :
		std::vector<std::list<std::pair<std::list<bool>, stateAmplitude> > >(),
		numberOfBits(0) {

}

Utilities::Operator::Operator(unsigned int n) :
		std::vector<std::list<std::pair<std::list<bool>, stateAmplitude> > >(),
		numberOfBits(n) {

	resize(1 << numberOfBits);

}

Utilities::Operator::Operator(const Operator& o) :
		std::vector<std::list<std::pair<std::list<bool>, stateAmplitude> > >(o),
		numberOfBits(o.numberOfBits) {

}

Utilities::Operator& Utilities::Operator::operator=(const Operator& o) {

	std::vector<std::list<std::pair<std::list<bool>, stateAmplitude> > >::operator=(
			o);
	return *this;

}

void Utilities::Operator::newSize(unsigned int n) {

	numberOfBits = n;
	resize(1 << numberOfBits);

}

void Utilities::Operator::appendState(const BaseState i, const State& s) {

	State temp;

	unsigned int sourceInt = i.toInteger();

	for (std::list<std::pair<std::list<bool>, stateAmplitude> >::const_iterator it =
			operator[](sourceInt).begin(); it != operator[](sourceInt).end();
			it++) {

		temp.add(it->first, it->second);

	}

	operator[](sourceInt).clear();

	temp += s;

	for (State::const_iterator it = temp.begin(); it != temp.end(); it++) {

		std::list<bool> bs;

		for (unsigned int i = 0; i < numberOfBits; i++) {
			bs.push_back(it->first.get(i));
		}

		operator[](sourceInt).push_back(
				std::pair<std::list<bool>, stateAmplitude>(bs, it->second));

	}

}

State Utilities::Operator::operator*(const State& s) const {

	State result;

	for (State::const_iterator stateIt = s.begin(); stateIt != s.end();
			stateIt++) {

		unsigned int sourceNumber = stateIt->first.toInteger();

		for (std::list<std::pair<std::list<bool>, stateAmplitude> >::const_iterator it =
				operator [](sourceNumber).begin();
				it != operator [](sourceNumber).end(); it++) {

			State::value_type resultingEntry(it->first,
					it->second * stateIt->second);

			result.add(it->first, it->second * stateIt->second);

		}
	}

	return result;

}

Utilities::Operator& Utilities::Operator::operator+=(const Operator& s) {

	if (numberOfBits < s.numberOfBits)
		newSize(s.numberOfBits);

	for (unsigned int i = 0; i < s.size(); i++) {

		State temp;

		for (std::list<std::pair<std::list<bool>, stateAmplitude> >::const_iterator it =
				s[i].begin(); it != s[i].end(); it++) {

			temp.add(it->first, it->second);

		}

		State temp2;

		for (std::list<std::pair<std::list<bool>, stateAmplitude> >::const_iterator it =
				operator[](i).begin(); it != operator[](i).end(); it++) {

			temp2.add(it->first, it->second);

		}

		operator[](i).clear();

		temp2 += temp;

		for (State::const_iterator it = temp2.begin(); it != temp2.end();
				it++) {

			std::list<bool> bs;

			for (unsigned int j = 0; j < numberOfBits; j++) {
				bs.push_back(it->first.get(j));
			}

			operator[](i).push_back(
					std::pair<std::list<bool>, stateAmplitude>(bs, it->second));

		}

	}

	return *this;

}

void Utilities::Operator::streamOutExample(std::ostream& stream) {

	stream
			<< "# The representation of an operator\n\n"
			<< "# Format:\n"
			<< "# Base states are represented by strings (i.e. |0101>). Factors are rational numbers\n\n"
			<< "# First specify the expansion parameter this operator is associated with.\n"
			<< "# Give the number of the expansion parameter or '-1' if the operator should not be associated\n"
			<< "# with an expansion parameter\n"
			<< "0\n\n"
			<< "# Now the Operator.\n"
			<< "|0101> ->  1 |1111> +4 |0111> -3 |0000>;\n"
			<< "|0111> -> -1 |0111> +2 |0111> +5 |0000>;\n\n"
			<< "# Means, that the state |0101> is mapped to the state |1111> + 4 * |0111> - 3 * |0000> ...\n";

}

void Utilities::Operator::streamIn(std::istream& stream) {

	std::map<BaseState, State> temp;

	int parameterNumber;

	stream >> parameterNumber;

	BaseState sourceState(0);

	std::string s;

	while (stream.good()) {

		char peek = stream.get();

		if (peek == '>' || peek == EOF)
			break;

		if (peek == '0' || peek == '1')
			s.push_back(peek);

	}

	numberOfBits = 0;

	for (std::string::const_iterator stringIt = s.begin(); stringIt != s.end();
			stringIt++) {

		switch (*stringIt) {

		case '0':
			numberOfBits++;
			break;
		case '1':
			sourceState.set(numberOfBits);
			numberOfBits++;
			break;

		}

	}

	if (numberOfBits > 0)
		resize(1 << numberOfBits);
	else
		resize(0);

	while (stream.good()) {

		stream.ignore(2048, '>');

		State targetState;

		stream >> targetState;

		if (!stream.good())
			break;

		if (targetState.size() != 0) {

			std::map<BaseState, State>::value_type insertValue(sourceState,
					targetState);
			std::pair<std::map<BaseState, State>::iterator, bool> insertResult =
					temp.insert(insertValue);

			if (!insertResult.second) {

				insertResult.first->second += targetState;
				if (insertResult.first->second.empty())
					temp.erase(insertResult.first);

			}

		}

		stream >> sourceState;

	}

	if (parameterNumber >= 0
			&& (unsigned int) parameterNumber
					< solver.modelProperties.numberOfExpansionParameters) {
		Exponents parameterExponents;
		parameterExponents.e[parameterNumber] = 1;
		ExpandedAmplitude parameterAmplitude;
		parameterAmplitude.add(amplitude(1), parameterExponents);

		stateAmplitude convertedAmplitude;
		Utilities::typeCast(parameterAmplitude, convertedAmplitude);

		for (std::map<BaseState, State>::iterator it = temp.begin();
				it != temp.end(); it++) {
			it->second *= convertedAmplitude;
		}

	}

	for (std::map<BaseState, State>::const_iterator it = temp.begin();
			it != temp.end(); it++) {

		appendState(it->first, it->second);

	}

}

std::ostream& Utilities::operator<<(std::ostream& o,
		const Utilities::Operator& d) {

	for (unsigned int i = 0; i < d.size(); i++) {

		if (!d[i].empty()) {

			o << "|" << Utilities::intToBinaryReverted(i, d.numberOfBits)
					<< " ->\n";

			std::list<std::pair<std::list<bool>, stateAmplitude> >::const_iterator it =
					d[i].begin();

			if (it != d[i].end()) {

				o << it->second;
				o << "|";
				for (std::list<bool>::const_iterator listIt = it->first.begin();
						listIt != it->first.end(); listIt++) {

					o << *listIt;

				}
				o << ">";

				it++;

			}

			while (it != d[i].end()) {

				o << "\n";
				o << it->second;

				o << "|";
				for (std::list<bool>::const_iterator listIt = it->first.begin();
						listIt != it->first.end(); listIt++) {

					o << *listIt;

				}
				o << ">";

				it++;

			}

			o << ";\n\n";

		}

	}

	o << "\n";

	return o;

}

