/*
 * Operator.h
 *
 *  Created on: 23.11.2010
 *      Author: klagges
 */

#ifndef OPERATOR_H_
#define OPERATOR_H_

#include <Utilities/ParameterSet.h>
#include <Interface/Interface.h>

namespace Utilities {

/** Represents an operator as a matrix.
 The matrix is represented as a vector of lists. Matrix entries are pairs of base states and ::stateAmplitude objects,
 where the base states are represented as lists of boolean values.
 Can be streamed in from files.*/
class Operator: public std::vector<
		std::list<std::pair<std::list<bool>, stateAmplitude> > >
		, public Utilities::ParameterSet {

public:

	void streamOutExample(std::ostream& stream);

	void streamIn(std::istream& stream);

	/** Default constructor*/
	Operator();

	/** Create an Operator acting in a Hilbert space of size 2^numberOfBits. */
	Operator(unsigned int numberOfBits);

	/** Copy constructor*/
	Operator(const Operator& o);

	/** Assignment operator*/
	Operator& operator=(const Operator& o);

	/** Resize the operator to act in a Hilbert space of size 2^numberOfBits.*/
	void newSize(unsigned int numberOfBits);

	/** Add s to the i-th column vector of the Operator, so that it maps the base state i
	 to the state s.*/
	void appendState(const BaseState i, const State& s);

	/** Apply the Operator the the state s and return the result*/
	State operator*(const State& s) const;

	/** Component vice sum of two operators.*/
	Operator& operator+=(const Operator& s);

	/** The number of bits to represent the hilbert Space this operator is acting on.
	 * The dimension of the Hilbert space is \f$ 2^{numberOfBits} \f$.*/
	unsigned int numberOfBits;

};

/** Stream out as a string like:
 |0111> -> 2|0000>+1|1111>-5|0111>;
 |0101> -> -1|0110>+3|0011>+|0000>;*/
std::ostream& operator<<(std::ostream& o, const Utilities::Operator& d);

}

#endif /* OPERATOR_H_ */
