/*
 * OperatorDescription.cpp
 *
 *  Created on: 21.02.2011
 *      Author: klagges
 */

#include <Utilities/OperatorDescription.h>
#include <Framework/Solver.h>

Utilities::OperatorDescription::OperatorDescription(bool aV, unsigned int rV,
		unsigned int nS) :
	applyV(aV), remainingV(rV), numberOfS(nS) {
}

Utilities::OperatorDescription::OperatorDescription() :
	applyV(true), remainingV(0), numberOfS(0) {
}

Utilities::OperatorDescription::OperatorDescription(char c) {

	switch (c) {

	case 'V':
		applyV = true;
		remainingV = 10000;
		numberOfS = 0;
		break;
	case 'S':
		applyV = false;
		remainingV = 10000;
		numberOfS = 1;
		break;
	default:
		applyV = false;
		remainingV = 0;
		numberOfS = 0;
		break;

	}

}

bool Utilities::OperatorDescription::operator==(const OperatorDescription& od) const {
	if (remainingV == od.remainingV && numberOfS == od.numberOfS && applyV
			== od.applyV)
		return true;
	return false;
}

void Utilities::OperatorDescription::apply(State& state) const {

	if (applyV) {

		State intermediateResult;

		solver.modules.pointerToModelProvider->applySSV(state,
				intermediateResult, remainingV, numberOfS);

		intermediateResult.swap(state);

	} else {

		if (remainingV > 0) {

			solver.modules.pointerToModelProvider->applyS(state, remainingV,
					numberOfS);

		} else {

			solver.modules.pointerToModelProvider->applyP(state);

		}

	}

}

void Utilities::OperatorDescription::apply(const State& source, State& result) const {

	if (applyV) {

		solver.modules.pointerToModelProvider->applySSV(source, result,
				remainingV, numberOfS);

	} else {

		if (remainingV > 0) {

			solver.modules.pointerToModelProvider->applyS(source, result,
					remainingV, numberOfS);

		} else {

			solver.modules.pointerToModelProvider->applyP(source, result);

		}
	}

}

std::ostream& Utilities::operator<<(std::ostream& o,
		const OperatorDescription& od) {

	if (od.applyV) {

		o << "V";

		for (unsigned int i = 0; i < od.numberOfS; i++)
			o << "S";

		if (od.remainingV == 0)
			o << "P";

	} else {

		if (od.remainingV > 0) {

			for (unsigned int i = 0; i < od.numberOfS; i++)
				o << "S";

		} else {

			o << "P";

		}
	}

	return o;

}
