/*
 * OperatorDescription.h
 *
 *  Created on: 21.02.2011
 *      Author: klagges
 */

#ifndef OPERATORDESCRIPTION_H_
#define OPERATORDESCRIPTION_H_

#include <Interface/Interface.h>

namespace Utilities {

/** A class to store the description of a short sequence of operators.

For example : SSV

They are intended to be stored in lists, trees or other data structures by the DataProcessor modules.*/
class OperatorDescription {

public:

	OperatorDescription(bool aV, unsigned int rV, unsigned int nS);
	OperatorDescription(char c);
	OperatorDescription();

	bool operator==(const OperatorDescription& od) const;

	/** Is true, if a V operator should be applied before the other operators.*/
	bool applyV;
	
	/** This is for optimization reasons. Gives the number of V operators that are
	expected to be applied after the application of this operator. Can be used to
	truncate the Hilbert space of possible states during the application of this operator.
	If raminingV = 0, a P operator should be applied.*/
	unsigned int remainingV;
	
	/** The number of S operators that should be applied (solely or after a V operator).
	 If numberOfS is zero, but remainingV > 0, a 1-P projector should be applied.*/
	unsigned int numberOfS;

	/** Apply the operator represented by this object in place. */
	void apply(State& state) const;
	
	/** Apply the operator represented by this object an write the output to target. */
	void apply(const State& source, State& result) const;

};

/** Stream out as a string.*/
std::ostream& operator<<(std::ostream& o, const OperatorDescription& od);

}

#endif /* OPERATORDESCRIPTION_H_ */
