/*
 * ParameterSet.cpp
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 * All representational classes derive from the class ParameterSet.
 * It defines functions to read and write from config and coefficients files.
 *
 */

#include <Framework/Solver.h>
#include <fstream>
#include <sstream>

bool Utilities::ParameterSet::readFromCoefficientsFile(std::string filename) {

	std::string fullFileName("Coefficients/");
	fullFileName.append(filename);

	std::ifstream ifs(fullFileName.c_str());

	if (ifs.good()) {

		ifs >> *this;

		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Loaded file: " << fullFileName << ".\n";
		ifs.close();

		return true;

	} else {

		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Failed to read " << fullFileName
					<< ". Writing example file.\n";
		ifs.close();
		std::ofstream ofs(fullFileName.c_str(),
				std::ios_base::out | std::ios_base::trunc);
		if (ofs.is_open())
			ofs << *this;
		ofs.close();
		return false;

	}

}

bool Utilities::ParameterSet::writeToCoefficientsFile(std::string filename) {

	std::string fullFileName("Coefficients/");
	fullFileName.append(filename);

	std::ofstream ofs(fullFileName.c_str(),
			std::ios_base::out | std::ios_base::trunc);
	if (ofs.is_open()) {

		ofs << *this;
		ofs.close();
		return true;

	} else {

		ofs.close();
		return false;

	}

}

bool Utilities::ParameterSet::readFromConfigFile(std::string filename) {

	std::string fullFileName(solver.configurationDirectory);
	fullFileName.push_back('/');
	fullFileName.append(filename);

	std::ifstream ifs(fullFileName.c_str());

	if (ifs.good()) {
		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Loading file: " << fullFileName << ".\n";
		ifs >> *this;
		ifs.close();

		return true;

	} else {

		if (solver.globalConfiguration.verbosityLevel > 0)
			std::cerr << "Failed to read " << fullFileName
					<< ". Writing example file.\n";
		ifs.close();
		std::ofstream ofs(fullFileName.c_str(),
				std::ios_base::out | std::ios_base::trunc);
		if (ofs.is_open())
			ofs << *this;
		ofs.close();
		return false;

	}

}

bool Utilities::ParameterSet::writeToConfigFile(std::string filename) {

	std::string fullFileName(solver.configurationDirectory);
	fullFileName.push_back('/');
	fullFileName.append(filename);

	std::ofstream ofs(fullFileName.c_str(),
			std::ios_base::out | std::ios_base::trunc);
	if (ofs.is_open()) {

		ofs << *this;
		ofs.close();
		return true;

	} else {

		ofs.close();
		return false;

	}

}

std::ostream& operator<<(std::ostream& o, Utilities::ParameterSet& d) {

	d.streamOutExample(o);
	return o;

}

std::istream& operator>>(std::istream& o, Utilities::ParameterSet& d) {

	std::stringstream ss;

	while (o.good()) {

		if (o.peek() == '#') {

			ss.put('\n');

			o.ignore(4096, '\n');

		} else {

			if (o.peek() != EOF)
				ss.put(o.get());
			else
				o.get();

		}

	}

	d.streamIn(ss);
	return o;

}
