/*
 * ParameterSet.h
 *
 *  Created on: 26.10.2010
 *      Author: klagges
 *
 */

#ifndef PARAMETERSET_H_
#define PARAMETERSET_H_

#include <iostream>

namespace Utilities {

/** Provides methods to read from and write to configuration files.*/
class ParameterSet {

public:

	/** Streams the content of the file 'Coefficients/filename' to the parameter set.*/
	bool readFromCoefficientsFile(std::string filename);

	/** Streams an example file out to the file 'Coefficients/filename'.*/
	bool writeToCoefficientsFile(std::string filename);

	/** Streams the content of the file 'solver.configurationDirectory/filename' to the parameter set.*/
	bool readFromConfigFile(std::string filename);

	/** Streams an example file out to the file 'solver.configurationDirectory/filename'.*/
	bool writeToConfigFile(std::string filename);

	// this are the virtual functions that need to be overloaded by the derived classes

	/** Writes an example parameter file to the output stream.
	 Feel free to use the '#' character to add comments and make the file more readable*/
	virtual void streamOutExample(std::ostream& stream) = 0;

	/** Reads the stream and fills the parameter set.*/
	virtual void streamIn(std::istream& stream) = 0;

};

}

/** Filters comments (lines started with a '#' character) out of the
 input stream and calls the streamIn() method.*/
std::istream& operator>>(std::istream& o, Utilities::ParameterSet& d);

/** The default implementation for the stream out operator is to call the
 streamOutExample() method.*/
std::ostream& operator<<(std::ostream& o, Utilities::ParameterSet& d);

#endif /* PARAMETERSET_H_ */
