/*
 * TypeCast.h
 *
 *  Created on: 20.09.2011
 *      Author: klagges
 */

#ifndef TYPECAST_H_
#define TYPECAST_H_

namespace Utilities {

/** Write the value of source into target.*/
template<typename S, typename T>
inline void typeCast(const S & source, T & target) {

	target = source;
}

template<typename T>
inline void typeCast(const mpq_class & source, std::complex<T> & target) {

	T temp;

	typeCast(source, temp);

	target = std::complex < T > (temp);
}

template<typename T>
inline void typeCast(const mpf_class & source, std::complex<T> & target) {

	T temp;

	typeCast(source, temp);

	target = std::complex < T > (temp);
}

template<typename T>
inline void typeCast(const double & source, std::complex<T> & target) {

	T temp;

	typeCast(source, temp);

	target = std::complex < T > (temp);
}

template<typename T>
inline void typeCast(const float & source, std::complex<T> & target) {

	T temp;

	typeCast(source, temp);

	target = std::complex < T > (temp);
}

template<typename S, typename cont, typename exp, typename amp>
inline void typeCast(const S & source,
		ExpandedAmplitudeTemplate<cont, exp, amp> & target) {

	amp temp;

	typeCast(source, temp);

	target = ExpandedAmplitudeTemplate<cont, exp, amp>(temp);

}

template<typename cont, typename exp, typename amp, typename T>
inline void typeCast(const ExpandedAmplitudeTemplate<cont, exp, amp> & source,
T & target) {

	amp temp(0);

	for (typename ExpandedAmplitudeTemplate<cont, exp, amp>::const_iterator it =
			source.begin(); it != source.end(); it++) {

		temp += it->second;

	}

	typeCast(temp, target);

}

template<typename cont, typename exp, typename amp>
inline void typeCast(const ExpandedAmplitudeTemplate<cont, exp, amp> & source,
ExpandedAmplitudeTemplate<cont, exp, amp> & target) {

	target = source;

}

template<>
inline void typeCast(const mpq_class & source, double & target) {

	target = source.get_d();
}

template<>
inline void typeCast(const mpq_class & source, float & target) {

	target = source.get_d();
}

}

#endif /* TYPECAST_H_ */
