/*
 * EDController.cpp
 *
 *  Created on: Sep 3, 2014
 *      Author: ixert
 */

#include<iostream>
#include "EDController.h"
#include "SeriesTemplate.h"
#include "SpinModelClassTemplate.h"
#include "SpinStateTemplate.h"
#include <gmpxx.h>
#include <gmp.h>
#include "LatticeSymmetryClass.h"
#include "nr3.h"
#include "eigen_sym.h"
#include <list>
#include <utility>

using namespace std;

void EDController::iterateParameterValues(const unsigned int &d, vector<Doub> &parametersValue){
	if(d==parametersValue.size()){
		// done with iterations
		// perform Calculation on the actual parametersValue Array

		// mutiply every parameter by the first one
		vector<Doub> parametersValue2=parametersValue;
		for(unsigned int i=0;i<multipleOfFirstParameter.size();i++){
			parametersValue2[multipleOfFirstParameter[i]]*=parametersValue2[0];
		}

		switch(calculationType){
			case 0:{
				performLanczosED(parametersValue2);
				break;
			}
			case 1:{
				performFullED(parametersValue2);
				break;
			}
		}
		return;
	}
	else{
		for(parametersValue[d]=parametersStart[d];parametersValue[d]<=parametersEnd[d];parametersValue[d]+=parametersStep[d]){
			iterateParameterValues(d+1,parametersValue);
		}
	}
}

void EDController::performLanczosED(vector<Doub> const &parametersValue){
	//create temporary maxNumberOfIterations
	cout << "parameters=";
	for(unsigned int i=0;i<parametersValue.size();i++){
		cout << parametersValue[i] << " ";
	}
	cout << "\n";

	vector<Doub> parametersValueTemp=parametersValue;
	if(useAngleParameter){
		cout << "convert parameters to cos/sin\n";
		parametersValueTemp[0]=cos(parametersValue[1]);
		parametersValueTemp[1]=sin(parametersValue[1]);
	}

	// evaluate series
	unsigned int evaluatedLocalSpinModelSize=referenceSpinModels.size();
	if(useLocalSpinModel){
		evaluatedLocalSpinModelSize++;
	}
	vector<LocalSpinModelClassTemplate<stateInt,Complex> > evaluatedLocalSpinModels(evaluatedLocalSpinModelSize);

	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		SpinModelClassTemplate<pair<SeriesDouble,SeriesDouble> > tempSpinModel=referenceSpinModels[i];
		//cout << "spinModel:\n";
		//cout << tempSpinModel << "\n";
		SpinModelClassTemplate<Complex> evaluatedSpinModel=tempSpinModel.evaluateSpinModel(parametersValueTemp);
		//cout << "evaluatedSpinModel:\n";
		//cout << evaluatedSpinModel << "\n";
		evaluatedLocalSpinModels[i]=LocalSpinModelClassTemplate<stateInt,Complex>(evaluatedSpinModel);
		//cout << "evaluatedLocalSpinModels[i]:\n";
		//cout << evaluatedLocalSpinModels[i] << "\n";
	}
	if(useLocalSpinModel){
		evaluatedLocalSpinModels[evaluatedLocalSpinModelSize-1]=localSpinModel.evaluateLocalSpinModel(parametersValueTemp);
	}

	stringstream ssEigenvaluesFile;
	ssEigenvaluesFile << "eigenvalues";
	if(periodicBC){
		ssEigenvaluesFile << "_" << xMax << "x" << yMax;
	}
	ssEigenvaluesFile << filenameAppendix << ".txt";
	fstream fsEigenvaluesFile;

	stringstream ssTransverseSpinStructureFactorFile;
	ssTransverseSpinStructureFactorFile << "transverseSpinStructureFactor";
	if(periodicBC){
		ssTransverseSpinStructureFactorFile << "_" << xMax << "x" << yMax;
	}
	ssTransverseSpinStructureFactorFile << filenameAppendix << ".txt";
	fstream fsTransverseSpinStructureFactorFile;

	stringstream ssSpinStructureFactorFile;
	ssSpinStructureFactorFile << "spinStructureFactor";
	if(periodicBC){
		ssSpinStructureFactorFile << "_" << xMax << "x" << yMax;
	}
	ssSpinStructureFactorFile << filenameAppendix << ".txt";
	fstream fsSpinStructureFactorFile;

	stringstream ssMagnetizationFile;
	ssMagnetizationFile << "magnetization";
	if(periodicBC){
		ssMagnetizationFile << "_" << xMax << "x" << yMax;
	}
	ssMagnetizationFile << filenameAppendix << ".txt";
	fstream fsMagnetizationFile;

	stringstream ssTrimerizationFile;
	ssTrimerizationFile << "trimerization";
	if(periodicBC){
		ssTrimerizationFile << "_" << xMax << "x" << yMax;
	}
	ssTrimerizationFile << filenameAppendix << ".txt";
	fstream fsTrimerizationFile;

	unsigned int tempMaxNumberOfIterations=maxNumberOfIterations;
	bool evConverged=false;;

	VecDoub NReigenVec;

	SpinStateComplex eigenVec;
	if(!numberOfEigenvectors){
		eigenVec.clear();
	}
	SpinStateComplex u,v,w;
	v.createRandomState();
	u.createRandomState(); //create second random state and add this state with imaginary prefactor to the first state...
	u*=Complex(0,1);
	v+=u;

	cout << "stateCont.size()=" << v.size() << "\n";
	if(v.size()==0){
		cout << "no representatives present! Aborting Lansczos Algorithm\n";
		return;
	}

	// normalize state
	v/=sqrt(real(v.normQuad()));

	if(numberOfEigenvectors){ // if the eigenvector should be calculated later on, the random state must be saved!
		eigenVec=v;
	}

	// this is the Lanczos-Algorithm!
	vector<Doub> lastEnergies(numberOfEigenvalues,0);
	vector<Doub> lastEnergiesConv(numberOfEigenvalues,0);
	vector<Doub> alpha;
	vector<Doub> beta;
	beta.push_back(0);
	// first lanczos step outside of loop
	u=v;

	v.applyLocalSpinHamiltonian(w,evaluatedLocalSpinModels,siteCombinationsToReferenceSpinModel,kx,ky,spinFlip,refx,refy);

	Doub temp=real(v*w);
	cout << "alpha=" << v*w << "\n";
	alpha.push_back(temp);
	temp*=-1;
	v*=temp;
	v+=w;
	temp=real(v.normQuad());

	cout << "u*v=" << u*v << "\n";

	if(myAbs(temp)>zeroEPS){
		temp=sqrt(temp);
		v/=temp;
		beta.push_back(temp);
	}
	else{
		cout << "reached zero norm!\n";
		tempMaxNumberOfIterations=0;
		evConverged=true;
	}

	lastEnergies[0]=alpha.back();

	for(unsigned int i=1;i<tempMaxNumberOfIterations;i++){
		cout << "i=" << i << "\n";
		//apply Lanczos Step
		v.applyLocalSpinHamiltonian(w,evaluatedLocalSpinModels,siteCombinationsToReferenceSpinModel,kx,ky,spinFlip,refx,refy);

		u*=beta.back();
		w-=u;

		cout << "alpha=" << v*w << "\n";
		temp=real(v*w);
		alpha.push_back(temp);

		temp*=-1;
		u=v;
		v*=temp;
		v+=w;

		temp=real(v.normQuad());
		if(myAbs(temp)>zeroEPS){
			temp=sqrt(temp);
			v/=temp;
			beta.push_back(temp);
		}
		else{
			cout << "reached zero norm in step " << i << "!\n";
			//break; // no break at this point! the actual matrix must be diagonalized!
			//i=tempMaxNumberOfIterations; // this is the exit condition after the loop is finished
			evConverged=true;
		}

		cout << "u*v=" << u*v << "\n";

		// calculate eigenvalues and check for convergence
		VecDoub aNR(alpha.size());
		VecDoub bNR(alpha.size());
		for(unsigned int j=0;j<alpha.size();j++){
			cout << "alpha[" << j << "]=" << alpha[j] << "\n";
			aNR[j]=alpha[j];
			bNR[j]=beta[j];
		}
		// diagonalize matrix
		Symmeig TSymmEig(aNR,bNR,false); // use tridiagonality!
		bool converged=true;
		cout << "lowest eigenvalue of Lanczos matrix:";
		cout << setprecision(16) << TSymmEig.d[alpha.size()-1] << "\n";
		// save convergence of eigenvalues
		if(alpha.size()<lastEnergies.size()){
			for(unsigned int j=0;j<alpha.size();j++){
				lastEnergiesConv[j]=myAbs(lastEnergies[j]-TSymmEig.d[alpha.size()-j-1]);
			}
		}
		else{
			for(unsigned int j=0;j<lastEnergies.size();j++){
				lastEnergiesConv[j]=myAbs(lastEnergies[j]-TSymmEig.d[alpha.size()-j-1]);
			}
		}
		// just check if gsEnergy is converged, but save the other energies!
		cout << "E(i)-E(i-1)=" << lastEnergiesConv[0] << "\n";
		if(lastEnergiesConv[0]>convergenceEPS){
			converged=false;
		}

		if(alpha.size()<lastEnergies.size()){
			converged=false;
			for(unsigned int j=0;j<alpha.size();j++){
				lastEnergies[j]=TSymmEig.d[alpha.size()-j-1];
			}
		}
		else{
			for(unsigned int j=0;j<lastEnergies.size();j++){
				lastEnergies[j]=TSymmEig.d[alpha.size()-j-1];
			}
		}
		if(converged){
			cout << "Eigenvalues are converged in step " << i << "!\n";
			evConverged=true;
		}

		if(evConverged){
			// if eigenvectors should be computed calculate the eigenvectors
			if(numberOfEigenvectors){
				Symmeig TSymmEig2(aNR,bNR,true);
				NReigenVec.resize(TSymmEig2.z.nrows());
				unsigned int colIndex=TSymmEig2.z.ncols()-1;
				for(unsigned int rowIndex=0;rowIndex<TSymmEig2.z.nrows();rowIndex++){
					NReigenVec[rowIndex]=TSymmEig2.z[rowIndex][colIndex];
				}
			}
			break;
		}
	}
	// save eigenvalues
	cout << "Eigenvalues:\n";
	for(unsigned int i=0;i<lastEnergies.size();i++){
		cout << lastEnergies[i] << "\n";
	}

	// save eigenvalues, independent of convergence, because now the program saves the convergenceValue also
	//if(evConverged){
	fsEigenvaluesFile.open(ssEigenvaluesFile.str().c_str(),fstream::out | fstream::app);
	fsEigenvaluesFile << setprecision(16);
	for(unsigned int i=0;i<parametersValue.size();i++){
		if(i!=0){
			fsEigenvaluesFile << " ";
		}
		fsEigenvaluesFile << parametersValue[i];
	}
	if(useSzSymm){
		fsEigenvaluesFile << " " << Sz;
	}
	if(periodicBC){
		fsEigenvaluesFile << " " << kx << " " << ky << " " << spinFlip;
	}
	if(useReflectionSymm){
		fsEigenvaluesFile << " " << refx << " " << refy;
	}
	// write eigenvalues to file
	for(unsigned int i=0;i<lastEnergies.size();i++){
		fsEigenvaluesFile << " " << lastEnergies[i];
	}
	// write convergence of eigenvalues to files
	for(unsigned int i=0;i<lastEnergiesConv.size();i++){
		fsEigenvaluesFile << " " << lastEnergiesConv[i];
	}
	fsEigenvaluesFile << "\n";
	fsEigenvaluesFile.close();
	//}
	
	if(evConverged && numberOfEigenvectors && NReigenVec.size()){
		// todo: works only for numberOfEigenvectors==1 at the moment
		if(numberOfEigenvectors==1){
			cout << "NReigenVec.size()=" << NReigenVec.size() << "\n";
			//now start the lansczos again to calculate the eigenvectors
			Doub betaV=0;
			v=eigenVec;
			eigenVec*=NReigenVec[0];

			u=v;
			v.applyLocalSpinHamiltonian(w,evaluatedLocalSpinModels,siteCombinationsToReferenceSpinModel,kx,ky,spinFlip,refx,refy);

			temp=real(v*w);;
			temp*=-1;
			v*=temp;
			v+=w;
			temp=real(v.normQuad());

			if(myAbs(temp)>zeroEPS){
				temp=sqrt(temp);
				v/=temp;
				betaV=temp;
				w=v;
				w*=NReigenVec[1];
				eigenVec+=w;
			}

			for(unsigned int i=2;i<NReigenVec.size();i++){
				cout << "i=" << i << "\n";
				//apply Lanczos Step
				v.applyLocalSpinHamiltonian(w,evaluatedLocalSpinModels,siteCombinationsToReferenceSpinModel,kx,ky,spinFlip,refx,refy);

				u*=betaV;
				w-=u;
				temp=real(v*w);
				temp*=-1;
				u=v;
				v*=temp;
				v+=w;
				temp=real(v.normQuad());
				if(myAbs(temp)>zeroEPS){
					temp=sqrt(temp);
					v/=temp;
					betaV=temp;
					w=v;
					w*=NReigenVec[i];
					eigenVec+=w;
				}
			}

			// now do something with this eigenvector!
			if(calculateTransverseSpinStructureFactor){
				vector<Complex> SxyQ((xMax+1)*(yMax+1),Complex(0));

				for(unsigned int i=0;i<numberOfSites;i++){
					siteCombinatinsOfOperatorModel[0][0][0]=i;
					for(unsigned int j=0;j<numberOfSites;j++){
						siteCombinatinsOfOperatorModel[0][0][1]=j;
						eigenVec.applyLocalOperator(u,operatorModels[0],siteCombinatinsOfOperatorModel[0],kx,ky,spinFlip,refx,refy);

						Complex ssf=eigenVec*u;
						for(int qx=0;qx<=xMax;qx++){
							for(int qy=0;qy<=yMax;qy++){
								Complex phase=0;
								LatticeSymmetryClass::calculatePhaseForSites(i,j,qx,qy,phase);
								SxyQ[qx*(yMax+1)+qy]+=phase*ssf;
							}
						}

					}
				}

				for(int qx=0;qx<=xMax;qx++){
					for(int qy=0;qy<=yMax;qy++){
						if(myAbs(SxyQ[qx*(yMax+1)+qy].imag())<=zeroEPS){
							fsTransverseSpinStructureFactorFile.open(ssTransverseSpinStructureFactorFile.str().c_str(),fstream::out | fstream::app);
							fsTransverseSpinStructureFactorFile << setprecision(16);
							for(unsigned int i=0;i<parametersValue.size();i++){
								if(i!=0){
									fsTransverseSpinStructureFactorFile << " ";
								}
								fsTransverseSpinStructureFactorFile << parametersValue[i];
							}
							if(useSzSymm){
								fsTransverseSpinStructureFactorFile << " " << Sz;
							}
							if(periodicBC){
								fsTransverseSpinStructureFactorFile << " " << kx << " " << ky << " " << spinFlip;
							}
							if(useReflectionSymm){
								fsTransverseSpinStructureFactorFile << " " << refx << " " << refy;
							}
							fsTransverseSpinStructureFactorFile << " " << qx << " " << qy;
							fsTransverseSpinStructureFactorFile << setprecision(16);
							fsTransverseSpinStructureFactorFile << " " << SxyQ[qy*(yMax+1)+qy].real() << "\n";
							fsTransverseSpinStructureFactorFile.close();
						}
						else{
							cout << "ERROR: Sxy is not real!\n";
						}
					}
				}
			}

			if(calculateSpinStructureFactor){
				vector<Complex> SzzQ((xMax+1)*(yMax+1),Complex(0));

				for(unsigned int i=0;i<numberOfSites;i++){
					siteCombinatinsOfOperatorModel[1][0][0]=i;
					for(unsigned int j=0;j<numberOfSites;j++){
						siteCombinatinsOfOperatorModel[1][0][1]=j;
						eigenVec.applyLocalOperator(u,operatorModels[1],siteCombinatinsOfOperatorModel[1],kx,ky,spinFlip,refx,refy);
						Complex ssf=eigenVec*u;
						for(int qx=0;qx<=xMax;qx++){
							for(int qy=0;qy<=yMax;qy++){
								Complex phase=0;
								LatticeSymmetryClass::calculatePhaseForSites(i,j,qx,qy,phase);
								SzzQ[qx*(yMax+1)+qy]+=phase*ssf;
							}
						}
					}
				}

				for(int qx=0;qx<=xMax;qx++){
					for(int qy=0;qy<=yMax;qy++){
						if(myAbs(SzzQ[qx*(yMax+1)+qy].imag())<=zeroEPS){
							fsSpinStructureFactorFile.open(ssSpinStructureFactorFile.str().c_str(),fstream::out | fstream::app);
							fsSpinStructureFactorFile << setprecision(16);
							for(unsigned int i=0;i<parametersValue.size();i++){
								if(i!=0){
									fsSpinStructureFactorFile << " ";
								}
								fsSpinStructureFactorFile << parametersValue[i];
							}
							if(useSzSymm){
								fsSpinStructureFactorFile << " " << Sz;
							}
							if(periodicBC){
								fsSpinStructureFactorFile << " " << kx << " " << ky << " " << spinFlip;
							}
							if(useReflectionSymm){
								fsSpinStructureFactorFile << " " << refx << " " << refy;
							}
							fsSpinStructureFactorFile << " " << qx << " " << qy;
							fsSpinStructureFactorFile << setprecision(16);
							fsSpinStructureFactorFile << " " << SzzQ[qx*(yMax+1)+qy].real() << "\n";
							fsSpinStructureFactorFile.close();
						}
						else{
							cout << "Error: Szz is not real!\n";
						}
					}
				}
			}

			if(calculateMagnetization){
				Complex M=0;

				for(unsigned int i=0;i<numberOfSites;i++){
					siteCombinatinsOfOperatorModel[2][0][0]=i;
					eigenVec.applyLocalOperator(u,operatorModels[2],siteCombinatinsOfOperatorModel[2],kx,ky,spinFlip,refx,refy);
					Complex Mtemp=eigenVec*u;;
					M+=Mtemp;
				}
				if(myAbs(M.imag())<=zeroEPS){
					fsMagnetizationFile.open(ssMagnetizationFile.str().c_str(),fstream::out | fstream::app);
					fsMagnetizationFile << setprecision(16);
					for(unsigned int i=0;i<parametersValue.size();i++){
						if(i!=0){
							fsMagnetizationFile << " ";
						}
						fsMagnetizationFile << parametersValue[i];
					}
					if(useSzSymm){
						fsMagnetizationFile << " " << Sz;
					}
					if(periodicBC){
						fsMagnetizationFile << " " << kx << " " << ky << " " << spinFlip;
					}
					if(useReflectionSymm){
						fsMagnetizationFile << " " << refx << " " << refy;
					}
					fsMagnetizationFile << setprecision(16);
					fsMagnetizationFile << " " << M.real() << "\n";
					fsMagnetizationFile.close();
				}
				else{
					cout << "Error: M is not real!\n";
				}
			}

			if(calculateTrimerization){
				Complex T=0;

				for(unsigned int i=0;i<numberOfSites;i+=3){
					//S_0*S_1
					siteCombinatinsOfOperatorModel[3][0][0]=i;
					siteCombinatinsOfOperatorModel[3][0][1]=i+1;
					eigenVec.applyLocalOperator(u,operatorModels[3],siteCombinatinsOfOperatorModel[3],kx,ky,spinFlip,refx,refy);
					Complex Ttemp=eigenVec*u;;
					T+=Ttemp;
					//S_0*S_2
					siteCombinatinsOfOperatorModel[3][0][0]=i;
					siteCombinatinsOfOperatorModel[3][0][1]=i+2;
					eigenVec.applyLocalOperator(u,operatorModels[3],siteCombinatinsOfOperatorModel[3],kx,ky,spinFlip,refx,refy);
					Ttemp=eigenVec*u;;
					T+=Ttemp;
					//S_1*S_2
					siteCombinatinsOfOperatorModel[3][0][0]=i+1;
					siteCombinatinsOfOperatorModel[3][0][1]=i+2;
					eigenVec.applyLocalOperator(u,operatorModels[3],siteCombinatinsOfOperatorModel[3],kx,ky,spinFlip,refx,refy);
					Ttemp=eigenVec*u;;
					T+=Ttemp;
				}
				if(myAbs(T.imag())<=zeroEPS){
					fsTrimerizationFile.open(ssTrimerizationFile.str().c_str(),fstream::out | fstream::app);
					fsTrimerizationFile << setprecision(16);
					for(unsigned int i=0;i<parametersValue.size();i++){
						if(i!=0){
							fsTrimerizationFile << " ";
						}
						fsTrimerizationFile << parametersValue[i];
					}
					if(useSzSymm){
						fsTrimerizationFile << " " << Sz;
					}
					if(periodicBC){
						fsTrimerizationFile << " " << kx << " " << ky << " " << spinFlip;
					}
					if(useReflectionSymm){
						fsTrimerizationFile << " " << refx << " " << refy;
					}
					fsTrimerizationFile << setprecision(16);
					fsTrimerizationFile << " " << T.real() << "\n";
					fsTrimerizationFile.close();
				}
				else{
					cout << "Error: T is not real!\n";
				}
			}
		}
		else{
			cout << "numberOfEigenvectors!=1: Eigenvectors were not calculated!\n";
		}
	}
}

void EDController::performFullED(vector<Doub> const &parametersValue){
	cout << "parameters=";
	for(unsigned int i=0;i<parametersValue.size();i++){
		cout << parametersValue[i] << " ";
	}
	cout << "\n";

	vector<Doub> parametersValueTemp=parametersValue;
	if(useAngleParameter){
		cout << "convert parameters to cos/sin\n";
		parametersValueTemp[0]=cos(parametersValue[1]);
		parametersValueTemp[1]=sin(parametersValue[1]);
	}

	// evaluate series
	vector<LocalSpinModelClassTemplate<stateInt,Complex> > evaluatedLocalSpinModels(referenceSpinModels.size());

	for(unsigned int i=0;i<referenceSpinModels.size();i++){
		SpinModelClassTemplate<pair<SeriesDouble,SeriesDouble> > tempSpinModel=referenceSpinModels[i];
		SpinModelClassTemplate<Complex> evaluatedSpinModel=tempSpinModel.evaluateSpinModel(parametersValueTemp);
		evaluatedLocalSpinModels[i]=LocalSpinModelClassTemplate<stateInt,Complex>(evaluatedSpinModel);
	}


	// now perform the full diagonalization for each Sz-Sector separately
	stringstream ssEigenvaluesFile;
	ssEigenvaluesFile << "eigenvaluesFull";
	if(periodicBC){
		ssEigenvaluesFile << "_" << xMax << "x" << yMax;
	}
	ssEigenvaluesFile << filenameAppendix << ".txt";
	fstream fsEigenvaluesFile;

	VecDoub NReigenVec;

	/////// diagonalize the complete matrix ///////

	SpinStateComplex testState;
	unsigned int repSize=testState.size();
	cout << "repSize=" << repSize << "\n";
	vector<SpinStateComplex> allTestStates;
	for(unsigned int i=0;i<repSize;i++){
		allTestStates.push_back(SpinStateComplex(i));
	}
	cout << "create hamilton matrix:\n";
	MatDoub A(repSize,repSize);
	for(unsigned int i=0;i<repSize;i++){
		for(unsigned int j=0;j<repSize;j++){
			allTestStates[j].applyLocalSpinHamiltonian(testState,evaluatedLocalSpinModels,siteCombinationsToReferenceSpinModel,kx,ky,spinFlip,refx,refy);
			A[i][j]=real(allTestStates[i]*testState);
		}
	}
	cout << "calculate eigenvalues:\n";
	Symmeig TSymmEigTest(A);
	/*
		cout << "full diagonalization eigenvalues:\n";
		for(unsigned int i=0;i<repSize;i++){
			cout << setprecision(16) << TSymmEigTest.d[i] << "\n";
		}
	 */
	//////////////////////////////////////////////


	// save eigenvalues
	fsEigenvaluesFile.open(ssEigenvaluesFile.str().c_str(),fstream::out | fstream::app);
	fsEigenvaluesFile << setprecision(16);
	for(unsigned int i=0;i<parametersValue.size();i++){
		fsEigenvaluesFile << parametersValue[i] << " ";
	}
	fsEigenvaluesFile << Sz;
	// write eigenvalues to file
	for(unsigned int i=1;i<=repSize;i++){
		fsEigenvaluesFile << " " << TSymmEigTest.d[repSize-i];
	}
	fsEigenvaluesFile << "\n";
	fsEigenvaluesFile.close();
}


EDController::EDController(string const &sParametersFile){
	// default values
	convergenceEPS=1e-12;
	createNewEigenvaluesFile=true;
	filenameAppendix="";
	calculationType=0;
	useSublatticeCoding=0;
	useAngleParameter=false;
	periodicBC=true;
	useSzSymm=true;
	useReflectionSymm=false;
	useLocalSpinModel=false;
	calculateMagnetization=false;
	calculateTrimerization=false;

	bool allSzValues=true;
	bool allKValues=true;
	bool allSpinFlipValues=true;
	bool allReflectionValues=true;

	bool allSpinModelOperators=true;
	vector<unsigned int> spinModelOperators;

	bool allSpinModelTerms=true;
	vector<unsigned int> spinModelTerms;

	// filenames
	string sReferenceSpinModelFile;
	string sSiteCombinationsFile;
	string sSiteMapFile;
	string sInterUnitCellVectorsFile;
	string sIntraUnitCellVectorsFile;

	string sReflectionMapFile;

	string sLocalSpinModelFile;

	unsigned int localSpinModelIndex=0;

	// load parameters file
	fstream fsParametersFile;
	fsParametersFile.open(sParametersFile.c_str(),fstream::in);
	string tempString;
	Doub tempDoub;
	unsigned int tempUint;
	mpq_class tempMpq;
	int tempInt;
	while(fsParametersFile>>tempString){

		// parameter values
		if(tempString=="parametersStart"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile>>tempDoub;
				parametersStart.push_back(tempDoub);
			}
		}
		if(tempString=="parametersStep"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile>>tempDoub;
				parametersStep.push_back(tempDoub);
			}
		}
		if(tempString=="parametersEnd"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile>>tempDoub;
				parametersEnd.push_back(tempDoub);
			}
		}
		if(tempString=="multipleOfFirstParameter"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile >> tempUint;
				multipleOfFirstParameter.push_back(tempUint);
			}
		}
		if(tempString=="useAngleParameter"){
			fsParametersFile >> useAngleParameter;
		}

		// calculation types
		if(tempString=="calculationType"){
			fsParametersFile >> calculationType;
		}
		if(tempString=="useSublatticeCoding"){
			fsParametersFile >> useSublatticeCoding;
		}

		// lattice specifications
		if(tempString=="singleSpin"){
			fsParametersFile >> tempMpq;
			tempMpq*=2;
			mpz_class tempMpz=tempMpq.get_num();
			totalS=tempMpz.get_ui();
		}
		if(tempString=="numberOfSites"){
			fsParametersFile >> numberOfSites;
		}
		if(tempString=="xMax"){
			fsParametersFile >> xMax;
		}
		if(tempString=="yMax"){
			fsParametersFile >> yMax;
		}
		if(tempString=="siteMapFile"){
			fsParametersFile >> sSiteMapFile;
		}
		if(tempString=="interUnitCellVectorsFile"){
			fsParametersFile >> sInterUnitCellVectorsFile;
		}
		if(tempString=="intraUnitCellVectorsFile"){
			fsParametersFile >> sIntraUnitCellVectorsFile;
		}
		if(tempString=="reflectionMapFile"){
			fsParametersFile >> sReflectionMapFile;
		}

		// symmetry parameters
		if(tempString=="useSzSymm"){
			fsParametersFile >> useSzSymm;
		}
		if(tempString=="allSzValues"){
			fsParametersFile >> allSzValues;
		}
		if(tempString=="Sz"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile >> tempMpq;
				SzValues.push_back(tempMpq);
			}
		}
		if(tempString=="periodicBC"){
			fsParametersFile >> periodicBC;
		}
		if(tempString=="allKValues"){
			fsParametersFile >> allKValues;
		}
		if(tempString=="kx"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile >> tempInt;
				kxValues.push_back(tempInt);
			}
		}
		if(tempString=="ky"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile >> tempInt;
				kyValues.push_back(tempInt);
			}
		}
		if(tempString=="allSpinFlipValues"){
			fsParametersFile >> allSpinFlipValues;
		}
		if(tempString=="spinFlipValues"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile>>tempInt;
				spinFlipValues.push_back(tempInt);
			}
		}
		if(tempString=="useReflectionSymm"){
			fsParametersFile>>useReflectionSymm;
		}
		if(tempString=="allReflectionValues"){
			fsParametersFile>>allReflectionValues;
		}
		if(tempString=="refx"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile>>tempInt;
				refxValues.push_back(tempInt);
			}
		}
		if(tempString=="refy"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile>>tempInt;
				refyValues.push_back(tempInt);
			}
		}

		if(tempString=="useLocalSpinModel"){
			fsParametersFile>>useLocalSpinModel;
		}
		if(tempString=="localSpinModelFile"){
			fsParametersFile>>sLocalSpinModelFile;
		}
		if(tempString=="localSpinModelIndex"){
			fsParametersFile>>localSpinModelIndex;
		}

		// filenames
		if(tempString=="referenceSpinModelFile"){
			fsParametersFile >> sReferenceSpinModelFile;

		}
		if(tempString=="siteCombinationsFile"){
			fsParametersFile >> sSiteCombinationsFile;
		}
		if(tempString=="filenameAppendix"){
			fsParametersFile >> filenameAppendix;
		}

		// spin model parameters
		if(tempString=="allSpinModelOperators"){
			fsParametersFile >> allSpinModelOperators;
		}
		if(tempString=="spinModelOperators"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile >> tempUint;
				spinModelOperators.push_back(tempUint);
			}
		}
		if(tempString=="allSpinModelTerms"){
			fsParametersFile >> allSpinModelTerms;
		}
		if(tempString=="spinModelTerms"){
			while(fsParametersFile.peek()!='\n'){
				fsParametersFile >> tempUint;
				spinModelTerms.push_back(tempUint);
			}
		}

		// output parameters
		if(tempString=="createNewEigenvaluesFile"){
			fsParametersFile >> createNewEigenvaluesFile;
		}

		// lanczos parameters
		if(tempString=="maxNumberOfIterations"){
			fsParametersFile >> maxNumberOfIterations;
		}
		if(tempString=="numberOfEigenvalues"){
			fsParametersFile >> numberOfEigenvalues;
		}
		if(tempString=="convergenceEPS"){
			fsParametersFile >> convergenceEPS;
		}
		if(tempString=="numberOfEigenvectors"){
			fsParametersFile >> numberOfEigenvectors;
		}

		// operator parameters
		if(tempString=="calculateTransverseSpinStructureFactor"){
			fsParametersFile >> calculateTransverseSpinStructureFactor;
		}
		if(tempString=="calculateSpinStructureFactor"){
			fsParametersFile >> calculateSpinStructureFactor;
		}
		if(tempString=="calculateMagnetization"){
			fsParametersFile >> calculateMagnetization;
		}
		if(tempString=="calculateTrimerization"){
			fsParametersFile >> calculateTrimerization;
		}

		// zero parameters
		if(tempString=="zeroEPS"){
			fsParametersFile >> zeroEPS;
		}
	}
	fsParametersFile.close();

	// if all Sz values should used clear SzValues and set them to all SzValues
	if(allSzValues){
		SzValues.clear();
		mpq_class maxSz(numberOfSites,2);
		for(mpq_class SzLoop=0;SzLoop<=maxSz;SzLoop+=1){
			SzValues.push_back(SzLoop);
		}
	}

	// if all kValues should used clear kxValues and kyValues and set them to all kValues
	if(allKValues){
		kxValues.clear();
		kyValues.clear();
		for(unsigned int i=0;i<=xMax;i++){
			kxValues.push_back(i);
		}
		for(unsigned int i=0;i<=yMax;i++){
			kyValues.push_back(i);
		}
	}

	if(allSpinFlipValues){
		spinFlipValues.clear();
		spinFlipValues.push_back(-1);
		spinFlipValues.push_back(1);
	}

	// if all reflectionValues should be used clear refxValues and refyValues and set them to all refValues
	if(allReflectionValues){
		refxValues.clear();
		refyValues.clear();
		refxValues.push_back(-1);
		refxValues.push_back(1);
		refyValues.push_back(-1);
		refyValues.push_back(1);
	}

	// from the totalS create the number of bits of the localHilbertSpace
	unsigned int start=2;
	localHilbertSpace=1;
	while(start<(totalS+1)){
		start*=2;
		localHilbertSpace++;
	}

	// test output
	cout << "Test Output of Parameters:\n";
	cout << "parametersStart=";
	for(unsigned int i=0;i<parametersStart.size();i++){
		cout << parametersStart[i] << " ";
	}
	cout << "\n";
	cout << "parametersStep=";
	for(unsigned int i=0;i<parametersStep.size();i++){
		cout << parametersStep[i] << " ";
	}
	cout << "\n";
	cout << "parametersEnd=";
	for(unsigned int i=0;i<parametersEnd.size();i++){
		cout << parametersEnd[i] << " ";
	}
	cout << "\n";
	cout << "multipleOfFirstParameter=";
	for(unsigned int i=0;i<multipleOfFirstParameter.size();i++){
		cout << multipleOfFirstParameter[i] << " ";
	}
	cout << "\n";
	cout << "useAngleParameter=" << useAngleParameter << "\n";
	cout << "calculationType=" << calculationType << "\n";
	cout << "useSublatticeCoding=" << useSublatticeCoding << "\n";
	cout << "singleSpin=" << mpq_class(totalS,2) << "\n";
	cout << "localHilbertSpace=" << localHilbertSpace << "\n";
	cout << "numberOfSites=" << numberOfSites << "\n";
	cout << "xMax=" << xMax << "\n";
	cout << "yMax=" << yMax << "\n";
	cout << "siteMapFile=" << sSiteMapFile << "\n";
	cout << "interUnitCellVectorsFile=" << sInterUnitCellVectorsFile << "\n";
	cout << "intraUnitCellVectorsFile=" << sIntraUnitCellVectorsFile << "\n";
	cout << "reflectionMapFile=" << sReflectionMapFile << "\n";
	cout << "useSzSymm=" << useSzSymm << "\n";
	cout << "allSzValues=" << allSzValues << "\n";
	cout << "Sz=";
	for(unsigned int i=0;i<SzValues.size();i++){
		cout << SzValues[i] << " ";
	}
	cout << "\n";
	cout << "periodicBC=" << periodicBC << "\n";
	cout << "allKValues=" << allKValues << "\n";
	cout << "kx=";
	for(unsigned int i=0;i<kxValues.size();i++){
		cout << kxValues[i] << " ";
	}
	cout << "\n";
	cout << "ky=";
	for(unsigned int i=0;i<kyValues.size();i++){
		cout << kyValues[i] << " ";
	}
	cout << "\n";
	cout << "allSpinFlipValues=" << allSpinFlipValues << "\n";
	for(unsigned int i=0;i<spinFlipValues.size();i++){
		cout << spinFlipValues[i] << " ";
	}
	cout << "\n";
	cout << "useReflectionSymm=" << useReflectionSymm << "\n";
	cout << "allReflectionValues=" << allReflectionValues << "\n";
	cout << "refx=";
	for(unsigned int i=0;i<refxValues.size();i++){
		cout << refxValues[i] << " ";
	}
	cout << "\n";
	cout << "refy=";
	for(unsigned int i=0;i<refyValues.size();i++){
		cout << refyValues[i] << " ";
	}
	cout << "\n";
	cout << "useLocalSpinModel=" << useLocalSpinModel << "\n";
	cout << "localSpinModelFile=" << sLocalSpinModelFile << "\n";
	cout << "localSpinModelIndex=" << localSpinModelIndex << "\n";
	cout << "referenceSpinModelFile=" << sReferenceSpinModelFile << "\n";
	cout << "siteCombinationsFile=" << sSiteCombinationsFile << "\n";
	cout << "filenameAppendinx=" << filenameAppendix << "\n";
	cout << "allSpinModelOperators=" << allSpinModelOperators << "\n";
	cout << "spinModelOperators=";
	for(unsigned int i=0;i<spinModelOperators.size();i++){
		cout << spinModelOperators[i] << " ";
	}
	cout << "\n";
	cout << "allSpinModelTerms=" << allSpinModelTerms << "\n";
	cout << "spinModelTerms=";
	for(unsigned int i=0;i<spinModelTerms.size();i++){
		cout << spinModelTerms[i] << " ";
	}
	cout << "\n";
	cout << "createNewEigenvaluesFile=" << createNewEigenvaluesFile << "\n";
	cout << "maxNumberOfIterations=" << maxNumberOfIterations << "\n";
	cout << "numberOfEigenvalues=" << numberOfEigenvalues << "\n";
	cout << "convergenceEPS=" << convergenceEPS << "\n";
	cout << "numberOfEigenvectors=" << numberOfEigenvectors << "\n";
	cout << "calculateTransverseSpinStructureFactor=" << calculateTransverseSpinStructureFactor << "\n";
	cout << "calculateSpinStructureFactor=" << calculateSpinStructureFactor << "\n";
	cout << "calculateMagnetization=" << calculateMagnetization << "\n";
	cout << "calculateTrimerization=" << calculateTrimerization << "\n";
	cout << "zeroEPS=" << zeroEPS << "\n";

	if(calculationType<2){
		// load referenceSpinModelsFile
		cout << "load referenceSpinModelsFile\n";

		vector<unsigned int> indexForRefModel; //save the index of the referenceSpinModel in the vector
		fstream fsReferenceSpinModels;
		fsReferenceSpinModels.open(sReferenceSpinModelFile.c_str(),fstream::in);
		while(fsReferenceSpinModels >> tempUint){
			SpinModelClassTemplate<pair<SeriesDouble,SeriesDouble> > tempSpinModel;
			do{
				SpinModelClassTemplate<pair<SeriesDouble,SeriesDouble> > readinSpinModel;
				fsReferenceSpinModels >> readinSpinModel;
				tempSpinModel+=readinSpinModel;

			}while(fsReferenceSpinModels.peek()!='\n');

			bool pushBack=allSpinModelOperators && allSpinModelTerms;
			if(!pushBack){
				if(!allSpinModelOperators){
					for(unsigned int i=0;i<spinModelOperators.size();i++){
						if(spinModelOperators[i]==tempUint){
							pushBack=true;
							break;
						}
					}
				}
				else{
					pushBack=true;
				}
				if(!allSpinModelTerms && pushBack){
					pushBack=false;
					for(unsigned int i=0;i<spinModelTerms.size();i++){
						if(spinModelTerms[i]==tempSpinModel.numberOfOperators()){
							pushBack=true;
							break;
						}
					}
				}
			}
			if(pushBack){
				referenceSpinModels.push_back(tempSpinModel);
				indexForRefModel.push_back(tempUint);
			}
		}
		fsReferenceSpinModels.close();

		// if localSpinModel should be used load the local spinModel and identify it with the index from parameters.config
		if(useLocalSpinModel){
			cout << "load localSpinModelFile\n";
			indexForRefModel.push_back(localSpinModelIndex);
			fstream fsLocalSpinModel;
			fsLocalSpinModel.open(sLocalSpinModelFile.c_str(),fstream::in);
			fsLocalSpinModel>>localSpinModel;
			fsLocalSpinModel.close();
		}

		// load siteCombinationsFile
		cout << "load siteCombinationsFile\n";
		fstream fsSiteCombinations;
		fsSiteCombinations.open(sSiteCombinationsFile.c_str(),fstream::in);
		getline(fsSiteCombinations,tempString);
		unsigned short tempUshort;
		while(fsSiteCombinations >> tempUint){
			vector<unsigned short> tempComb;
			while(fsSiteCombinations >> tempUshort){
				tempComb.push_back(tempUshort);
				if(fsSiteCombinations.peek()=='\n'){
					// check if sites are redundant in this particular combination! => no site should be contained more than once!
					bool sitesOnlyOnce=true;
					for(unsigned int tempCombIndex=0;tempCombIndex<tempComb.size()-1;tempCombIndex++){
						for(unsigned int tempCombIndex2=tempCombIndex+1;tempCombIndex2<tempComb.size();tempCombIndex2++){
							if(tempComb[tempCombIndex]==tempComb[tempCombIndex2]){
								sitesOnlyOnce=false;
								break;
							}
						}
						if(!sitesOnlyOnce){
							break;
						}
					}
					if(sitesOnlyOnce){
						bool insertSiteComb=allSpinModelTerms;
						if(!allSpinModelTerms){
							for(unsigned int i=0;i<spinModelTerms.size();i++){
								if(spinModelTerms[i]==tempComb.size()){
									insertSiteComb=true;
									break;
								}
							}
						}
						if(!allSpinModelOperators && insertSiteComb){
							insertSiteComb=false;
							for(unsigned int i=0;i<spinModelOperators.size();i++){
								if(spinModelOperators[i]==tempUint){
									insertSiteComb=true;
									break;
								}
							}
						}

						if(insertSiteComb){
							for(unsigned int j=0;j<indexForRefModel.size();j++){
								if(indexForRefModel[j]==tempUint){
									siteCombinationsToReferenceSpinModel.push_back(pair<vector<unsigned short>,unsigned int>(tempComb,j));
									break;
								}
							}
						}
					}
					else{
						cout << "WARNING: same site is contained twice! combination is ignored!\n";
						cout << "referenceSpinModelIndex=" << tempUint << "\n";
						cout << "sites=";
						for(unsigned int i=0;i<tempComb.size();i++){
							cout << tempComb[i] << " ";
						}
						cout << "\n";
					}
					break;
				}
			}
		}
		fsSiteCombinations.close();


		// test output of referenceSpinModels
		cout << "test output of referenceSpinModels:\n";
		for(unsigned int i=0;i<referenceSpinModels.size();i++){
			cout << "i=" << i << "\n";
			cout << referenceSpinModels[i] << "\n";
		}

		cout << "test output of localSpinModel:\n";
		cout << localSpinModel << "\n";
		/*
		cout << "test output of siteCombinations:\n";
		map<vector<unsigned short>,unsigned int>::iterator itTest;
		for(itTest=siteCombinationsToReferenceSpinModel.begin();itTest!=siteCombinationsToReferenceSpinModel.end();itTest++){
			cout << itTest->second << ": ";
			for(unsigned int i=0;i<itTest->first.size();i++){
				cout << itTest->first[i] << " ";
			}
			cout << "\n";
		}
		 */
	}

	// load siteMap and interUnitCellVectors
	if(periodicBC){
		vector<pair<int,int> > invSiteMap(numberOfSites);
		vector<pair<int,int> > interUnitCellVectors;
		vector<pair<int,int> > intraUnitCellVectors;

		// siteMap
		cout << "load siteMap\n";
		fstream fsSiteMap;
		fsSiteMap.open(sSiteMapFile.c_str(),fstream::in);
		int x,y,siteIndex;
		char tempChar;
		while(fsSiteMap>>x){
			fsSiteMap >> y;
			fsSiteMap >> skipws >> tempChar;
			fsSiteMap >> tempChar;
			fsSiteMap >> siteIndex;
			invSiteMap[siteIndex]=pair<int,int>(x,y);
		}
		fsSiteMap.close();

		for(unsigned int i=0;i<invSiteMap.size();i++){
			cout << invSiteMap[i].first << " " << invSiteMap[i].second << "\n";
		}

		// interUnitCellVectors
		cout << "load interUnitCellVectors\n";
		fstream fsInterUnitCellVectors;
		fsInterUnitCellVectors.open(sInterUnitCellVectorsFile.c_str(),fstream::in);
		getline(fsInterUnitCellVectors,tempString);
		while(fsInterUnitCellVectors>>x){
			fsInterUnitCellVectors>>y;
			interUnitCellVectors.push_back(pair<int,int>(x,y));
		}
		fsInterUnitCellVectors.close();

		cout << "load intraUnitCellVectors\n";
		fstream fsIntraUnitCellVectors;
		fsIntraUnitCellVectors.open(sIntraUnitCellVectorsFile.c_str(),fstream::in);
		getline(fsIntraUnitCellVectors,tempString);
		while(fsIntraUnitCellVectors>>tempChar){
			if(fsIntraUnitCellVectors>>x){
				fsIntraUnitCellVectors>>y;
				intraUnitCellVectors.push_back(pair<int,int>(x,y));
				getline(fsIntraUnitCellVectors,tempString);
			}
			else{
				break;
			}
		}
		fsIntraUnitCellVectors.close();

		cout << "interUnitCellVectors:\n";
		for(unsigned int i=0;i<interUnitCellVectors.size();i++){
			cout << interUnitCellVectors[i].first << " " << interUnitCellVectors[i].second << "\n";
		}
		cout << "intraUnitCellVectors:\n";
		for(unsigned int i=0;i<intraUnitCellVectors.size();i++){
			cout << intraUnitCellVectors[i].first << " " << intraUnitCellVectors[i].second << "\n";
		}

		cout << "initializeSiteMap\n";
		LatticeSymmetryClass::initializeSiteMap(xMax+1,yMax+1,invSiteMap,interUnitCellVectors,intraUnitCellVectors); // here xMax/yMax must be increased by one, since this is the maximum Number of unitCells in one direction
	}

	// if reflection symmetry should be used initialize the site map in the same way as for translation symmetry (IMPORTANT: at the moment reflection symmetry and translation symmetry are not compatible)
	if(useReflectionSymm){
		// load reflectionMap
		vector<pair<int,int> > reflectionMap(numberOfSites);
		int site,x,y;
		fstream fsReflectionMap;
		fsReflectionMap.open(sReflectionMapFile.c_str(),fstream::in);
		getline(fsReflectionMap,tempString);
		while(fsReflectionMap>>site){
			fsReflectionMap>>x;
			fsReflectionMap>>y;
			reflectionMap[site].first=x;
			reflectionMap[site].second=y;
		}
		fsReflectionMap.close();

		// create shiftedSites within LatticeSymmetryClass
		LatticeSymmetryClass::initializeShiftedSitesForReflectionSymmetry(reflectionMap);
	}

	// initialize Operators
	if(calculateTransverseSpinStructureFactor){
		// generate operators
		SpinOperatorsOnSitesPMBasis tempOperator1;
		tempOperator1.insertNewOperator('p',0);
		tempOperator1.insertNewOperator('m',1);
		SpinOperatorsOnSitesPMBasis tempOperator2;
		tempOperator2.insertNewOperator('m',0);
		tempOperator2.insertNewOperator('p',1);
		SpinModelClassTemplate<Complex> tempSpinModel(Complex(1.0),tempOperator1);
		tempSpinModel+=SpinModelClassTemplate<Complex>(Complex(1.0),tempOperator2);

		LocalSpinModelClassTemplate<stateInt,Complex> tempLocalSpinModel(tempSpinModel);
		operatorModels.push_back(tempLocalSpinModel);
		cout << "tempLocalSpinModel:\n";
		cout << tempLocalSpinModel << "\n";

		// generate siteCombinations
		vector<unsigned short> vec1(2,0);
		vector<vector<unsigned short> > vec2;
		vec2.push_back(vec1);
		siteCombinatinsOfOperatorModel.push_back(vec2);
	}
	if(calculateSpinStructureFactor){
		// as this is the 2nd operator, add a dummy operator if the size of the operatorModels is zero
		if(operatorModels.size()<1){
			operatorModels.push_back(LocalSpinModelClassTemplate<stateInt,Complex>());
			siteCombinatinsOfOperatorModel.push_back(vector<vector<unsigned short> >());
		}
		SpinOperatorsOnSitesPMBasis tempOperator1;
		tempOperator1.insertNewOperator('z',0);
		tempOperator1.insertNewOperator('z',1);
		SpinModelClassTemplate<Complex> tempSpinModel(Complex(1.0),tempOperator1);
		//cout << "tempSpinModel=" << tempSpinModel << "\n";
		LocalSpinModelClassTemplate<stateInt,Complex> tempLocalSpinModel(tempSpinModel);
		//cout << "tempLocalSpinModel=" << tempLocalSpinModel << "\n";
		operatorModels.push_back(tempLocalSpinModel);

		// generate siteCombination 0 0, which is just a dummy and must be changed during the calculation
		vector<unsigned short> vec1(2,0);
		vector<vector<unsigned short> > vec2;
		vec2.push_back(vec1);
		siteCombinatinsOfOperatorModel.push_back(vec2);
	}
	if(calculateMagnetization){
		// as this is the 3rd operator, add dummy operators if the size of the operatorModels does not match
		if(operatorModels.size()<2){
			if(operatorModels.size()<1){
				operatorModels.push_back(LocalSpinModelClassTemplate<stateInt,Complex>());
				siteCombinatinsOfOperatorModel.push_back(vector<vector<unsigned short> >());
			}
			operatorModels.push_back(LocalSpinModelClassTemplate<stateInt,Complex>());
			siteCombinatinsOfOperatorModel.push_back(vector<vector<unsigned short> >());
		}
		SpinOperatorsOnSitesPMBasis tempOperator1;
		tempOperator1.insertNewOperator('z',0);
		SpinModelClassTemplate<Complex> tempSpinModel(Complex(1.0),tempOperator1);
		//cout << "tempSpinModel=" << tempSpinModel << "\n";
		LocalSpinModelClassTemplate<stateInt,Complex> tempLocalSpinModel(tempSpinModel);
		//cout << "tempLocalSpinModel=" << tempLocalSpinModel << "\n";
		operatorModels.push_back(tempLocalSpinModel);

		// generate siteCombination 0, which is just a dummy and must be changed during the calculation
		vector<unsigned short> vec1(1,0);
		vector<vector<unsigned short> > vec2;
		vec2.push_back(vec1);
		siteCombinatinsOfOperatorModel.push_back(vec2);
	}
	if(calculateTrimerization){
		// as this is the 4th operator, add dummy operators if the size of the operatorModels does not match
		for(unsigned int i=operatorModels.size();i<3;i++){
			operatorModels.push_back(LocalSpinModelClassTemplate<stateInt,Complex>());
			siteCombinatinsOfOperatorModel.push_back(vector<vector<unsigned short> >());
		}
		SpinOperatorsOnSitesPMBasis tempOperator1;
		tempOperator1.insertNewOperator('p',0);
		tempOperator1.insertNewOperator('m',1);
		SpinOperatorsOnSitesPMBasis tempOperator2;
		tempOperator2.insertNewOperator('m',0);
		tempOperator2.insertNewOperator('p',1);
		SpinOperatorsOnSitesPMBasis tempOperator3;
		tempOperator3.insertNewOperator('z',0);
		tempOperator3.insertNewOperator('z',1);

		SpinModelClassTemplate<Complex> tempSpinModel(Complex(2.0),tempOperator1);
		tempSpinModel+=SpinModelClassTemplate<Complex>(Complex(2.0),tempOperator2);
		tempSpinModel+=SpinModelClassTemplate<Complex>(Complex(1.0),tempOperator3);

		LocalSpinModelClassTemplate<stateInt,Complex> tempLocalSpinModel(tempSpinModel);
		operatorModels.push_back(tempLocalSpinModel);

		// generate siteCombination 0, which is just a dummy and must be changed during the calculation
		vector<unsigned short> vec1(2,0);
		vector<vector<unsigned short> > vec2;
		vec2.push_back(vec1);
		siteCombinatinsOfOperatorModel.push_back(vec2);
	}
}

void EDController::startCalculation(){
	// create new file if required
	if(createNewEigenvaluesFile){
		stringstream ssEigenvaluesFile;
		ssEigenvaluesFile << "eigenvalues";
		if(calculationType==1){
			ssEigenvaluesFile << "Full";
		}
		if(periodicBC){
				ssEigenvaluesFile << "_" << xMax << "x" << yMax;
		}
		ssEigenvaluesFile << filenameAppendix << ".txt";
		fstream fsEigenvaluesFile;
		fsEigenvaluesFile.open(ssEigenvaluesFile.str().c_str(), fstream::out);
		fsEigenvaluesFile << "#";
		for(unsigned int i=0;i<parametersStart.size();i++){
			if(i!=0){
				fsEigenvaluesFile << " ";
			}
			fsEigenvaluesFile << "h" << i;
		}
		if(useSzSymm){
			fsEigenvaluesFile << " Sz";
		}
		if(periodicBC){
			fsEigenvaluesFile << " kx ky spinFlip";
		}
		if(useReflectionSymm){
			fsEigenvaluesFile << " refx refy";
		}
		for(unsigned int i=0;i<numberOfEigenvalues;i++){
			fsEigenvaluesFile << " E" << i;
		}
		for(unsigned int i=0;i<numberOfEigenvalues;i++){
			fsEigenvaluesFile << " convE" << i;
		}
		fsEigenvaluesFile << "\n";
		fsEigenvaluesFile.close();

		if(calculateTransverseSpinStructureFactor){
			stringstream ssTransverseSpinStructureFactorFile;
			ssTransverseSpinStructureFactorFile << "transverseSpinStructureFactor";
			if(periodicBC){
				ssTransverseSpinStructureFactorFile << "_" << xMax << "x" << yMax;
			}
			ssTransverseSpinStructureFactorFile << filenameAppendix << ".txt";
			fstream fsTransverseSpinStructureFactorFile;
			fsTransverseSpinStructureFactorFile.open(ssTransverseSpinStructureFactorFile.str().c_str(), fstream::out);
			fsTransverseSpinStructureFactorFile << "#";
			for(unsigned int i=0;i<parametersStart.size();i++){
				if(i!=0){
					fsTransverseSpinStructureFactorFile << " ";
				}
				fsTransverseSpinStructureFactorFile << "h" << i;
			}
			if(useSzSymm){
				fsTransverseSpinStructureFactorFile << " Sz";
			}
			if(periodicBC){
				fsTransverseSpinStructureFactorFile << " kx ky spinFlip";
			}
			if(useReflectionSymm){
				fsTransverseSpinStructureFactorFile << " refx refy";
			}
			fsTransverseSpinStructureFactorFile << " qx qy";
			fsTransverseSpinStructureFactorFile << " Sxy";
			fsTransverseSpinStructureFactorFile << "\n";
			fsTransverseSpinStructureFactorFile.close();
		}
		if(calculateSpinStructureFactor){
			stringstream ssSpinStructureFactorFile;
			ssSpinStructureFactorFile << "spinStructureFactor";
			if(periodicBC){
				ssSpinStructureFactorFile << "_" << xMax << "x" << yMax;
			}
			ssSpinStructureFactorFile << filenameAppendix << ".txt";
			fstream fsSpinStructureFactorFile;
			fsSpinStructureFactorFile.open(ssSpinStructureFactorFile.str().c_str(),fstream::out);
			fsSpinStructureFactorFile << "#";
			for(unsigned int i=0;i<parametersStart.size();i++){
				if(i!=0){
					fsSpinStructureFactorFile << " ";
				}
				fsSpinStructureFactorFile << "h" << i;
			}
			if(useSzSymm){
				fsSpinStructureFactorFile << " Sz";
			}
			if(periodicBC){
				fsSpinStructureFactorFile << " kx ky spinFlip";
			}
			if(useReflectionSymm){
				fsSpinStructureFactorFile << " refx refy";
			}
			fsSpinStructureFactorFile << " qx qy";
			fsSpinStructureFactorFile << " Szz";
			fsSpinStructureFactorFile << "\n";
			fsSpinStructureFactorFile.close();
		}
		if(calculateMagnetization){
			stringstream ssMagnetizationFile;
			ssMagnetizationFile << "magnetization";
			if(periodicBC){
				ssMagnetizationFile << "_" << xMax << "x" << yMax;
			}
			ssMagnetizationFile << filenameAppendix << ".txt";
			fstream fsMagnetizationFile;
			fsMagnetizationFile.open(ssMagnetizationFile.str().c_str(),fstream::out);
			fsMagnetizationFile << "#";
			for(unsigned int i=0;i<parametersStart.size();i++){
				if(i!=0){
					fsMagnetizationFile << " ";
				}
				fsMagnetizationFile << "h" << i;
			}
			if(useSzSymm){
				fsMagnetizationFile << " Sz";
			}
			if(periodicBC){
				fsMagnetizationFile << " kx ky spinFlip";
			}
			if(useReflectionSymm){
				fsMagnetizationFile << " refx refy";
			}
			fsMagnetizationFile << " M";
			fsMagnetizationFile << "\n";
			fsMagnetizationFile.close();
		}
		if(calculateTrimerization){
			stringstream ssTrimerizationFile;
			ssTrimerizationFile << "trimerization";
			if(periodicBC){
				ssTrimerizationFile << "_" << xMax << "x" << yMax;
			}
			ssTrimerizationFile << filenameAppendix << ".txt";
			fstream fsTrimerizationFile;
			fsTrimerizationFile.open(ssTrimerizationFile.str().c_str(),fstream::out);
			fsTrimerizationFile << "#";
			for(unsigned int i=0;i<parametersStart.size();i++){
				if(i!=0){
					fsTrimerizationFile << " ";
				}
				fsTrimerizationFile << "h" << i;
			}
			if(useSzSymm){
				fsTrimerizationFile << " Sz";
			}
			if(periodicBC){
				fsTrimerizationFile << " kx ky spinFlip";
			}
			if(useReflectionSymm){
				fsTrimerizationFile << " refx refy";
			}
			fsTrimerizationFile << " Trimerization(only up)";
			fsTrimerizationFile << "\n";
			fsTrimerizationFile.close();
		}
	}
	// loop over all parameters
	for(unsigned int SzIndex=0;SzIndex<SzValues.size();SzIndex++){
		Sz=SzValues[SzIndex];
		for(unsigned int kxIndex=0;kxIndex<kxValues.size();kxIndex++){
			kx=kxValues[kxIndex];
			for(unsigned int kyIndex=0;kyIndex<kyValues.size();kyIndex++){
				ky=kyValues[kyIndex];
				for(unsigned int spinFlipIndex=0;spinFlipIndex<spinFlipValues.size();spinFlipIndex++){
					spinFlip=spinFlipValues[spinFlipIndex];
					for(unsigned int refxIndex=0;refxIndex<refxValues.size();refxIndex++){
						refx=refxValues[refxIndex];
						for(unsigned int refyIndex=0;refyIndex<refyValues.size();refyIndex++){
							refy=refyValues[refyIndex];
							cout << "Sz=" << Sz << " kx=" << kx << " ky=" << ky << " spinFlip=" << spinFlip << " refx=" << refx << " refy=" << refy << "\n";
							SpinStateComplex::createRepresentatives(Sz,kx,ky,spinFlip,refx,refy,useSzSymm,periodicBC,useSublatticeCoding,useReflectionSymm);
							vector<Doub> parametersValue(parametersStart.size());
							iterateParameterValues(0,parametersValue);
						}
					}
				}
			}
		}
	}
}
