/*
 * EDController.h
 *
 *  Created on: Sep 3, 2014
 *      Author: ixert
 */

#ifndef EDCONTROLLER_H_
#define EDCONTROLLER_H_

#include "nr3.h"
#include<utility>
#include<vector>
#include "SpinModelClassTemplate.h"
#include "LocalSpinModelClassTemplate.h"
#include<gmpxx.h>
#include<gmp.h>

using namespace std;

class EDController {
private:
	vector<Doub> parametersStart;
	vector<Doub> parametersStep;
	vector<Doub> parametersEnd;

	vector<unsigned int> multipleOfFirstParameter;
	bool useAngleParameter;

	unsigned int calculationType;
	unsigned int useSublatticeCoding;

	bool periodicBC;
	bool useSzSymm;
	bool useReflectionSymm;

	bool useLocalSpinModel;

	bool createNewEigenvaluesFile;

	unsigned int maxNumberOfIterations;
	unsigned int numberOfEigenvalues;
	Doub convergenceEPS;
	unsigned int numberOfEigenvectors;

	bool calculateTransverseSpinStructureFactor;
	bool calculateSpinStructureFactor;
	bool calculateMagnetization;
	bool calculateTrimerization;

	vector<LocalSpinModelClassTemplate<stateInt,Complex> > operatorModels; // defines operators
	vector<vector<vector<unsigned short> > > siteCombinatinsOfOperatorModel; // on which sites should the corresponding operator act

	vector<SpinModelClassTemplate<pair<SeriesDouble,SeriesDouble> > > referenceSpinModels;
	LocalSpinModelClassTemplate<stateInt,SeriesDouble> localSpinModel;
	list<pair<vector<unsigned short>,unsigned int> > siteCombinationsToReferenceSpinModel;

	string filenameAppendix;

	unsigned int xMax;
	unsigned int yMax;
	vector<int> kxValues;
	vector<int> kyValues;
	vector<mpq_class> SzValues;
	vector<int> spinFlipValues;
	vector<int> refxValues;
	vector<int> refyValues;

	// actual values of kx,ky,Sz,spinFlip
	int kx;
	int ky;
	mpq_class Sz;
	int spinFlip;
	int refx;
	int refy;

	//// private functions ////
	void iterateParameterValues(unsigned int const &d, vector<Doub> &parametersValue);
	void performLanczosED(vector<Doub> const &parametersValue);
	void performFullED(vector<Doub> const &parametersValue);

public:
	EDController(string const &sParametersFile); //load parameters file

	void startCalculation();

};

#endif /* EDCONTROLLER_H_ */
