/*
 * LatticeSymmetryClass.cpp
 *
 *  Created on: Sep 11, 2014
 *      Author: ixert
 */

#include "LatticeSymmetryClass.h"
#include<math.h>
#include<tuple>
#include "functions.h"

using namespace std;
unsigned int LatticeSymmetryClass::xMax=0;
unsigned int LatticeSymmetryClass::yMax=0;
vector<tuple<int,int,int> > LatticeSymmetryClass::invSiteMap;
map<tuple<int,int,int>, unsigned int> LatticeSymmetryClass::siteMap;

vector<vector<vector<int> > > LatticeSymmetryClass::shiftedSites;

void LatticeSymmetryClass::initializeSiteMap(unsigned int const &xMaxValue, unsigned int const &yMaxValue, vector<pair<int,int> > const &invSiteMapValue, vector<pair<int,int> > const &interUnitCellVectorsValue, vector<pair<int,int> > const &intraUnitCellVectorsValue){
	// invSiteMapValue: all sites (from siteMapFile), whereas the index is the siteNumber and the pair is (x,y)-Coordinate
	// interUnitCellVectors contains the two interUCVectors and the intraUCVector (3rd element)

	xMax=xMaxValue;
	yMax=yMaxValue;
	// the Matrix is written as (a,b;c,d)
	int a=interUnitCellVectorsValue[0].first;
	int b=interUnitCellVectorsValue[1].first;
	int c=interUnitCellVectorsValue[0].second;
	int d=interUnitCellVectorsValue[1].second;
	int det=a*d-b*c;

	invSiteMap.resize(invSiteMapValue.size());
	//siteType.resize(invSiteMapValue.size());
	cout << interUnitCellVectorsValue[0].first << "*";
	cout << interUnitCellVectorsValue[1].second << "-";
	cout << interUnitCellVectorsValue[1].first << "*";
	cout << interUnitCellVectorsValue[0].second << "\n";
	cout << "det=" << det << "\n";

	for(unsigned int i=0;i<invSiteMapValue.size();i++){
		for(unsigned int j=0;j<intraUnitCellVectorsValue.size();j++){
			int vx=invSiteMapValue[i].first-intraUnitCellVectorsValue[j].first;
			int vy=invSiteMapValue[i].second-intraUnitCellVectorsValue[j].second;
			int xComp=d*vx-b*vy;
			int yComp=a*vy-c*vx;
			// check if this is possible without rest
			if(((xComp%det)==0) && ((yComp%det)==0)){
				get<0>(invSiteMap[i])=xComp/det;
				get<1>(invSiteMap[i])=yComp/det;
				get<2>(invSiteMap[i])=j;
			}
		}
	}

	for(unsigned int i=0;i<invSiteMapValue.size();i++){
		siteMap.insert(pair<tuple<int,int,int>,unsigned int>(invSiteMap[i],i));
	}


	//test output
	cout << "siteMap:\n";
	map<tuple<int,int,int>, unsigned int>::const_iterator itSiteMap;
	for(itSiteMap=siteMap.begin();itSiteMap!=siteMap.end();itSiteMap++){
		cout << get<0>(itSiteMap->first) << " " << get<1>(itSiteMap->first) << " " << get<2>(itSiteMap->first) << " -> " << itSiteMap->second << "\n";
	}
	cout << "invSiteMap:\n";
	for(unsigned int i=0;i<invSiteMap.size();i++){
		//for(unsigned int j=0;j<invSiteMap[i].size();j++){
			cout << get<0>(invSiteMap[i]) << " " << get<1>(invSiteMap[i]) << " " << get<2>(invSiteMap[i]);
		//}
		cout << "\n";
	}

	// shift all sites
	for(unsigned int i=0;i<invSiteMap.size();i++){
		vector<vector<int> > vviTemp;
		for(unsigned int jx=0;jx<xMax;jx++){
			vector<int> viTemp;
			for(unsigned int jy=0;jy<yMax;jy++){
				if(jx==0 && jy==0){
					viTemp.push_back(i);
				}
				else{
					tuple<int,int,int> translatedSite=invSiteMap[i];
					get<0>(translatedSite)+=jx;
					get<1>(translatedSite)+=jy;
					// periodic boundary conditions!
					while(get<0>(translatedSite)>=xMax){
						get<0>(translatedSite)-=xMax;
					}
					while(get<1>(translatedSite)>=yMax){
						get<1>(translatedSite)-=yMax;
					}
					int j=siteMap[translatedSite];
					viTemp.push_back(j);
				}
			}
			vviTemp.push_back(viTemp);
		}
		shiftedSites.push_back(vviTemp);
	}

	//test output for shifted sites
	for(unsigned int i=0;i<shiftedSites.size();i++){
		for(unsigned int jx=0;jx<shiftedSites[i].size();jx++){
			for(unsigned int jy=0;jy<shiftedSites[i][jx].size();jy++){
				cout << i << " --(" << jx << "," << jy << ")--> " << shiftedSites[i][jx][jy] << "\n";
			}
		}
	}
}

void LatticeSymmetryClass::initializeShiftedSitesForReflectionSymmetry(vector<pair<int,int> > const &reflectionMap){
	// run over the reflection map and perform
	shiftedSites.resize(reflectionMap.size(),vector<vector<int> >(2,vector<int>(2)));
	for(unsigned int i=0;i<shiftedSites.size();i++){
		shiftedSites[i][0][0]=i;
		shiftedSites[i][0][1]=reflectionMap[i].second;
		shiftedSites[i][1][0]=reflectionMap[i].first;
		shiftedSites[i][1][1]=reflectionMap[reflectionMap[i].first].second;
	}

	//test output for shifted sites
	for(unsigned int i=0;i<shiftedSites.size();i++){
		for(unsigned int jx=0;jx<shiftedSites[i].size();jx++){
			for(unsigned int jy=0;jy<shiftedSites[i][jx].size();jy++){
				cout << i << " --(" << jx << "," << jy << ")--> " << shiftedSites[i][jx][jy] << "\n";
			}
		}
	}
}

unsigned int LatticeSymmetryClass::getxMax(){
	return xMax;
}
unsigned int LatticeSymmetryClass::getyMax(){
	return yMax;
}
