/*
 * LocalSpinModelClassTemplate.h
 *
 *  Created on: Dec 4, 2014
 *      Author: ixert
 */

#ifndef LOCALSPINMODELCLASSTEMPLATE_H_
#define LOCALSPINMODELCLASSTEMPLATE_H_

#include "nr3.h"
#include<list>
#include<utility>
#include "SeriesTemplate.h"
#include "SpinModelClassTemplate.h"
#include<iostream>
#include<map>
#include "LatticeSymmetryClass.h"
#include "SpinStateTemplate.h"

template<class S,class T>
class SpinStateTemplate;

using namespace std;

template<class S, class T>
class LocalSpinModelClassTemplate {
template<class Q,class R>
friend istream& operator>>(istream &o, LocalSpinModelClassTemplate<Q,R> &s);
template<class Q,class R>
friend ostream& operator<<(ostream &o, LocalSpinModelClassTemplate<Q,R> &s);

protected:
	map<S,list<pair<T,S> > > localSpinModel;

	//static set<S> allLocalStates;

public:
	LocalSpinModelClassTemplate();
	LocalSpinModelClassTemplate(map<S,list<pair<T,S> > > const &mapValue);
	LocalSpinModelClassTemplate(SpinModelClassTemplate<T> const &spinModel);

	void addElement(S const &inputState,list<pair<T,S> > const &outputState);

	void applyLocalSpinModelOnBitStateOnSites(vector<T> &newStateCont, S const &state, T const &prefactor, vector<unsigned short> const &sites, int const &kx, int const &ky, int const &spinFlip, int const &refx, int const &refy, unsigned int const &repMethod) const;


	LocalSpinModelClassTemplate<stateInt,Complex> evaluateLocalSpinModel(vector<Doub> const &values);
};

//template<class S,class T>
//set<S> LocalSpinModelClassTemplate<S,T>::allLocalStates=set<S>();

// class function implementation //

template<class S,class T>
LocalSpinModelClassTemplate<S,T>::LocalSpinModelClassTemplate(){
}
template<class S,class T>
LocalSpinModelClassTemplate<S,T>::LocalSpinModelClassTemplate(map<S,list<pair<T,S> > > const &mapValue){
	localSpinModel=mapValue;
}
template<class S,class T>
LocalSpinModelClassTemplate<S,T>::LocalSpinModelClassTemplate(SpinModelClassTemplate<T> const &spinModel){
	//cout << "spinModel:\n";
	//cout << spinModel << "\n";

	// 1) find number of sites
	int siteNumber=spinModel.numberOfOperators();
	if(siteNumber<0){
		cout << "Number of Sites is not the same in this SpinModel!\n";
	}
	else{
		// 2) create all spinStates
		// create maximum state
		S maxState=(S(1)<<(localHilbertSpace*siteNumber))-1;
		// create all States up to the maximum State
		for(S state=S(0);state<=maxState;state++){
			localSpinModel.insert(pair<S,list<pair<T,S> > >(state,list<pair<T,S> >()));
		}
		// 3) act with the spinModel on all of this States
		typename map<S,list<pair<T,S> > >::iterator localSpinModelIt;
		for(localSpinModelIt=localSpinModel.begin();localSpinModelIt!=localSpinModel.end();localSpinModelIt++){
			map<S,T> sum;
			spinModel.applyOperatorSequenceOnBitState(sum,localSpinModelIt->first,T(1));
			// add the sum to the localSpinModel
			typename map<S,T>::iterator sumIt;
			for(sumIt=sum.begin();sumIt!=sum.end();sumIt++){
				// try to find the state within the list
				typename list<pair<T,S> >::iterator itList;
				bool foundState=false;
				for(itList=localSpinModelIt->second.begin();itList!=localSpinModelIt->second.end();itList++){
					if(itList->second==sumIt->first){
						itList->first+=sumIt->second;
						foundState=true;
						break;
					}
				}
				// if the state is not found insert it into the list
				if(!foundState){
					localSpinModelIt->second.push_back(pair<T,S>(sumIt->second,sumIt->first));
				}
			}
		}
	}
}

template<class S,class T>
void LocalSpinModelClassTemplate<S,T>::addElement(S const &inputState,list<pair<T,S> > const &outputState){
	 pair<typename map<S,list<pair<T,S> > >::iterator,bool> ret;
	 ret=localSpinModel.insert(pair<S,list<pair<T,S> > >(inputState,outputState));
	 if(!ret.second){
		 cout << "could not insert element!\n";
	 }
}

template<class S,class T>
void LocalSpinModelClassTemplate<S,T>::applyLocalSpinModelOnBitStateOnSites(vector<T> &newStateCont, S const &state, T const &prefactor, vector<unsigned short> const &sites, int const &kx, int const &ky, int const &spinFlip, int const &refx, int const &refy, unsigned int const &repMethod) const{
	// 1) extract sites from state
	S localState=S(0);
	S remainingState=state;
	S fullLocalState=(S(1)<<localHilbertSpace)-1;
	for(unsigned int i=0;i<sites.size();i++){
		S extractor=(fullLocalState<<(localHilbertSpace*sites[i]));
		S extractedBit=state & extractor;
		remainingState^=extractedBit;
		extractedBit>>=(localHilbertSpace*sites[i]);
		extractedBit<<=(localHilbertSpace*i);
		localState|=extractedBit;
	}

	// 2) find localState in spinModel and apply all operators with assigned sites
	typename map<S,list<pair<T,S> > >::const_iterator itMap=localSpinModel.find(localState);
	if(itMap!=localSpinModel.end()){
		// 3) shift all states in the sum to the correct positions
		typename list<pair<T,S> >::const_iterator itList;
		for(itList=itMap->second.begin();itList!=itMap->second.end();itList++){
			S tempState=S(0);
			for(unsigned int i=0;i<sites.size();i++){
				S extractor=(fullLocalState<<(localHilbertSpace*i));
				S extractedBit=itList->second & extractor;
				extractedBit>>=(localHilbertSpace*i);
				extractedBit<<=(localHilbertSpace*sites[i]);
				tempState|=extractedBit;
			}
			S resultingState=tempState | remainingState;

			// use the corresponding repMethod for finding the representativeIndex
			switch(repMethod){
				case 1:{
				}
				case 2:{
					// 4) find representative and phase for the resulting state
					// 4a) create from resultingState the sublattice states
					subStateInt resultingState_A, resultingState_B;
					getSublatticeStates(resultingState, resultingState_A, resultingState_B);

					// 4b) find representative and translation of sublattice states
					unsigned int ix,iy;
					bool spinFlip_i;
					unsigned int repIndex=SpinStateTemplate<S,T>::getRepresentativeIndex(resultingState_A,resultingState_B,ix,iy,spinFlip_i);

					if(repIndex!=-1){
						T phase;
						LatticeSymmetryClass::calculatePhase(ix,iy,kx,ky,phase);
						if(spinFlip && spinFlip_i){
							phase*=T(spinFlip);
						}

						// 5) add phase*prefactor*norm*spinCoupling to the stateVector
						newStateCont[repIndex]+=conj(phase)*prefactor*T(SpinStateTemplate<S,T>::getNormalizationFactor(repIndex))*itList->first;
					}
					break;
				}
				case 3:{
					// 4) index of resulting state is just the integer of the state
					newStateCont[resultingState]+=prefactor*itList->first;
					break;
				}
				case 4:{
					// 4) find index of resulting state
					unsigned int repIndex=SpinStateTemplate<S,T>::getStateToRepresentativeIndex(resultingState);
					newStateCont[repIndex]+=prefactor*itList->first;
					break;
				}
				case 5:{
					unsigned int repIndex=SpinStateTemplate<S,T>::getStateToRepresentativeIndex(resultingState);
					if(repIndex!=-1){
						unsigned int ix=SpinStateTemplate<S,T>::getStateToTranslationNumber_x(resultingState);
						unsigned int iy=SpinStateTemplate<S,T>::getStateToTranslationNumber_y(resultingState);
						bool spinFlip_i=SpinStateTemplate<S,T>::getStateToTranslationNumber_spinFlip(resultingState);
						T phase;
						LatticeSymmetryClass::calculatePhase(ix,iy,kx,ky,phase);
						if(spinFlip && spinFlip_i){
							phase*=T(spinFlip);
						}
						newStateCont[repIndex]+=conj(phase)*prefactor*T(SpinStateTemplate<S,T>::getNormalizationFactor(repIndex))*itList->first;
					}
					break;
				}
				case 6:{
					unsigned int repIndex=SpinStateTemplate<S,T>::getStateToRepresentativeIndex(resultingState);
					//cout << "repIndex=" << repIndex << "\n";
					int rx=SpinStateTemplate<S,T>::getStateToTranslationNumber_x(resultingState);
					int ry=SpinStateTemplate<S,T>::getStateToTranslationNumber_y(resultingState);
					if(repIndex!=-1){
						T phase=(1+(refx-1)*rx)*(1+(refy-1)*ry);
						//cout << "phase=" << phase << "\n";
						newStateCont[repIndex]+=phase*prefactor*T(SpinStateTemplate<S,T>::getNormalizationFactor(repIndex))*itList->first;
					}
					break;
				}
				case 7:{
					// 4) find index of resulting state
					unsigned int repIndex=convertToBasis(resultingState);
					newStateCont[repIndex]+=prefactor*itList->first;
					break;
				}
				case 8:{
					subStateInt resultingState_A, resultingState_B;
					getSublatticeStates(resultingState, resultingState_A, resultingState_B);
					unsigned int Ja=SpinStateTemplate<S,T>::getStateToTranslationNumber_x(resultingState_A);
					unsigned int Jb=SpinStateTemplate<S,T>::getStateToTranslationNumber_y(resultingState_B);
					unsigned int repIndex=Ja+Jb;
					newStateCont[repIndex]+=prefactor*itList->first;
					break;
				}
			}
		}
	}
	//else{
	//	cout << "could not find localState within localSpinModel...\n";
	//	cout << "localState=" << localState << "\n";
	//}
}

template<class S,class T>
LocalSpinModelClassTemplate<stateInt,Complex> LocalSpinModelClassTemplate<S,T>::evaluateLocalSpinModel(vector<Doub> const &values){
	LocalSpinModelClassTemplate<stateInt,Complex> evaluatedLocalSpinModel;
	typename map<S,list<pair<T,S> > >::iterator itMap;
	for(itMap=localSpinModel.begin();itMap!=localSpinModel.end();++itMap){
		list<pair<Complex,stateInt> > evaluatedList;
		typename list<pair<T,S> >::iterator itList;
		for(itList=itMap->second.begin();itList!=itMap->second.end();++itList){
			evaluatedList.push_back(pair<Complex,stateInt>(Complex(itList->first),itList->second));
		}
		evaluatedLocalSpinModel.addElement(itMap->first,evaluatedList);
	}
	return evaluatedLocalSpinModel;
}

template<>
inline LocalSpinModelClassTemplate<stateInt,Complex> LocalSpinModelClassTemplate<stateInt,SeriesDouble>::evaluateLocalSpinModel(vector<Doub> const &values){
	LocalSpinModelClassTemplate<stateInt,Complex> evaluatedLocalSpinModel;
	map<stateInt,list<pair<SeriesDouble,stateInt> > >::iterator itMap;
	for(itMap=localSpinModel.begin();itMap!=localSpinModel.end();++itMap){
		list<pair<Complex,stateInt> > evaluatedList;
		list<pair<SeriesDouble,stateInt> >::iterator itList;
		for(itList=itMap->second.begin();itList!=itMap->second.end();++itList){
			Complex evaluatedSeries(itList->first.evaluate(values));
			evaluatedList.push_back(pair<Complex,stateInt>(evaluatedSeries,itList->second));
		}
		evaluatedLocalSpinModel.addElement(itMap->first,evaluatedList);
	}
	return evaluatedLocalSpinModel;
}

template<>
inline LocalSpinModelClassTemplate<stateInt,Complex> LocalSpinModelClassTemplate<stateInt,pair<SeriesDouble,SeriesDouble> >::evaluateLocalSpinModel(vector<Doub> const &values){
	LocalSpinModelClassTemplate<stateInt,Complex> evaluatedLocalSpinModel;
	map<stateInt,list<pair<pair<SeriesDouble,SeriesDouble>,stateInt> > >::iterator itMap;
	for(itMap=localSpinModel.begin();itMap!=localSpinModel.end();++itMap){
		list<pair<Complex,stateInt> > evaluatedList;
		list<pair<pair<SeriesDouble,SeriesDouble>,stateInt> >::iterator itList;
		for(itList=itMap->second.begin();itList!=itMap->second.end();++itList){
			Complex evaluatedSeries(itList->first.first.evaluate(values),itList->first.second.evaluate(values));
			evaluatedList.push_back(pair<Complex,stateInt>(evaluatedSeries,itList->second));
		}
		evaluatedLocalSpinModel.addElement(itMap->first,evaluatedList);
	}
	return evaluatedLocalSpinModel;
}

template<class S,class T>
istream& operator>>(istream &o, LocalSpinModelClassTemplate<S,T> &s){
	string tempString;
	char tempChar;
	S state1,state2;
	T tempValue;
	while(o.peek()=='|'){
		getline(o,tempString);
		stringstream iss(tempString);
		iss>>tempChar; // |
		iss>>state1;
		iss>>tempChar; // >
		iss>>skipws>>tempChar; // -
		iss>>skipws>>tempChar; // >
		iss>>noskipws;
		list<pair<T,S> > tempList;
		while(iss.peek()==' '){
			iss>>skipws>>tempValue;
			iss>>skipws>>tempChar; // |
			iss>>state2;
			iss>>tempChar; // >
			iss>>noskipws;
			tempList.push_back(pair<T,S>(tempValue,state2));
		}
		s.localSpinModel.insert(pair<S,list<pair<T,S> > >(state1,tempList));
	}
	return o;
}

template<class S,class T>
ostream& operator<<(ostream &o, LocalSpinModelClassTemplate<S,T> &s){
	typename map<S,list<pair<T,S> > >::iterator localSpinModelIt;
	for(localSpinModelIt=s.localSpinModel.begin();localSpinModelIt!=s.localSpinModel.end();localSpinModelIt++){
		o << "|" << localSpinModelIt->first << "> ->";
		typename list<pair<T,S> >::iterator itList;
		for(itList=localSpinModelIt->second.begin();itList!=localSpinModelIt->second.end();itList++){
			o << " " << itList->first << " |" << itList->second << ">";
		}
		o << "\n";
	}
	return o;
}

#endif /* LOCALSPINMODELCLASSTEMPLATE_H_ */
