/*
 * SpinOperatorsOnSites.cpp
 *
 *  Created on: 02.09.2014
 *      Author: dominik-ixert
 */

#include "SpinOperatorsOnSites.h"
#include<iostream>

using namespace std;

// class function implementation //
SpinOperatorsOnSites::SpinOperatorsOnSites(){
}

unsigned int SpinOperatorsOnSites::numberOfOperators() const{
	return operatorSequence.size();
}

vector<unsigned short> SpinOperatorsOnSites::getSites(){
	vector<unsigned short> tempVec(operatorSequence.size());
	for(unsigned int i=0;i<operatorSequence.size();i++){
		tempVec[i]=operatorSequence[i].first;
	}
	return tempVec;
}

bool SpinOperatorsOnSites::areSameSites(vector<unsigned short> const &sites){
	if(sites.size()!=operatorSequence.size()){
		return false;
	}
	for(unsigned int i=0;i<operatorSequence.size();i++){
		if(sites[i]!=operatorSequence[i].first){
			return false;
		}
	}
	return true;
}
bool SpinOperatorsOnSites::containsSite(unsigned short const &site){
	for(unsigned int i=0;i<operatorSequence.size();i++){
		if(operatorSequence[i].first==site){
			return true;
		}
	}
	return false;
}

SpinOperatorsOnSites SpinOperatorsOnSites::permutate(vector<unsigned short> const &positions){
	// positions are the new operator positions, e.g. positions={2 3 4 1} means: op1*op2*op3*op4 -> op2*op3*op4*op1
	SpinOperatorsOnSites soRet;
	if(positions.size()!=operatorSequence.size()){
		cout << "SpinOperators::permutate : positions.size() does not match number of operators!\n";
		return *this;
	}
	for(unsigned int i=0;i<positions.size();i++){
		soRet.operatorSequence.push_back(operatorSequence[positions[i]]);
	}
	return soRet;
}

void SpinOperatorsOnSites::assignNewSites(){
	//this function assigns the new sites 0,1,2,..
	for(unsigned short i=0;i<operatorSequence.size();i++){
		operatorSequence[i].first=i;
	}
}
void SpinOperatorsOnSites::assignNewSites(vector<unsigned short> const &newSites){
	//this function sets the sites to the newSites, BUT does not order them!
	if(newSites.size()!=operatorSequence.size()){
		cerr << "SpinOperators::assignNewSites : newSites.size() does not match number of operators!\n";
	}
	else{
		for(unsigned int i=0;i<newSites.size();i++){
			operatorSequence[i].first=newSites[i];
		}
	}
}
void SpinOperatorsOnSites::assignNewSites(vector<unsigned short> const &oldSites, vector<unsigned short> const &newSites){
	//this function sets the sites in oldSites to the sites in newSites, but does not order them.
	for(unsigned int i=0;i<operatorSequence.size();i++){
		for(unsigned int j=0;j<oldSites.size();j++){
			if(operatorSequence[i].first==oldSites[j]){
				operatorSequence[i].first=newSites[j];
				break;
			}
		}
	}
}

void SpinOperatorsOnSites::orderAgainstSites(){
	// this function orders the operatorSequence in ascending order with respect to the siteNumbers
	SpinOperatorsOnSites soRet;
	for(unsigned int i=0;i<operatorSequence.size();i++){
		unsigned short minSite=operatorSequence[i].first;
		unsigned int minSiteIndex=i;
		for(unsigned int j=i+1;j<operatorSequence.size();j++){
			if(operatorSequence[j].first<minSite){
				minSite=operatorSequence[j].first;
				minSiteIndex=j;
			}
		}
		if(minSiteIndex!=i){
			pair<unsigned short,char> tempPair=operatorSequence[minSiteIndex];
			operatorSequence[minSiteIndex]=operatorSequence[i];
			operatorSequence[i]=tempPair;
		}
	}
}

void SpinOperatorsOnSites::insertNewOperator(char const &opType, unsigned short const &opSite){
	operatorSequence.push_back(pair<unsigned short,char>(opSite,opType));
}

bool SpinOperatorsOnSites::isUnity(){
	// returns true if the Operator is the Unity-Operator on ONE SINGLE SITE!
	if(operatorSequence.size()==1){
		if(operatorSequence[0].second=='E'){
			return true;
		}
	}
	return false;
}

bool operator==(SpinOperatorsOnSites const& lhs, SpinOperatorsOnSites const& rhs){
	return (lhs.operatorSequence==rhs.operatorSequence);
}

// free function implementation //
bool operator != (SpinOperatorsOnSites  const& lhs, SpinOperatorsOnSites  const& rhs){
  return ! (lhs==rhs);
}
