/*
 * SpinOperators.h
 *
 *  Created on: Feb 3, 2014
 *      Author: ixert
 */

#ifndef SPINOPERATORS_H_
#define SPINOPERATORS_H_

#include<iostream>
#include<vector>

class SpinOperatorsOnSitesPMBasis;

using namespace std;

class SpinOperatorsOnSites {
friend bool operator==(SpinOperatorsOnSites const& lhs, SpinOperatorsOnSites const& rhs);
friend ostream& operator<<(ostream &o, SpinOperatorsOnSitesPMBasis &s);
protected:
	vector<pair<unsigned short, char> > operatorSequence; // unsigned short is site, char is Operator with p=Sp, m=Sm, x=Sx, y=Sy, z=Sz, E=E (where Sx=sigma_x=Sp+Sm, Sy=i*sigma_y=Sp-Sm, Sz=sigma_z)
	//02.09.2014: i.e. no complex amplitudes are here! therefore the prefactor could be ommited => the prefactor for the correct application of the operators is then in SpinModelClass => saving memory!
	//11.09.2014: this class is now parentClass of SpinOperatorsOnSitesXYBasis and SpinOperatorsOnSitesPMBasis => there subclasses implement the correct routines!
public:
	SpinOperatorsOnSites();
	unsigned int numberOfOperators() const;
	vector<unsigned short> getSites();
	bool areSameSites(vector<unsigned short> const &sites);
	bool containsSite(unsigned short const &site);
	SpinOperatorsOnSites permutate(vector<unsigned short> const &positions);
	void assignNewSites();
	void assignNewSites(vector<unsigned short> const &newSites);
	void assignNewSites(vector<unsigned short> const &oldSites, vector<unsigned short> const &newSites);
	void orderAgainstSites();

	void insertNewOperator(char const &opType, unsigned short const &opSite);

	bool isUnity();
};

// friend function declaration
bool operator==(SpinOperatorsOnSites const& lhs, SpinOperatorsOnSites const& rhs);
ostream& operator<<(ostream &o, SpinOperatorsOnSitesPMBasis &s);

// free function declaration //
bool operator != (SpinOperatorsOnSites  const& lhs, SpinOperatorsOnSites  const& rhs);


#endif /* SPINOPERATORS_H_ */
