/*
 * SpinOperatorsOnSitesPMBasis.h
 *
 *  Created on: Sep 11, 2014
 *      Author: ixert
 */

#ifndef SPINOPERATORSONSITESPMBASIS_H_
#define SPINOPERATORSONSITESPMBASIS_H_

#include "SpinOperatorsOnSites.h"
#include <math.h>
#include "globalVariables.h"

using namespace std;

class SpinOperatorsOnSitesPMBasis: public SpinOperatorsOnSites {
friend ostream& operator<<(ostream &o, SpinOperatorsOnSitesPMBasis const &s);
public:
	SpinOperatorsOnSitesPMBasis();

	template<class S>
	double applyOperatorSequenceOnBitState(S &state) const; // returns the sign of the state and changes the input state!
};

template<class S>
double SpinOperatorsOnSitesPMBasis::applyOperatorSequenceOnBitState(S &state) const{
	// returns the sign of the state and changes the input state!
	// applies the complete Operator Sequence (product of Sp, Sm, Sz and E)

	// Sz is defined as int(state)-2*totalS
	S fullLocalState=(S(1)<<localHilbertSpace)-1;

	//cout << "state=" << state << "\n";

	double vz=1;
	for(unsigned int i=0;i<operatorSequence.size();i++){
		// comment 10.09.14: this is the "wrong" order of application! BUT: as soon as all sites are different the operators commute, so... everything is fine!
		unsigned short k=operatorSequence[i].first;
		switch(operatorSequence[i].second){
			case 'p':{
				S shiftedState=(state>>(localHilbertSpace*k))&fullLocalState;
				int Sz=2*int(shiftedState)-totalS;
				int radikand=totalS*(totalS+2)-Sz*(Sz+2);
				if(radikand==0){
					return 0;
				}
				double ladderPrefactor=0.5*sqrt(radikand);
				//cout << "ladderPrefactor=" << ladderPrefactor
				vz*=ladderPrefactor;
				// now set the state to next higher one
				state^=(shiftedState<<(localHilbertSpace*k)); // this XOR deletes the localState at position k
				shiftedState+=S(1);
				state|=(shiftedState<<(localHilbertSpace*k)); // this OR sets the localState at position k to the next higher value
				break;
			}
			case 'm':{
				S shiftedState=(state>>(localHilbertSpace*k))&fullLocalState;
				int Sz=2*int(shiftedState)-totalS;
				int radikand=totalS*(totalS+2)-Sz*(Sz-2);
				if(radikand==0){
					return 0;
				}
				double ladderPrefactor=0.5*sqrt(radikand);
				vz*=ladderPrefactor;
				// now set the state to next lower one
				state^=(shiftedState<<(localHilbertSpace*k)); // this XOR deletes the localState at position k
				shiftedState-=S(1);
				state|=(shiftedState<<(localHilbertSpace*k)); // this OR sets the localState at position k to the next higher value
				break;
			}
			case 'z':{
				S shiftedState=(state>>(localHilbertSpace*k))&fullLocalState;
				int Sz=2*int(shiftedState)-totalS;
				if(Sz==0){
					return 0;
				}
				vz*=Sz;
				break;
			}
		}
	}
	return vz;
}


#endif /* SPINOPERATORSONSITESPMBASIS_H_ */

