/*
 * functions.h
 *
 *  Created on: 10.11.2011
 *      Author: ixert
 */

#ifndef FUNCTIONS_H_
#define FUNCTIONS_H_

#include<iostream>
#include<vector>
#include<time.h>
#include<tuple>
#include "globalVariables.h"
using namespace std;

void printProcessingTime(time_t startingTime);

template<class S>
S zipperProduct(S const &s1, S const &s2){
	// creates the zipper product of s1 and s2
	// for this version this is just res=s1s2
	// numberOfSites must be even (this function must only be used within the sublattice coding scheme for one or two site unit cells!)
	S res=s2;
	res<<=(localHilbertSpace*numberOfSites/2);
	res|=s1;
	return res;
}

template<class S>
inline void getSublatticeStates(S const &state, subStateInt &resultingState_A, subStateInt &resultingState_B){
	// this function is the inverse of the zipper product and returns the sublattice states of state
	// again, this function assumes that numberOfSites is even
	unsigned int shiftVar=localHilbertSpace*numberOfSites/2;
	S lowerMask=(1<<shiftVar)-1;
	resultingState_A=state&lowerMask;
	resultingState_B=((state&(~lowerMask))>>shiftVar);
}

template<class S>
unsigned int calculateNumberOfSetBits(S v){
	// calculate the number of set bits in v
	unsigned int setBits=0;
	while(v){
		setBits++;
		v=v&(v-1);
	}
	return setBits;
}

template<class S>
bool calculateSz(S v, unsigned int const &numberOfSitesSub, int &Sz){
	// calculate Sz (without the factor 1/2) of the state v
	// this is the naive way in which one just loops over all sites
	// return true if the calculation was succesfull
	Sz=0;
	S fullLocalState=(S(1)<<localHilbertSpace)-1;
	S shiftedState;
	for(unsigned int i=0;i<numberOfSitesSub;i++){
		shiftedState=(v>>(localHilbertSpace*i))&fullLocalState;
		int temp=2*int(shiftedState)-totalS;
		if(temp>totalS){ // this is important for states which do not use the full local hilbertSpace (e.g. S=1 only needs 00,01,10 and not 11)
			//cout << "invalid state!\n";
			return false;
		}
		Sz+=temp;
	}
	return true;;
}

template<class S>
S nextHigherNumberWithSameNumberOfSetBits(S const & x){
	S c=(x&-x); // isolate lowest set 1
	S r=x+c; // adding lowest 1 makes a shift of all ones to the left
	return (((r^x)>>2)/c)|r;
}

template<class S>
inline unsigned int convertToBasis(S x){
	// this function converts the state x to a different basis
	// this is useful for obtaining the index of a state when no symmetries are taken into account and partialBits are used
	// e.g. the state 38=10 01 10 in a Spin-1 system is converted to 23=2 1 2 (basis 3); this is the index in the stateCont
	unsigned int index=0;
	unsigned int powerFactor=1;
	unsigned int multFactor=totalS+1;
	S fullLocalState=(S(1)<<localHilbertSpace)-1;
	while(x){
		unsigned int site=x&fullLocalState;
		index+=site*powerFactor;
		powerFactor*=multFactor;
		x>>=localHilbertSpace;
	}
	return index;
}

#endif /* FUNCTIONS_H_ */
