/*
 * ExampleSpecialLatticeModel.h
 *
 *  Created on: 04.04.2011
 *      Author: klagges
 */

#ifndef EXAMPLESPECIALLATTICEMODEL_H_
#define EXAMPLESPECIALLATTICEMODEL_H_

#include <Modules/ModelProvider/GeneralLatticeModel/GeneralLatticeModel.h>

/** An example implementation for a ModelProvider that inherits from the GeneralLatticeModel.
 *
 * If you have implemented a lattice model within the GenerlLatticeModel module and you want:
 *
 * - Encode additional information into a BaseState object to for example:
 *  - Exploit problem specific knowledge to speed up the calculation.
 *  - Keep track of additional information for debugging or post processing.
 * - Want to change implementations of the GeneralLatticeModel for performance reasons.
 * - Implement additional functionality.
 *
 * you can create an own ModelProvider, that inherits from the GeneralLatticeModel.
 *
 * To create a ModelProvider, follow the instructions in
 * Modules/ModelProvider/TemplateModelProvider/TemplateModelProvider.h.
 *
 * In this class you can find an example how you inherit from the GeneralLatticeModel and overload
 * some of the methods.
 *
 * In this example additional information is added to the BaseStates to keep track of which bond
 * has been touched, and performance is improved by making sure that no bond is touched twice.
 *
 * It is intended to be used with the SpinLadder lattice model, with a bond file where matrix elements
 * between sites are calculated, where only a term with maximal order can occur. The
 * GeneralLatticeModel::BondFile::numberOfExtraBits value should match the number of bonds in the bond file.
 *
 * */
class ExampleSpecialLatticeModel: public GeneralLatticeModel {

	bool acceptInputState(const BaseState& inputState,
			const stateAmplitude& inputAmplitude, const Bond& bond);

	bool updateExtraInformation(BaseState& outputState,
			const BaseState& inputState,
			const stateAmplitude& inputAmplitude, const Bond& bond);

};

#endif /* EXAMPLESPECIALLATTICEMODEL_H_ */
